#!/bin/sh
# ----------------------------------------------------------------
# 	xqbridge - Shell-script for running the XQuark Bridge
# ----------------------------------------------------------------

# Platform check

case "`uname`" in
	CYGWIN | Darwin)
		echo "Windows environment is not supported by this script."
		echo "Please use xqbridge.bat instead."
		exit 1
		;;
	*)
		if [ -z "$JAVA_HOME" ]
		then
			JAVA_HOME=${JAVA_HOME}
		fi
		;;
esac

# XQuark Bridge home setting

if [ -z "$XQBRIDGE_HOME" ]
then
	scriptdir=`pwd`
	XQBRIDGE_HOME=`dirname "$0"`/..

	# make XQBRIDGE_HOME a fully-qualified, straight path
	XQBRIDGE_HOME=`cd "$XQBRIDGE_HOME" && pwd`

	cd $scriptdir
fi

# Locate JVM

if [ -n "$JAVA_HOME"  ]
then
	JAVA="$JAVA_HOME/bin/java"
else
    JAVA=java
	echo "WARNING: JAVA_HOME not set. Trying with java found in the execution path."
fi
 
if [ ! -x "$JAVA" ]
then
  echo "ERROR: ${JAVA} cannot be executed. Please check JAVA_HOME value."
  exit 1
fi

# Set classpath

CP=$XQBRIDGE_HOME/bin/:$XQBRIDGE_HOME/lib/xquark-bridge.jar:.
if [ -n "$XQ_CLASSPATH" ]
then
	CP=$CP:$XQ_CLASSPATH
fi
if [ -n "$CLASSPATH" ]
then
	CP=$CP:$CLASSPATH
fi

# Third-party libraries
CP=$CP:$XQBRIDGE_HOME/lib/crimson.jar
CP=$CP:$XQBRIDGE_HOME/lib/mysql-connector-java-3.0.9-stable-bin.jar
CP=$CP:$XQBRIDGE_HOME/lib/log4j-1.2.8.jar

# Run XQuark Bridge

"$JAVA" -classpath "$CP" org.xquark.bridge.Main "$@"
