/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Static utilities for relational attributes
 */
public final class AlgebraTools {

	private static final String RCSRevision = "$Revision: 1.14 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	private static Log log = LogFactory.getLog(AlgebraTools.class);

	public static List clone(Collection collection) throws CloneNotSupportedException {
		return cloneCopy(collection, new ArrayList());
	}

	public static List cloneCopy(Collection source, List dest) throws CloneNotSupportedException {
		Object obj = null;
		Iterator it = source.iterator();
		while (it.hasNext()) {
			obj = it.next();

			if (obj instanceof Expression) {
				dest.add(((Expression) obj).clone());
			} else if (obj instanceof MapItem) {
				dest.add(((MapItem) obj).clone());
			} else if (obj instanceof SqlType) {
				dest.add(((SqlType) obj).clone());
			} else if (obj instanceof String) {
				dest.add(obj);
			} else if (null == obj) {
				dest.add(null);
			} else {
				log.error("object to be cloned is not Expression" + obj);
				throw new CloneNotSupportedException("Error in cloning ArrayList" + obj);
			}
		}
		return dest;
	}

	public static List cloneCopyExpressionsWithoutDoubles(Collection source, List dest) throws CloneNotSupportedException {
		Expression expr = null;
		Iterator it = source.iterator();
		while (it.hasNext()) {
			expr = (Expression) it.next();

			if (!UnOpProject.hasNameConflict(expr, dest))
				dest.add(((Expression) expr).clone());
		}

		return dest;
	}

	public static List clone(List list, Map clonedExprs) throws CloneNotSupportedException {
		if (null == list) {
			return null;
		}
		List retVal = new ArrayList();

		for (int i = 0; i < list.size(); i++) {
			Object obj = list.get(i);
		
			if (obj instanceof Expression) {
				retVal.add(((Expression) obj).clone(clonedExprs));
			} else if (obj instanceof MapItem) {
				retVal.add(((MapItem) obj).clone());
			} else if (obj instanceof SqlType) {
				retVal.add(((SqlType) obj).clone());
			} else if (obj instanceof String) {
				retVal.add(new String((String) obj));
			} else {
				log.error("object to be cloned is not Expression" + obj);
				throw new CloneNotSupportedException("Error in cloning ArrayList" + obj);
			}
		}

		return retVal;
	}

	/* Compares deeply expression without considering order */
	public static boolean areExprListEquivalent(List exprList1, List exprList2) {
		int len1 = exprList1.size(), len2 = exprList2.size();
		boolean ret = (len1 == len2), found;
		LinkedList optList2 = new LinkedList(exprList2);
		// clone is performed for optimization
		// and matching expression are removed (linked list is more efficient)
		Expression expr;
		ListIterator it;

		/* Browse the reference list */
		for (int i = 0; ret && i < len1; i++) {
			expr = (Expression) exprList1.get(i);
			it = optList2.listIterator();
			found = false;
			/* browse the compared cloned list and remove item that matches */
			while (it.hasNext()) {
				if (expr.deepEquals(it.next())) {
					it.remove();
					found = true;
				}
			}
			ret &= found;
		}
		return ret;
	}

	public static void applyVisitorOnCollection(AlgebraVisitor visitor, List exprList) {
		applyVisitorOnCollection(visitor, exprList, true);
	}

	public static void applyVisitorOnCollection(AlgebraVisitor visitor, List exprList, boolean preserve) {
		if (exprList != null) {
			ListIterator it = exprList.listIterator();
			while (it.hasNext()) {
				((Expression) it.next()).accept(visitor);
				if (!preserve)
					it.set(visitor.getVisitedExpression());
			}
		}
	}
}
