/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.xfunctions.*;

public interface AlgebraVisitor {

    public Expression getVisitedExpression();

	public void visit(Attribute arg) throws SqlWrapperException;

	public void visit(AttributeExpression arg) throws SqlWrapperException;

	public void visit(BinOpArithmetic arg) throws SqlWrapperException;

	public void visit(BinOpBoolean arg) throws SqlWrapperException;

	public void visit(BinOpCompare arg) throws SqlWrapperException;

	public void visit(BinOpCompareAny arg) throws SqlWrapperException;

	public void visit(BinOpDifference arg) throws SqlWrapperException;

	public void visit(BinOpIntersect arg) throws SqlWrapperException;

	public void visit(BinOpMerge arg) throws SqlWrapperException;

	public void visit(BinOpOuterJoin arg) throws SqlWrapperException;

	public void visit(BinOpUnion arg) throws SqlWrapperException;

	public void visit(DummyTable arg) throws SqlWrapperException;

    public void visit(ExternalVariable arg) throws SqlWrapperException;
    
	public void visit(UnOpExists arg) throws SqlWrapperException;

	public void visit(FunAggregate arg) throws SqlWrapperException;

	public void visit(IfThenElse arg) throws SqlWrapperException;

	public void visit(Join arg) throws SqlWrapperException;

	public void visit(LitBoolean arg) throws SqlWrapperException;

	public void visit(LitDate arg) throws SqlWrapperException;

	public void visit(LitDouble arg) throws SqlWrapperException;

	public void visit(LitFloat arg) throws SqlWrapperException;

	public void visit(LitDecimal arg) throws SqlWrapperException;

	public void visit(LitInteger arg) throws SqlWrapperException;

	public void visit(LitNull arg) throws SqlWrapperException;

	public void visit(LitList arg) throws SqlWrapperException;

	public void visit(LitString arg) throws SqlWrapperException;

	public void visit(OuterJoinPredicate arg) throws SqlWrapperException;

	public void visit(RenameItem arg) throws SqlWrapperException;

	public void visit(RenameRelation arg) throws SqlWrapperException;

	public void visit(SortSpecification arg) throws SqlWrapperException;

	public void visit(TupleExpression arg) throws SqlWrapperException;

	public void visit(Table arg) throws SqlWrapperException;

	public void visit(TempValue arg) throws SqlWrapperException;

	public void visit(ConstantTempTable arg) throws SqlWrapperException;

	public void visit(TempTable arg) throws SqlWrapperException;

	public void visit(UnOpAggregate arg) throws SqlWrapperException;

	public void visit(UnOpGroup arg) throws SqlWrapperException;

	public void visit(UnOpIsNull arg) throws SqlWrapperException;

	public void visit(UnOpMinus arg) throws SqlWrapperException;

	public void visit(UnOpNot arg) throws SqlWrapperException;

	public void visit(UnOpProject arg) throws SqlWrapperException;

	public void visit(UnOpRestrict arg) throws SqlWrapperException;

	public void visit(UnOpSort arg) throws SqlWrapperException;

	public void visit(AfCast arg) throws SqlWrapperException;

	public void visit(AfTrim arg) throws SqlWrapperException;

	public void visit(XfCast arg) throws SqlWrapperException;

	public void visit(XfAbs arg) throws SqlWrapperException;

	public void visit(XfCeiling arg) throws SqlWrapperException;

	public void visit(XfConcat arg) throws SqlWrapperException;

	public void visit(XfContains arg) throws SqlWrapperException;

	public void visit(XfCurrentDateTime arg) throws SqlWrapperException;

	public void visit(XfGetCenturyFromDate arg) throws SqlWrapperException;

	public void visit(XfGetHourFromTime arg) throws SqlWrapperException;

	public void visit(XfGetMinutesFromtime arg) throws SqlWrapperException;

	public void visit(XfGetSecondsFromtime arg) throws SqlWrapperException;

	public void visit(XfDate arg) throws SqlWrapperException;

	public void visit(XfDateTime arg) throws SqlWrapperException;

	public void visit(XfEndsWith arg) throws SqlWrapperException;

	public void visit(XfFloor arg) throws SqlWrapperException;

	public void visit(XfLowerCase arg) throws SqlWrapperException;

	public void visit(XfRound arg) throws SqlWrapperException;

	public void visit(XfStartsWith arg) throws SqlWrapperException;

	public void visit(XfStringLength arg) throws SqlWrapperException;

	public void visit(XfSubString arg) throws SqlWrapperException;

	public void visit(XfTime arg) throws SqlWrapperException;

	public void visit(XfUpperCase arg) throws SqlWrapperException;

}
