/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.MessageLibrary;
import org.xquark.extractor.common.SqlWrapperException;


public class AnalyzePredicateVisitor extends DefaultSimpleVisitor{
	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";

	AnalyzePredicateHandUpVisitor  _aphv = null;

	public AnalyzePredicateVisitor() {
	}

	public void reinit() {
		if (null != _aphv) {
			_aphv.reinit();
		}
		else {
			_aphv = new AnalyzePredicateHandUpVisitor();
		}

	}

	public void visit(UnOpRestrict arg) throws SqlWrapperException {
		//Trace.enter(this, "void visit(UnOpRestrict arg)", arg);
		super.visit((UnaryOperator) arg);

		Set visibleTI = arg.visibleTableInstances();
		List notSatisfiedPredicateList = new ArrayList();
		List list = arg.getPredicateList();
		Expression expr = null;
		Set referredTI = null;
		if (null != list) {
			/* find out not satisfied ( mal plac ) predicate */
			for (int i = list.size() - 1; i >=0  ; i--) {
				expr = (Expression)list.get(i);
				referredTI = expr.getReferredTableInstances();
				if (visibleTI != null /** @todo  */&& !visibleTI.containsAll(referredTI)) {
					//Trace.message(this, "visit(UnOpRestrict arg)", "predicat mal plac:");
					//Trace.message(this, "visit(UnOpRestrict arg)", expr);
					notSatisfiedPredicateList.add(expr);
					list.remove(i);
				}
			}

			Expression father = null;
			if (! notSatisfiedPredicateList.isEmpty()) {
					/* try to hand up the predicates*/
					_aphv.reinit();
					_aphv.setPredicateList(notSatisfiedPredicateList);
					father = arg.getFather();
					father.accept(_aphv);

					if (_aphv.isBlocked()) {

						resolveDistinct(arg, _aphv.getUnresolvedPredicateList());
					}
			}
		}
		//Trace.exit(this, "void visit(UnOpRestrict arg)", arg);
	}

	private void resolveDistinct( Expression origine, List predicateList) {
		//Trace.enter(this, "resolveDistinct( Expression origine, List predicateList)");

		Set visibleTableInstances = ((Relation)origine).visibleTableInstances();
		Set notVisibleTableInstances = getNotVisibleTableInstances(predicateList, visibleTableInstances);

		Expression breakPoint = findBreakPoint(origine, notVisibleTableInstances);
		Expression blockingBranch = findBlockingBranch(origine,notVisibleTableInstances);
		Expression mainBranch = extractMainBranch(breakPoint,notVisibleTableInstances);


		/* */
		Expression father = origine.getFather();

		Join join = new Join();
		join.setPredicateList(predicateList);
		join.addOperand(mainBranch);
		join.addOperand(origine);
		/* insert the join in argebra tree just above origine */
		father.replaceChild(origine, join);

		/* add items to blocking project distinct when neccessary */
		UnOpProject blockingNode = (UnOpProject)_aphv.getBlockingNode();

		Set notVisibleAttributeExpressions = getNotVisibleAttributeExpressions(predicateList, visibleTableInstances);

		AttributeExpression[] nVA = new AttributeExpression[notVisibleAttributeExpressions.size()];
		notVisibleAttributeExpressions.toArray(nVA);
		AttributeExpression[] mirror = new AttributeExpression[nVA.length];



		AttributeExpression attr = null;
		Expression          addedExpr = null;
		AttributeExpression newAttr = null;
		for (int i = 0 ; i < nVA.length; i++ ) {
			attr = (AttributeExpression)nVA[i];
			addedExpr = blockingNode.addItem(attr);
			newAttr = new AttributeExpression(null,addedExpr.getName());
			newAttr.setUnderlyingExpr(addedExpr);
			/** @todo newAttr.setOriginalXExpr() ?? */
			mirror[i] = newAttr;
		}

		upDateMirror(mirror, blockingNode.getFather(), breakPoint);
		/**/
		father = breakPoint.getFather();

		join = (Join)breakPoint;

		Expression predicate = null;
		for (int i = 0; i < mirror.length; i++) {
			predicate = new BinOpCompare(Constants.EQ_COMPOP, nVA[i], mirror[i]);
			join.addPredicate(predicate);
		}

		father.replaceChild(breakPoint,join);

		//Trace.exit(this, "resolveDistinct( Expression origine, List predicateList)");
	}


	private  Set getNotVisibleTableInstances(List exprList, Collection visibleTableInstances) {
		//Trace.enter(this, "getNotVisibleTableInstances(List exprList, Collection visibleTableInstances)");
		Set retVal = new HashSet();
		Expression expr = null;
		for (int i = 0; i < exprList.size(); i++) {
			expr = (Expression)exprList.get(i);
			retVal.addAll(expr.getReferredTableInstances());
		}
		retVal.removeAll(visibleTableInstances);

		//Trace.exit(this, "getNotVisibleTableInstances(List exprList, Collection visibleTableInstances)");
		return retVal;
	}

	private  Set getNotVisibleAttributeExpressions(List exprList, Collection visibleTableInstances) {
		//Trace.enter(this, "getNotVisibleAttributeExpressions(List exprList, Collection visibleTableInstances)");
		Set retVal = new HashSet();
		Expression expr = null;
		for (int i = 0; i < exprList.size(); i++) {
			expr = (Expression)exprList.get(i);
			retVal.addAll(expr.getReferredAttributes());
		}

		AttributeExpression attr = null;
		Object [] attrList = retVal.toArray();
		for (int i = 0; i < attrList.length; i++) {
			attr = (AttributeExpression)attrList[i];
			if (visibleTableInstances.contains(attr.getTableInstance())) {
				retVal.remove(attr);
			}
		}

		//Trace.exit(this, "getNotVisibleAttributeExpressions(List exprList, Collection visibleTableInstances)");
		return retVal;
	}

	private Expression findBreakPoint(Expression startNode, Collection tableInstances) {
		//Trace.enter(this, "findBreakPoint(Expression startNode, Collection tableInstances)");
		Expression retVal = startNode;
		while (!((Relation)retVal).visibleTableInstances().containsAll(tableInstances)) {
			retVal = ((Expression)retVal).getFather();
		}

		//Trace.exit(this, "findBreakPoint(Expression startNode, Collection tableInstances)");
		return retVal;
	}

	private Expression findBlockingBranch(Expression startNode, Collection notVisibleTableInstances) {
		//Trace.enter(this, "findBlockingBranch(Expression startNode, Collection notVisibleTableInstances)");
		Expression retVal = startNode;
		while (!((Relation)startNode).visibleTableInstances().containsAll(notVisibleTableInstances)) {
			retVal = startNode;
			startNode = ((Expression)startNode).getFather();
		}

		//Trace.exit(this, "findBlockingBranch(Expression startNode, Collection notVisibleTableInstances)");
		return retVal;
	}

	private Expression extractMainBranch(Expression breakPoint, Collection concernedTableInstances) {
		//Trace.enter(this, "extractMainBranch(Expression breakPoint, Set concernedTableInstances)");
		Expression retVal = null;
		if (breakPoint instanceof Join) {
			retVal = extractJoin((Join)breakPoint, concernedTableInstances);
		}
		else {
			Debug.nyi("");
		}

		//Trace.exit(this, "extractMainBranch(Expression breakPoint, Set concernedTableInstances)");
		return retVal;
	}

	private Expression extractJoin(Join breakPoint, Collection concernedTableInstances) {
		//Trace.enter(this, "extractMainBranch(Join breakPoint, Collection concernedTableInstances)");
		Expression retVal = null;
		List predicateList  = new ArrayList();
		List operandList    = new ArrayList();

		try {
			List list = null;
			Expression expr = null;

			list = breakPoint.getPredicateList();
			if ( null != list) {
				for (int i = 0; i < list.size(); i++) {
					expr = (Expression)list.get(i);
					if (concernedTableInstances.contains(expr.getReferredTableInstances())) {
						predicateList.add(expr.clone());
					}
				}
			}

			list = breakPoint.getOperandList();
			for (int i = 0; i < list.size(); i++) {
				expr = (Expression)list.get(i);
				if (concernedTableInstances.contains(expr)) {
					operandList.add(expr.clone());
				}
			}

		}
		catch (java.lang.CloneNotSupportedException ex) {
			throw new SqlWrapperException(MessageLibrary.getMessage("IE_ERR"),null);
		}

		if (1 < operandList.size()) {
			retVal = new Join(operandList, predicateList);
		}
		else if (1 == operandList.size() ) {
			if (0 < predicateList.size()) {
				retVal = new UnOpRestrict((Expression)operandList.get(0), predicateList);
			}
			else {
				retVal = (Expression)operandList.get(0);
			}
		}
		else {
			Debug.assertTrue(false, "Internal logic error");
		}

		//Trace.exit(this, "extractMainBranch(Join breakPoint, Collection concernedTableInstances)");
		return retVal;
	}

	private void  upDateMirror(AttributeExpression[] mirror, Expression start, Expression end) {
		//Trace.enter(this, "upDateMirror(AttributeExpression[] mirror, Expression start, Expression end)");

		Expression cursor = start;
		AttributeExpression attr = null;
		do {
			if (cursor instanceof RenameRelation) {
				for (int i = 0; i < mirror.length; i++) {
					attr = (AttributeExpression)mirror[i];
					attr.setTableInstance((RenameRelation)cursor);
				}
			}
//            else if () {
//
//            }
			else {
				Debug.nyi("upDateMirror()");
			}
			cursor = cursor.getFather();
		} while (cursor != end);

		//Trace.exit(this, "upDateMirror(AttributeExpression[] mirror, Expression start, Expression end)");
	}
}
