/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.List;
import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.jdbc.typing.DbType;

public class Attribute extends Expression
{
    private static final String RCSRevision = "$Revision: 1.9 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";


    protected org.xquark.extractor.metadata.Attribute     _attribute;

    /**
    @param attribute
    @roseuid 3BD1863001C7
     */
    public Attribute(org.xquark.extractor.metadata.Attribute attribute)
    {
        setAttribute(attribute);
    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        //Trace.enter(this, "clone(HashMap clonedExpr)");

        Object retVal = super.clone(clonedExprs);

        clonedExprs.put(this, retVal);
        //Trace.exit(this, "clone(HashMap clonedExpr)");
        return retVal;
    }

    /**
    Access method for the _attribute property.

    @return   the current value of the _attribute property
     */
    public org.xquark.extractor.metadata.Attribute getAttribute()
    {
        return _attribute;
    }

    /**
    Sets the value of the _attribute property.

    @param aAttribute the new value of the _attribute property
     */
    public void setAttribute(org.xquark.extractor.metadata.Attribute aAttribute)
    {
        _attribute = aAttribute;
    }

    public String getName () {
        return _attribute.getName();
    }

    public String getAlias () {
        return _attribute.getAlias();
    }

    public SqlType getType()
    {
        return _type;
    }

    public DbType getPrimitiveType()
    {
        return _attribute.getMappingInfo().getDBType();
    }

    public List getOperands()
    {
        return null;
    }

    public String getTableName()
    {
        return _attribute.getFather().getName();
    }

    public String getSchemaName()
    {
        return _attribute.getFather().getFather().getName();
    }

    public String getCatalogName()
    {
        return _attribute.getFather().getFather().getFather().getName();
    }

    public String getFullName()
    {
        return _attribute.getFullName();
    }

    public String pprint()
    {
        return getName();
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * The semantics chosen for equals of algebra nodes is "the same algebra operator
	 * hierarchy referring to the same database objects".
	 * @param o the expression to be compared with this.
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof Attribute)
		{
			Attribute att = (Attribute)o;
			return	_attribute.equals(att.getAttribute()); // NOTE: assuming metadata object are not duplicated...
		}
		return false;
	}
}
