/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public final class BinOpOuterJoin extends BinaryOperator implements Relation
{
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";

	protected int _joinType;
	protected List _predicateList;

	List _keys;

	public BinOpOuterJoin(Expression leftOperand, Expression rightOperand, int joinType, List predicateList)
	{
		super ( leftOperand, rightOperand );
		setJoinType  (joinType ) ;
		setPredicateList ( predicateList );
	}

	synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
	{
		BinOpOuterJoin retVal = (BinOpOuterJoin)super.clone(clonedExprs);
		retVal.setPredicateList(AlgebraTools.clone(getPredicateList(), clonedExprs));
		clonedExprs.put(this, retVal);
		return  retVal;
	}

	/**
	 * Access method for the _joinType property.
	 * 
	 * @return the current value of the _joinType property
	 */
	public int getJoinType() { return _joinType;}

	/**
	 * Sets the value of the _joinType property.
	 * 
	 * @param aJoinType
	 *            the new value of the _joinType property
	 */
	public void setJoinType(int aJoinType) { _joinType = aJoinType;}

	/**
	 * Access method for the _predicate property.
	 * 
	 * @return the current value of the _predicate property
	 */
	public List getPredicateList() { return _predicateList;}

	public void setPredicateList(List predicateList) {
		_predicateList = predicateList;

		if ( null != _predicateList )
		{
			Iterator    itr ;
			itr = _predicateList.iterator() ;
			while ( itr.hasNext () ) {
				( (Expression)itr.next() ).setFather(this) ;
			}
		}
	}


	public void addPredicate(Expression predicate)
	{
		if (predicate == null)
		   return;

		if (null == _predicateList)
			_predicateList = new ArrayList ();

		_predicateList.add(predicate);
		predicate.setFather(this);
	}


	public Set providedTableInstances(){
		return visibleTableInstances();
	}

	public AttributeExpression findNonNullAttribute() {
		AttributeExpression ret = ((Relation)_leftOperand).findNonNullAttribute();
		if (ret == null)
			ret = ((Relation)_rightOperand).findNonNullAttribute();
		return ret;
	}

	public boolean replaceChild(Expression oldChild, Expression newChild)
	{
		boolean retVal = false;
		List list = getPredicateList();

		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			if (expr.equals(oldChild)) {
				list.set(i, newChild);
				newChild.setFather(this);
				retVal = true;
				break;
			}
		}

		if (getLeftOperand().equals(oldChild)) {
			setLeftOperand(newChild);
			retVal = true;
		}
		else if (getRightOperand().equals(oldChild)) {
			setRightOperand(newChild);
			retVal = true;
		}
		return retVal;
	}


	public void setKeys(List keys) {
		  _keys = keys;
	}

	public List getKeys() {
	 return _keys;
	}

	public List nameTest(String name)
	{
		Debug.nyi("BinOpOuterJoin::nameTest(String name)");
		return ((Relation)getLeftOperand()).nameTest(name);
	}

	public Set participants() {
		Set retVal = new HashSet();
		BuildReferredAttributes visitor = new BuildReferredAttributes();
		Expression predicate = null;
		for (int i = 0; i < _predicateList.size(); i++) {
			predicate = (Expression)_predicateList.get(i);
			predicate.accept(visitor);
			retVal.addAll(predicate.getReferredTableInstances());
		}

		return retVal;
	}

	public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
	{
		return visitor.visit(this);
	}

	public void accept (AlgebraVisitor visitor) throws SqlWrapperException
	{
		visitor.visit(this);
	}
}







