/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.xfunctions.*;

/**
 *
 * This base implementation of {@link org.xquark.extractor.algebra.AlgebraVisitor}
 * perform a call to the super implementation for each algebra node visited.
 */
public abstract class DefaultAlgebraVisitor implements AlgebraVisitor {
    private static final String RCSRevision = "$Revision: 1.6 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

	protected Expression expr = null;

    public void visit(Expression arg) throws SqlWrapperException {
        expr = arg;
    }

    public void visit(Attribute arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(AttributeExpression arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(BinaryOperator arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(BinaryAtomicOp arg) throws SqlWrapperException {
        visit((BinaryOperator)arg);
    }

    public void visit(BinaryAlgebra arg) throws SqlWrapperException {
        visit((BinaryOperator)arg);
    }

    public void visit(BinOpArithmetic arg) throws SqlWrapperException {
        visit((BinaryAtomicOp)arg);
    }

    public void visit(BinOpBoolean arg) throws SqlWrapperException {
        visit((BinaryAtomicOp)arg);
    }

    public void visit(BinOpCompare arg) throws SqlWrapperException {
        visit((BinaryAtomicOp)arg);
    }

    public void visit(BinOpCompareAny arg) throws SqlWrapperException {
        visit((BinaryAtomicOp)arg);
    }

    public void visit(BinOpDifference arg) throws SqlWrapperException {
        visit((BinaryAlgebra)arg);
    }

    public void visit(BinOpIntersect arg) throws SqlWrapperException {
        visit((BinaryAlgebra)arg);
    }

    public void visit(BinOpMerge arg) throws SqlWrapperException {
        visit((BinaryAlgebra)arg);
    }

    public void visit(BinOpOuterJoin arg) throws SqlWrapperException {
        visit((BinaryOperator)arg);
    }

    public void visit(BinOpUnion arg) throws SqlWrapperException {
        visit((BinaryAlgebra)arg);
    }

    public void visit(DummyTable arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(FunAggregate arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(IfThenElse arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(Join arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(Literal arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(ExternalVariable arg) throws SqlWrapperException {
        visit((Literal)arg);
    }

    public void visit(LitBoolean arg) throws SqlWrapperException {
        visit((Literal)arg);
    }

    public void visit(LitDate arg) throws SqlWrapperException {
        visit((Literal)arg);
    }

    public void visit(LitDecimal arg) throws SqlWrapperException{
        visit((Literal)arg);
    }

    public void visit(LitDouble arg) throws SqlWrapperException {
        visit((Literal)arg);
    }

    public void visit(LitFloat arg) throws SqlWrapperException {
        visit((Literal)arg);
    }

    public void visit(LitInteger arg) throws SqlWrapperException {
        visit((Literal)arg);
    }

    public void visit(LitList arg) throws SqlWrapperException{
        visit((Expression)arg);
    }
    
    public void visit(LitNull arg) throws SqlWrapperException{
        visit((Literal)arg);
    }

    public void visit(LitString arg) throws SqlWrapperException {
        visit((Literal)arg);
    }


    public void visit(OuterJoinPredicate arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(RenameItem arg) throws SqlWrapperException {
        visit((UnaryAtomicOp)arg);
    }

    public void visit(RenameRelation arg) throws SqlWrapperException {
        visit((UnaryAlgebra)arg);
    }

    public void visit(SortSpecification arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(Table arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(TupleExpression arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(TempValue arg) throws SqlWrapperException {
        visit((UnaryOperator)arg);
    }

    public void visit(ConstantTempTable arg) throws SqlWrapperException {
        visit((Table)arg);
    }

    public void visit(UnaryOperator arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(UnaryAlgebra arg) throws SqlWrapperException {
        visit((UnaryOperator)arg);
    }

    public void visit(UnaryAtomicOp arg) throws SqlWrapperException {
        visit((UnaryOperator)arg);
    }

    public void visit(UnOpAggregate arg) throws SqlWrapperException {
        visit((UnOpProject)arg);
    }

    public void visit(UnOpExists arg) throws SqlWrapperException {
        visit((UnaryAtomicOp)arg);
    }

	public void visit(UnOpGroup arg) throws SqlWrapperException {
		visit((UnaryAlgebra)arg);
	}

	public void visit(TempTable arg) throws SqlWrapperException {
		visit((UnaryAlgebra)arg);
	}

    public void visit(UnOpIsNull arg) throws SqlWrapperException {
        visit((UnaryAtomicOp)arg);
    }

    public void visit(UnOpMinus arg) throws SqlWrapperException {
        visit((UnaryAtomicOp)arg);
    }

    public void visit(UnOpNot arg) throws SqlWrapperException {
        visit((UnaryAtomicOp)arg);
    }

    public void visit(UnOpProject arg) throws SqlWrapperException {
        visit((UnaryAlgebra)arg);
    }

    public void visit(UnOpRestrict arg) throws SqlWrapperException {
        visit((UnaryAlgebra)arg);
    }

    public void visit(UnOpSort arg) throws SqlWrapperException {
        visit((UnaryAlgebra)arg);
    }

    public void visit(XFunction arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(Xfunction_Boolean arg) throws SqlWrapperException {
        visit((XFunction)arg);
    }

    public void visit(Xfunction_Date arg) throws SqlWrapperException {
        visit((XFunction)arg);
    }

    public void visit(Xfunction_DateTime arg) throws SqlWrapperException {
        visit((XFunction)arg);
    }

    public void visit(Xfunction_Integer arg) throws SqlWrapperException {
        visit((XFunction)arg);
    }

    public void visit(Xfunction_String arg) throws SqlWrapperException {
        visit((XFunction)arg);
    }

    public void visit(AfTrim arg) throws SqlWrapperException {
        visit((Xfunction_String)arg);
    }

    public void visit(AfCast arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(XfCast arg) throws SqlWrapperException {
        visit((Expression)arg);
    }

    public void visit(XfAbs arg) throws SqlWrapperException{
        visit((XFunction)arg);
    }

    public void visit(XfCeiling arg) throws SqlWrapperException {
        visit((Xfunction_Integer)arg);
    }

    public void visit(XfConcat arg) throws SqlWrapperException {
        visit((Xfunction_String)arg);
    }

    public void visit(XfContains arg) throws SqlWrapperException {
        visit((Xfunction_Boolean)arg);
    }

    public void visit(XfCurrentDateTime arg) throws SqlWrapperException {
        visit((XFunction)arg);
    }

    public void visit(XfGetCenturyFromDate arg) throws SqlWrapperException {
        visit((Xfunction_Integer)arg);
    }

    public void visit(XfGetHourFromTime arg) throws SqlWrapperException {
        visit((Xfunction_Integer)arg);
    }

    public void visit(XfGetMinutesFromtime arg) throws SqlWrapperException {
        visit((Xfunction_Integer)arg);
    }

    public void visit(XfGetSecondsFromtime arg) throws SqlWrapperException {
        visit((Xfunction_Integer)arg);
    }

    public void visit(XfDate arg) throws SqlWrapperException{
        visit((XFunction)arg);
    }

    public void visit(XfDateTime arg) throws SqlWrapperException{
        visit((XFunction)arg);
    }

    public void visit(XfEndsWith arg) throws SqlWrapperException {
        visit((Xfunction_Boolean)arg);
    }

    public void visit(XfFloor arg) throws SqlWrapperException {
        visit((Xfunction_Integer)arg);
    }

    public void visit(XfLowerCase arg) throws SqlWrapperException {
        visit((Xfunction_String)arg);
    }

    public void visit(XfRound arg) throws SqlWrapperException {
        visit((Xfunction_Integer)arg);
    }

    public void visit(XfStartsWith arg) throws SqlWrapperException {
        visit((Xfunction_Boolean)arg);
    }

    public void visit(XfStringLength arg) throws SqlWrapperException {
        visit((Xfunction_Integer)arg);
    }

    public void visit(XfSubString arg) throws SqlWrapperException {
        visit((Xfunction_String)arg);
    }

    public void visit(XfTime arg) throws SqlWrapperException{
        visit((XFunction)arg);
    }

    public void visit(XfUpperCase arg) throws SqlWrapperException {
        visit((Xfunction_String)arg);
    }

    public Expression getVisitedExpression() {
        return expr;
    }
}
