/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.List;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.xfunctions.AfCast;
import org.xquark.extractor.xfunctions.XFunction;
import org.xquark.extractor.xfunctions.XfCast;

/**
 *
 * This base implementation of {@link org.xquark.extractor.algebra.AlgebraVisitor}
 * performs a recursive depth-first browsing of an algebra tree. Operands, items,
 * predicates, expressions, parameters and arguments are browsed.
 */
public class DefaultCompleteVisitor extends DefaultAlgebraVisitor{
    private static final String RCSRevision = "$Revision: 1.5 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";


    public void visit(BinaryOperator arg) throws SqlWrapperException {
        arg.getLeftOperand().accept(this);
        arg.getRightOperand().accept(this);
    }

    public void visit(BinOpOuterJoin arg) throws SqlWrapperException {
        arg.getLeftOperand().accept(this);
        arg.getRightOperand().accept(this);

        List list = arg.getPredicateList();
        Expression expr = null;
        if (null != list) {
            for (int i = 0; i < list.size(); i++) {
                expr = (Expression)list.get(i);
                expr.accept(this);
            }
        }
    }

    public void visit(FunAggregate arg) throws SqlWrapperException {
        arg.getOperand().accept(this);
    }

    public void visit(IfThenElse arg) throws SqlWrapperException {
        arg.getIf().accept(this);
        arg.getThen().accept(this);
        arg.getElse().accept(this);
    }

    public void visit(Join arg) throws SqlWrapperException {
        List list = arg.getOperandList();
        Expression expr = null;
        if (null != list) {
            for (int i = 0; i < list.size(); i++) {
                expr = (Expression)list.get(i);
                expr.accept(this);
            }
        }

        list = arg.getPredicateList();
        if (null != list) {
            for (int i = 0; i < list.size(); i++) {
                expr = (Expression)list.get(i);
                expr.accept(this);
            }
        }
    }

    public void visit(SortSpecification arg) throws SqlWrapperException {
        arg.getSortExpression().accept(this);
    }

    public void visit(TupleExpression arg) throws SqlWrapperException {
        List list = arg.getItemList();
        Expression expr = null;
        if (null != list) {
            for (int i = 0; i < list.size(); i++) {
                expr = (Expression)list.get(i);
                expr.accept(this);
            }
        }
    }

    public void visit(UnaryAlgebra arg) {
        arg.getOperand().accept(this);

        List list = arg.getParameterList();
        Expression expr = null;
        if (null != list) {
            for (int i = 0; i < list.size(); i++) {
                expr = (Expression)list.get(i);
                expr.accept(this);
            }
        }
    }

    public void visit(UnaryAtomicOp arg) {
        arg.getOperand().accept(this);
    }

    public void visit(AfCast arg) throws SqlWrapperException {
        arg.getExpression().accept(this);
    }

    public void visit(XfCast arg) throws SqlWrapperException {
        arg.getExpression().accept(this);
    }

    public void visit(XFunction arg)throws SqlWrapperException {
        List list = arg.getArguments();
        Expression expr = null;
        if (null != list) {
            for (int i = 0; i < list.size(); i++) {
                expr = (Expression)list.get(i);
                expr.accept(this);
            }
        }
    }
}
