/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.xquery.parser.XQueryExpression;

public abstract class Expression implements Cloneable, Constants
{
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";

	protected SqlType   _type;
	protected Expression _father;
	protected XQueryExpression _OrginalXExpr;
	protected Set _referredAttributes;
	protected Mapper _mapper;

	/**
	 * Access method for the _father property.
	 * 
	 * @return the current value of the _father property
	 */
	public org.xquark.extractor.algebra.Expression getFather()
	{
		return _father;
	}

	/**
	 * Sets the value of the _father property.
	 * 
	 * @param aFather
	 *            the new value of the _father property
	 */
	public void setFather(org.xquark.extractor.algebra.Expression aFather)
	{
		_father = aFather;
	}

	/**
	 * Access method for the _OrginalXExpr property.
	 * 
	 * @return the current value of the _OrginalXExpr property
	 */
	public XQueryExpression getOrginalXExpr()
	{
		return _OrginalXExpr;
	}

	/**
	 * Sets the value of the _OrginalXExpr property.
	 * 
	 * @param aOrginalXExpr
	 *            the new value of the _OrginalXExpr property
	 */
	public void setOrginalXExpr(XQueryExpression aOrginalXExpr)
	{
		_OrginalXExpr = aOrginalXExpr;
	}


	public void setType(SqlType type)
	{
		_type = type;
	}

	/**
	 * Access method for the _referredAttributes property.
	 * 
	 * @return the current value of the _referredAttributes property
	 */
	public Set getReferredAttributes()
	{
		return _referredAttributes;
	}

	/**
	 * Sets the value of the _referredAttributes property.
	 * 
	 * @param aReferredAttributes
	 *            the new value of the _referredAttributes property
	 */
	public void setReferredAttributes(Set aReferredAttributes)
	{
		_referredAttributes = aReferredAttributes;
	}

	/**
	 * Access method for the _mapper property.
	 * 
	 * @return the current value of the _mapper property
	 */
	public Mapper getMapper()
	{
		/* default implementation*/
		return _mapper;
	}

	/**
	 * Sets the value of the _mapper property.
	 * 
	 * @param aMapper
	 *            the new value of the _mapper property
	 */
	public void setMapper(Mapper aMapper)
	{
		_mapper = aMapper;
	}

	public abstract List getOperands();

	public SqlType getType()
	{
		return _type;
	}

	public List getChildren()
	{
		Debug.nyi("List getChildren()");
		return null;
	}

	public String pprint()
	{
		String name = this.getClass().getName();
		return name.substring(name.lastIndexOf('.') + 1);
	}

	public String toString() {
		return pprint();
	}

	// TODO: implement a deep clone without map
	public synchronized Object clone() throws CloneNotSupportedException
	{
		Object retVal = null;
		Map clonedExprs = new HashMap(500);
		retVal = clone(clonedExprs);
		clonedExprs.clear();
		clonedExprs = null;

		return  retVal;
	}

	synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
	{
		return super.clone();
	}

	public String getName()
	{
		return null;
	}

	public boolean replaceChild(Expression oldChild, Expression newChild)
	{
		Debug.assertTrue(false,"NYI!! for " + this.pprint());
		return false;
	}

	public abstract SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException;

	public abstract void accept (AlgebraVisitor visitor) throws SqlWrapperException;

	public Set getReferredTableInstances()
	{
		Set retVal = new HashSet();
		if (null != _referredAttributes) {
			Iterator iter = _referredAttributes.iterator();
			AttributeExpression item;
			while (iter.hasNext()) {
				item = (AttributeExpression)iter.next();
				retVal.add(item.getTableInstance());
			}
		}
		return retVal;
	}
	
	/**
	 * The semantics chosen for equals of algebra nodes is "the same algebra operator
	 * hierarchy (aliases ignored) referring to the same database objects".
	 * @param o the expression to be compared with this.
	 */
	public abstract boolean deepEquals(Object o);
}

