/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.jdbc.typing.MappingInfo;
import org.xquark.schema.SimpleType;

/**
 * Features an external variable.
 * NOTE: Not really correct to extend Literal, but in fact it is processed like
 * a literal
 */
public class ExternalVariable extends Literal {
    private String _expandedName = null;
    public MappingInfo _mappingInfo = null;
    
    public ExternalVariable(String ns, String localName, SimpleType sType) {
        _expandedName = getExpandedName(ns, localName);
        if (sType != null)
            _mappingInfo = new MappingInfo(sType);
    }

    public static String getExpandedName(String ns, String varName) {
        StringBuffer sb = new StringBuffer();
	    if (ns != null) {
	        sb.append('{');
	        sb.append(ns);
	        sb.append('}');
	    }
	    sb.append(varName);
	    return sb.toString();
	}
	
    public SqlExpression accept(GenSqlVisitor visitor) throws SqlWrapperException {
        return visitor.visit(this);
    }

    /**
     * @return Returns the _expandedName.
     */
    public String getExpandedName() {
        return _expandedName;
    }
    
    public MappingInfo getMappingInfo() {
        return _mappingInfo;
    }

    public SimpleType getSimpleType() {
        return _mappingInfo.getXMLType();
    }

    public void accept(AlgebraVisitor visitor) throws SqlWrapperException {
        visitor.visit(this);
    }

    public boolean deepEquals(Object o) {
		if (o instanceof ExternalVariable)
		{
		    ExternalVariable cast = (ExternalVariable)o;
			return _expandedName.equals(cast.getExpandedName());
		}
		return false;
    }
}
