/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.sql.Types;
import java.util.List;

import org.xquark.extractor.xfunctions.XfCast;
import org.xquark.jdbc.typing.DbType;


/**
 * Formats top-level projections to retrieve a schema valid output for dates.
 * Adds a cast to string on top of datetime top projection items.
 * UNUSED
 */
public final class FormatDateTime {

    private static final String RCSRevision = "$Revision: 1.9 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    public FormatDateTime() {
    }

    public static void formatDateTimes(Expression algebraTree)
    {
        //Trace.enter("FormatDateTime","formatDateTime(Expression algebraTree)");
        TopLevelProjectsVisitor  tlpv = new TopLevelProjectsVisitor();
        tlpv.reinit();
        algebraTree.accept(tlpv);
        List projectList = tlpv.getProjectList();
        UnOpProject project = null;
        for (int i = 0; i < projectList.size(); i++) {
            project = (UnOpProject)projectList.get(i);
            formatDateTimes(project);
        }
        //Trace.exit("FormatDateTime","formatDateTime(Expression algebraTree)");
    }

    private static void formatDateTimes(UnOpProject project)
    {
        //Trace.enter("FormatDateTime","formatDateTime(UnOpProject project)");

        List itemList = project.getItemList();
        Expression item = null;
        SqlTypeAtom type = null;
        DbType stringType = new DbType(Types.CHAR);
        String itemName = null;
        for (int i = 0; i < itemList.size(); i++) {
            item = (Expression)itemList.get(i);
            type = (SqlTypeAtom)item.getType();
            if (type.isDataTime()) {
                itemName = item.getName();
                if (item instanceof RenameItem) {
                    item = ((RenameItem)item).getOperand();
                }
                item = new XfCast(item,stringType);
                if (null != itemName) {
                    item = new RenameItem(item, itemName);
                }
                project.setItem(i, item);
            }
        }
        //Trace.exit("FormatDateTime","formatDateTime(UnOpProject project)");
    }











}
