/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public final class FunAggregate extends UnaryOperator
{

    private static final String RCSRevision = "$Revision: 1.5 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    public final static int AVG = 0;
    public final static int COUNT = 1;
    public final static int MAX = 2;
    public final static int MIN = 3;
    public final static int SUM = 4;

    public final static String[] AGGREGATE_FUNCTIONS = {
                    "AVG",
                    "COUNT",
                    "MAX",
                    "MIN",
                    "SUM"
                    };

    private boolean _distinct;
    private int _operator ;

    /**
     * @param operand
     * @param distinct
     * @roseuid 3AB864A901C7
     */
    public FunAggregate( int operator ,Expression operand, boolean distinct)
    {

        super ( operand );
        setDistinct (distinct );
        setOperator (operator) ;
    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        //Trace.enter(this, "clone(Map clonedExprs)");
        FunAggregate retVal = (FunAggregate)super.clone(clonedExprs);
        clonedExprs.put(this, retVal);
        //Trace.exit(this, "clone(Map clonedExprs)");
        return retVal;
    }

    /**
     * @return boolean
     * @roseuid 3AB87D14027A
     */
    public boolean getDistinct()
    {
        return _distinct ;
    }

    /**
     * @param distinct
     * @roseuid 3B288B1300CD
     */
    public void setDistinct(boolean distinct)
    {
        _distinct = distinct ;
    }
    public void setOperator ( int operator )
    {
        _operator = operator ;
    }

    public int getOperator ()
    {
        return _operator ;
    }


    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        //Trace.enter(this, "replaceChild(Expression oldChild, Expression newChild)");
        boolean retVal = false;

        if (getOperand().equals(oldChild)) {
            setOperand(newChild);
            retVal = true;
        }

        //Trace.exit(this, "replaceChild(Expression oldChild, Expression newChild)");
        return retVal;
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof FunAggregate)
		{
			FunAggregate cast = (FunAggregate)o;
			return super.deepEquals(o) && _distinct == cast.getDistinct()
						&& _operator == cast.getOperator();
		}
		return false;
	}
}
