/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public final class LitDouble extends Literal
{

    private static final String RCSRevision = "$Revision: 1.5 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    private Double _value;

    /**
     * @param value
     * @roseuid 3B277B3700C3
     */
    public LitDouble(Double value)
    {
        _value = value ;
    }

    public LitDouble(double value)
    {
        _value = new Double(value) ;
    }

//    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
//    {
//        Trace.enter(this, "clone(Map clonedExprs)");
//        LitDouble retVal = (LitDouble)super.clone();
//        retVal.setValue((getValue() == null) ? null : new Double(getValue().floatValue()));
//        clonedExpr.put(this, retVal);Trace.exit(this, "clone(Map clonedExprs)");return retVal;
//        return  retVal;
//    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        //Trace.enter(this, "clone(Map clonedExprs)");

        LitDouble retVal = (LitDouble)super.clone(clonedExprs);
        retVal.setValue((getValue() == null) ? null : new Double(getValue().doubleValue()));

        clonedExprs.put(this, retVal);
        //Trace.exit(this, "clone(Map clonedExprs)");
        return retVal;
    }

    /**
     * Access method for the _value property.
     *
     * @return   the current value of the _value property
     */
    public Double getValue()
    {
        return _value;
    }

    /**
     * Sets the value of the _value property.
     *
     * @param aValue the new value of the _value property
     */
    public void setValue(Double aValue)
    {
        _value = aValue;
    }

    public String pprint()
    {
        return  _value.toString();
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof LitDouble)
		{
			LitDouble cast = (LitDouble)o;
			return	 _value.equals(cast.getValue());
		}
		return false;
	}
}
