/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.xfunctions.AfCast;
import org.xquark.extractor.xfunctions.XFunction;
import org.xquark.extractor.xfunctions.XfCast;

public final class RemoveProjectReplaceVisitor extends DefaultAlgebraVisitor {
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	public RemoveProjectReplaceVisitor() {
	}

	public RemoveProjectReplaceVisitor(Map map) {
		setMap(map);
	}

	private Map _map;

	/**
	 * Access method for the _map property.
	 * 
	 * @return the current value of the _map property
	 */
	public Map getMap() {
		return _map;
	}

	/**
	 * Sets the value of the _map property.
	 * 
	 * @param aMapping
	 *            the new value of the _map property
	 */
	public void setMap(Map aMap) {
		_map = aMap;
	}

	public void updateTableInstance(RenameRelation tableInstance) {
		HashMap newMap = new HashMap();
		Set keySet = _map.keySet();
		Iterator iter = keySet.iterator();
		AttributeExpression item = null;
		Object value = null;
		while (iter.hasNext()) {
			item = (AttributeExpression) iter.next();
			value = _map.get(item);

			item.setTableInstance(tableInstance);
			newMap.put(item, value);
		}
		setMap(newMap);
	}

	public void visit(Expression arg) throws SqlWrapperException {
		Debug.assertTrue(false, "NYI!! for " + arg.pprint());
	}

	public void visit(Attribute arg) throws SqlWrapperException {
		Debug.assertTrue(false, "exceptional case:" + arg.toString());
	}

	public void visit(AttributeExpression arg) throws SqlWrapperException {
		Expression replacer = (Expression) _map.get(arg);
		if (null != replacer) {
			arg.getFather().replaceChild(arg, replacer);
		}
		else {
			Object tableInstance = _map.get(arg.getTableInstance());
			if (null == tableInstance) {
			} else if (tableInstance instanceof NullPointer) {
				arg.setTableInstance(null);
			} else {
				arg.setTableInstance((RenameRelation) tableInstance);
			}
		}

		replacer = (Expression) _map.get(arg.getUnderlyinExpr());
		if (null != replacer)
			arg.getFather().replaceChild(arg, replacer);
	}

	public void visit(BinaryAtomicOp arg) throws SqlWrapperException {
		arg.getLeftOperand().accept(this);
		arg.getRightOperand().accept(this);
	}

	public void visit(BinaryAlgebra arg) throws SqlWrapperException {
		/* implemention for Merg, Union, Difference and except, but not for BinOpOuterJoin */
		arg.getLeftOperand().accept(this);
		arg.getRightOperand().accept(this);
	}

	public void visit(BinOpOuterJoin arg) throws SqlWrapperException {
		List list = arg.getPredicateList();
		Debug.assertTrue(null != list, "null!=list");
		Expression expr = null;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression) list.get(i);
			expr.accept(this);
		}

		arg.getLeftOperand().accept(this);
		arg.getRightOperand().accept(this);
	}

	public void visit(FunAggregate arg) throws SqlWrapperException {
		arg.getOperand().accept(this);
	}

	public void visit(Join arg) throws SqlWrapperException {
		List list = arg.getPredicateList();
		Debug.assertTrue(null != list, "null!=list");
		Expression expr = null;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression) list.get(i);
			expr.accept(this);
		}

		list = arg.getOperands();
		Debug.assertTrue(null != list, "null!=list");
		expr = null;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression) list.get(i);
			expr.accept(this);
		}
	}

	public void visit(Literal arg) throws SqlWrapperException {}

	public void visit(RenameRelation arg) throws SqlWrapperException {
		arg.getOperand().accept(this);
	}

	public void visit(Table arg) throws SqlWrapperException {}

	public void visit(UnaryAtomicOp arg) throws SqlWrapperException {
		arg.getOperand().accept(this);
	}

	public void visit(UnOpProject arg) throws SqlWrapperException {
		List list = arg.getItemList();
		Debug.assertTrue(null != list, "null!=list");
		Expression expr = null;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression) list.get(i);
			expr.accept(this);
		}

		arg.getOperand().accept(this);
	}

	public void visit(UnOpRestrict arg) throws SqlWrapperException {
		List list = arg.getPredicateList();
		Debug.assertTrue(null != list, "null!=list");
		Expression expr = null;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression) list.get(i);
			expr.accept(this);
		}

		arg.getOperand().accept(this);
	}

	public void visit(UnOpSort arg) throws SqlWrapperException {
		List list = arg.getSortSpecificationList();
		Debug.assertTrue(null != list, "null!=list");
		SortSpecification spec = null;
		Expression expr = null;
		for (int i = 0; i < list.size(); i++) {
			spec = (SortSpecification) list.get(i);
			expr = spec.getSortExpression();
			expr.accept(this);
		}

		arg.getOperand().accept(this);
	}

	public void visit(AfCast arg) throws SqlWrapperException {
		arg.getExpression().accept(this);
	}

	public void visit(XfCast arg) throws SqlWrapperException {
		arg.getExpression().accept(this);
	}

	public void visit(XFunction arg) throws SqlWrapperException {
		if (arg.getArgumentNumber() > 0) {
			List argList = arg.getArguments();
			Object argument = null;
			for (int i = 0; i < argList.size(); i++) {
				argument = argList.get(i);
				if (argument instanceof Expression) {
					((Expression) argument).accept(this);
				}
			}
		}
	}

	public String printMap() {
		String newLine = System.getProperty("line.separator");
		StringBuffer retVal = new StringBuffer();
		retVal.append(newLine);
		retVal.append("<================ RemoveProjectReplaceVisitor ===========>");
		retVal.append(newLine);
		Object keyList[] = _map.keySet().toArray();
		Expression key = null;
		Object value = null;

		for (int i = 0; i < keyList.length; i++) {
			key = (Expression) keyList[i];
			retVal.append(Integer.toString(i));
			retVal.append("\t");

			retVal.append(key.pprint());
			retVal.append("\t");

			value = _map.get(key);
			if (value instanceof Expression) {
				retVal.append(((Expression) value).pprint());
			} else {
				retVal.append(value.toString());
			}
			retVal.append(newLine);
		}

		retVal.append("</================ RemoveProjectReplaceVisitor ===========>");

		return retVal.toString();
	}

}
