/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.Collection;

import org.xquark.extractor.xfunctions.XFunction;

/**
 * This visitor changes the relation name in AttributeExpressions by the substitution
 * one if it matches one of the provided relation names. It blocks on RenameRelation
 * nodes since it is not necessary to rename references to table that can not be seen.
 * This visitor may be used when using a temporary table to rename attribute references
 * in query using the temporary table.
 */
public class RenameAttributeExpressionVisitor extends DefaultCompleteVisitor {

	private Collection _relations;
	private RenameRelation _substitution;
	private boolean _blockOnRenameItems = false;
	private boolean _exclude = false;
	
	public RenameAttributeExpressionVisitor() {}
    
	public RenameAttributeExpressionVisitor(Collection renameRelations, RenameRelation substitution) {
		reinit(renameRelations, substitution);
	}
    
	public void reinit(Collection renameRelations, RenameRelation substitution,
	        boolean blockOnRenameItems, boolean exclude) {
		_relations = renameRelations;
		_substitution = substitution;
		_blockOnRenameItems = blockOnRenameItems;
		_exclude = exclude;
	}
	public void reinit(Collection renameRelations, RenameRelation substitution) {
		reinit(renameRelations, substitution, false, false);
	}
	
	public void reinit(RenameRelation substitution, boolean blockOnRenameItems) {
		reinit(null, substitution, blockOnRenameItems, false);
	}
	
	public void reinit(RenameRelation substitution) {
		reinit(substitution, false);
	}
	
	public Expression getVisitedExpression() {
	    return expr;
	}
    
    public void visit(AttributeExpression arg) {
        if (_relations == null || (_relations.contains(arg.getTableInstance()) == !_exclude))
    		arg.setTableInstance(_substitution);
        expr = arg;
    }
    
    public void visit(RenameItem arg) {
        if (_blockOnRenameItems) {
	        AttributeExpression attExpr = new AttributeExpression(_substitution, arg.getName());
	        attExpr.setUnderlyingExpr(arg.getOperand());
	        arg._father.replaceChild(arg, attExpr);
	        expr = attExpr;
        }
        else {
    	    super.visit((UnaryAtomicOp)arg);
            expr = arg;
        }
    }
    
    public void visit(RenameRelation arg) {	expr = arg;/* block */}

	public void visit(BinaryAtomicOp arg){
	    super.visit(arg);
        expr = arg;
	}
	
	public void visit(UnaryAtomicOp arg) {
	    super.visit(arg);
        expr = arg;
	}
	
	public void visit(XFunction arg) {
	    super.visit(arg);
        expr = arg;
	}
}
