/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import org.xquark.extractor.common.Debug;

public abstract class SqlType implements Cloneable
{
	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";

	protected int _multiplicity = ONE;
	public static int ONE = 1;
	public static int MANY = 2;

	public Object clone()
	{
		try {
			return super.clone();
		}
		catch (CloneNotSupportedException ex) {
			Debug.assertTrue(false, "error in implementation of clone");
			return null;
		}
	}

	/**
	Access method for the _multiplicity property.

	@return   the current value of the _multiplicity property
	 */
	public int getMultiplicity()
	{
		return _multiplicity;
	}

	/**
	Sets the value of the _multiplicity property.

	@param aMultiplicity the new value of the _multiplicity property
	 */
	public void setMultiplicity(int aMultiplicity)
	{
		_multiplicity = aMultiplicity;
	}

	public abstract boolean isRelation();

	/**
	@return boolean
	@roseuid 3B2786970092
	 */
	public boolean isColumn()
	{
	 return true;
	}

	/**
	@return boolean
	@roseuid 3B278A310386
	 */
	public abstract boolean isTuple();

	public abstract boolean isAtom();


	public abstract boolean isBoolean();
	public abstract boolean isNumeric();
	public abstract boolean isInteger();
	public abstract boolean isString();
	public abstract boolean isDataTime();
	public abstract String  pprint ();


	/**
	@param type
	@return boolean
	@roseuid 3B27520E0141
	 */
	public abstract boolean isCompatibleTo(SqlType type);
	public  boolean isComparableTo(SqlType type)  { /** @todo  */ return true;};
}
