/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */


package org.xquark.extractor.algebra;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xquark.extractor.common.Debug;

public final class SqlTypeStructure extends SqlType{
	protected String _name = null;
	protected List _attributes = null;

	private static Log log = LogFactory.getLog(SqlTypeStructure.class);

	public SqlTypeStructure()
	{
	}

	public SqlTypeStructure(String name, List attributes,int multiplicity )
	{
		setName(name);
		setAttributes (attributes);
		setMultiplicity (multiplicity);
	}

	public SqlTypeStructure(List attributes,int multiplicity )
	{
		setAttributes (attributes);
		setMultiplicity (multiplicity);
	}

	public SqlTypeStructure(List attributes )
	{
		setAttributes (attributes);
	}

	public Object clone() {
		//Trace.enter(this, "clone()");
		try {
			SqlTypeStructure retVal = (SqlTypeStructure)super.clone();
			if (null != _attributes) {
				retVal.setAttributes( AlgebraTools.clone(_attributes));
			}
			//Trace.exit(this, "clone()");
			return retVal;
		}
		catch (CloneNotSupportedException ex) {
			Debug.assertTrue(false, "error in implementation of clone");
			//Trace.exit(this, "clone()");
			return null;
		}

	}

	public String getName()
	{
		return _name;
	}

	public void setName(String aName)
	{
		_name = aName;
	}

	public List getAttributes()
	{
		return _attributes;
	}


	public void setAttributes(List aAttributes)
	{
		_attributes = aAttributes;
	}

	public void addAttribute (SqlTypeAtom atom )
	{
		if (null==_attributes){
			_attributes = new ArrayList();
		}
		_attributes.add(atom);
	}

	public void addAttributes (List attributes)
	{
		if (null==_attributes){
			_attributes = new ArrayList();
		}
		_attributes.addAll(attributes);
	}


	public List getTypes()
	{
		ArrayList retVal = new ArrayList();

		Iterator iter = _attributes.iterator();
		while (iter.hasNext()) {
			retVal.add( ((SqlTypeAtom)iter.next()).getType() );
		}
		return retVal;
	}


	public List getNames()
	{
		ArrayList retVal = new ArrayList();
		SqlTypeAtom   atom ;
		Iterator iter = _attributes.iterator();
		while (iter.hasNext()) {
			atom = (SqlTypeAtom)iter.next();
			retVal.add( atom.getName() );
		}
		return retVal;
	}

	public boolean isRelation()
	{
		return MANY == _multiplicity;
	}

	public boolean isTuple()
	{
		return ONE == _multiplicity;
	}

	public boolean isColumn()
	{
	 return false;
	}

	public boolean isAtom()
	{
	 return false;
	}

	public  boolean isBoolean()
	{
		Iterator iter = _attributes.iterator();
		SqlTypeAtom attr;
		while (iter.hasNext()) {
			attr = (SqlTypeAtom)iter.next();
			if (!attr.isBoolean()) {
				return false;
			}
		}
		return true ;
	};

	public  boolean isNumeric()
	{
		Iterator iter = _attributes.iterator();
		SqlTypeAtom attr;
		while (iter.hasNext()) {
			attr = (SqlTypeAtom)iter.next();
			if (!attr.isNumeric()) {
				return false;
			}
		}
		return true ;
	};
	public  boolean isInteger()
	{
		Iterator iter = _attributes.iterator();
		SqlTypeAtom attr;
		while (iter.hasNext()) {
			attr = (SqlTypeAtom)iter.next();
			if (!attr.isInteger()) {
				return false;
			}
		}
		return true ;
	};
	public  boolean isString()
	{
		Iterator iter = _attributes.iterator();
		SqlTypeAtom attr;
		while (iter.hasNext()) {
			attr = (SqlTypeAtom)iter.next();
			if (!attr.isString()) {
				return false;
			}
		}
		return true ;
	};
	public  boolean isDataTime()
	{
		Iterator iter = _attributes.iterator();
		SqlTypeAtom attr;
		while (iter.hasNext()) {
			attr = (SqlTypeAtom)iter.next();
			if (!attr.isDataTime()) {
				return false;
			}
		}
		return true ;
	};

	public String pprint ()
	{
		StringBuffer retVal = null;
		if ( MANY == _multiplicity) {
			retVal.append("Relation(");
		}
		else {
			retVal.append("Tuple(");
		}

		for (int i = 0; i < _attributes.size(); i++) {
			retVal.append(((SqlTypeAtom)_attributes.get(i)).ppprint());
			retVal.append(",");
		}
		retVal.append(")");
		return retVal.toString();
	}

	public boolean isCompatibleTo(SqlType type)
	{
		log.warn("isCompatibleTo(SqlType type) not yet implemented.");
		return true;
	}
}
