/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;


public class TempValue extends UnaryOperator {

    private static final String RCSRevision = "$Revision: 1.5 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    
	Literal _value = null;

    public TempValue() {
    }

    public TempValue(Expression operand) {
		super(operand);
    }

    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        //Trace.enter(this, "replaceChild(Expression oldChild, Expression newChild)");

        boolean retVal = false;
        if (getOperand() == oldChild) {
            setOperand(newChild);
            retVal = true;
        }

        //Trace.exit(this, "replaceChild(Expression oldChild, Expression newChild)");
        return retVal;
    }

	public void setValue(Literal value) {
		_value = value;
	}

	public Literal getValue() {
		return _value;
	}

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof TempValue)
		{
			TempValue cast = (TempValue)o;
			return super.deepEquals(o) && _value.equals(cast.getValue());
		}
		return false;
	}
}
