/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;

/**
 * This visitor browse an algebra tree to replace leaf expressions (not operators)
 * by others following a provided map. Attribute expressions are also modified to
 * point to new relations if their relations are changed.
 */
public class TopDownReplacer extends DefaultCompleteVisitor {

	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	Map _replaceMap = null;

	public TopDownReplacer() {
	}

	public TopDownReplacer(Map replaceMap) {
		_replaceMap = replaceMap;
	}

	public void setReplaceMap(Map replaceMap) {
		_replaceMap = replaceMap;
	}

	public Map getReplaceMap() {
		return _replaceMap;
	}

	public void visit(Expression arg) {
		//Trace.enter(this, "visit(Expression arg)", arg);
		Expression replacer = (Expression) _replaceMap.get(arg);
		/* try to find replacer for this Expression */
		if (null != replacer) {
			arg.getFather().replaceChild(arg, replacer);
			//Trace.message(this,"visit(AttributeExpression arg)", arg.pprint()+" replaced by " + replacer.pprint());
		}
		//Trace.exit(this, "visit(Expression arg)", arg);
	}

	public void visit(AttributeExpression arg) throws SqlWrapperException {
		//Trace.enter(this,"visit(AttributeExpression arg)", arg);

		Expression replacer = (Expression) _replaceMap.get(arg);
		/* try to find replacer for this AttributeExpression */
		if (null != replacer) {
			arg.getFather().replaceChild(arg, replacer);
			//Trace.message(this,"visit(AttributeExpression arg)", arg.pprint()+" replaced by " + replacer.pprint());
		} else {
			/* try to find replacer for the table instance of this AttributeExpression */
			Object ti = arg.getTableInstance();
			Object tableInstance = null;
			if (null != ti) {
				tableInstance = _replaceMap.get(ti);
			}

			if (null == tableInstance) {
			} else if (tableInstance instanceof NullPointer) {
				arg.setTableInstance(null);
			} else {
				arg.setTableInstance((RenameRelation) tableInstance);
			}
		}
		//Trace.exit(this,"visit(AttributeExpression arg)", arg);
	}

	public void visit(SortSpecification arg) throws SqlWrapperException {
		//Trace.enter(this,"visit(SortSpecification arg)", arg);

		AttributeExpression sortExpr = (AttributeExpression) arg.getSortExpression();
		sortExpr.accept(this);
		Expression uderlyingExpr = sortExpr.getUnderlyinExpr();
		if (uderlyingExpr instanceof AttributeExpression) {
			Expression replacer = (Expression) _replaceMap.get(uderlyingExpr);
			/* try to find replacer for this uderlyingExpr */
			if (null != replacer) {
				sortExpr.setUnderlyingExpr(replacer);
			}
		}
		//Trace.exit(this,"visit(SortSpecification arg)", arg);
	}
}
