/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.xquery.parser.XQueryExpression;


/**
 * NOTE: This 'Expression' does not seem to appear in the generated relational
 * algebra. It correspond to XQuery variables bound to complex elements that are
 * locally stored in the variable stack by the @link org.xquark.extractor.algebra.GenAlgebraVisitor
 * but does not appear in the return clause and thus in the relational algebra tree.
 */
public final class TupleExpression extends Expression {

    private static final String RCSRevision = "$Revision: 1.6 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";


    protected List              _itemList;
    protected RenameRelation   _tableInstance;

    public TupleExpression() {
    }

    public TupleExpression(RenameRelation tableInstance , List itemList, XQueryExpression aOrginalXExpr)
    {
        setTableInstance(tableInstance);
        setItemList(itemList);
        setOrginalXExpr(aOrginalXExpr);
    }


    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        //Trace.enter(this, "clone(Map clonedExprs)");
        TupleExpression retVal = (TupleExpression)super.clone(clonedExprs);
        retVal.setItemList(AlgebraTools.clone(_itemList, clonedExprs));
        clonedExprs.put(this, retVal);
        //Trace.exit(this, "clone(Map clonedExprs)");
        return retVal;
    }


    public RenameRelation getTableInstance()
    {
        return _tableInstance ;
    }

    public void setTableInstance(RenameRelation tableInstance)
    {
        _tableInstance = tableInstance;
    }

    /**
    Access method for the _itemList property.

    @return   the current value of the _itemList property
     */
    public List getItemList()
    {
        return _itemList ;
    }

    public List getOperands() {
        return null;
    }

    /**
    Sets the value of the _itemList property.

    @param aItemList the new value of the _itemList property
     */
    public void setItemList(List itemList)
    {
        //Trace.enter(this,"setItemList");

        _itemList = itemList;

        if (null==_itemList) {
            return ;
        }

        Iterator iter = _itemList.iterator();
        while (iter.hasNext()) {
            Expression item = (Expression)iter.next();
            item.setFather(this);
        }

        //Trace.exit(this,"setItemList");
    }


    /**
    @param itemExpr
    @roseuid 3B2787F00323
     */
    public void addItem(AttributeExpression itemExpr)
    {
        //Trace.enter(this,"addItem");

        if ( null == _itemList )
        {
            _itemList = new ArrayList ();
        }

        _itemList.add ( itemExpr ) ;
        itemExpr.setFather(this);

        //Trace.exit(this,"addItem");
    }


    public void arithmeticOperation(int operater,Expression leftOperand,Expression rightOperand)
    {
        //Trace.enter(this,"ArithmeticOperation");

//        Iterator iter = _itemList.iterator();
//        Expression item ;
//        List newItemList = new ArrayList();
//        Expression  arOp ;
//        while (iter.hasNext()) {
//            item = getRealExpression((Expression)iter.next());
//            if (null != leftOperand) {
//                arOp = new BinOpArithmetic (operater, leftOperand, item);
//            }
//            else {
//                arOp = new BinOpArithmetic (operater, item, rightOperand);
//            }
//            newItemList.add (arOp);
//        }
//        setItemList(newItemList);
        //Trace.exit (this,"ArithmeticOperation");
    }




    public void addNameTest (String name)
    {
        //Trace.enter( this , "addNameTest");
//        /** @todo!!!  */
//        List items = ((Relation)_operand).nameTest(name);
//        Iterator iter = items.iterator();
//        while (iter.hasNext()) {
//            Object item = iter.next();
//            addItem((Expression)item);
//        }
        //Trace.exit( this , "addNameTest");
    }

    public List nameTest (String name)
    {
        //Trace.enter(this,"nameTest",name);

        List retVal = null;

//        if (name.equalsIgnoreCase("*") || null == name) {
//            retVal = _itemList;
//            // in this case , there will be a  problem when there are duplicated name
//            Trace.warn(this,"nameTest","wild card used in nameTest");
//        }
//        else {
//            // Name is not a wild card
//            retVal = new ArrayList ();
//            for (int i = 0; i < _itemNames.size(); i++) {
//                if (name.equalsIgnoreCase((String)_itemNames.get(i))) {
//                    retVal.add (_itemList.get(i));
//                }
//            }
//            if (0==retVal.size()) {
//                retVal = null;
//            }
//        }

        //Trace.exit (this,"nameTest",name);
        return retVal;
    }

    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        //Trace.enter(this, "replaceChild(Expression oldChild, Expression newChild)");
        boolean retVal = false;

        List list = getItemList();

        Expression expr = null ;
        for (int i = 0; i < list.size(); i++) {
            expr = (Expression)list.get(i);
            if (expr.equals(oldChild)) {
                list.set(i, newChild);
				newChild.setFather(this);
                retVal = true;
                break;
            }
        }

        //Trace.exit(this, "replaceChild(Expression oldChild, Expression newChild)");
        return retVal;
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof TupleExpression)
		{
			TupleExpression cast = (TupleExpression)o;
			return _tableInstance.deepEquals(cast.getTableInstance()) // NOTE: assuming metadata object are not duplicated...
					&& AlgebraTools.areExprListEquivalent(_itemList, cast.getItemList());
		}
		return false;
	}
}
