/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.List;
import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.sql.SqlExpression;

public final class UnOpGroup extends UnaryAlgebra
{

    private static final String RCSRevision = "$Revision: 1.6 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";


    private List _groupExpressionList;

    public UnOpGroup(Expression operand, List groupExpressionList)
    {
        super(operand);
        setGroupExpressionList ( groupExpressionList ) ;
    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        UnOpGroup retVal = (UnOpGroup)super.clone(clonedExprs);
        retVal.setGroupExpressionList(AlgebraTools.clone(_groupExpressionList,clonedExprs));

        clonedExprs.put(this, retVal);
        return retVal;
    }

    /**
	 * Access method for the _groupExpressionList property.
	 * 
	 * @return the current value of the _groupExpressionList property
	 */
    public List getGroupExpressionList()
    {
        return _groupExpressionList ;
    }

    public List getParameterList()
    {
        return getGroupExpressionList();
    }

    /**
	 * Sets the value of the _groupExpressionList property.
	 * 
	 * @param aGroupExpressionList
	 *            the new value of the _groupExpressionList property
	 */
    public void setGroupExpressionList(List aGroupExpressionList)
    {
        _groupExpressionList = aGroupExpressionList;
		if ( null != _groupExpressionList) {
			Expression expr = null;
			for (int i = 0; i < _groupExpressionList.size(); i++) {
				expr = (Expression)_groupExpressionList.get(i);
				expr.setFather(this);
			}
		}
    }

    public List getKeys() {
        return _groupExpressionList;
    }

    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        boolean retVal = false;

        List list = getGroupExpressionList();

        Expression expr = null ;
        for (int i = 0; i < list.size(); i++) {
            expr = (Expression)list.get(i);
            if (expr.equals(oldChild)) {
                list.set(i, newChild);
				newChild.setFather(this);
                retVal = true;
                break;
            }
        }

        if (getOperand().equals(oldChild)) {
            setOperand(newChild);
            retVal = true;
        }

        return retVal;
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof UnOpGroup)
		{
			UnOpGroup cast = (UnOpGroup)o;
			return super.deepEquals(o)
				&& AlgebraTools.areExprListEquivalent(_groupExpressionList, cast.getGroupExpressionList()); 
		}
		return false;
	}
}
