/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

public abstract class UnaryOperator extends Expression
{

    private static final String RCSRevision = "$Revision: 1.6 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";


    protected Expression _operand;

    /**
    @roseuid 3B2793B4013C
     */
    public UnaryOperator()
    {

    }

    /**
    @param operand
    @roseuid 3B2793910056
     */
    public UnaryOperator(Expression operand)
    {
        setOperand( operand);
    }

    synchronized Object clone(Map clonedExprs)  throws CloneNotSupportedException
    {
        //Trace.enter(this, "clone(Map clonedExprs)");
        UnaryOperator retVal = (UnaryOperator)super.clone(clonedExprs);
        retVal.setOperand((getOperand () == null) ? null : (Expression) getOperand().clone (clonedExprs));

        clonedExprs.put(this, retVal);
        //Trace.exit(this, "clone(Map clonedExprs)");
        return retVal;
    }

    /**
    Access method for the _operand property.

    @return   the current value of the _operand property
     */
    public Expression getOperand()
    {
        return _operand ;
    }

    public List getOperands() {
        List retVal = new ArrayList();
        retVal.add(this);
        return retVal;
    }

    /**
    Sets the value of the _operand property.

    @param aOperand the new value of the _operand property
     */
    public void setOperand(Expression aOperand)
    {
        _operand = aOperand;
        _operand.setFather(this);
    }

    /**
    @return java.util.Set
    @roseuid 3B27939102B9
     */
    // change LARS 30/04/2004
//    public Set referfedColumnExpressions()
//    {
//        //Trace.enter( this, "referfedColumnExpressions" );
//        Trace.warn ( this, "referfedColumnExpressions" , "unexpected function call " );
//        //Trace.exit ( this, "referfedColumnExpressions" );
//        return null  ;
//    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof UnaryOperator)
		{
			UnaryOperator cast = (UnaryOperator)o;
			return	 _operand.deepEquals(cast.getOperand());
		}
		return false;
	}
}
/**

UnaryOperator.getOperands(){
        ArrayList  retVal = new ArrayList () ;
        retVal.add( _operand ) ;
        return retVal ;
    }

 */
