/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;

public final class VariableTable {
	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";


	VariableTable _defautVariableTable;
	HashMap _varTable;

	public VariableTable ()
	{
		//Trace.enter(this,"VariableTable ()");
		_varTable = new HashMap();
		//Trace.exit(this,"VariableTable ()");
	}
	/**
	Access method for the _defautVariableTable property.

	@return   the current value of the _defautVariableTable property
	 */
	public VariableTable getDefautVariableTable()
	{
		//Trace.enter(this,"getDefautVariableTable()");
		//Trace.exit(this,"getDefautVariableTable()");
		return _defautVariableTable;
	}

	/**
	Sets the value of the _defautVariableTable property.

	@param aDefautVariableTable the new value of the _defautVariableTable property
	 */
	public void setDefautVariableTable(VariableTable aDefautVariableTable)
	{
		//Trace.enter(this,"setDefautVariableTable(VariableTable aDefautVariableTable)");
		_defautVariableTable = aDefautVariableTable;
		//Trace.exit(this,"setDefautVariableTable(VariableTable aDefautVariableTable)");
	}


	/**
	 * get the translations of the variable.
	 *
	 * if the varName is null, throw java.lang.NullPointerException
		@param varName
		@return Expression
		@throws java.lang.NullPointerException
		@roseuid 3BD16E3602C0
	 */
	public Expression get(String varName)
	{
		//Trace.enter(this,"get(String varName)");

		Expression retVal = null;

		retVal = (Expression)_varTable.get(varName);
		if (null==retVal && _defautVariableTable != null) {
			retVal = _defautVariableTable.get(varName);
		}

		//Trace.exit(this,"get(String varName)");
		return retVal;
	}

	/**
	 * put a variable name and the translation of the variable in this object.
	 *
	 * if either of the parameters is null, throw  ???? Exception @todo
	@param varName  : the name of variable. case-sensitive.
	@param translation : the translations of this variable, often a tree.
	@roseuid 3BD16E6E03B1
	 */
	public void put(String varName, Expression translation)
	{
		//Trace.enter(this,"put(String varName, Expression translation)", "$" + varName);
		Debug.assertTrue(null!=varName,"null!=varName");
		Debug.assertTrue(null!=translation,"null!=translation");
		_varTable.put(varName, translation);
		//Trace.exit(this,"put(String varName, Expression translation)", "$" + varName);
	}

	public HashSet getCurrentLevelTableInstances()
	{
		//Trace.enter(this,"getPredefinedTableInstances()");
		HashSet retVal = new HashSet();
		Collection varTrans = _varTable.values();
		Iterator iter = varTrans.iterator();
		while (iter.hasNext()) {
			Object item = iter.next();
			if (item instanceof TupleExpression) {
				retVal.add( ((TupleExpression)item).getTableInstance() );
			}
			else if (item instanceof AttributeExpression) {
				retVal.add( ((AttributeExpression)item).getTableInstance() );
			}
		}
		//Trace.exit(this,"getPredefinedTableInstances()");
		return retVal;
	}
	public HashSet getPredefinedTableInstances()
	{
		//Trace.enter(this,"getPredefinedTableInstances()");
		HashSet retVal = new HashSet();
		VariableTable nextVT  = getDefautVariableTable();
		while (null != nextVT ) {
			retVal.addAll(nextVT.getCurrentLevelTableInstances());
			nextVT = nextVT.getDefautVariableTable();
		}
		//Trace.exit(this,"getPredefinedTableInstances()");
		return retVal;
	}
}
