/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.metadata;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Base type for Extractor metadata.
 */
// QUESTION : Why is XTreeNode not used ?
public abstract class NamedNode {

    private static final String RCSRevision = "$Revision: 1.5 $";
    private static final String RCSName = "$Name: XQ1_1_1 $";

    protected String _name;
    protected List _children;
    protected NamedNode _father;
    protected String _alias;

	public NamedNode() {}

    /**
     * @param name
     */
    public NamedNode(String name) {
        setName(name);
    }

    /**
    Access method for the _name property.
    
    @return   the current value of the _name property
     */
    public String getName() {
        return _name;
    }

    /**
     * Sets the value of the _name property.
     * @param aName the new value of the _name property
     */
    public void setName(String aName) {
        _name = aName;
    }

    public String getAlias() {
        return _alias;
    }

    public void setAlias(String alias) {
        _alias = alias;
    }

    /**
     * @param child
     */
    public void addChild(NamedNode child) {
        if (null == _children)
            _children = new ArrayList();
        _children.add(child);
        child.setFather(this);
        return;
    }

    /**
     * @param children
     */
    public void setChildren(List children) {
        _children = children;
        if (null != _children) {
            for (int i = 0; i < _children.size(); i++) {
                ((NamedNode) _children.get(i)).setFather(this);
            }
        }
    }

    /**
     * @param childName
     * @return NamedNode
     */
    public NamedNode findChild(String childName) {
        NamedNode retVal = null;

        if (null != _children && null != childName) {
            Iterator iter = _children.iterator();
            NamedNode item;
            while (iter.hasNext()) {
                item = (NamedNode) iter.next();
                if (childName.equalsIgnoreCase(item.getName())) {
                    retVal = item;
                    break;
                }
            }
        }
        return retVal;
    }

    public NamedNode findChildByAlias(String childName) {
        NamedNode retVal = null;

        if (null != _children && null != childName) {
            Iterator iter = _children.iterator();
            NamedNode item;
            while (iter.hasNext()) {
                item = (NamedNode) iter.next();
                if (childName.equals(item.getAlias())) {
                    retVal = item;
                    break;
                }
            }
        }
        return retVal;
    }

    /**
     * @return org.xquark.xqueryparser.xquery.List
     */
    public List getChildren() {
        return _children;
    }

    public void setFather(NamedNode father) {
        _father = father;
    }

    public NamedNode getFather() {
        return _father;
    }

    public abstract String pprint();
}
