/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.mysql;

import java.util.ArrayList;
import java.util.List;

import org.xquark.extractor.algebra.*;
import org.xquark.extractor.mysql.sql.SqlCreateTempTableAs;
import org.xquark.extractor.runtime.IDProvider;
import org.xquark.extractor.sql.*;

public final class MySqlGenSqlVisitor extends AbstractGenSqlVisitor {

	private static final String RCSRevision = "$Revision: 1.9 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	private IDProvider _tmpTableIDProvider;

	public MySqlGenSqlVisitor(IDProvider relIDProvider, IDProvider tmpTableIDProvider) {
		super((SqlFactory) MySqlFactory.getInstance(), relIDProvider);
		_tmpTableIDProvider = tmpTableIDProvider;
	}

	public void reinit() {
		super.reinit();
	}

	public SqlExpression visit(UnOpAggregate arg) {
		SqlExpression retVal = visit((UnOpProject) arg);
		return retVal;
	}

	// TODO: generate also subquery for sorts...

	public SqlExpression visit(UnOpProject arg) {
		SqlExpression retVal;

		SqlSelect outerExpression = null;
		if (!_nestStack.isEmpty())
			outerExpression = (SqlSelect) _nestStack.firstElement();

		SqlSelect select = _factory.createSelect(arg);
		_nestStack.push(select);

		retVal = selectAddSelectClause(select, arg); // After potential attributes renaming

		/* if _nestStack not empty (subquery) create a temporary table (supposed
		 * to remain only in FROMs */
		if (outerExpression != null) {
			String tempTableName = getTemporaryTableName();
			SqlCreateTempTableAs sCreateTempTable = new SqlCreateTempTableAs(tempTableName, select);
			outerExpression.appendPrecedent(sCreateTempTable);
			retVal = _factory.createTable(tempTableName);
		}
		_nestStack.pop();

		return retVal;
	}

	protected SqlSelect selectAddFromClause(SqlSelect select, BinOpOuterJoin arg) {
		Expression[] relations = { arg.getLeftOperand(), arg.getRightOperand()};
		SqlExpression[] sOprnd = new SqlExpression[2];

		for (int i = 0; i < relations.length; i++) {
			// NOTE: a list of if (instanceof) used to check algebra integrity have
			// been removed because these tests make visitor automatic type switch worthless...
			sOprnd[i] = relations[i].accept(this);
		}

		/* Outer join predicate creation */
		List aPredicateList = arg.getPredicateList();
		List sPredicateList = null;
		if (null != aPredicateList) {
			/* translate predicates */
			sPredicateList = new ArrayList();
			Expression aPredicate = null;
			org.xquark.extractor.sql.SqlExpression sPredicate = null;

			for (int i = 0; i < aPredicateList.size(); i++) {
				aPredicate = (Expression) aPredicateList.get(i);
				sPredicate = aPredicate.accept(this);
				sPredicateList.add(sPredicate);
			}
		}

		/* Outer join clause creation */
		org.xquark.extractor.sql.SqlExpression retVal = new org.xquark.extractor.sql.SqlBinOpOuterJoin(sOprnd[0], sOprnd[1], arg.getJoinType(), sPredicateList);

		select.addFromClause(retVal);

		return select;
	}

	protected String getTemporaryTableName() {
		return TMP_TABLE_PREFIX + _tmpTableIDProvider.getID();
	}
}
