/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.oracle;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.xquark.extractor.common.MetaDataException;
import org.xquark.extractor.metadata.*;
import org.xquark.extractor.runtime.Selection;
import org.xquark.jdbc.typing.DBMSInfo;
import org.xquark.jdbc.typing.DbType;

public final class Loader extends org.xquark.extractor.metadata.Loader {

	private static final String RCSRevision = "$Revision: 1.12 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	public Loader(Connection connection, DBMSInfo dbmsInfo, QNameEncoder encoder) throws SQLException {
		super(connection, dbmsInfo, encoder);
	}

	protected void loadKeys(String catalogName, String schemaName, Table table, Selection selection) throws MetaDataException {
		//Trace.enter(this, "loadKeys(String catalogName, String schemaName, Table table, Selection selection)", table.getName());
		/* load Primary Keys for this table */
		//Trace.message(this, "loadTable(...)", "creating pseudoAttribute column...");
		List key = new ArrayList();

		PseudoAttribute rowId = new PseudoAttribute("ROWID");
		rowId.setMappingInfo(new ExtractorMappingInfo(new DbType(DbType.ORACLE_ROWID), _dbmsInfo.getTypeMap()));
		key.add(rowId);
		table.addKey(key);
		super.loadKeys(catalogName, schemaName, table, selection);
		//Trace.exit(this, "loadKeys(String catalogName, String schemaName, Table table, Selection selection)", table.getName());
	}

	protected String createQuery(String catalogName, String schemaName, String origTableName) {
		StringBuffer retVal = new StringBuffer();

		retVal.append("SELECT * FROM ");
		if (null != catalogName && 0 < catalogName.trim().length()) {
			retVal.append('"');
			retVal.append(catalogName.trim());
			retVal.append("\".");
		}
		if (null != schemaName && 0 < schemaName.trim().length()) {
			retVal.append('"');
			retVal.append(schemaName.trim());
			retVal.append("\".");
		}
		if (null != origTableName && 0 < origTableName.trim().length()) {
			retVal.append('"');
			retVal.append(origTableName.trim());
			retVal.append('"');
		}
		retVal.append(" WHERE 0=1");
		return retVal.toString();
	}
}
