/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.oracle;

import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.xquark.extractor.algebra.*;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.extractor.metadata.MetaDataManager;
import org.xquark.extractor.runtime.Query;
import org.xquark.extractor.runtime.SynchronizedResultSet;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.metadata.resolver.MetadataAccess;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;

public class Oracle7QueryFactory extends OracleQueryFactory {
	private static final String RCSRevision = "$Revision: 1.7 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	private PreEvaluationVisitor _pev;

	public Oracle7QueryFactory(MetaDataManager metadataManager, MetadataAccess metadata) throws XMLDBCException {
		super(metadataManager, metadata);
	}

	protected void createVisitors() {
		TypeInterpreter ti = new TypeInterpreter();
		_genAlgebraVisitor = new GenAlgebraVisitor(ti, _metadataManager, _metadata, _attIDProvider, _relIDProvider);

		_algebraTypeVisitor = new AlgebraTypeVisitor();
		_removeProjectVisitor = new RemoveIntermediateProjectVisitor();
		_brav = new BuildReferredAttributes();
		_apv = new AnalyzePredicateVisitor();

		_genSqlVisitor = new org.xquark.extractor.oracle.OracleGenSqlVisitor(_relIDProvider);
		_genSqlVisitor.setMetadataManager(_metadataManager);

		_pev = new PreEvaluationVisitor();
	}

	protected Query.SQLExecutionInfo compileSingleQDB(XQueryExpression xQuery, boolean keyNeeded)
	throws XQueryException, XMLDBCException {
		//Trace.enter(this, "executeSingleQDB(XQueryExpression xQuery, boolean keyNeeded)");

		SynchronizedResultSet retVal = null;
		Expression algebraTree = null;
		Mapper mapper = null;

		/* Algebra tree building */
		_genAlgebraVisitor.reinit();

		if (keyNeeded) { // Used when Sorted Outer Union is needed to identify rows
			List souArgList = new ArrayList();
			souArgList.add(xQuery);
			_genAlgebraVisitor.noticSortedOuterUnion(souArgList);
		}
		xQuery.accept(_genAlgebraVisitor);
		algebraTree = _genAlgebraVisitor.getAlgebraTree();

		// INFO : YS types of visitors : GenAlgebraVisitor (a ParserVisitor impl),
		// AlgebraVisitor, GenSQLVisitor
		_removeProjectVisitor.reinit();
		algebraTree.accept(_removeProjectVisitor);

		algebraTree.accept(_brav);

		/* resolve invalid predicates */
		_apv.reinit();
		algebraTree.accept(_apv);

		algebraTree.accept(new RemoveNestedAggregation(_attIDProvider, _relIDProvider));

		_algebraTypeVisitor.reinit();
		algebraTree.accept(_algebraTypeVisitor);

		_pev.setGenSqlVisitor(_genSqlVisitor);
		try {	// TODO: why temp tables should be created here ? 
		    Statement jdbcStatement = _stmt.createJdbcStatement();
			_pev.setJdbcStatement(jdbcStatement);
            jdbcStatement.close();
		}
    	catch (SQLException ex) {
    	    throw new XMLDBCException(ex.getMessage(), ex);
    	}
		algebraTree.accept(_pev);

		mapper = algebraTree.getMapper();

		_genSqlVisitor.reinit();
		SqlExpression sql = algebraTree.accept(_genSqlVisitor);

		/* Generate string to be ready for prepared statements execution */
		sql.generateRequestList();
		return new Query.SQLExecutionInfo(sql, algebraTree.getMapper());
	}
}
