/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.oracle;

import org.xquark.extractor.algebra.BinOpArithmetic;
import org.xquark.extractor.algebra.DummyTable;
import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.runtime.IDProvider;
import org.xquark.extractor.sql.AbstractGenSqlVisitor;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.extractor.sql.SqlFactory;

public final class OracleGenSqlVisitor extends AbstractGenSqlVisitor {

	private static final String RCSRevision = "$Revision: 1.11 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	public OracleGenSqlVisitor(IDProvider relIDProvider) {
		super((SqlFactory) OracleFactory.getInstance(), relIDProvider);
	}

	public SqlExpression visit(BinOpArithmetic arg) {
		SqlExpression retVal = null;

		Expression aLOprnd = arg.getLeftOperand();
		SqlExpression sLOprnd = aLOprnd.accept(this);

		Expression aROprnd = arg.getRightOperand();
		SqlExpression sROprnd = aROprnd.accept(this);

		int oprtr = arg.getOperator();

		retVal = _factory.createBinOpArithmetic(oprtr, sLOprnd, sROprnd);

		if (false /** @todo   INT_DIVIDE_ARITHMETICS == oprtr &&
					aLOprnd.getType().isInteger() &&
					aROprnd.getType().isInteger()*/
			) {
			/** @todo pay attention that the round funcion on negative values */
			retVal = _factory.createSfRound(retVal);
		}
		return retVal;
	}

	protected org.xquark.extractor.sql.SqlSelect selectAddFromClause(org.xquark.extractor.sql.SqlSelect select, DummyTable operand) {
		org.xquark.extractor.sql.SqlTable newFrom = _factory.createTable("dual");
		select.addFromClause(newFrom);

		return select;
	}
}
