/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.runtime;

import java.util.ArrayList;
import java.util.List;

import org.xquark.extractor.algebra.Mapper;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.extractor.sql.StatementInfo;
import org.xquark.xquery.normalize.TransformXQueryExpression;
import org.xquark.xquery.parser.XQueryModule;

/**
 * An extractor query as an XQuery string and in the compiled form (compilation
 * is performed by this class).
 */
public class Query {

	private String _query = null;
	private List _cQuery = null;
	private QueryFactory _compiler = null;
	private XQueryModule _module = null;

	//    public Query(String query, QueryFactory compiler, XQueryModule module) {
	//        this._query = query;
	//        _compiler = compiler;
	//    }

	public Query(String query, QueryFactory compiler, List compiledQuery, XQueryModule module) {
		//this(query, compiler, module);
		_query = query;
		_compiler = compiler;
		_cQuery = compiledQuery;
		_module = module;
	}

	//    public void compile() throws XMLDBCException {
	//        _compiler.compileQuery(_query);
	//    }

	/**
	 * @return Returns the compiled query.
	 */
	public List getCompiledQuery() {
		return _cQuery;
	}

	/**
	 * @return Returns the query as String.
	 */
	public String getQuery() {
		return _query;
	}

	/**
	 * @return Returns the parent module of the query.
	 */
	public XQueryModule getModule() {
		return _module;
	}

	public static class CompiledExpression {
		public SQLExecutionInfo sqlInfoTree = null;
		public TransformXQueryExpression expr = null;
		public CompiledExpression(SQLExecutionInfo sqlInfoTree, TransformXQueryExpression expr) {
			this.sqlInfoTree = sqlInfoTree;
			this.expr = expr;
		}
	}

	/**
	 * A tree of queries and mapper that let us to postpone the JDBC execution
	 * for subclass prepared statement mainly.
	 */
	public static class SQLExecutionInfo {
		public SqlExpression sql = null;
		public StatementInfo sInfo = null;
		public Mapper mapper = null;
		public List childrenQueries = null;
		public int[] idPosition = null;
		public int idCount = -1;

		public SQLExecutionInfo() {
		}
		public SQLExecutionInfo(SqlExpression sql, Mapper mapper) {
			this.sql = sql;
			this.mapper = mapper;
		}
		public void addChild(SQLExecutionInfo child) {
			if (childrenQueries == null)
				childrenQueries = new ArrayList();
			childrenQueries.add(child);
		}
		public boolean isUnion() {
			return (sql == null);
		}
	}
}
