/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.runtime;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.xerces.impl.xpath.regex.RegularExpression;
import org.xquark.extractor.common.Constants;

/**
 * Path tree loaded from configuration file featuring a filter on database
 * objects allowing to narrow objects seen by Extractor.
 */
// QUESTION : Why XTreeNode not used ?
public class Selection implements Constants {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName = "$Name: XQ1_1_1 $";

    private static RegularExpression regexNCName =
        new RegularExpression("[\\i-[:]][\\c-[:]]*", "X");
    private static RegularExpression regexEntry =
        new RegularExpression("", "X");

    private byte type = NONE;
    private String name = null;
    private boolean nameIsRegex = false;
	//    private RegularExpression regExpr = null;
	private String targetNamespace = null;
	private HashMap map = null;
    private String alias = null;
    private int elementFormDefault = UNQUALIFIED;

    /* for a column element, pkColumn values true means that this column takes
     * part in primary key
     */
    private boolean pkColumn = false;


    public Selection(
        String name,
        String targetNamespace,
        byte type,
        HashMap map) {
        this.name = name;
        this.nameIsRegex = false;
        this.targetNamespace = targetNamespace;
        this.type = type;
        this.map = map;
    }

    public Selection(
        String name,
        boolean nameIsRegex,
        String targetNamespace,
        byte type,
        HashMap map) {
        this.name = name;
        this.nameIsRegex = nameIsRegex;
        this.targetNamespace = targetNamespace;
        this.type = type;
        this.map = map;
    }

    public String getName() {
        return name;
    }
    
    public void setNameForDefault(String name) {
    	if (this.name == null)
    		this.name = name;
    }
    
    public String getTargetNamespace() {
        return targetNamespace;
    }
    public byte getType() {
        return type;
    }
    public HashMap getMap() {
        return map;
    }

    public void setElementFormDefault(int elementFormDefault) {
        this.elementFormDefault = elementFormDefault;
    }

    public int getElementFormDefault() {
        return elementFormDefault;
    }

    public void addToMap(String name, Selection selection) {
        if (map == null)
            map = new HashMap();
        map.put(name, selection);
    }

    public void setAlias(String alias) {
        this.alias = alias;
    }

    public String getAlias() {
        return this.alias;
    }

    //    public void setRegularExpression(String regularExpression) {
    //        this.regExpr = new RegularExpression(regularExpression);
    //    }

    //    public boolean isSelected(String subElementName) {
    //        boolean retVal = false;
    //        retVal = (null == includesSubElement(subElementName));
    //        if (!retVal &&
    //            null != subElementName
    //            && null != regExpr) {
    //            retVal = regExpr.matches(subElementName);
    //        }
    //        return retVal;
    //    }

    public boolean noSubElementDef() {
        return (null == map || map.isEmpty());
    }

    public Selection includesSubElement(String str) {
        if (map == null || map.isEmpty())
            return null;
        if (null == str) {
            return (Selection) this.map.get(null);
        }
        //		if (!verifyName(str)) return null;
        Collection values = map.values();
        Iterator it = values.iterator();
        Selection retSelection = null;
        while (it.hasNext()) {
            Selection value = (Selection) it.next();
            // TODO: explodes if schema element contains nothing... <=> all tables ? Add a constraint in schema ? A default include all ?
            if (value == null || value.getName() == null)
            	continue;
            // TODO: Store regular expression statically
            regexEntry.setPattern(value.getName());
            if (regexEntry.matches(str)) {
                if (value.getType() == Selection.EXCLUDES)
                    return null;
                else if (value.getType() == Selection.INCLUDES)
                    if (retSelection == null)
                        retSelection = value;
            }
        }
        return retSelection;
    }

    public void setPKColumn(boolean pkColumn) {
        this.pkColumn = pkColumn;
    }

    public boolean getPKColumn() {
        return this.pkColumn;
    }

    static public boolean verifyName(String name) {
        return regexNCName.matches(name);
    }

    /*
    private boolean includesElement(String str) {
    	if (type != INCLUDES) return false;
    	if (name != null && name.equals(str)) return true;
    	return false;
    }
    */

	public String print() {
		return print(0);
	}
	
    public String print(int tab) {
		StringBuffer bufTab = new StringBuffer();
		for (int i=0;i<tab;i++)
			bufTab.append("\t");
        StringBuffer buf = new StringBuffer();     	
        buf.append("\n");
        buf.append(bufTab.toString());
        buf.append("Name : ");
        buf.append(name);
        buf.append("\n");
		buf.append(bufTab.toString());
        buf.append("TargetNamespace : ");
        buf.append(targetNamespace);
        buf.append("\n");
		buf.append(bufTab.toString());
		buf.append("Type : ");
        buf.append(type);
        if (map != null) {
            Collection col = map.values();
            Iterator it = col.iterator();
            while (it.hasNext()) {
                buf.append("\n");
				buf.append(bufTab.toString());
				buf.append("Map entry -->");
                buf.append(((Selection) it.next()).print(tab+1));
            }
        }
        return buf.toString();
    }

    public String toString() {
        return print();
    }
}
