/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.runtime;

import java.io.DataOutputStream;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;

import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.AttributesImpl;
import org.xquark.extractor.algebra.MapItem;
import org.xquark.extractor.algebra.Mapper;
import org.xquark.extractor.common.MessageLibrary;
import org.xquark.schema.validation.PSVInfoSetProvider;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.parser.Step;
import org.xquark.xquery.xdbc.XDBCResultSetInterface;

/**
 * NOTE: This result set is not currently used. Could be reactivated if the
 * SOU was enabled again ? This is the vesion with one big result set...
 */
public class XDBCResultSet implements XDBCResultSetInterface {
	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	private ContentHandler _contenthandler = null;
	private LexicalHandler _lexicalhandler = null;
	private ErrorHandler _errorhandler = null;
	private Mapper _mapper = null;
	private LookAheadResultSet _sqlResultSet = null;

	/*  if the sql resultset contains multiple part, partItemName is the
	 *   name of the attribute that indicates the part */
	private String _partAttributeName = null;

	/* for represent identifiers whoes value is null */
	private List _nullIdentifier = new ArrayList(1);

	/* used by generate to avoid create new object on echo call*/
	private AttributesImpl _nullAttribute = new AttributesImpl();

	private String _namespace = null;

	private boolean _isClosed = false;

	private Statement _jdbcStatement = null;

	public void setNamespace(String namespace) {
		this._namespace = namespace;
	}

	//    public  XDBCResultSet(List requestList, Mapper mapper, Statement jdbcStatement) throws XMLDBCException
	//    {
	//        /* execute  "queries" and obtain a jdbc resulset*/
	//        _jdbcStatement = jdbcStatement;
	//        ResultSet sqlResultSet = null;
	//        try {
	//            sqlResultSet = executeRequests(requestList, jdbcStatement);
	//        }
	//        catch (SQLException ex) {
	//             throw new XMLDBCException("XDBCResultSet constructor failed .", ex);
	//        }
	//
	//        _nullIdentifier.add(null);
	//
	//        _mapper = mapper;
	//
	//        if (_mapper.isMultiPart()) {
	//            _partAttributeName = _mapper.getPartAttributeName();
	//            try {
	//                _sqlResultSet = new EliminatedSqlResultSet(sqlResultSet,_mapper.getEliminationGuide());
	//            }
	//            catch (Exception ex) {
	//                throw new XMLDBCException("XDBCResultSet constructor failed .", ex);
	//            }
	//        }
	//        else {
	//            try {
	//                _sqlResultSet = new LookAheadSqlResultSet(sqlResultSet);
	//            }
	//            catch (Exception ex) {
	//                throw new XMLDBCException("XDBCResultSet constructor failed .", ex);
	//            }
	//        }
	//    }
	//
	//    protected ResultSet executeRequests(List requestList, Statement jdbcStatement) throws SQLException {
	//        Trace.enter(this, "executeRequests(List requestList, Statement jdbcStatement)");
	//        ResultSet retVal = null;
	//
	//        String request = null;
	//        int number = requestList.size();
	//        for (int i = 0; i < number-1; i++) {
	//            request = (String)requestList.get(i);
	//            jdbcStatement.executeUpdate(request);
	//        }
	//        request = (String)requestList.get(number-1);
	//        retVal = jdbcStatement.executeQuery(request);
	//
	//        Trace.exit(this, "executeRequests(List requestList, Statement jdbcStatement)");
	//        return retVal;
	//    }

	public XDBCResultSet(ResultSet sqlResultSet, Mapper mapper) throws XMLDBCException {
		//Trace.enter(this, "XDBCResultSet(ResultSet sqlResultSet, Mapper mapper)");
		if (sqlResultSet == null || mapper == null)
			throw new XMLDBCException("The arguments of the XDBCResultSet constructor cannot be null.", null);

		/*
		try {
				org.xquark.extractor.common.Debug.showResultSet(sqlResultSet);
				sqlResultSet.beforeFirst();
		}
		catch (Exception e) {}
		 */

		_nullIdentifier.add(null);

		_mapper = mapper;

		if (_mapper.isMultiPart()) {
			_partAttributeName = _mapper.getPartAttributeName();
			try {
				_sqlResultSet = new EliminatedSqlResultSet(sqlResultSet, _mapper.getEliminationGuide());
			} catch (SQLException ex) {
				throw new XMLDBCException("XDBCResultSet constructor failed .", ex);
			}
		} else {
			try {
				_sqlResultSet = new LookAheadSqlResultSet(sqlResultSet);
			} catch (SQLException ex) {
				throw new XMLDBCException("XDBCResultSet constructor failed .", ex);
			}
		}

		//Trace.exit(this, "XDBCResultSet(ResultSet sqlResultSet, Mapper mapper)");
	}

	public XDBCResultSet(ResultSet sqlResultSet, Mapper mapper, boolean heavyElimination) throws XMLDBCException {
		//Trace.enter(this, "XDBCResultSet(ResultSet sqlResultSet, Mapper mapper, boolean heavyElimination)");
		if (sqlResultSet == null || mapper == null)
			throw new XMLDBCException("The arguments of the XDBCResultSet constructor cannot be null.", null);

		/*
		try {
				org.xquark.extractor.common.Debug.showResultSet(sqlResultSet);
				sqlResultSet.beforeFirst();
		}
		catch (Exception e) {}
		 */

		_nullIdentifier.add(null);

		_mapper = mapper;

		if (heavyElimination && _mapper.isMultiPart()) {
			_partAttributeName = _mapper.getPartAttributeName();
			try {
				_sqlResultSet = new HeavyElimination(sqlResultSet, _mapper.getEliminationGuide());
				//                ((EliminatedSqlResultSet)_sqlResultSet).print();
			} catch (SQLException ex) {
				throw new XMLDBCException("XDBCResultSet constructor failed .", ex);
			}

		} else if (_mapper.isMultiPart()) {
			_partAttributeName = _mapper.getPartAttributeName();
			try {
				_sqlResultSet = new EliminatedSqlResultSet(sqlResultSet, _mapper.getEliminationGuide());
				//                ((EliminatedSqlResultSet)_sqlResultSet).print();
			} catch (SQLException ex) {
				throw new XMLDBCException("XDBCResultSet constructor failed .", ex);
			}
		} else {
			try {
				_sqlResultSet = new LookAheadSqlResultSet(sqlResultSet);
			} catch (SQLException ex) {
				throw new XMLDBCException("XDBCResultSet constructor failed .", ex);
			}
		}

		//Trace.exit(this, "XDBCResultSet(ResultSet sqlResultSet, Mapper mapper, boolean heavyElimination)");
	}

	/**
	 * @see XDBCResultSetInterface#fetch
	 */
	// TODO if used again used psvisp when necessary
	public String fetch(String path, int nodeAccessor, String loopID, PSVInfoSetProvider psvisp) throws XMLDBCException {
		return fetch(path, nodeAccessor);
	}

	public String fetch(String path, int nodeAccessor, String loopID) throws XMLDBCException {
		return fetch(path, nodeAccessor);
	}
	public String fetch(String path, int nodeAccessor) throws XMLDBCException {
		//org.xquark.xml.misc.Debug.print(this,"START XDBCResultSet::FETCH - path = " + path + " - nodeAccessor = " + nodeAccessor);
		//Trace.enter(this, "fetch(String path)", path);

		if (_isClosed)
			throw new XMLDBCException("XMLResultSet is already closed");
		String retVal;
		try {
			MapItem mapItem = _mapper.map(path);
			if (mapItem != null)
				retVal = _sqlResultSet.getString(mapItem.getItemName());
			else
				throw new XMLDBCException("(fetch) This path \"" + path + "\" can not be found in the result set.", null);
		} catch (SQLException sqle) {
			throw new XMLDBCException(sqle.getMessage(), sqle);
		}

		//Trace.exit(this, "fetch(String path)", path);
		//org.xquark.xml.misc.Debug.print(this,"STOP  XDBCResultSet::FETCH - path = " + path + " -> " + retVal);
		return retVal;
	}

	/**
	 * @see XDBCResultSetInterface#generate
	 */
	// TODO if used again used psvisp when necessary
	public void generate(String path, int nodeAccessor, String loopID, PSVInfoSetProvider psvisp) throws XMLDBCException {
		generate(path, nodeAccessor);
	}

	public void generate(String path, int nodeAccessor, String loopID) throws XMLDBCException {
		generate(path, nodeAccessor);
	}
	public void generate(String path, int nodeAccessor) throws XMLDBCException {
		//org.xquark.xml.misc.Debug.print(this,"START XDBCResultSet::GENERATE - path = " + path + " - nodeAccessor = " + nodeAccessor);
		//org.xquark.xml.misc.Debug.print(this,"START XDBCResultSet::GENERATE - _mapper = " + _mapper.pprint());
		//Trace.enter(this, "generate(String path, ContentHandler handler)", path);
		if (_isClosed)
			throw new XMLDBCException("XMLResultSet is already closed");
		MapItem mapItem = _mapper.map(path);
		//org.xquark.xml.misc.Debug.print(this,"IN	XDBCResultSet::GENERATE - mapItem = " + mapItem);

		if (null == mapItem) {
			throw new XMLDBCException(MessageLibrary.getMessage("IE_INV_ARG", "generate(String path, int nodeAccessor, String loopID, org.xml.sax.ContentHandler handler)", "path = " + path), null);
		}
		try {
			String tag = mapItem.getElementName();
			String namespace = mapItem.getNamespace();
			if (namespace == null)
				namespace = "";
			//org.xquark.xml.misc.Debug.print(this,"IN	XDBCResultSet::GENERATE - tag = " + tag);
			//org.xquark.xml.misc.Debug.print(this,"IN	XDBCResultSet::GENERATE - mapItem.isLeafPath() = " + mapItem.isLeafPath());
			if (mapItem.isLeafPath()) {
				//org.xquark.xml.misc.Debug.print(this,"IN	XDBCResultSet::GENERATE - mapItem.getItemName() = " + mapItem.getItemName());
				//org.xquark.xml.misc.Debug.print(this,"IN	XDBCResultSet::GENERATE - _sqlResultSet.getString(mapItem.getItemName() = " + _sqlResultSet.getString(mapItem.getItemName()));
				//org.xquark.xml.misc.Debug.print(this,"IN	XDBCResultSet::GENERATE - mapItem.getItemName() = " + mapItem.getItemName());
				String value = _sqlResultSet.getString(mapItem.getItemName());
				//org.xquark.xml.misc.Debug.print(this,"IN	XDBCResultSet::GENERATE - value = " + value);
				if (value != null) {
					if (null != tag) {
						if (nodeAccessor == 0 || nodeAccessor == Step.CHILD_NODE) {
						//if (nodeAccessor == 0 || nodeAccessor == XDBCResultSetInterface.NODE_ACCESSOR_NODE) {
							if (_contenthandler != null) {
								_contenthandler.startElement(namespace, tag, "", _nullAttribute);
								_contenthandler.characters(value.toCharArray(), 0, value.length());
								_contenthandler.endElement(namespace, tag, "");
							}
						} else if (nodeAccessor == Step.CHILD_TEXT) {
						//} else if (nodeAccessor == XDBCResultSetInterface.NODE_ACCESSOR_TEXT) {
							//org.xquark.xml.misc.Debug.print(this,"IN    XDBCResultSet::generate - STOP startElement = " + tag);
							if (_contenthandler != null)
								_contenthandler.characters(value.toCharArray(), 0, value.length());
						}
					} else if (_contenthandler != null) {
						_contenthandler.characters(value.toCharArray(), 0, value.length());
					}
				}
			} else {
				if (nodeAccessor == 0) {
					List children = mapItem.getChildren();
					if (null != tag) {
						if (_contenthandler != null) {
							_contenthandler.startElement(namespace, tag, "", _nullAttribute);
							for (int i = 0; i < children.size(); i++)
								generate((String) children.get(i), nodeAccessor);
							_contenthandler.endElement(namespace, tag, "");
						}
					} else {
						for (int i = 0; i < children.size(); i++) {
							generate((String) children.get(i), nodeAccessor);
						}
					}
				} else if (nodeAccessor == Step.CHILD_NODE) {
				//} else if (nodeAccessor == XDBCResultSetInterface.NODE_ACCESSOR_NODE) {
					List children = mapItem.getChildren();
					for (int i = 0; i < children.size(); i++) {
						generate((String) children.get(i), (tag == null) ? nodeAccessor : 0);
					}
				}
			}
		} catch (SAXException e) {
			throw new XMLDBCException(e.getMessage());
		} catch (SQLException e) {
			throw new XMLDBCException(e.getMessage());
		}
		//Trace.exit(this, "generate(String path)", path);
	}

	/**
	 * @see XDBCResultSetInterface#getIdentifiers
	 */
	public void getIdentifiers(Object[] identifiers) throws XMLDBCException {
		//Trace.enter(this, "getIdentifiers(Object[] identifiers)");

		if (_isClosed)
			throw new XMLDBCException("XMLResultSet is already closed");
		List keyList = _mapper.getIdentifierList();
		//org.xquark.xml.misc.Debug.print(this,"IN GETIDENTIFIER - keyList: "+keyList);
		MapItem keyItem;
		try {
			if (null == keyList) {
				throw new XMLDBCException("There is no identifier in the resultset.");
			}
			if (null != _partAttributeName) {
				for (int i = 0; i < keyList.size(); i++) {

					//org.xquark.xml.misc.Debug.print(this,"IN GETIDENTIFIER - _partItemName: "+_partItemName);

					long currentPart = _sqlResultSet.getLong(_partAttributeName);

					//org.xquark.xml.misc.Debug.print(this,"IN GETIDENTIFIER - currentPart: "+currentPart);
					keyItem = (MapItem) keyList.get(i);

					//org.xquark.xml.misc.Debug.print(this,"IN GETIDENTIFIER - keyItem: "+keyItem);
					if (keyItem.isLeafPath()) {
						identifiers[i] = _sqlResultSet.getString(keyItem.getItemName());
						if (null == identifiers[i] && keyItem.isPartOf(currentPart)) {
							identifiers[i] = _nullIdentifier;
						}
					} else {
						if (keyItem.isPartOf(currentPart)) {
							List idAttrs = new ArrayList();
							List children = _mapper.getChildren(keyItem);
							for (int j = 0; j < children.size(); j++) {
								MapItem child = (MapItem) children.get(j);
								String attrValue = _sqlResultSet.getString(child.getItemName());
								idAttrs.add(attrValue);
							}
							identifiers[i] = idAttrs;
						} else {
							identifiers[i] = null;
						}

					}

					//org.xquark.xml.misc.Debug.print(this,"IN GETIDENTIFIER - identifiers["+i+"]: "+identifiers[i]);

					//org.xquark.xml.misc.Debug.print(this,"IN GETIDENTIFIER - identifiers[i]: "+identifiers[i]);

				}
			} else {
				for (int i = 0; i < keyList.size(); i++) {
					keyItem = (MapItem) keyList.get(i);
					//org.xquark.xml.misc.Debug.print(this,"IN GETIDENTIFIER - keyItem: "+keyItem);
					identifiers[i] = _sqlResultSet.getString(keyItem.getItemName());
					if (null == identifiers[i]) {
						identifiers[i] = _nullIdentifier;
					}
					//org.xquark.xml.misc.Debug.print(this,"IN GETIDENTIFIER - identifiers["+i+"]: "+identifiers[i]);
				}
			}
		} catch (SQLException sqle) {
			throw new XMLDBCException(sqle.getMessage());
		}

		//Trace.exit(this, "getIdentifiers(Object[] identifiers)");
	}

	/**
	 * @see XDBCResultSetInterface#getNextIdentifiers
	 */
	public void getNextIdentifiers(Object[] identifiers) throws XMLDBCException {
		//Trace.enter(this, "getNextIdentifiers(Object[] identifiers)");

		if (_isClosed)
			throw new XMLDBCException("XMLResultSet is already closed");
		boolean hasNext;
		//		try
		//		{
		hasNext = _sqlResultSet.lookAhead();
		if (hasNext) {
			getIdentifiers(identifiers);
			_sqlResultSet.goBack();
		} else {
			identifiers = null;
		}
		//        }
		//		catch (SQLException sqle)
		//		{
		//			throw new XDBCException(sqle.getMessage());
		//		}

		//Trace.exit(this, "getNextIdentifiers(Object[] identifiers)");
	}

	/**
	 * @see XDBCResultSetInterface#next
	 */
	public boolean next() throws XMLDBCException {
		//Trace.enter(this, "next()");

		if (_isClosed)
			throw new XMLDBCException("XMLResultSet is already closed");
		boolean retVal = false;
		try {
			retVal = _sqlResultSet.next();

			///* For debugging */
			//				java.sql.ResultSetMetaData metaData = _sqlResultSet.getSqlResultSet().getMetaData();
			//				StringBuffer sb = new StringBuffer();
			//				for (int i = 1; i <= metaData.getColumnCount(); i++)
			//				{
			//					if (i > 1) sb.append(", " );
			//					sb.append(_sqlResultSet.getSqlResultSet().getString(i));
			//				}
			//org.xquark.xml.misc.Debug.print(this,"******************************************************************************");
			//org.xquark.xml.misc.Debug.print(this,"current ResultSet: ["+sb+"]");
			//org.xquark.xml.misc.Debug.print(this,"******************************************************************************");

		} catch (SQLException sqle) {
			//org.xquark.xml.misc.Debug.print(this,"IN	XDBCResultSet::next() -> error!!!!!!!!!!");
			throw new XMLDBCException(sqle.getMessage(), sqle);
		}

		//Trace.exit(this, "next()");
		return retVal;
	}

	/**
	 * @see XDBCResultSetInterface#hasNext
	 */

	public boolean hasNext() throws XMLDBCException {
		//Trace.enter(this, "hasNext()");
		if (_isClosed)
			throw new XMLDBCException("XMLResultSet is already closed");
		boolean retVal = _sqlResultSet.lookAhead();
		//org.xquark.xml.misc.Debug.print(this,"IN	hasNext::retVal -> "+retVal);
		if (retVal) {
			_sqlResultSet.goBack();
		}
		//Trace.exit(this, "hasNext()");
		return retVal;
	}

	/**
	 * @see XDBCResultSetInterface#setContentHandler
	 */
	public void setContentHandler(ContentHandler handler) {
		if (handler == null)
			throw new NullPointerException("ContentHandler can not be null.");
		this._contenthandler = handler;
	}

	/**
	 * @see XDBCResultSetInterface#setLexicalHandler
	 */
	public void setLexicalHandler(LexicalHandler handler) {
		if (handler == null)
			throw new NullPointerException("LexicalHandler can not be null.");
		this._lexicalhandler = handler;
	}

	/**
	 * @see XDBCResultSetInterface#setErrorHandler
	 */
	public void setErrorHandler(ErrorHandler handler) {
		if (handler == null)
			throw new NullPointerException("ErrorHandler can not be null.");
		this._errorhandler = handler;
	}

	/** @todo this function is to be removed after debug  */
	public ResultSet getSqlResultSet() {
		return _sqlResultSet.getSqlResultSet();
	}

	public Mapper getMapper() {
		return _mapper;
	}

	public void close() throws XMLDBCException {
		if (!_isClosed) {
			try {
				_isClosed = true;
				_sqlResultSet.close();
			} catch (SQLException ex) {
				throw new XMLDBCException(ex.getMessage(), ex);
			}
		}
	}

	class LookAheadSqlResultSet implements LookAheadResultSet {
		private static final String RCSRevision = "$Revision: 1.4 $";
		private static final String RCSName = "$Name: XQ1_1_1 $";

		private int BOR = 0; /* begin of resultset*/
		private int BUFFER = 1;
		private int RESULTSET = 2;

		private ResultSet _resultset;
		private List _buffer = new ArrayList();
		private List _attributeNameList = new ArrayList();
		private int _attributeNumber;

		private boolean _bufferNotEmpty;
		private boolean _resultSetNotEmpty;
		private int _cursor;

		public LookAheadSqlResultSet(ResultSet resultset) throws SQLException {
			//Trace.enter(this, "LookAheadSqlResultSet()");
			_resultset = resultset;

			ResultSetMetaData _rsm = _resultset.getMetaData();

			_attributeNumber = _rsm.getColumnCount();
			for (int i = 1; i <= _attributeNumber; i++) {
				_attributeNameList.add(_rsm.getColumnName(i));
			}

			_bufferNotEmpty = _resultset.next();
			if (_bufferNotEmpty) {
				refreshBuffer();
				_resultSetNotEmpty = _resultset.next();
			}
			_cursor = BOR;
			//Trace.exit(this, "LookAheadSqlResultSet()");
		}

		public boolean next() throws SQLException {
			//Trace.enter(this, "next()");
			boolean retVal;
			if (BOR == _cursor) {
				_cursor = BUFFER;
			} else {
				if (_resultSetNotEmpty) {
					refreshBuffer();
					_resultSetNotEmpty = _resultset.next();
				} else {
					_resultset.close();
					_bufferNotEmpty = false;
				}
			}
			//Trace.exit(this, "next()");
			return _bufferNotEmpty;
		}

		public boolean hasNext() {
			return _resultSetNotEmpty;
		}

		public boolean lookAhead() {
			//Trace.enter(this, "lookAhead()");
			boolean retVal = false;
			if (BOR == _cursor) {
				retVal = _bufferNotEmpty;
				//org.xquark.xml.misc.Debug.print(this,"IN	lookAhead (BOF == _cursor)::retVal -> "+retVal);
				if (retVal) {
					_cursor = BUFFER;
				}
			} else if (BUFFER == _cursor) {
				retVal = _resultSetNotEmpty;
				//org.xquark.xml.misc.Debug.print(this,"IN	lookAhead (BUFFER == _cursor)::retVal -> "+retVal);
				if (retVal) {
					_cursor = RESULTSET;
				}
			} else {
				org.xquark.extractor.common.Debug.assertTrue(false, "Logic error");
			}

			//Trace.exit(this, "lookAhead()");
			return retVal;
		}

		public boolean goBack() {
			//Trace.enter(this, "goBack()");
			boolean retVal = false;
			if (BUFFER == _cursor) {
				_cursor = BOR;
				retVal = true;
			} else if (RESULTSET == _cursor) {
				_cursor = BUFFER;
				retVal = true;
			} else if (BOR == _cursor) {
				retVal = false;
			}

			//Trace.exit(this, "goBack()");
			return retVal;
		}

		public long getLong(String name) throws SQLException {
			//Trace.enter(this, "getLong(String name)");
			long retVal = 0;
			if (BUFFER == _cursor) {
				retVal = getLongFromBuffer(name);
			} else {
				try {
					retVal = _resultset.getLong(name);
				} catch (SQLException e) {
				}
			}
			//Trace.exit(this, "getLong(String name)");
			return retVal;
		}

		public String getString(String name) throws SQLException {
			//Trace.enter(this, "getSting(String name)");
			String retVal = null;
			if (BUFFER == _cursor) {
				retVal = getStringFromBuffer(name);
			} else {
				try {
					retVal = _resultset.getString(name);
				} catch (SQLException e) {
				}
			}
			//Trace.exit(this, "getSting(String name)");
			return retVal;
		}

		private void refreshBuffer() throws SQLException {
			//Trace.enter(this, "refreshBuffer()");
			_buffer.clear();
			for (int i = 1; i <= _attributeNumber; i++) {
				_buffer.add(_resultset.getString(i));
			}
			//Trace.exit(this, "refreshBuffer()");
		}

		public String getStringFromBuffer(String name) {
			//Trace.enter(this, "getStingFromBuffer(String name)");
			String retVal = null;
			int index = -1;
			for (int i = 0; i < _attributeNameList.size(); i++) {
				if (name.equalsIgnoreCase((String) _attributeNameList.get(i))) {
					index = i;
					break;
				}
			}
			if (-1 != index) {
				if (!_buffer.isEmpty()) // added   22/03/2002
					retVal = (String) _buffer.get(index);
			}
			//Trace.exit(this, "getStingFromBuffer(String name)");
			return retVal;
		}

		public long getLongFromBuffer(String name) {
			//Trace.enter(this, "getStingFromBuffer(String name)");
			long retVal = 0;
			int index = -1;
			for (int i = 0; i < _attributeNameList.size(); i++) {
				if (name.equalsIgnoreCase((String) _attributeNameList.get(i))) {
					index = i;
					break;
				}
			}
			if (-1 != index) {
				if (!_buffer.isEmpty()) // added   22/03/2002
					retVal = Long.parseLong((String) _buffer.get(index));
			}
			//Trace.exit(this, "getStingFromBuffer(String name)");
			return retVal;
		}

		/** @todo this function is to be removed after debug  */
		public ResultSet getSqlResultSet() {
			return _resultset;
		}

		public void write(DataOutputStream dos) throws java.io.IOException {
			//Trace.enter(this, "write(DataOutputStream dos)");

			String newLine = System.getProperty("line.separator");

			dos.writeBytes("<LookAheadResultSet>");
			dos.writeBytes(newLine);

			dos.writeBytes("<header>");
			for (int i = 0; i < _attributeNameList.size(); i++) {
				Object obj = _attributeNameList.get(i);
				if (null != obj) {
					dos.writeBytes((String) obj);
					dos.writeBytes(",\t");
				}
			}
			dos.writeBytes("</header>");
			dos.writeBytes(newLine);

			try {
				while (next()) {
					dos.writeBytes("<row>");
					for (int i = 0; i < _buffer.size(); i++) {
						Object obj = _buffer.get(i);
						if (null != obj) {
							dos.writeBytes((String) obj);
							dos.writeBytes(",\t");
						}
					}
					dos.writeBytes("</row>");
					dos.writeBytes(newLine);
				}

				dos.writeBytes("</LookAheadResultSet>");
				dos.writeBytes(newLine);
				;
			} catch (SQLException ex) {
				ex.printStackTrace();
			}

			//Trace.exit(this, "write(DataOutputStream dos)");
		}
		public void close() throws SQLException {
			try {
				_resultset.close();
			} catch (SQLException ex) {
			}
		}
	}

	class HeavyElimination extends EliminatedSqlResultSet {
		List _keyColumnList = new ArrayList();
		String partAttributeName = _partAttributeName;
		HeavyElimination(ResultSet resultSet, int[] eliminationGuide) throws SQLException {
			super(resultSet, eliminationGuide);
			_keyColumnList = _mapper.getAllIdentifierAttributeNames();
		}

		protected boolean feedBuffer2() throws SQLException {
			//Trace.enter(this, "feedBuffer2()");
			boolean retVal = false;

			int oldCursor = _cursor;
			_cursor = BUFFER2;
			boolean isRedundant = false;
			if (_resultSetNotEmpty) {
				fillBuffer2();
				_resultSetNotEmpty = _resultSet.next();
				if (_buffer2NotEmpty) {
					if (_resultSetNotEmpty) {
						isRedundant = isRedundant();
						if (isRedundant) {
							retVal = feedBuffer2();
						} else {
							retVal = true;
						}
					} else {
						_resultSetNotEmpty = false;
						retVal = true;
					}
				} else {
					retVal = false;
				}
			} else {
				retVal = false;
				_buffer2NotEmpty = false;
				_resultSet.close();
			}
			_cursor = oldCursor;

			//Trace.exit(this, "feedBuffer2()");
			return retVal;
		}

		private boolean isRedundant() throws SQLException {
			//Trace.enter(this, "isIdenticalLines()");
			boolean retVal = true;

			int oldCursor = _cursor;
			_cursor = BUFFER2;
			String idInBuffer = null;
			String idInResultset = null;
			String keyColumnName = null;
			for (int i = 0; i < __keyColumnList.size(); i++) {
				keyColumnName = (String) __keyColumnList.get(i);
				idInBuffer = getStringFromBuffer(keyColumnName);
				idInResultset = _resultSet.getString(keyColumnName);
				if (null != idInBuffer && null != idInResultset) {
					retVal = idInBuffer.equals(idInResultset);
				}
				//                else if (null == idInBuffer && null != idInResultset) {
				//                    retVal = false;
				//                }
				else if (null != idInBuffer && null == idInResultset) {
					retVal = false;
				}
				if (!retVal) {
					break;
				}
			}
			_cursor = oldCursor;
			//Trace.exit(this, "isIdenticalLines()", retVal ? "true" : "false");
			return retVal;
		}
	}

	class EliminatedSqlResultSet implements LookAheadResultSet {
		private static final String RCSRevision = "$Revision: 1.4 $";
		private static final String RCSName = "$Name: XQ1_1_1 $";

		protected int BOR = 0; /* begin of resultset*/
		protected int BUFFER1 = 1;
		protected int BUFFER2 = 2;
		protected int RESULTSET = 3;

		protected ResultSet _resultSet;
		protected List _buffer1;
		protected List _buffer2;

		protected List _attributeNameList = new ArrayList();
		protected int _attributeNumber;

		protected boolean _buffer1NotEmpty = false;
		protected boolean _buffer2NotEmpty = false;
		;
		protected boolean _resultSetNotEmpty = false;
		;
		protected int _cursor = BOR;

		int[] _eliminationGuide;

		/* added for heavyElimination*/
		protected String __partAttributeName = null;
		protected List __keyColumnList = new ArrayList();
		/* end of added for heavyElimination*/

		public EliminatedSqlResultSet(ResultSet resultSet, int[] eliminationGuide) throws SQLException {
			//Trace.enter(this, "EliminatedSqlResultSet( ResultSet resultset, int[] eliminationGuide)");

			_resultSet = resultSet;
			_eliminationGuide = eliminationGuide;

			/* added for heavyElimination*/
			__partAttributeName = _partAttributeName;
			__keyColumnList = _mapper.getAllIdentifierAttributeNames();
			/* end of added for heavyElimination*/
			ResultSetMetaData _rsm = _resultSet.getMetaData();

			_resultSetNotEmpty = _resultSet.next();

			if (_resultSetNotEmpty) {
				_attributeNumber = _rsm.getColumnCount();
				for (int i = 1; i <= _attributeNumber; i++) {
					_attributeNameList.add(_rsm.getColumnName(i));
				}

				_buffer1 = new ArrayList(_attributeNumber);
				_buffer2 = new ArrayList(_attributeNumber);
				for (int i = 0; i < _attributeNumber; i++) {
					_buffer1.add(null);
					_buffer2.add(null);
				}

				feedBuffer2();
				feedBuffer1();
				feedBuffer2();

			}
			_cursor = BOR;

			//Trace.exit(this, "EliminatedSqlResultSet( ResultSet resultset, int[] eliminationGuide)");
		}

		protected boolean feedBuffer1() {
			//Trace.enter(this, "feedBuffer1()");
			boolean retVal = false;

			List tmp = _buffer1;

			_buffer1 = _buffer2;
			_buffer1NotEmpty = _buffer2NotEmpty;
			retVal = _buffer1NotEmpty;

			_buffer2 = tmp;
			_buffer2NotEmpty = false;

			//Trace.exit(this, "feedBuffer1()");
			return retVal;
		}

		protected boolean feedBuffer2() throws SQLException {
			//Trace.enter(this, "feedBuffer2()");
			boolean retVal = false;

			int oldCursor = _cursor;
			_cursor = BUFFER2;
			if (_resultSetNotEmpty) {
				fillBuffer2();
				_resultSetNotEmpty = _resultSet.next();
				if (_buffer2NotEmpty) {
					if (_resultSetNotEmpty) {
						long currentPartBitmap = getLongFromBuffer(_partAttributeName);
						int curQueryNo = 1;
						while (1 != currentPartBitmap) {
							currentPartBitmap = currentPartBitmap >> 1;
							curQueryNo++;
						}

						long nextParttBitmap = _resultSet.getLong(_partAttributeName);
						int nextQueryNo = 1;
						while (1 != nextParttBitmap) {
							nextParttBitmap = nextParttBitmap >> 1;
							nextQueryNo++;
						}

						if ((nextQueryNo <= _eliminationGuide[curQueryNo - 1]) && (nextQueryNo > curQueryNo)) {
							/* current data in buffer2 is not neccessary */
							//Trace.message(this, "feedBuffer2()", "tuple of query : " + Integer.toString(curQueryNo) + " is eliminated");
							retVal = feedBuffer2();
							org.xquark.extractor.common.Debug.assertTrue(retVal, "retVal");
						} else {
							retVal = true;
						}
					} else {
						_resultSetNotEmpty = false;
						retVal = true;
					}
				} else {
					retVal = false;
				}
			} else {
				retVal = false;
				_buffer2NotEmpty = false;
				_resultSet.close();
			}
			_cursor = oldCursor;

			//Trace.exit(this, "feedBuffer2()");
			return retVal;
		}

		protected void fillBuffer2() throws SQLException {
			//Trace.enter(this, "fillBuffer2()");
			for (int i = 1; i <= _attributeNumber; i++) {
				_buffer2.set(i - 1, _resultSet.getString(i));
			}
			_buffer2NotEmpty = true;

			//Trace.exit(this, "fillBuffer2()");
		}

		public boolean next() throws SQLException {
			//Trace.enter(this, "next()");
			boolean retVal = false;

			if (BOR == _cursor) {
				_cursor = BUFFER1;
				retVal = _buffer1NotEmpty;
			} else if (BUFFER1 == _cursor) {
				retVal = feedBuffer1();
				feedBuffer2();
			} else {
				org.xquark.extractor.common.Debug.assertTrue(false, "Logic error");
			}

			//Trace.exit(this, "next()");
			return retVal;
		}

		public boolean lookAhead() {
			//Trace.enter(this, "lookAhead()");
			boolean retVal = false;
			if (BOR == _cursor) {
				retVal = _buffer1NotEmpty;
				if (retVal) {
					_cursor = BUFFER1;
				}
			} else if (BUFFER1 == _cursor) {
				retVal = _buffer2NotEmpty;
				if (retVal) {
					_cursor = BUFFER2;
				}
			} else {
				org.xquark.extractor.common.Debug.assertTrue(false, "Logic error");
			}

			//Trace.exit(this, "lookAhead()");
			return retVal;
		}

		public boolean goBack() {
			//Trace.enter(this, "goBack()");
			boolean retVal = false;
			if (BUFFER1 == _cursor) {
				_cursor = BOR;
				retVal = true;
			} else if (BUFFER2 == _cursor) {
				_cursor = BUFFER1;
				retVal = true;
			} else if (BOR == _cursor) {
				retVal = false;
			}

			//Trace.exit(this, "goBack()");
			return retVal;
		}

		public long getLong(String name) throws SQLException {
			long retVal = getLongFromBuffer(name);
			return retVal;
		}

		public String getString(String name) throws SQLException {
			//Trace.enter(this, "getSting(String name)");

			String retVal = null;
			if (BUFFER1 == _cursor || BUFFER2 == _cursor) {
				retVal = getStringFromBuffer(name);
			} else {
				//                try {
				retVal = _resultSet.getString(name);
				//                }
				//                catch (Exception e) {}
			}

			//Trace.exit(this, "getSting(String name)");
			return retVal;
		}

		public String getStringFromBuffer(String name) {
			//Trace.enter(this, "getStingFromBuffer(String name)");
			String retVal = null;

			int index = -1;
			if (BUFFER1 == _cursor) {
				for (int i = 0; i < _attributeNameList.size(); i++) {
					if (name.equalsIgnoreCase((String) _attributeNameList.get(i))) {
						index = i;
						break;
					}
				}
				if (-1 != index) {
					retVal = (String) _buffer1.get(index);
				}
			} else if (BUFFER2 == _cursor) {
				for (int i = 0; i < _attributeNameList.size(); i++) {
					if (name.equalsIgnoreCase((String) _attributeNameList.get(i))) {
						index = i;
						break;
					}
				}
				if (-1 != index) {
					retVal = (String) _buffer2.get(index);
				}

			} else {
				org.xquark.extractor.common.Debug.assertTrue(false, "Logic error : cursor is not on buffers when calling getStringFromBuffer(String name)");
			}

			//Trace.exit(this, "getStingFromBuffer(String name)");
			return retVal;
		}

		public long getLongFromBuffer(String name) {
			//Trace.enter(this, "getLongFromBuffer(String name)");
			long retVal = 0;

			int index = -1;
			if (BUFFER1 == _cursor) {
				for (int i = 0; i < _attributeNameList.size(); i++) {
					if (name.equalsIgnoreCase((String) _attributeNameList.get(i))) {
						index = i;
						break;
					}
				}
				if (-1 != index) {
					retVal = Long.parseLong((String) _buffer1.get(index));
				}
			} else if (BUFFER2 == _cursor) {
				for (int i = 0; i < _attributeNameList.size(); i++) {
					if (name.equalsIgnoreCase((String) _attributeNameList.get(i))) {
						index = i;
						break;
					}
				}
				if (-1 != index) {
					retVal = Long.parseLong((String) _buffer2.get(index));
				}
			} else {
				org.xquark.extractor.common.Debug.assertTrue(false, "Logic error : cursor is not on buffers when calling getLongFromBuffer(String name)");
			}

			//Trace.exit(this, "getLongFromBuffer(String name)");
			return retVal;
		}

		/** @todo this function is to be removed after debug  */
		public ResultSet getSqlResultSet() {
			return _resultSet;
		}

		public void close() throws SQLException {
			try {
				_resultSet.close();
			} catch (SQLException ex) {
			}
		}

		public void print() throws SQLException {
			System.err.println(this.toString());
			System.err.println("===============================================");
			for (int i = 0; i < _attributeNameList.size(); i++) {
				System.err.print(_attributeNameList.get(i));
				System.err.print('\t');
			}
			System.err.println();

			while (next()) {
				for (int i = 0; i < _attributeNameList.size(); i++) {
					System.err.print(getString((String) _attributeNameList.get(i)));
					System.err.print('\t');
				}
				System.err.println();
			}
		}
	}
	interface LookAheadResultSet {
        public boolean next() throws SQLException;
        public boolean lookAhead();
        public boolean goBack();
        public long getLong(String name)throws SQLException;
        public long getLongFromBuffer(String name);
        public String getString(String name)throws SQLException;
        public String getStringFromBuffer(String name);
        public ResultSet getSqlResultSet();
        public void close() throws SQLException;
}
}
