/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.sql;

import java.util.Iterator;
import java.util.List;

public class SqlBinOpOuterJoin extends SqlBinaryOperator
{

	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";



	public static final int LEFT_OUTER_jOIN = 0;
	public static final int RIGHT_OUTER_jOIN = 0;

	protected int _outerJoinType ;
    protected List _predicateList;

    /**
    @roseuid 3BD17C0503E3
     */
    public SqlBinOpOuterJoin() {
    }


    public SqlBinOpOuterJoin(SqlExpression leftOperand, SqlExpression rightOperand, int outerJoinType, List predicateList)
    {
        super (leftOperand, rightOperand);
		setOuterJoinType(outerJoinType);
        setPredicateList(predicateList);
    }

	public int getOuterJoinType() {
		return _outerJoinType;
	}

	public void setOuterJoinType(int outerJoinType) {
		_outerJoinType = outerJoinType;
	}

    public List getPredicateList()
    {
        return _predicateList;
    }


    public void setPredicateList(List predicateList)
    {
        _predicateList = predicateList;
    }

    public String toSql (Context context)
    {
        //Trace.enter(this,"toSql(Context context)");

        StringBuffer retVal = new StringBuffer();

		retVal.append(getLeftOperand().toSql(context));

		if ( getOuterJoinType() == Constants.LEFT_OUTER_JOIN ) {
			retVal.append(" LEFT OUTER JOIN ");
		}
		else {
			retVal.append(" RIGHT OUTER JOIN ");
		}

		retVal.append(getRightOperand().toSql(context));

		retVal.append(" ON ");

		predicteListToSql(retVal, context);

        //Trace.exit(this,"toSql(Context context)");
        return retVal.toString();
    }

    protected StringBuffer predicteListToSql(StringBuffer statementBuffer, Context context) {
        //Trace.enter(this, "predicteListToSql(StringBuffer statementBuffer, Context context)");

        Iterator tmpItr = null;
        if ( null != _predicateList)
        {
            tmpItr = _predicateList.iterator();
            while (tmpItr.hasNext()) {
				statementBuffer.append('(');
				statementBuffer.append(((SqlExpression)tmpItr.next()).toSql(context));
				statementBuffer.append(") AND ");
            }
            statementBuffer.setLength(statementBuffer.length() - 5);
        }

        //Trace.exit(this, "predicteListToSql(StringBuffer statementBuffer, Context context)");
        return statementBuffer;
    }
}
