/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.sql;

import org.xquark.extractor.common.Debug;

/**
 * For Oracle-style outer join predicate specification ( (+)= in the where clause)  
 */
public class SqlOuterJoinPredicate extends SqlExpression
{
	private static final String RCSRevision = "$Revision: 1.5 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";

	private SqlExpression _predicate;
	protected SqlRenameRelation _innerTable;

	/**
	@param predicate
	@param nullGenerateTable
	@param outerTable
	@roseuid 3BD177F80226
	 */
	public SqlOuterJoinPredicate(SqlExpression predicate, SqlRenameRelation innerTable)
	{
		setPredicate (predicate);
		setInnerTable (innerTable);
	}

	/**
	@param predicate
	@param nullGenerateTable
	@roseuid 3BD177F801F4
	 */
	public SqlOuterJoinPredicate() {
	}

	/**
	Access method for the _predicate property.

	@return   the current value of the _predicate property
	 */
	public SqlExpression getPredicate()
	{
	  return _predicate;
	}

	/**
	Sets the value of the _predicate property.

	@param aPredicate the new value of the _predicate property
	 */
	public void setPredicate(SqlExpression aPredicate)
	{
		Debug.assertTrue(null!=aPredicate,"null!=aPredicate");
		_predicate = aPredicate;
	}

	public SqlRenameRelation getInnerTable()
	{
		return _innerTable;
	}

	public void setInnerTable(SqlRenameRelation innerTable)
	{
		Debug.assertTrue(null != innerTable, "null != innerTable");
		_innerTable = innerTable;
	}

	public String toSql(Context context)
	{
		//Trace.enter(this,"toSql(Context context)");
		context.outerJoinPredicate = true;
		context.innerTable = getInnerTable();

		String retVal = getPredicate().toSql(context);

		context.outerJoinPredicate = false;
		context.innerTable = null;

		//Trace.exit(this,"toSql(Context context)");
		return retVal;
	}
}
