/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.sql;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xquark.extractor.algebra.Expression;

public class SqlSelect extends SqlExpression implements Executable
{

	private static final String RCSRevision = "$Revision: 1.7 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";

    private boolean _selectDistinct;
    private SqlExpression _whereClause;
    private List _selectList;
    private List _orderbyClause;
    private List _groupbyClause;
    private SqlExpression _havingClause;
    private List _fromClause;
	private Expression _originalExpression;
    /**
    @roseuid 3BD177FC0041
     */
    public SqlSelect(Expression originalExpression) {
    	_originalExpression = originalExpression;
    }

    /**
    Access method for the _whereClause property.

    @return   the current value of the _whereClause property
     */
    public SqlExpression getWhereClause()
    {
        return _whereClause;
    }

    public void setWhereClause(SqlExpression aWhereClause)
    {
        //_whereClause = predicate ;
        _whereClause = aWhereClause;
    }

    /**
    Access method for the _selectList property.

    @return   the current value of the _selectList property
     */
    public List getSelectList()
    {
        return _selectList;
    }

    /**
    Sets the value of the _selectList property.

    @param aSelectList the new value of the _selectList property
     */
    public void setSelectList(List aSelectList)
    {
        _selectList = aSelectList;
    }

    /**
    Access method for the _orderbyClause property.

    @return   the current value of the _orderbyClause property
     */
    public List getOrderbyClause()
    {
        return _orderbyClause;
    }

    /**
    Sets the value of the _orderbyClause property.

    @param aOrderbyClause the new value of the _orderbyClause property
     */
    public void setOrderbyClause(List aOrderbyClause)
    {
        //_orderbyClause = sortSpecList ;
        _orderbyClause = aOrderbyClause;
    }

    /**
    Access method for the _groupbyClause property.

    @return   the current value of the _groupbyClause property
     */
    public List getGroupbyClause()
    {
        return _groupbyClause ;
    }

    /**
    Sets the value of the _groupbyClause property.

    @param aGroupbyClause the new value of the _groupbyClause property
     */
    public void setGroupbyClause(List aGroupbyClause)
    {
        //_groupbyClause = groupbyClause ;
        _groupbyClause = aGroupbyClause;
    }

    /**
    Access method for the _havingClause property.

    @return   the current value of the _havingClause property
     */
    public SqlExpression getHavingClause()
    {
        return _havingClause;
    }

    /**
    Sets the value of the _havingClause property.

    @param aHavingClause the new value of the _havingClause property
     */
    public void setHavingClause(SqlExpression aHavingClause)
    {
        _havingClause = aHavingClause;
    }

    /**
    Access method for the _fromClause property.

    @return   the current value of the _fromClause property
     */
    public List getFromClause()
    {
     return _fromClause;
    }

    /**
    Sets the value of the _fromClause property.

    @param aFromClause the new value of the _fromClause property
     */
    public void setFromClause(List aFromClause)
    {
        _fromClause = aFromClause;
    }

    /**
    @return boolean
    @roseuid 3BD177FC00FF
     */
    public boolean getSelectDistinct()
    {
        return _selectDistinct;
    }

    public void setSelectDistinct(boolean selectDistinct )
    {
        _selectDistinct = selectDistinct ;
    }

    /**
    Sets the value of the _selectClause property.
    @param aSelectClause the new value of the _selectClause property
    @param selectDistinct
    @param selectList
    @roseuid 3BD177FC013C
     */
    public void setSelectClause(boolean selectDistinct, List selectList)
    {
        _selectDistinct = selectDistinct ;
        _selectList = selectList ;
    }


//    /**
//    @param predicate
//    @roseuid 3BD177FC0358
//     */
//    public void addWhereClause(SqlExpression predicate)
//    {
//        if ( null == predicate ) {
//           return ;
//        }
//
//        if (null == _whereClause) {
//            _whereClause = new ArrayList();
//        }
//        _whereClause.add(predicate);
//    }



    /**
    @param sortSpecList
    @return java.util.List
    @roseuid 3BD177FE00D0
     */
    public List getOrderbyClause(List sortSpecList)
    {
        return _orderbyClause  ;
    }


    public void addFromClause(SqlExpression fromItem)
    {
        if ( null == _fromClause )
            _fromClause = new ArrayList () ;

        _fromClause.add(fromItem);
    }

    public String toSql(Context context)
    {
        Context newContext = new Context(context);

        StringBuffer retVal = new StringBuffer()  ;
        selectToSql(retVal, newContext);
        fromToSql(retVal, newContext);
        whereToSql(retVal, newContext);
        groupbyToSql(retVal, newContext);
        havingToSql(retVal, newContext);
        orderbyToSql(retVal, newContext);

        return retVal.toString() ;
    }

    protected StringBuffer selectToSql(StringBuffer statementBuffer, Context context) {
        statementBuffer.append("SELECT") ;
        if ( _selectDistinct )
            statementBuffer.append(" DISTINCT ");
        else
            statementBuffer.append(" ALL ");

        String tmpStr = "";
        Iterator    tmpItr ;
        context.selectList = true;
        if ( null != _selectList)
        {
            tmpItr = _selectList.iterator();
            while (tmpItr.hasNext()) {
                tmpStr += ((SqlExpression)tmpItr.next()).toSql(context) + " , ";
            }
            tmpStr = tmpStr.substring(0 ,tmpStr.length()-3 );
        }
        else
            tmpStr += " * " ;
        context.selectList = false;
        statementBuffer.append(tmpStr);

        return statementBuffer;
    }

    protected StringBuffer fromToSql(StringBuffer statementBuffer, Context context) {
        Iterator tmpItr = null;

        if ( null != _fromClause )
        {
			statementBuffer.append("\nFROM ");

            tmpItr = _fromClause.iterator();
            while (tmpItr.hasNext()) {
                SqlExpression relation = (SqlExpression)tmpItr.next();

                if (relation instanceof SqlSelect) {
					statementBuffer.append('(');
					statementBuffer.append(relation.toSql(context));
					statementBuffer.append(')');
                }
                else
					statementBuffer.append(relation.toSql(context));

                statementBuffer.append(' ');	// NOTE: not needed anymore since a SqlJoin node exists
            }
        }
        return statementBuffer;
    }

    protected StringBuffer whereToSql(StringBuffer statementBuffer, Context context) {
        if ( null != _whereClause )
        {
            statementBuffer.append("\nWHERE ");
            statementBuffer.append(_whereClause.toSql(context));
        }
        return statementBuffer;
    }

    protected StringBuffer groupbyToSql(StringBuffer statementBuffer, Context context) {
        // groupby clause
        String tmpStr = "" ;
        Iterator tmpItr = null;
        if ( null != _groupbyClause )
        {
            tmpItr = _groupbyClause.iterator();
            while (tmpItr.hasNext()) {
                tmpStr += ((SqlExpression)tmpItr.next()).toSql(context) + " , ";
            }
            tmpStr = tmpStr.substring(0 ,tmpStr.length()-3 );

            statementBuffer.append("\nGROUP BY ");
            statementBuffer.append(tmpStr);
        }
        return statementBuffer;
    }

    protected StringBuffer havingToSql(StringBuffer statementBuffer, Context context) {
        // having clause
        if ( null != _havingClause )
        {
            statementBuffer.append("\nHAVING ");
            statementBuffer.append(_havingClause.toSql(context));
        }
        return statementBuffer;
    }

    protected StringBuffer orderbyToSql(StringBuffer statementBuffer, Context context) {
        String tmpStr = "" ;
        Iterator tmpItr = null;
        if ( null != _orderbyClause && !_orderbyClause.isEmpty())
        {
            tmpItr = _orderbyClause.iterator();
            while (tmpItr.hasNext()) {
                tmpStr += ((SqlExpression)tmpItr.next()).toSql(context) + " , ";
            }
            tmpStr = tmpStr.substring(0 ,tmpStr.length()-3 );

            statementBuffer.append("\nORDER BY ");
            statementBuffer.append(tmpStr);
        }
        return statementBuffer;
    }

    public Expression getOriginalExpression() {
        return _originalExpression;
    }

}
