/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.xfunctions;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;

/**
6.2.4 xf:time
xf:time(string $srcval) => time
If the value of $srcval conforms to the lexical representation of a time as defined in [XML Schema
Part 2: Datatypes], the constructor returns the time corresponding to that representation.
Otherwise, the constructor returns the error value as defined in [XQuery 1.0 and XPath 2.0 Data
Model].

6.2.4.1 Examples
xf:time("11:33:24") returns a time value corresponding to 33 minutes and 24 seconds past 11 AM in
an unspecified timezone.

xf:time("23:33:24.35-05:00") returns a time value corresponding to 33 minutes and 24.35 seconds
past 11 PM for a timezone which is 5 hours behind Coordinated Universal Time (UTC).
 */
public final class XfTime extends XFunction
{

	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";


    public XfTime()
    {
        super("XfTime");
    }
    /**
    @param timeString
    @roseuid 3BD191EB0105
     */
    public XfTime(Expression timeString)
    {
        super("XfTime");
        setArgument(0,timeString);
    }

    /**
    @param visitor
    @return org.xquark.extractor.sql.SqlExpression
    @throws org.xquark.extractor.common.SqlWrapperException
    @roseuid 3BD1922B028E
     */
    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }
}
