/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.extractor.xfunctions;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;

/**
3.5.15 xf:upper-case
3.5.15.1 Signature
xf:upper-case(string $srcval) => string

3.5.15.2 Parameters
$srcval  static type is string

3.5.15.3 Semantics
Returns the value of $srcval after translating every lower-case letter to its upper-case
correspondent. Every lower-case letter that does not have an upper-case correspondent, and every
character that is not a lower-case letter, is included in the returned value in its original form.

A "lower-case letter" is a character whose Unicode General Category class includes "Ll". The
corresponding upper-case letter is determined using [Unicode Case Mappings].

3.5.15.4 Examples
xf:upper-case("abCd0") returns "ABCD0".
 */
public final class XfUpperCase extends Xfunction_String
{

	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";

    public XfUpperCase()
    {
      super("XfUpperCase");
    }

    public XfUpperCase(Expression arg )
    {
      super("XfUperCase");
      setArgument(0,arg);
    }

    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }
}
