/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.jdbc.typing;

import java.sql.Types;


    /**
    private String [] type_names ={"CHAR","VARCHAR", "LONGVARCHAR",
    "NUMERIC","DECIMAL","BIT","TINYINT","SMALLINT","INTEGER","BIGINT","REAL",
    "FLOAT","DOUBLE",
    "BINARY","VARBINARY","LONGVARBINARY",
    "DATE","TIME","TIMESTAMP"};
    @return java.lang.Object
    @roseuid 3B2798BF00B5
     */

public class DbType implements Cloneable
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";

	public static final int ORACLE_ROWID = 111;
	public static final int BOOLEAN = 16;	// because java.sql.Types.BOOLEAN does not exist in JDK 1.3 !!!


	/** @todo  */
	private int jdbcType;
	
	/** The size of this column */
	private long length = 0;

	/** Number of digit after decimal point */
	private int scale = 0;
	
    /** The native name of this column's data type */
	private String nativeType;
	
    /** Whether this column is a 'full text' type */
	private boolean isLongType = false;
	
    /** Whether this column is a LOB type */
	private boolean isLOB = false;

	public DbType() {}
	
	public DbType(int jdbcType) {
		this.jdbcType = jdbcType;
	}
	
	public DbType(TypeMap.SQLType sqlType) {
	    this(sqlType.getJDBCType());
		setNativeType(sqlType.getNativeTypeName());
	}
	
	/**
	 * @param jdbcType
	 * @param length
	 * @param scale
	 */
	public DbType(int jdbcType, int length, int scale) {
		this(jdbcType, length);
		this.scale = scale;
	}
	
	public DbType(int jdbcType, int size) {
		this.jdbcType = jdbcType;
		this.length = size;
	}

	public DbType(int length, int scale, TypeMap.SQLType sqlType) {
		this(sqlType);
		// if no length is pecified take the maximum for the type
		if (length == 0)
		    this.length = sqlType.getPrecision();
		else
		    this.length = length;
		this.scale = scale;
	}

	public Object clone () throws CloneNotSupportedException
    {
        return super.clone();
    };

    public boolean isCompatibleTo(DbType type)
    {
        return true;
    }

    /**
    Access method for the _name property.
    
    @return   the current value of the _name property
     */
    public String getName() {
        return getNameFromCode(jdbcType);
    }

    public int getPrimitiveTypeCode() {
        return jdbcType;
    }

    public boolean isPrimitive() {
        return true;
    }

    public boolean isUdt() {
        return false;
    }

    /**
    Access method for the _code property.
    
    @return   the current value of the _code property
     */
    public int getJDBCType() {
        return jdbcType;
    }

    /**
    Sets the value of the _code property.
    
    @param aCode the new value of the _code property
     */
    public void setJDBCType(int aCode) {
        jdbcType = aCode;
    }

    public String name() {
        return getNameFromCode(jdbcType);
    }

    /**
    @param code
    @return String
    @roseuid 3BD14FB5018A
     */
    protected static String getNameFromCode(int code) {
        String retVal = null;
        switch (code) {
            case Types.ARRAY:
                retVal = "ARRAY";
                break;
            case Types.BIGINT:
                retVal = "BIGINT";
                break;
            case Types.BINARY:
                retVal = "BINARY";
                break;
            case Types.BIT:
                retVal = "BIT";
                break;
            case Types.BLOB:
                retVal = "BLOB";
                break;
            case Types.CHAR:
                retVal = "CHAR";
                break;
            case Types.CLOB:
                retVal = "CLOB";
                break;
            case Types.DATE:
                retVal = "DATE";
                break;
            case Types.DECIMAL:
                retVal = "DECIMAL";
                break;
            case Types.DISTINCT:
                retVal = "DISTINCT";
                break;
            case Types.DOUBLE:
                retVal = "DOUBLE";
                break;
            case Types.FLOAT:
                retVal = "FLOAT";
                break;
            case Types.INTEGER:
                retVal = "INTEGER";
                break;
            case Types.JAVA_OBJECT:
                retVal = "JAVA_OBJECT";
                break;
            case Types.LONGVARBINARY:
                retVal = "LONGVARBINARY";
                break;
            case Types.LONGVARCHAR:
                retVal = "LONGVARCHAR";
                break;
            case Types.NULL:
                retVal = "NULL";
                break;
            case Types.NUMERIC:
                retVal = "NUMERIC";
                break;
            case Types.REAL:
                retVal = "REAL";
                break;
            case Types.SMALLINT:
                retVal = "SMALLINT";
                break;
            case Types.STRUCT:
                retVal = "STRUCT";
                break;
            case Types.TIME:
                retVal = "TIME";
                break;
            case Types.TIMESTAMP:
                retVal = "TIMESTAMP";
                break;
            case Types.TINYINT:
                retVal = "TINYINT";
                break;
            case Types.VARBINARY:
                retVal = "VARBINARY";
                break;
            case Types.VARCHAR:
                retVal = "VARCHAR";
                break;
            case ORACLE_ROWID:
                retVal = "ORACLE_ROWID";
                break;
            default:
                break;
        }
    
        return retVal;
    }

    public boolean isNumeric() {
        return  Types.INTEGER   == jdbcType ||
                Types.DECIMAL   == jdbcType ||
                Types.DOUBLE    == jdbcType ||
                Types.FLOAT     == jdbcType ||
                Types.REAL     == jdbcType  ||
                Types.NUMERIC   == jdbcType ||
                Types.SMALLINT  == jdbcType ||
                Types.BIGINT    == jdbcType ||
                Types.TINYINT   == jdbcType ;
    }

    public boolean isDecimal() {
        return  Types.DECIMAL   == jdbcType;
    }

    public boolean isInteger() {
        return  Types.INTEGER == jdbcType ||
                Types.SMALLINT== jdbcType ||
                Types.BIGINT== jdbcType ||
                Types.TINYINT == jdbcType ;
    }

    public boolean isString() {
        return  Types.CHAR  == jdbcType ||
        		Types.CLOB == jdbcType ||
                Types.VARCHAR == jdbcType ||
                Types.LONGVARCHAR == jdbcType ;
    }

    public boolean isDataTime() {
        return  Types.DATE == jdbcType ||
                Types.TIMESTAMP == jdbcType ||
                Types.TIME == jdbcType ;
    }

    public boolean isNull() {
        return  Types.NULL == jdbcType;
    }

    public String pprint() {
        return DbType.getNameFromCode(jdbcType) + "(" + Integer.toString(jdbcType) + ")";
    }
    public long getLength() {
        return length;
    }

    public void setLength(long length) {
        this.length = length;
    }

    public int getScale() {
        return scale;
    }

    public void setScale(int scale) {
        this.scale = scale;
    }

    public boolean isLOB() {
        return isLOB;
    }

    public void setLOB(boolean isLOB) {
        this.isLOB = isLOB;
    }

    public boolean isLongType() {
        return isLongType;
    }

    public void setLongType(boolean isLongType) {
        this.isLongType = isLongType;
    }

    public String getNativeType() {
        return nativeType;
    }

    public void setNativeType(String nativeType) {
        this.nativeType = nativeType;
    }

}
