/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper;

import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLDriverManager;

/** This class is used for maintaining unicity of access to repositories.
 * @deprecated The {@link RepositoryDataSource} or {@link RepositoryDriver}
 * approach is preferred.
 * @see org.xquark.mapper.RepositoryConnection
 */
final public class AccessManager
{
    private static final String RCSRevision = "$Revision: 1.2 $"; 
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    static
    {
        try
        {
            Class.forName("org.xquark.mapper.RepositoryDriver");
        }
        catch (ClassNotFoundException e)
        {
            // no op
        }
    }
    
    /** Constructor not accessible (factory model).
     */
    private AccessManager() {}
    
    /** Returns a RepositoryConnection object in order to access
     * XQuark Repository features.
     * You can open as many connections with the same parameters for use in a
     * multi-threaded context for instance.
     * <p><B>WARNING :</B>
     * Do not use differents URL for connecting the same relational
     * database or the AccessManager will consider different instances and
     * allocate multiple RepositoryConnection instantiations leading
     * to unpredictable results.</p>
     * @param URL JDBC string containing driver and database specification.
     * @param user rdbms user name.
     * @param password login password for rdbms user.
     * @throws java.sql.SQLException if a database-access error occurs.
     * @throws RepositoryException API exception.
     * @return the new RepositoryConnection object allocated.
     * @see RepositoryConnection
     */
    public static synchronized RepositoryConnection connect(String URL, String user, String password)
    throws XMLDBCException
    {
        return (RepositoryConnection)XMLDriverManager.getConnection(
                                            RepositoryDriver.REPOSITORY_URL_PREFIX + URL,
                                            user,
                                            password
                                            );
    }
}
