/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper;

import java.util.List;

import org.xquark.xml.xdbc.XMLCollection;
import org.xquark.xml.xdbc.XMLDBCException;

/** A <code>RepositoryCollection</code> object grants access to a collection of XML
 * documents. Using the SAX 2.0 or DOM level 2 interfaces, this object
 * will allow you to perform storing, reading and removal of XML documents.
 *
 * <p>A RepositoryCollection object is created every time every time you call 
 * the {@link RepositoryConnection#getRepositoryCollection(String) method. It
 * must be closed after use in order to release resources, escpecially  JDBC
 * resources.</p>
 * <p>This object is not synchronized since a {@link RepositoryConnection} is
 * monothread.</p>
 * @see org.xquark.mapper.RepositoryConnection
 */
public interface RepositoryCollection extends XMLCollection
{
    /** Creates a new {@link RepositoryDocument} object.
     * @param documentID the document string ID.
     * @return a RepositoryDocument object that may or may not exist. If not a
     * subsequent exception will be raised when performing data access.
     * @throws XMLDBCException API exception.
     */
     RepositoryDocument getRepositoryDocument(String documentID)
     throws XMLDBCException;
    
    /** Returns a list of{@link RepositoryDocument} objects.
     * @return a List object.
     * @throws XMLDBCException API exception.
     */
     List getRepositoryDocumentList() throws XMLDBCException;
    
    /** Creates indexes on storage tables.
     * <p>To use after document insertion when indexes have been dropped
     * to improve performance.</p>
     * <p><B>Warning :</B> Index reconstruction can be sometimes a quite
     * long process.</p>
     * @throws XMLDBCException API exception.
     */
     void createIndexes() throws XMLDBCException;

    /** Drops indexes on the storage tables.
     * <p>To use in order to improve performance during storage of a lot
     * of data in the XML collection.</p>
     * @throws XMLDBCException API exception.
     */
     void dropIndexes() throws XMLDBCException;
     
    /** Update DBMS statistics on collection tables to improve query performance.
     * @throws XMLDBCException API exception.
     */
     void updateStatistics() throws XMLDBCException;
}
