/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper;

import java.util.Date;

import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLDocument;

/** A <code>RepositoryDocument</code> object stands for an XML document stored
 * in the repository.
 * <p>It allows, in addition to the {@link org.xquark.xml.xdbc.XMLCollection XMLCollection}
 * API methods, to access specific information of repository XML documents.</p>
 * @see org.xquark.mapper.RepositoryCollection
 */
public interface RepositoryDocument extends XMLDocument
{
    /** Returns the number of XML nodes in the document (according to the XQuark
     * storage model). Nodes are elements, attributes, comments, text nodes...
     * @return XQuark storage model node count.
     * @throws RepositoryException API exception.
     */
     long getNodeCount() throws XMLDBCException;
     
    /** Returns the average number of characters of XML nodes of the document
     * @return a character count.
     * @throws RepositoryException API exception.
     */
     int getAverageNodeDataLength() throws XMLDBCException;
     
    /** Returns an estimation of the document length.
     * <p>The XML serialized data may differ in a significant way from this
     * estimation for several reasons:</p>
     * <ul>
     * <li>Character encoding,</li>
     * <li>Indentation characters,</li>
     * <li>Markup normalization,</li>
     * <li>etc.</li>
     * </ul>
     * @return a character count, not a byte count.
     * @throws RepositoryException API exception.
     */
     long getEstimatedDocumentLength() throws XMLDBCException;
     
    /** Returns the storage date of the XML file in the repository.
     * @throws RepositoryException API exception.
     * @return a java.util.Date.
     */
     Date getCreationDate() throws XMLDBCException;
     

}
