/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper;

import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLDataSource;

/**
 * XMLDBC driver implementation
 */
public final class RepositoryDriver implements org.xquark.xml.xdbc.XMLDriver
{
    private static final String RCSRevision = "$Revision: 1.4 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    public static final String REPOSITORY_URL_PREFIX = "xdbc:xquark:repository:";

	private static RepositoryDriver instance = new RepositoryDriver();
    
    /* Hashtable of currently opened data sources. */
    private Map openedSources = Collections.synchronizedMap(new HashMap());
    
    /** Constructor not accessible (factory model).
     */
    private RepositoryDriver() {
		org.xquark.xml.xdbc.XMLDriverManager.registerDriver(this);
    }
    
    public boolean acceptsURI(String uri) throws XMLDBCException
    {
        return  (uri.startsWith(REPOSITORY_URL_PREFIX));
    }
    
	public String getSpecificPart(String uri) throws XMLDBCException {
		if (acceptsURI(uri)) return uri.substring(REPOSITORY_URL_PREFIX.length());
		else throw new XMLDBCException("Unrecognized URI "+uri);
	}
    
    /**
     * To obtain a handle to a data source with the specified URI.
     * @param uri an URI corresponding to a data source.
     * @return an instance of the specified data source,
     *        or null if the driver does not know this type of data source
     * @throws XMLDBCException if a data source access error occurs.
     */
    public XMLDataSource getDataSource(String uri) throws XMLDBCException
    {
        if (!acceptsURI(uri))
            return null;
        
        XMLDataSource ds = (XMLDataSource)openedSources.get(uri);
        
        try
        {
            if (ds == null)
            {
                Class dsImpl = Class.forName("org.xquark.mapper.RepositoryDataSource");
                Class[] paramTypes = new Class[] {String.class};
                Object[] params = new Object[] { getSpecificPart(uri) };
                ds = (XMLDataSource)dsImpl.getConstructor(paramTypes).newInstance(params);
                openedSources.put(uri, ds);
            }
        }
        catch (InvocationTargetException e)
        {
        	if (e.getTargetException() instanceof XMLDBCException)
            	throw (XMLDBCException)e.getTargetException();
            else
//             	throw (RuntimeException)e.getTargetException();
           	throw new RepositoryException(RepositoryException.INTERNAL_ERROR,
            	"Error while instantiating data source ", e.getTargetException());
            	
        }
        catch (Exception e)
        {
            throw new XMLDBCException("internal error : XMLDataSource implementation could not be loaded.", e);
        }
        return ds;
   }

   /**
	* To obtain a handle to a data source with the specified URI, user name and password.
	* @param uri a URI corresponding to a data source.
	* @param user a user name
	* @param password a user password
	* @return an instance of the specified data source,
	*         or null if the driver does not know this type of data source
	* @throws XMLDBCException if a data source access error occurs.
	*/
    public XMLDataSource getDataSource(String uri, String user, String password) throws XMLDBCException {
        return null;
    }

}
