/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper;

import org.xquark.xml.xdbc.XMLDBCException;
/**
 * XMLCollection application exception that can encapsulate an original exception,
 * for instance, a SQLException
 *
 */
public class RepositoryException extends XMLDBCException
{
private static final String RCSRevision = "$Revision: 1.3 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
    // COMMENT IT SEEMS NOT SO COOL TO MAINTAIN MESSAGES TWO TIMES (here with comments and with constructor...)

	public static final int PERMISSION_DENIED = 2;
	/** Access denied */
	public static final int ACCESS_REFUSED = 3;
	/** Object already used in repository */
	public static final int ALREADY_EXISTS = 4;
	/** A failure occured with the SAX parser */
	public static final int PARSER_ERROR = 5;
	/** The data mapped in the rdbms is incoherent */
	public static final int CONSISTENCY_ERROR = 6;
        /** A SAX handling exception thrown by the output SAX Handler */
        public static final int SAX_OUTPUT_ERROR = 7;
        /** Data that should have been saved to database couldn't */
        public static final int DATA_LOSS = 8;
        /** Extra data loss (PI, Comment) */
        public static final int EXTRA_DATA_LOSS = 9;
        /** Object does not exists in repository */
        public static final int NOT_EXISTS = 10;
        /** A paramater passed to the repository API is not allowed */
        public static final int ILLEGAL_EXPRESSION = 11;
        /** The function is not supported yet */
        public static final int NOT_IMPLEMENTED = 12;
        /** This action is not allowed */
        public static final short NOT_ALLOWED = 13;
        /** The user database was found inconsistent : for instance some data that is to be fetched is missing */
        public static final int DB_CONSISTENCY_ERROR = 14;
        /** An internal error occured */
        public static final int INTERNAL_ERROR = 15;
        /** A system error like an IO error */
        public static final int SYSTEM_ERROR = 16;
        /** The metadata information for mapping is incoherent */
        public static final int MAPPING_CONSISTENCY_ERROR = 17;
        /** Error while XQuark XMLCollection was retrieving configuration information */
        public static final int XQUARK_CONF = 18;
        /** RDBMS raised a non-fatal error (e.g. a statement couldn't be closed) */
        public static final int DB_WARNING = 19;
        /** XML schema unconsistency */
        public static final int SCHEMA_ERROR = 20;
        /** The underlying database is not initialized by the repository */
        public static final int NO_INIT = 21;
        /** Attemp to modify read-only data */
        public static final int READ_ONLY = 22;
        /** Action interrupted by user */
        public static final int USER_ABORT = 23;
        /** RDBMS raised an error for no reason explicable by the application */
        public static final int DB_ERROR = 24;
        /** The value is out of repository bounds. Maybe space for resource is exhausted */
        public static final int OUT_OF_RANGE = 25;
        
        /** Constructor.
         * @param code the code featuring the kind of error
         * (see RepositoryException constants).
         * @param message A litteral message bringing precisions on the error.
         */
        public RepositoryException(int code, String message)
        {
            super(code, message);
        }
        
        /** Constructor for application exceptions encapsulating an underlying exception.
         * @param code the code featuring the kind of error
         * (see RepositoryException constants).
         * @param message A litteral message bringing precisions on the error.
         * @param underlyingException the original Exception that carried out this
         * exception.
         */
        public RepositoryException(int code, String message, Throwable underlyingException)
        {
            super(code, message, underlyingException);
        }

        /** Returns a String describing the Exception and the underlying
         * exception if any.
         * @return a printable string.
         */
        public String toString()  // TO RETROFIT 1.0? (change from getMessage to toString)
        {
            String message = "XQuark Storage exception : " + super.getMessage();
            if ((exception != null) && 
                ((errorCode == DB_ERROR) || (errorCode == PARSER_ERROR)
                || (errorCode == SAX_OUTPUT_ERROR) || (errorCode == SYSTEM_ERROR)))
                message += "\nUnderlying exception : " + exception.getMessage();
            return message;
        }
       
        /** Prints the stack of the Exception or the underlying one if existing
         */
        public void printStackTrace()
        {
            if (exception != null)
                exception.printStackTrace();
            else
                super.printStackTrace();
        }
}
