/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.dbms;


/**
 * Class containing information for relational table constraint creation (shared object).
 *
 */
public class ConstraintSpec
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    /* dbms table constraintstypes */
    public static final byte CONSTRAINT_UNIQUE = 0;
    
    private static final String uniqueString = "unique";
    
    protected byte type;
    protected ColumnSpec[] columns = null;
    
    //////////////////////////////////////////////////////////////////////////
    //                          CONSTRUCTORS
    //////////////////////////////////////////////////////////////////////////
    protected ConstraintSpec() {}
    
    public ConstraintSpec(String type)
    {
        this.type = parseType(type);
    }
    
    public String generateConstraintClause()
    {
        StringBuffer clause = new StringBuffer();
        clause.append("UNIQUE");
        clause.append(generateColumnList());
        return clause.toString();
    }
    
    public String generateColumnList()
    {
        StringBuffer clause = new StringBuffer();
        clause.append('(');
        for (int i = 0; i < columns.length; i++)
        {
            clause.append(columns[i].getName());
            clause.append(",");
        }
        clause.setCharAt(clause.length() - 1, ')');
        return clause.toString();
    }
    
    //////////////////////////////////////////////////////////////////////////
    //                          ACCESSORS
    //////////////////////////////////////////////////////////////////////////
    public byte getType()
    {
        return type;
    }
    
    public void setColumns(ColumnSpec[] columns)
    {
        this.columns = columns;
    }

    private byte parseType(String type)
    {
        if (!type.equals(uniqueString))
            throw new RuntimeException("The DBMS table type" + type + " used in tableSpec.xml is unknown.");
        return CONSTRAINT_UNIQUE;
    }
}
