/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.dbms;

import java.sql.*;

import org.xquark.jdbc.typing.DBMSInfo;

/**
 * Base implementation of AbstractConnection for JDBC 2 databases.
 *
 * <p>This class is abstract because some methods like table creation
 * are inevitably specific.</p>
 */
public class JDBC2Connection extends DefaultConnectionImpl
{
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    public JDBC2Connection(Connection connection, short dbms)
    throws SQLException
    {
        super(connection, dbms);
    }

    protected JDBC2Connection(Connection connection, short dbms, DBMSInfo dbmsInfo)
    throws SQLException
    {
        super(connection, dbms, dbmsInfo);
    }

    /////////////////////////////////////////////////////
    //                   JDBC2 specific methods
    ///////////////////////////////////////////////////////////////
    /** Add a SQL statement in a batch for delayed execution.
     * @param stmt the JDBC statement wich the SQL statement is to be added to
     */    
    public void addBatch(PreparedStatement stmt) throws SQLException
    {
        stmt.addBatch();
    }
    
    /** Add a SQL statement in a batch for delayed execution.
     * @param stmt the JDBC statement which the SQL statement is to be added to
     * @param SQLStatement the SQL statement that is to be added
     */    
    public void addBatch(Statement stmt, String SQLStatement) throws SQLException
    {
        stmt.addBatch(SQLStatement);
    }
    /** Execute a batch created with the addBatch method.
     * @param stmt the JDBC statement wich the SQL statement is to be added to
     * @return an array of update counts containing one element for each command
     * in the batch. The elements of the array are ordered according to the
     * order in which commands were added to the batch.
     */    
    public int[] executeBatch(Statement stmt) throws SQLException
    {
        int[] ret = stmt.executeBatch();
        stmt.clearBatch();
        return ret;
    }
    
    /** Gives the JDBC driver a hint as to the number of rows that should be fetched
     * from the database when more rows are needed.
     * @param stmt the JDBC statement wich the SQL statement is to be added to
     * @param rows  the number of rows to fetch
     */
    public void setFetchSize(Statement stmt,int rows) throws SQLException
    {
        stmt.setFetchSize(rows);
    }
}
