/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * TableSpecLoader.java
 *
 * Created on 12 janvier 2001, 09:25
 */

package org.xquark.mapper.dbms;

import java.net.URL;
import java.util.*;

import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import org.xquark.mapper.metadata.RepositoryConstants;
import org.xquark.schema.validation.*;

/**
 *
 */
public class TableSpecLoader extends DefaultHandler
{
    private static final String RCSRevision = "$Revision: 1.2 $"; 
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    
    /** Use to "filter" repositories, in order to ignore repositories for
     * which the current software is unsuited.
     * <p><B>This version is not necessarily the repository runtime version
     * but the last version since any modification to the dbms data structure
     * has been made. IT IS A DATA_COMPATIBILITY VERSION</B></p>
     */
    public static String REPOSITORY_DATA_VERSION;

    /////////////////////////
    //  Constants
    /////////////////////////
    /* Element names */
    private final static String SCHEMA_FILENAME = "TableSpec.xsd";
    private final static String DATA_FILENAME = "TableSpec.xml";
    private final static String ROOT_ELEMENT = "tables";
    private final static String VERSION_ATTR = "version";
    private final static String TABLE_ELEMENT = "table";
    private final static String TYPE_ATTR = "type";
    private final static String SUFFIX_ATTR = "suffix";
    private final static String SEQ_ATTR = "sequence";
    private final static String CAT_ATTR = "cat";
    private final static String DBMS_ATTR = "dbms";
    private final static String DBMS_TYPE_ATTR = "dbmsType";
    
    private final static String COLUMN_ELEMENT = "column";
    private final static String NAME_ATTR = "name";
    private final static String NULLABLE_ATTR = "nullable";
    private final static String KEY_ATTR = "key";
    
    private final static String JDBC_TYPE_ELEMENT = "type";
    private final static String JDBC_TYPE_ATTR = "jdbc";
    private final static String MAX_LEN_ELEMENT = "maxLen";
    private final static String PRECISION_ELEMENT = "precision";
    private final static String SCALE_ELEMENT = "scale";
    private final static String PARAM_ATTR = "param";
    
    private final static short ANY_DBMS = 0;

    private final static String CONSTRAINT_ELEMENT = "constraint";
    private final static String COLUMNS_ATTR = "columns";
    
    private final static String INDEX_ELEMENT = "index";
    
    /* Automaton status */
    private final static short UNKNOWN = 0;
    private final static short ROOT = 1;
    private final static short TABLE = 2;
    private final static short COLUMN = 3;
    private final static short JDBC_TYPE = 4;
    private final static short KEY = 5;
    private final static short NULLABLE = 6;
    private final static short CONSTRAINT = 7;
    private final static short INDEX = 8;
    private final static short MAX_LEN = 9;
    private final static short PRECISION = 10;
    private final static short SCALE = 11;
    
    /////////////////////////
    //  Global variables
    /////////////////////////
    private static TableSpecLoader singleton;
    private List[] tables = new List[TableSpec.CAT_COUNT];  // index is category number
    // accs  liste pour une catgorie
    // plus accs pour une categorie, type, sequence (parcours de liste)
    private SchemaValidationContext context;
    private short status = UNKNOWN;
    private TableSpec wTable;
    private ColumnSpec wColumn;
    private List columns = new ArrayList();
    private Map columnMap = new HashMap();
    private List constraints = new ArrayList();
    private List indexes = new ArrayList();
    
    static
    {
        singleton = new TableSpecLoader();
    }
    
    private TableSpecLoader()
    {
        for(int i = 0; i < TableSpec.CAT_COUNT; i++)
            tables[i] = new ArrayList();
        load();
    }
    
    public static TableSpecLoader getInstance()
    {
        return singleton;
    }
    
    private void load()
    {
        // use URL to provide a base URI for schema loading (relative URI)
        URL in = TableSpecLoader.class.getResource(RepositoryConstants.RESOURCES_FOLDER + DATA_FILENAME);
        if (in == null)
            throw new RuntimeException("Resource " + DATA_FILENAME + " couldn't be found.");
        InputSource resource = new InputSource(in.toString());
        XMLReader parser;
        try
        {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            factory.setNamespaceAware(true);
            parser = factory.newSAXParser().getXMLReader();
        }
        catch (Exception e)
        {
            throw new RuntimeException("JAXP exception while intantiating a SAX parser for internal use (" + e.getMessage() + ").");
        }
        
        context = new SchemaValidationContext();
        ValidatingSchemaFilter filter = new ValidatingSchemaFilter(parser, context);
        filter.setContentHandler(this);
        try
        {
            filter.parse(resource);
        }
        catch (Exception e)
        {
            throw new RuntimeException("Exception while parsing Repository table specifications (" + e.getMessage() + ").");
        }
        //            parser.parse(resource); why does not work ? Why ValidatingSchemaFilter extends DecoratorFilter
    }
    
    /* this method only works if tableSpec.xml is unmodified */
    public List getTableSpecs(byte cat)
    {
        return tables[cat];
    }
    
    /* this method only works if tableSpec.xml is unmodified */
    public List getTableSpecs(byte cat, short vendor)
    {
        ArrayList res = new ArrayList();
        TableSpec wTable;
        Iterator it = tables[cat].iterator();
        while (it.hasNext())
        {
            wTable = (TableSpec)it.next();
            if (wTable.getDBMSVendor() == ANY_DBMS || wTable.getDBMSVendor() == vendor )
                res.add(wTable);
        }
        return res;
    }
    
    /* this method only works if tableSpec.xml is unmodified */
    public TableSpec getTableSpec(byte cat, byte tableType)
    {
        TableSpec res = null, wTable;
        Iterator it = tables[cat].iterator();
        while (it.hasNext())
        {
            wTable = (TableSpec)it.next();
            if (wTable.getType() == tableType)
            {
                res = wTable;
                break;
            }
        }
        return res;
    }
    
    public void startElement(String ns, String local, String qName, Attributes atts)
    throws SAXException
    {
        /* Setting automaton status */
        switch (status)
        {
            case UNKNOWN:
                if (local.equals(ROOT_ELEMENT))
                    status = ROOT;
                break;
            case ROOT:
                if (local.equals(TABLE_ELEMENT))
                    status = TABLE;
               break;
            case TABLE:
                if (local.equals(COLUMN_ELEMENT))
                    status = COLUMN;
                else if (local.equals(CONSTRAINT_ELEMENT))
                    status = CONSTRAINT;
                else if (local.equals(INDEX_ELEMENT))
                    status = INDEX;
                break;
            case COLUMN:
                if (local.equals(JDBC_TYPE_ELEMENT))
                    status = JDBC_TYPE;
                break;
            case JDBC_TYPE:
                if (local.equals(MAX_LEN_ELEMENT))
                    status = MAX_LEN;
                else if (local.equals(PRECISION_ELEMENT))
                    status = PRECISION;
                else if (local.equals(SCALE_ELEMENT))
                    status = SCALE;
                break;
             default:
        }
        
        /* Creating parent objects */
        ElementPSVInfoset infoset = context.getCurrentInfoset();
        switch (status)
        {
            case ROOT:
                REPOSITORY_DATA_VERSION = context.getCurrentInfoset().getAttributePSVInfoset(null, VERSION_ATTR).getNormalizedValue();
                break;
            case TABLE:
                wTable = new TableSpec();   // because IndexSpec objects reference it
                break;
            case COLUMN:
                wColumn = new ColumnSpec();
               break;
            default:
        }
        
      }
    
    public void endElement(String ns, String local, String qName)
    throws SAXException
    {
        /* processing leaf elements */
        ElementPSVInfoset infoset = context.getCurrentInfoset();
        switch (status)
        {
            case TABLE:
                completeTable(infoset);
                break;
                
            case COLUMN:
                completeColumn(infoset);
                break;
                
            case JDBC_TYPE:
                fillColumnType(infoset);
                break;

            case MAX_LEN:
                fillSize(infoset);
                break;

            case PRECISION:
                fillPrecision(infoset);
                break;

            case SCALE:
                fillScale(infoset);
                break;

            case INDEX:
                createIndex(infoset);
                break;
                
            case CONSTRAINT:
                createConstraint(infoset);
                break;
             default:
        }
        
        /* Setting automaton status */
        switch (status)
        {
            case ROOT:
                status = UNKNOWN;
            break;
            case TABLE:
                status = ROOT;
            break;
            case COLUMN:
            case INDEX:
            case CONSTRAINT:
                status = TABLE;
            break;
            case JDBC_TYPE:
            case KEY:
            case NULLABLE:
                status = COLUMN;
            break;
            case MAX_LEN:
            case PRECISION:
            case SCALE:
                status = JDBC_TYPE;
            break;
            default:
        }
    }
    
    void completeTable(ElementPSVInfoset infoset)
    {
        String dbmsType = null;
        String suffix = null;
        boolean seq = false;
        byte cat = -1;
        short dbms = -1;
        byte tableType = -1;
        
        int attCount = infoset.getAttributeCount();
        PSVInfoset attInfoSet = null;
        
        for (int i = 0; i < attCount; i++)
        {
            attInfoSet = infoset.getAttributePSVInfoset(i);
            if (attInfoSet.getLocalName().equals(DBMS_TYPE_ATTR))
                dbmsType = attInfoSet.getNormalizedValue();
            else if (attInfoSet.getLocalName().equals(TYPE_ATTR))
                tableType = ((Number)attInfoSet.getActualValue()).byteValue();
            else if (attInfoSet.getLocalName().equals(CAT_ATTR))
                cat = ((Number)attInfoSet.getActualValue()).byteValue();
            else if (attInfoSet.getLocalName().equals(DBMS_ATTR))
                dbms = ((Number)attInfoSet.getActualValue()).byteValue();
            else if (attInfoSet.getLocalName().equals(SUFFIX_ATTR))
                suffix = attInfoSet.getNormalizedValue();
            else if (attInfoSet.getLocalName().equals(SEQ_ATTR))
                seq = ((Boolean)attInfoSet.getActualValue()).booleanValue();
        }
        
        wTable.set(cat, tableType, seq, suffix, dbms, dbmsType);
        wTable.setColumns((ColumnSpec[])columns.toArray(new ColumnSpec[columns.size()]));
        wTable.setConstraints((ConstraintSpec[])constraints.toArray(new ConstraintSpec[constraints.size()]));
        wTable.setIndexes((IndexSpec[])indexes.toArray(new IndexSpec[indexes.size()]));

        /* registrating column */
        ((List)tables[cat]).add(wTable);
        
        /* cleaning lists */
        columns.clear();
        indexes.clear();
        constraints.clear();
    }
    
    void fillColumnType(ElementPSVInfoset infoset)
    {
        wColumn.setDataType(infoset.getAttributePSVInfoset(
                                        null, JDBC_TYPE_ATTR).getNormalizedValue());
    }
    
    void fillSize(ElementPSVInfoset infoset)
    {
        wColumn.setSize (
        ((Number)infoset.getActualValue()).longValue(),
        ((Number)infoset.getAttributePSVInfoset(null, PARAM_ATTR).getActualValue()).shortValue()
        );
    }
    
    void fillPrecision(ElementPSVInfoset infoset)
    {
        wColumn.setPrecision (
        ((Number)infoset.getActualValue()).intValue(),
        ((Number)infoset.getAttributePSVInfoset(null, PARAM_ATTR).getActualValue()).shortValue()
        );
    }
    
    void fillScale(ElementPSVInfoset infoset)
    {
        wColumn.setScale (
        ((Number)infoset.getActualValue()).intValue(),
        ((Number)infoset.getAttributePSVInfoset(null, PARAM_ATTR).getActualValue()).shortValue()
        );
    }

    void completeColumn(ElementPSVInfoset infoset)
    {
        int attCount = infoset.getAttributeCount();
        PSVInfoset attInfoSet = null;
        
        String name = null;
        
        for (int i = 0; i < attCount; i++)
        {
            attInfoSet = infoset.getAttributePSVInfoset(i);
            if (attInfoSet.getLocalName().equals(NAME_ATTR))
            {
                name = attInfoSet.getNormalizedValue();
                wColumn.setName(name);
            }
            else if (attInfoSet.getLocalName().equals(KEY_ATTR))
                wColumn.setKeyRole(((Boolean)attInfoSet.getActualValue()).booleanValue());
            else if (attInfoSet.getLocalName().equals(NULLABLE_ATTR))
                wColumn.setNullable(((Boolean)attInfoSet.getActualValue()).booleanValue());
        }
        /* registrating column */
        if (name != null)
        {
            columnMap.put(name, wColumn);
            columns.add(wColumn);
        }
    }
    
    void createIndex(ElementPSVInfoset infoset)
    {
        String type = null;
        String suffix = null;
        List indexColumns = new ArrayList();
        
        int attCount = infoset.getAttributeCount();
        PSVInfoset attInfoSet = null;
        
        for (int i = 0; i < attCount; i++)
        {
            attInfoSet = infoset.getAttributePSVInfoset(i);
            if (attInfoSet.getLocalName().equals(TYPE_ATTR))
                type = attInfoSet.getNormalizedValue();
            else if (attInfoSet.getLocalName().equals(SUFFIX_ATTR))
                suffix = attInfoSet.getNormalizedValue();
            else if (attInfoSet.getLocalName().equals(COLUMNS_ATTR))
            {
            	List columnNames = (List)attInfoSet.getActualValue();
            	for (Iterator it = columnNames.iterator();it.hasNext();) {
            		String columnName = (String)it.next();
					wColumn = (ColumnSpec)columnMap.get(columnName);
					if (wColumn == null)
						throw new RuntimeException("The " + columnName
						+ " column used in an index defined in tableSpec.xml does not exist.");
					else
						indexColumns.add(wColumn);
            	}
            	/*
                String columnNames = (String)attInfoSet.getActualValue();
                StringTokenizer tokenizer = new StringTokenizer(columnNames);
                String columnName;
                while (tokenizer.hasMoreTokens())
                {
                    columnName = tokenizer.nextToken();
                    wColumn = (ColumnSpec)columnMap.get(columnName);
                    if (wColumn == null)
                        throw new RuntimeException("The " + columnName
                        + " column used in an index defined in tableSpec.xml does not exist.");
                    else
                        indexColumns.add(wColumn);
                }
                */
            }
        }
        IndexSpec wIndex = new IndexSpec(wTable, type, suffix);
        wIndex.setColumns((ColumnSpec[])indexColumns.toArray(new ColumnSpec[indexColumns.size()]));
        indexes.add(wIndex);
    }
    
    void createConstraint(ElementPSVInfoset infoset)
    {
        String type = null;
        List constraintColumns = new ArrayList();
        
        int attCount = infoset.getAttributeCount();
        PSVInfoset attInfoSet = null;
        
        for (int i = 0; i < attCount; i++)
        {
            attInfoSet = infoset.getAttributePSVInfoset(i);
            if (attInfoSet.getLocalName().equals(TYPE_ATTR))
                type = attInfoSet.getNormalizedValue();
            else if (attInfoSet.getLocalName().equals(COLUMNS_ATTR))
            {
				List columnNames = (List)attInfoSet.getActualValue();
				for (Iterator it = columnNames.iterator();it.hasNext();) {
					String columnName = (String)it.next();
					wColumn = (ColumnSpec)columnMap.get(columnName);
					if (wColumn == null)
						throw new RuntimeException("The " + columnName
						+ " column used in an index defined in tableSpec.xml does not exist.");
					else
					constraintColumns.add(wColumn);
				}
				/*
                String columnNames = (String)attInfoSet.getActualValue();
                StringTokenizer tokenizer = new StringTokenizer(columnNames);
                String columnName;
                while (tokenizer.hasMoreTokens())
                {
                    columnName = tokenizer.nextToken();
                    wColumn = (ColumnSpec)columnMap.get(columnName);
                    if (wColumn == null)
                        throw new RuntimeException("The " + columnName
                        + " column used in a constraint defined in tableSpec.xml does not exist.");
                    else
                        constraintColumns.add(wColumn);
                }
                */
            }
        }
        ConstraintSpec wIndex = new ConstraintSpec(type);
        wIndex.setColumns((ColumnSpec[])constraintColumns.toArray(new ColumnSpec[constraintColumns.size()]));
        constraints.add(wIndex);
    }
    
}
