/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.dbms;

import org.xquark.mapper.RepositoryConnection;
import org.xquark.mapper.metadata._Repository;
import org.xquark.mapper.storage.RepositoryReader;
import org.xquark.schema.SchemaLocator;
import org.xquark.schema.SchemaManager;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 * Provides functions for manipulating repositories.
 * <B>This is the internal version of the Repository interface (with methods
 * that don't belong to API)</B>
 */
public interface _RepositoryConnection
extends RepositoryConnection, SchemaLocator
{
	/**
	 * Accessor to the underlying JDBC connection (in fact an interface
	 * encapsulating the connection managing DBMS specificities).
	 * @return the "portable" RDBMS connection
	 */
    AbstractConnection getConnection();
    
	/**
	 * Accessor to the repository metadata object.
	 * @return a readonly interface to repository metadata object.
	 */
    _Repository getMetadata();
    
	/**
	 * Accessor to the XML schema manager used by this connection to repository.
	 * @return the repository memory instance schema manager
	 */
    SchemaManager getSchemaManager();
    
	/**
	 * Creates a SAX XMLReader on mapping system collection
	 * @return a Repository specific implementation of XMLReader
	 * @throws XMLDBCException application exception
	 */
    RepositoryReader getMappingReader() throws XMLDBCException;
}


