/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.dbms.microsoft;

import java.sql.*;

import org.xquark.jdbc.typing.DBMSInfo;
import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.dbms.JDBC2Connection;

/**
 * Implementation of AbstractConnection for Microsoft SQL Server 2000.
 *
 */
public class SQS2000Connection extends JDBC2Connection
{
    private static final String RCSRevision = "$Revision: 1.3 $"; 
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    
    /** Creates new SQS2000Connection */
    public SQS2000Connection(Connection connection, short dbms, DBMSInfo dbmsInfo)
    throws SQLException
    {
        super(connection, dbms, dbmsInfo);
    }

	protected void setSchemaName() throws SQLException
	{
		Statement stmt = null;
		try {
			stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery("select CURRENT_USER"); 	// user_name() works too
			if (rs.next())
				schemaName = rs.getString(1);
			rs.close();
		}
		finally {
			stmt.close();
		}
	}

    ///////////////////////////////////////////////////////////////
    //                      Table methods
    ///////////////////////////////////////////////////////////////
    public void dropTemporaryTable(String tableName) throws SQLException
    {
        super.dropTemporaryTable("#" + tableName);
    }
    
    public String getDeleteUserTableRowsStatement(String tableName, String documentRowsSelectionClause)
    throws SQLException, RepositoryException
    {
        StringBuffer deletionStmt = new StringBuffer();
        deletionStmt.append("DELETE ");
        deletionStmt.append(tableName);
        deletionStmt.append(' ');
        deletionStmt.append(documentRowsSelectionClause);
        // IF PROBLEM, USE the syntax DELETE A FROM A INNER JOIN B ON A.ID=B.ID WHERE B BETWEEN X AND Y
        
        return deletionStmt.toString();
    }
    
    // TO IMPLEMENT ?    public String getPathIndexStatement()
    
	public boolean useStringDelimitor()
	{
		return false;
		// COULD BE FALSE IF "SET ANSI_DEFAULTS ON" was called : TO DO
		// COULD be controlled for generic and extra (user data (mapping) does not use this flag)
	}

	public void onInitConnection() throws SQLException
	{
		executeUpdate("SET ANSI_DEFAULTS ON");
	}

    ///////////////////////////////////////////////////////////////
    //                      Private utilities
    ///////////////////////////////////////////////////////////////
    // TO IMPLEMENT ?    public void updateStatistics(String tableName) throws SQLException
    
}
