/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.dbms.sybase;

import java.sql.*;

import org.xquark.jdbc.typing.DBMSInfo;
import org.xquark.jdbc.typing.DbType;
import org.xquark.jdbc.typing.JavaTypeInfo;
import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.dbms.JDBC2Connection;

/**
 * Implementation of AbstractConnection for Sybase Adaptive Server 11.9.x.
 *
 */
public class ASE119Connection extends JDBC2Connection
{
    private static final String RCSRevision = "$Revision: 1.3 $"; 
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    
    /** Creates new ASE119Connection */
    public ASE119Connection(Connection connection, short dbms, DBMSInfo dbmsInfo)
    throws SQLException
    {
        super(connection, dbms, dbmsInfo);
   }

	protected void setSchemaName() throws SQLException
	{
		Statement stmt = null;
		try {
			stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery("select USER_NAME()");
			if (rs.next())
				schemaName = rs.getString(1);
			rs.close();
		}
		finally {
			stmt.close();
		}
	}
    ///////////////////////////////////////////////////////////////
    //                   Sybase ASE 11.x specific methods
    ///////////////////////////////////////////////////////////////
	public void setObject(PreparedStatement pStmt, int index, Object o, int javaType, DbType sqlType) throws SQLException
	{
		if ((javaType == JavaTypeInfo.JAVA_BIG_DECIMAL)
			&& (sqlType.getJDBCType() == Types.DECIMAL)
			&& (sqlType.getScale() <= 2))
			pStmt.setFloat(index, ((Number)o).floatValue());
		else
			pStmt.setObject(index, o);
	}

    ///////////////////////////////////////////////////////////////
    //                      Table methods
    ///////////////////////////////////////////////////////////////
    public void dropTemporaryTable(String tableName) throws SQLException
    {
        super.dropTemporaryTable("#" + tableName);
    }
    
    public String getDeleteUserTableRowsStatement(String tableName, String documentRowsSelectionClause)
    throws SQLException, RepositoryException
    {
        StringBuffer deletionStmt = new StringBuffer();
        deletionStmt.append("DELETE ");
        deletionStmt.append(tableName);
        deletionStmt.append(' ');
        deletionStmt.append(documentRowsSelectionClause);
        
        return deletionStmt.toString();
    }
    
    // TO IMPLEMENT ?    public String getPathIndexStatement()
    
    public boolean useStringDelimitor()
    {
        return true;
    }
    
    ///////////////////////////////////////////////////////////////
    //                      Private utilities
    ///////////////////////////////////////////////////////////////
    // TO IMPLEMENT ?    public void updateStatistics(String tableName) throws SQLException
    
}
