/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.mapping;

import org.xml.sax.SAXException;
import org.xquark.schema.*;
import org.xquark.xpath.NodeKind;
import org.xquark.xpath.XNode;

/** This class allows builds a repository mapping iterating through it
 * following the associated schema.
 * <p>Used in particular by the mapping loader</p>
 * @see org.xquark.mapper.mapping.Loader
 */
public class BuilderMappingContext extends SchemaContext
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
    //
    //  DATA
    //
    protected MappingFactory factory;
    protected RepositoryMapping mapping;
    protected MappingNode wNode;
    protected XNode pattern = new XNode();
    
    //
    //  CONSTRUCTORS
    //
    
    public BuilderMappingContext(MappingFactory factory)
    {
        super(factory.getSchemaManager(), factory.getSchemaLocator());
        this.factory = factory;
        mapping = (RepositoryMapping)factory.getTree();
        wNode = (MappingNode)mapping.getRoot();
    }
    
    //
    //  MappingContext OVERLOADING
    //
    
    //
    //  public Methods
    //
    
    public void createElementMapping(ElementDeclaration decl, boolean typeMapping)
    {
        factory.createElementNode(wNode, decl, typeMapping);
        // must be called after the node creation
        push(decl);   // careful if overidden !
    }
    
        /* Create a root node for Complex type */
    public org.xquark.schema.Type createTypeMapping(String namespace, String localName) throws SAXException
    {
        org.xquark.schema.Type type = factory.findType(namespace, localName);
        wNode = factory.createTypeNode(type);
        return type;
    }
    
        /* Remove Type node from mapping context stack */
    public void popType()
    {
        wNode = (MappingNode)mapping.getRoot();
    }
    
        /* Register a mapping */
    public void register(Mapping mapping)
    {
        SchemaComponent comp = mapping.getSchemaComponent();
        if (comp instanceof AttributeDeclaration)
            this.mapping.register((MappingNode)factory.createNamedNodeIfNotExist(wNode,
            comp.getNamespace(),
            comp.getName(),
		NodeKind.ATTRIBUTE), mapping);
        else // ElementDeclaration, Type
            this.mapping.register(wNode, mapping);
    }
    
    //
    //  SchemaContext OVERLOADING
    //
    
    public final void push(ElementDeclaration decl)
    {
        super.push(decl);
        pattern.set(decl.getNamespace(), decl.getName(), NodeKind.ELEMENT);
        wNode = (MappingNode)wNode.getChild(pattern);
    }
    
    public final ElementDeclaration pop()
    {
        ElementDeclaration decl = super.pop();
        wNode = (MappingNode)wNode.getParent();
        return decl;
    }
    
    /**
     * @return
     */
    MappingNode getCurrentNode() {
        return wNode;
    }

}
