/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.mapping;

import org.xquark.schema.SchemaComponent;

/**
 * Base interface for classes providing information on XML data mapping on
 * standard relational database management system.
 *
*/
public interface Mapping extends MappingConstants
{

	/**
	 * Accessor to the W3C XML schema component associated to the mapping.
	 * @return the component (element or attribute declaration or type) on
	 * which the mapping is defined
	 */
	public SchemaComponent getSchemaComponent();
	
	/**
	 * Returns the arbitrary table index allocated during mapping file loading.
	 * @return an integer index beginning with 0.
	 */
	public int getTableIndex();
	
	/**
	 * Accessor to the name of the user table on which the mapping is defined.
	 * @return the user table name
	 */
	public String getTableName();
}
