/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.mapping;

/**
 * Interface giving common access to user and the corresponding
 * XQuark index tables information (including SQL main statements).
 */
public interface MappingInfo
{


    
	/**
	 * Returns XQuarkindex table name.
	 * @return the native DBMS table name
	 */
    public String getOIDTableName();

	/**
	 * Returns the SQL statement for inserting data in the XQuarkindex table.
	 * @return a native DBMS insert SQL statement.
	 */
    public String getOIDInsertStatement();
    
	/**
	 * Returns the SQL statement for inserting data in the user table.
	 * @return a native DBMS insert SQL statement
	 */
    public String getInsertStatement();
	/**
	 * Returns the SQL statement for updating the user table.
	 * @return a native DBMS update SQL statement
	 */
    public String getUpdateStatement();
	/**
	 * Returns the SQL statement for retrieving data in the user table.
	 * @return a native DBMS select SQL statement
	 */
    public String getSelectStatement();
    
	/**
	 * Accessor to user table mapping information.
	 * @return a memory reprezentation of XQuark table mapping.
	 */
    public TableMapping getTableMapping();
    
	/**
	 * Says if all statements can be shared among the MappingInfo set
	 * defined on the same user table.
	 * @return boolean
	 */
	public boolean isShareable();

    
}
