/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.mapping;

import java.util.List;

import org.xquark.xpath.PathExpr;

/** This interface allows to access table and column mappings for a given context.
 */
public interface MappingLocator
{
    /** Get the table mappings for the current context
     */
    List currentTableMappings();
    
    /** Get the column mappings for the current context
     */
    List currentColumnMappings();
    
    /** Get the table mappings for a particular subelement in the current context
     */
    List getElementTableMappings(String namespace, String localName);
    
    /** Get the column mappings for a particular subelement in the current context
     */
    List getElementColumnMappings(String namespace, String localName);
    
    /** Get the table mappings for a particular attribute in the current context
     */
    List getAttributeTableMappings(String namespace, String localName);
    
    /** Get the column mappings for a particular attribute in the current context
     */
    List getAttributeColumnMappings(String namespace, String localName);
    
    /** Return the absolute position of the locator.
     * <p>i.e., !isDiscardable()</p>
     */
    PathExpr getLocation();
    
     /** Get the table mappings for a particular subelement in the current context
     */
    //	public List getElementTableMappings(ElementDeclaration decl);
    
    /** Get the column mappings for a particular subelement in the current context
     */
    //	public List getElementColumnMappings(ElementDeclaration decl);
    
    /** Get the table mappings for a particular attribute in the current context
     */
    //	public List getAttributeTableMappings(AttributeDeclaration decl);
    
    /** Get the column mappings for a particular attribute in the current context
     */
    //	public List getAttributeColumnMappings(AttributeDeclaration decl);
    
    /** get the number of mapped attributes for the current context
     */
    //        public int getMappedAttributesCount();
    
}
