/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.mapping;

/**
 * Interface for accessing table mappings attached to an XML collection
 * or a standalone mapping file.
 *
 */
public interface MappingSet
{
	/**
	 * Returns a list of objects giving access to user table mapping
	 * information.
	 * @return a specific iterator for table doubles management.
	 */
    public MappingSetIterator getMappingSetIterator();
    
	/**
	 * Returns mapping information for a particular table mapping index.
	 * @param index the table mapping index. This index is arbitrary and
	 * allocated when mapping file is loaded.
	 * @return a {@link MappingInfo} object.
	 */
    public MappingInfo getMappingStatements(int index);
    
	/**
	 * Returns the numbers of user table mappings in the mapping set.
	 * @return the number of {@link MappingInfo} object in this object.
	 */
    public int getMappingStatementsCount();
    
	/**
	 * Returns a list of objects giving access to user table mapping
	 * information in a relational dependency (foreign keys) order.
	 * This methods allows, for example, to delete rows in the correct order.
	 * @param mostDependantFirst if true, the list will be returned in the
	 * "delete" order.
	 * @return a specific iterator for table doubles management.
	 */
    public MappingSetIterator getSortedTableMappingSetIterator(boolean mostDependantFirst);

	/**
	 * Returns a list of objects giving access to user table mapping
	 * information in a relational dependency (foreign keys) order.
	 * This methods allows, for example, to delete rows in the correct order.
	 * @param mostDependantFirst if true, the list will be returned in the
	 * "delete" order.
	 * @return a specific iterator for table doubles management.
	 */
//	public MappingSetIterator getNameSortedTableMappingStatements();
}

