/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.mapping;

import java.util.HashMap;

/**
 * Iterator for iterating through a MappingSet performing memorization of the
 * tables encoutered to allow doubles management.
 */
public class MappingSetIteratorImpl implements MappingSetIterator
{
	private int current = -1;
	private int currentIndex = -1;
	private int firstOccurenceIndex = -1;
	private MappingInfo[] array;
	private boolean reverse = false;
	private HashMap tableFirstOccurences = new HashMap();

	/**
	 * Constructor for MappingSetIterator.
	 */
	public MappingSetIteratorImpl(MappingInfo[] tms, boolean reverse)
	{
		array = tms;
		this.reverse = reverse;
		if (tms != null)
		{
			if (reverse)
				current = array.length;
			else
				current = -1;
		}
	}

	public MappingSetIteratorImpl(MappingInfo[] tms)
	{
		this(tms, false);
	}

//	public MappingSetIterator(List tms, boolean reverse)
//	{
//		this((MappingInfo[])tms.toArray(new MappingInfo[tms.size()]), reverse);
//	}
//
//	public MappingSetIterator(List tms)
//	{
//		this((MappingInfo[])tms.toArray(new MappingInfo[tms.size()]), false);
//	}
//

	public boolean hasNext()
	{
		if (array == null)
			return false;
		if (reverse)
			return current > 0;
		else
			return (current + 1) < array.length;
	}

	public MappingInfo next()
	{
		MappingInfo ret = null;
		if (hasNext())
		{
			// update position
			if (reverse)
				current--;
			else
				current++;
			registerTable();
			ret = array[current];
		}
		return ret;
	}
	
	public boolean isTableFirstOccurence()
	{
		return firstOccurenceIndex == currentIndex;
	}
	
	public int getFirstOccurenceTableIndex()
	{
		return firstOccurenceIndex;
	}
	
	private void registerTable()
	{
		if (hasNext())
		{
			TableMapping wTable = array[current].getTableMapping();
			currentIndex = wTable.getTableIndex();
			Integer firstOccurenceObject = (Integer)tableFirstOccurences.get(wTable.getTableName());
			if (firstOccurenceObject == null)	// first time encoutered
			{
				tableFirstOccurences.put(wTable.getTableName(), new Integer(currentIndex));
				firstOccurenceIndex = currentIndex;
			}
			else
				firstOccurenceIndex = firstOccurenceObject.intValue();
		}
	}

}
