/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.mapping;

import java.math.BigDecimal;

import org.xquark.mapping.StorageContext;
import org.xquark.mapping.UserGenerator;
import org.xquark.mapper.metadata.RepositoryConstants;

public abstract class SystemVariableGenerator implements UserGenerator, MappingConstants
{
    private static final String RCSRevision = "$Revision: 1.3 $"; 
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    public abstract Object getValue(StorageContext context);
    public abstract String getXMLType();
    public abstract short getType();
    
    static SystemVariableGenerator createGenerator(String sysvar)
    {
        if (DOCID_VALUE.equals(sysvar))
            return new SystemVariableGenerator()
            {
                public Object getValue(StorageContext context)
                {
                    return context.getDocumentID();
                }
                public String getXMLType()
                {
                    return "<simpleType><restriction base=\"string\"><maxLength value=\""
                            + String.valueOf(RepositoryConstants.ID_LENGTH)
                            + "\"/></restriction></simpleType>";
                }
                public short getType()
                {
                    return DOCID_CODE;
                }
            };
        else if (DOCOID_VALUE.equals(sysvar))
            return new SystemVariableGenerator()
            {
                public Object getValue(StorageContext context)
                {
                    return new Long(context.getDocumentOID());
                }
                public String getXMLType()
                {
                    return "int";
                }
                public short getType()
                {
                    return DOCOID_CODE;
                }
            };
//        else if (BUCKET_VALUE.equals(sysvar))
//            return new SystemVariableGenerator()
//            {
//                public Object getValue(StorageContext context)
//                {
//                    return new Long(context.getBucketOID());
//                }
//                public String getXMLType()
//                {
//                    return "long";
//                }
//                public short getType()
//                {
//                    return BUCKET_CODE;
//                }
//            };
        else if (UOID_VALUE.equals(sysvar))
            return new SystemVariableGenerator()
            {
                public Object getValue(StorageContext context)
                {
                    return new Long(context.getUOID());
                }
                public String getXMLType()
                {
                    return "long";
                }
                public short getType()
                {
                    return UOID_CODE;
                }
            };
        else if (PATHID_VALUE.equals(sysvar))
            return new SystemVariableGenerator()
            {
                public Object getValue(StorageContext context)
                {
                    return new Integer(context.getPathOID());
                }
                public String getXMLType()
                {
                    return "short";
                }
                public short getType()
                {
                    return PATHID_CODE;
                }
            };
        else if (TAG_VALUE.equals(sysvar))
            return new SystemVariableGenerator()
            {
                public Object getValue(StorageContext context)
                {
                    return context.getQName();
                }
                public String getXMLType()
                {
                    return "<simpleType><restriction base=\"string\"><maxLength value=\"255\"/></restriction></simpleType>";
                }
                public short getType()
                {
                    return TAG_CODE;
                }
            };
        else if (RANK_VALUE.equals(sysvar))
            return new SystemVariableGenerator()
            {
                public Object getValue(StorageContext context)
                {
                    return BigDecimal.valueOf(context.getNodeRank());
                }
                public String getXMLType()
                {
                    return "int";
                }
                public short getType()
                {
                    return RANK_CODE;
                }
            };
        else if (LOCAL_VALUE.equals(sysvar))
            return new SystemVariableGenerator()
            {
                public Object getValue(StorageContext context)
                {
                    return context.getLocalName();
                }
                public String getXMLType()
                {
                    return "<simpleType><restriction base=\"string\"><maxLength value=\"255\"/></restriction></simpleType>";
                }
                public short getType()
                {
                    return LOCAL_CODE;
                }
            };
        else if (URI_VALUE.equals(sysvar))
            return new SystemVariableGenerator()
            {
                public Object getValue(StorageContext context) {
                    return context.getNamespaceURI();
                }
                public String getXMLType()
                {
                    return "<simpleType><restriction base=\"string\"><maxLength value=\"255\"/></restriction></simpleType>";
                }
                public short getType()
                {
                    return URI_CODE;
                }
            };
        else return null;
    }
}
