/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.mapping;

import java.util.Collection;

import org.xquark.jdbc.typing.ColumnMetaData;

/**
 * Table properties gained from the JDBC API.
 *
 * Owns a set of metadata objects for table's columns.
 *
 */
public interface TableMetaData 
{


    
	/**
	 * Accessor to table name.
	 * @return the table name.
	 */
    public String getTableName();

	/**
	 * Returns the number of columns in the table.
	 * @return the number of columns in the table.
	 */
    public int getColumnCount();
    
	/**
	 * Accessor to metadata for a particular column by name.
	 * @param cname the relational name of the column.
	 * @return column metadata or null if name does not exist.
	 */
    public ColumnMetaData getColumnMetaData(String cname);

	/**
	 * Accessor to metadata for all the table's columns.
	 * @return a collection of {@link ColumnMetaData} objects.
	 */
    public Collection getColumnsMetaData();
    
	/**
	 * Accessor to table relational unique constraints.
     * @return a Collection of Collection of {@link ColumnMetaData} objects.
	 */
    public Collection getUniqueConstraints();
    
	/**
	 * Says if all statements can be shared among the table mappings defined on
	 * this user table.
	 * @return boolean
	 */
	public boolean areStatementsShareable();
	
	/**
	 * Setter for the StatementsNotShareable flag.
	 */
	public void setStatementsNotShareable();

	/**
	 * Says if this user table has multiple table mappings.
	 * @return boolean
	 */
	public boolean isShared();

	/**
	 * Setter for the Shared flag.
	 */
	public void setShared();

}
