/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.metadata;

import org.xquark.mapper.util.RecyclingStack;
import org.xquark.schema.SchemaConstants;

/**
 * Data structure that corresponds to XML extra-data nodes (PI, comments & prefix definition).
 *
 */
public class ExtraNode implements Cloneable, RecyclingStack.StackObject
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    public long anchor = -1;
    public short rowNum = 0;
    public short path = -2;  // -1 is ROOT path ID !
    
    /* Code for setting the type of node */
    public short type = -1;
    public short position = 0;
    public int offset = 0;
    public String data =null;
    
    public ExtraNode()
    {}
    
/*    public ExtraNode(int anchorOID, short path, short type)
    {
        set(anchorOID, path, type);
    }
 */
    public ExtraNode(long anchorOID, short rowNum, short path, short type, short position)
    {
        set(anchorOID, rowNum, path, type, position);
    }
    
    public ExtraNode(long anchorOID, short rowNum, short path, short type, short position, int offset, String data)
    {
        set(anchorOID, rowNum, path, type, position, offset, data);
    }
    
    /**
     * This method is for recycling objects. This method is prefered to the constructor call.
     */
    public void set(long anchorOID, short path, short type, short position)
    {
        this.anchor = anchorOID;
        this.path = path;
        this.type = type;
        this.position = position;
    }
    /**
     * This method is for recycling objects. This method is prefered to the constructor call.
     */
    public void set(long anchorOID, short rowNum, short path, short type, short position)
    {
        set(anchorOID, path, type, position);
        this.rowNum = rowNum;
    }
    
    public void set(long anchorOID, short rowNum, short path, short type, short position, int offset, String data)
    {
        set(anchorOID, rowNum, path, type, position);
        this.data = data;
        this.offset = offset;
    }
    
    public boolean isNil()
    {
        return (type == RepositoryConstants.XSI_NIL) 
                    && data.equalsIgnoreCase(SchemaConstants.TRUE_VALUE);
    }
    
    public void clear()
    {
        anchor = -1;
        path = -2;  // -1 is ROOT path ID !
        type = -1;
        position = 0;
        offset = 0;
        data = null;
    }
    
    public String toString()
    {
        return "[" + anchor + ", " + path+ ", " + type + ", " + position
                + ", " + offset + ", " + data + "]";
    }
    
    public Object clone()
    {
        Object o = null;
        try {
            o =  super.clone();
        }
        catch (CloneNotSupportedException e) {
        }
        return o;
    }
}
