/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.metadata;

/**
 * Data structure that corresponds to XML logical structure nodes.
 *
 */
public class Node implements Cloneable
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    public long oid = -1;
    public long last = -1;
    public short path = -2;  // -1 is ROOT path ID !
    
    /* Code for setting the type of node */
    public short type = -1;
    
  // not stored
    
    public Node()
    {}
    
    public Node(long oid, short path, short type)
    {
        set(oid, path, type);
    }
    
    public Node(long oid, long last, short path, short type)
    {
        set(oid, last, path, type);
    }
    
    public void setNodeType(short lt)
    {
        this.type = lt;
    }
    
    public int getNodeType()
    {
        return type;
    }
    
    /**
     * This method is for recycling objects. This method is prefered to the constructor call.
     */
    public void set(long oid, short path, short type)
    {
        this.oid = oid;
        this.path = path;
        this.type = type;
    }
    
    public void set(long oid, long last, short path, short type)
    {
        set(oid, path , type);
        this.last = last;
    }
    
    public void clear()
    {
        set( -1, -1, (short)-2,(short)-1);  // -1 is ROOT path ID !
    }
    
    public String toString()
    {
        return "[" + oid + ", " + last + ", " + path+ ", " + type + "]";
    }
    
    public Object clone()
    {
        Object o = null;
        try {
            o =  super.clone();
        }
        catch (CloneNotSupportedException e) {
        }
        return o;
    }
}
