/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.metadata;

import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.mapping.MappingFactory;
import org.xquark.mapper.mapping.RepositoryMapping;
import org.xquark.xpath.*;

/**
 * Specific implementation of XTreeBuilder.
 * <p><B>Warning:</B> This factory is not synchronized since path set is.
 *
 */
public class PathSetFactory extends XTreeBuilder
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    private CollectionMetadata collection;
    private MappingFactory mappingFactory;
    private PathSet pathSet;   
    /**
     * To use when createTree() is going to be called.
     **/
    public PathSetFactory(CollectionMetadata collection, MappingFactory mappingFactory)
    {
        set(collection, mappingFactory);
    }
    
    private void set(CollectionMetadata collection, MappingFactory mappingFactory)
    {
        this.collection = collection;
        this.mappingFactory = mappingFactory;
    }
    
    public RepositoryMapping getMapping()
    {
        return (RepositoryMapping)mappingFactory.getTree();
    }
    
    //
    //  XModelFactory IMPLEMENTATION
    //
    public XTree createTree()
    {
        pathSet = (PathSet)super.createTree();
        pathSet.initRoot();
        
        // create an empty (no pid) copy of the repository mapping tree
        // TO IMPROVE : now that all ps table is browsed, only perform duplicate when ps is new ? (browse symetrically Reposiory mapping to attach mapping nodes)
        mappingFactory.duplicate(this);
        try {
            if (!pathSet.isNew())
                pathSet.refresh();
        }
        catch (RepositoryException e) {
            throw new XTreeRuntimeException(e);
        }
        return pathSet;
    }
    public XTree allocateTree()
    {
        return new PathSet(collection);
    }
    
    public XTreeNode allocateNode(XTreeNode parent, String namespace, String localName, byte type)
    {
        return new PathNode((PathSet)tree, (PathNode)parent, namespace, localName, type);
    }
    
    /** This version is used direcly by the mapping duplication (mapping node is not
     * allocated, it is copied from the RepositoryMapping by customizeDuplicate())
     */
    public XTreeNode createNamedNode(XTreeNode parent, String namespace, String localName, byte type)
    {
        XTreeNode newNode = super.createNamedNode(parent, namespace, localName, type);
        try
        {
            pathSet.allocateID((PathNode)newNode);
        }
        catch (RepositoryException e)
        {
            throw new XTreeRuntimeException(e);
        }
        return newNode;
    }
    
    /** This version is to be used by the pathset construction during document
     * instance storage <B>only</B>.
     *
     */
    public synchronized XTreeNode createNamedNodeIfNotExist(XTreeNode parent, String namespace, String localName, byte type)
    {
        /* Then create */
        PathNode node = (PathNode)super.createNamedNodeIfNotExist(parent, namespace, localName, type);
        if (node.getMapping() == null)
        {
            try
            {
                node.setDefaultMapping(mappingFactory, type);
            }
            catch (RepositoryException e)
            {
                throw new XTreeRuntimeException(e);
            }
        }
        return node;
    }
}
