/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.metadata;

/** General constants for XMLCollection.
 *
 */
public interface RepositoryConstants
{
    //
    //  XMLCollection Current Version
    //
    
    ////////////////////
    //  NODE TYPES (= extension of XPath types see org.xquark.xml.xpath)
    ////////////////////
    public byte XSI_NIL = 0x09;
    public byte XSI_TYPE = 0x0A;
    public byte XSI_SCHEMA_LOCATION = 0x0B;
    public byte XSI_NO_NAMESPACE_SCHEMA_LOCATION = 0x0C;
    public byte START_CDATA_SECTION = 0x0D;
    public byte END_CDATA_SECTION = 0x0E;
    
    public String ATTRIBUTE_TYPES[] = {"ID", "IDREF", "IDREFS", "ENTITY", "ENTITIES", "NMTOKEN", "NMTOKENS", "CDATA"};

    public short ROOT_PATH_ID = 0;
    
    public int SEQUENCE_FIRST_VALUE = 1;
    
    // OID limits
    public byte UOID_SIZE = 63;
    public byte CID_SIZE_DEFAULT_VALUE = 8;
    public byte CID_SIZE_MIN_VALUE = 8;
    public byte CID_SIZE_MAX_VALUE = 16;
    public byte DID_SIZE_DEFAULT_VALUE = 24;
    public byte DID_SIZE_MIN_VALUE = 8;
    public byte DID_SIZE_MAX_VALUE = 48;
    
    //
    // TABLE NAMES
    //
    public String TABLE_PREFIX = "XQUARK$";
    
    public String LEGACY_REPOSITORY_CHECKER = TABLE_PREFIX + "CONF%";
    
    /* Column index in table specifications loaded by TableSpecLoader */
    // Default XML data table
    public int V_UOID_INDEX = 0;
    public int V_PATHID_INDEX = 1;
    public int V_CHAR_VALUE_INDEX = 2;
    public int V_NUM_VALUE_INDEX = 3;
    public int V_DATE_VALUE_INDEX = 4;
    
    //  XMLCollection tables features
    public int DEFAULT_METADATA_CACHE_SIZE = 10;
    
    public byte DEFAULT_CID_SIZE = 8;
    public int ID_LENGTH = 255;
    public int DEFAULT_DATA_LENGTH = 255;   // minimum value due to Sybase

    //
    // CONFIGURATION
    //
    public String VERSION_FILE = "Repository.ver";
    public String CONF_FILE = "Repository.properties";
    public String DEFAULT_CONF_FILE = "DefaultRepository.properties";
    public String CONF_PACKAGE = "/com/xquark/";
    public String RESOURCES_FOLDER = "resources/";
    public String MAPPING_PACKAGE = "mapping/";
    public String MAPPING_SCHEMA_FILE = "mapping.xsd";
    
    // MAPPER
	public String CONF_USER_BATCHSIZE = "org.xquark.mapper.batchSize";
	 
    // REPOSITORY SPECIFIC
    public String CONF_USER_FETCHSIZE = "org.xquark.repository.dbms.user.fetchSize";
    public String CONF_TREE_FETCHSIZE = "org.xquark.repository.dbms.tree.fetchSize";
    public String CONF_DATA_FETCHSIZE = "org.xquark.repository.dbms.data.fetchSize";
	public String CONF_DATA_BATCHSIZE = "org.xquark.repository.data.batchSize";
    public String CONF_TREE_BATCHSIZE = "org.xquark.repository.dbms.tree.batchSize";
    public String CONF_DOCID_BATCHSIZE = "org.xquark.repository.dbms.docID.batchSize";
    public String CONF_PATH_BATCHSIZE = "org.xquark.repository.dbms.path.batchSize";
    public String CONF_REMOVE_BATCHSIZE = "org.xquark.repository.dbms.remove.batchSize";
    public String CONF_RENAME_BATCHSIZE = "org.xquark.repository.dbms.rename.batchSize";
    public String CONF_COLLECTION_METADATA_CACHE = "org.xquark.repository.memory.collectionCache";
	public String CONF_TRIM_WS = "org.xquark.repository.trimWhiteSpaces";
   
    // OTHER
    public String REPOSITORY_SOURCE_NAME = "Repository";
    public String NO_NAMESPACE_SCHEMA_ID = "xquark:noNamespace";
}
