/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * CollectionConfig.java
 *
 * Created on 20 juillet 2000, 14:24
 */
 
package org.xquark.mapper.metadata;

import org.xquark.mapper.dbms.TableSpecLoader;

/** Data structure that holds a repository features.
 */

public class RepositoryInfo implements Cloneable
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
	
    /** Repository logical ID.
     */
    protected String name = "XQuark";

    protected byte colIDSize = RepositoryConstants.CID_SIZE_DEFAULT_VALUE;
    
    protected String version = TableSpecLoader.REPOSITORY_DATA_VERSION;

    public RepositoryInfo()
    {
    }
    
    public RepositoryInfo(String name, byte colIDSize)
    {
        this.name = name;
        setColIDSize(colIDSize);
    }

    /** Builds a String in order to displays the information of this configuration.
     * @return the string representing the repository features.
     */
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append("\nName :\t");
        buffer.append(name);
        buffer.append("\nCollection ID size in bits:\t");
        buffer.append(colIDSize);
        buffer.append("\nModel version :\t");
        buffer.append(version);
        buffer.append("\n");
        
        return buffer.toString();
    }
    
    /** Builds a copy of this object.
     * @return a CollectionConfig object.
     */
    public Object clone()
    {
        Object copy = null;
        try {
            copy = super.clone();
        }
        catch(CloneNotSupportedException e) {
        }
        return copy;
    }
    
    public String getName()
    {
        return name;
    }
    
    public String getVersion()
    {
        return version;
    }
    
    public byte getColIDSize()
    {
        return colIDSize;
    }
    
    public short getColIDMaxValue()
    {
        return (short)((1 << colIDSize) - 1);
    }
    
    public void setName(String name)
    {
        this.name = name;
    }
    
    public void setVersion(String version)
    {
        this.version = version;
    }
    
    public void setColIDSize(byte colSize)
    {
        if (colSize < RepositoryConstants.CID_SIZE_MIN_VALUE)
            colIDSize = RepositoryConstants.CID_SIZE_MIN_VALUE;
        else if (colSize > RepositoryConstants.CID_SIZE_MAX_VALUE)
            colIDSize = RepositoryConstants.CID_SIZE_MAX_VALUE;
        else
            colIDSize = colSize;
    }
}
