/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.metadata;

import org.xquark.mapper.dbms.TableInfo;
import org.xquark.mapper.mapping.MappingSet;
import org.xquark.mapper.mapping.RepositoryMapping;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 * Read-only internal collection metadata interface.
 * @see org.xquark.mapper.AccessManager
  */
public interface _Collection extends MappingSet
{



    /** Returns the string name of this colection.
     * @return the string ID of the collection.
     */
    public String getCollectionName();
    
    /** Returns the numeric ID of this colection used especially for table
     * naming.
     * @return the numeric ID of the collection.
     */
    public short getCollectionID();

    /** Returns the pathSet information of this collection.
     * @return an implementation of {@link org.xquark.xml.xpath.XTree}
     */
    public PathSet getPathSet();
    
    /** Returns the mapping information for this collection.
     * <B>May be shared with other collections.</B>
     * @return an implementation of {@link org.xquark.xml.xpath.XTree}
     */
    public RepositoryMapping getMapping();
    
    /**
     * Get the OID table names (customized with collection and user table ID).
     *
     * @param tableMappingIndex index of table mapping.
     * @return the OID table name.
     */
    public String getOIDTableName(int tableMappingIndex) throws XMLDBCException;
    
    /**
     * Access table information for colletcion tables (OID tables excluded)
     * @see org.xquark.mapper.dbms.TableSpec
     */
    public TableInfo getTableInfo(byte type);
    
    /**
     * Returns the object performing OID manipulations.
     * return the UOIDManager of the collection.
     */
    public UOIDManager getUOIDManager();
    
    /**
     * Update statistics for the collection tables using the dbms native features
     */
    public void updateStatistics() throws XMLDBCException;
    
    /** Synchronizes metadata with database.
     */
    public void refresh() throws XMLDBCException;
    
}
