/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.metadata;


import java.util.List;

import org.xquark.mapper.dbms.TableInfo;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.metadata.resolver.MetadataAccess;
import org.xquark.xquery.metadata.resolver.SourceMetadata;


/**
 * Read-only internal repository metadata interface.
 * @see org.xquark.mapper.AccessManager
  */
public interface _Repository extends MetadataAccess, SourceMetadata
{


    
    /**
     * Access table information for repository tables
     * @see org.xquark.mapper.dbms.TableSpec
     */
    public TableInfo getTableInfo(byte type);
    
    /**
     * Return the collections names that exist in the repository.
     *
     * @return repository names in a string List.
     * @throws RepositoryException application exception.
     */
    public List getCollectionList() throws XMLDBCException;
    
    /**
     * Return collection metadata for an opened collection.
     *
     * @return null if the collection does not exist or is not opened.
     * @throws RepositoryException application exception.
     */
    public _Collection getCollection(String name) throws XMLDBCException;
    
    /** Returns the dbms user name used by this object.
     * @return the user name supplied at opening time.
     */
    public String getUserName();
    
    /** Returns the JDBC string used by this object.
     * @return the JDBC string supplied at opening time.
     */
    public String getURL();
    
    /** Returns the logical name for this repository.
     * @return a string.
     */
    public String getName();
    
    /** Returns the Repository data model version used by this object.
     * @return a string version ID.
     */
    public String getVersion();
    
    /** Synchronizes metadata with database.
     */
    public void refresh() throws XMLDBCException;
    
}
