/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.*;

import org.xquark.jdbc.typing.DBMSConstants;
import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.dbms.TableInfo;
import org.xquark.mapper.metadata.RepositoryConstants;

/**
 * Base implementation for XML structure node storage.
 *
 */
public abstract class AbstractPersistentNode implements RepositoryConstants, DBMSConstants
{
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    /////////////////////////////////////
    //  Storage JDBC variables
    /////////////////////////////////////

    /** java.sql.Statement object used to fetch the whole data set from the database. */
    protected TableInfo tableInfo = null;
    protected Statement stmt = null;
    protected ResultSet rs = null;
    
    protected String insertStmt;

    /** JDBC data used for storing rows in the database with batch style.
     * IMPORTANT  : when storing, and contrary to reading only one batch
     * is used for all Node objects.
     */
    protected PreparedStatement pStmt = null;

    protected long docOID = -1;
    protected long uoid = -1;
    protected int rsColumnOffset = 1;

    public AbstractPersistentNode(TableInfo tableInfo)
    {
        this.tableInfo = tableInfo;
    }
    
    public void set(long docOID) {this.docOID = docOID;}
    
    public long getUOID() { return uoid;}
    
    public void close() throws SQLException
    {
        reset();
        // release statement
        if (pStmt != null)
            pStmt.close();
        pStmt = null;
    }
    
    public void reset() throws SQLException
    {
        // release statement
        if (rs != null)
            rs.close();
        if (stmt != null)
            stmt.close();
        docOID = -1;
        uoid = -1;
        rs = null;
        stmt = null;
    }

    //
    //          STATIC METHODS
    //
    
    /** Delete the links of a given document.
     * @param connection JDBC connection used for removal.
     * @param name repository name.
     * @param first first UOID of the range.
     * @param last last UOID of the range.
     * @throws SQLException JDBC error
     */
    public static PreparedStatement getDeleteStatement(AbstractConnection connection, TableInfo tableInfo)
    throws SQLException
    {
        return connection.getConnection().prepareStatement("DELETE FROM "
            + tableInfo.getName() + " WHERE UOID >= ? and UOID <= ?");
    }

}
