/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.io.IOException;

import org.xml.sax.*;
import org.xml.sax.ext.LexicalHandler;
import org.xquark.util.SAXConstants;
import org.xquark.xml.xdbc.XMLDBCException;


/** Common part ot the {@link org.xquark.mapper.XMLCollection XMLCollection}
 * implementations.
 *
 */

public abstract class BaseXMLReaderImpl extends XMLBuilder
implements RepositoryReader, SAXConstants
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    //
    // Data
    //
    
    // the repository instance

    //
    //  Options
    //
    protected boolean restoreEnumeratedAttrTypes = false;   // if true NMTOKEN attribute types generated
                                                          // (contrary as specified by SAX 2.0)
     /** Constructor.
     * @param name The name of the repository.
     * @param config A CollectionConfig object.
     * @exception java.sql.SQLException RDBMS exception.
     */

    public BaseXMLReaderImpl(_RepositoryCollection collection)
    throws XMLDBCException
    {
        super(collection);
    }

    ///////////////////////////////////////////////////////////////////////////
    //                     XMLReader implementation
    ///////////////////////////////////////////////////////////////////////////
    public void parse(InputSource input)
    throws IOException, SAXException
    {
        if (input.getSystemId() != null)
            parse(input.getSystemId());
        else
            parse(input.getPublicId());
    }
    
    public boolean getFeature(String name) 
    throws SAXNotRecognizedException, SAXNotSupportedException
    {
        if (name.equals(SAX_NAMESPACE_FEATURE))
            return true;
        else if (name.equals(SAX_PREFIX_FEATURE))
            return false;
        else if (name.equals(SAX_VALIDATION_FEATURE))
            return false;
        else
            throw new SAXNotSupportedException("Unrecognized feature.");
    }
    public Object getProperty (String name)
	throws SAXNotRecognizedException, SAXNotSupportedException
    {
        if (name.equals(SAX_LEXICAL_PROPERTY))
            return lexicalHandler;
        else
            throw new SAXNotSupportedException("Unrecognized property.");
    }

    public ContentHandler getContentHandler () 
    {
        return contentHandler;
    }
    
    public ErrorHandler getErrorHandler () 
    {
        return errorHandler;
    }
    
    public EntityResolver getEntityResolver()
    {
        return null;
    }

    public DTDHandler getDTDHandler ()
    {
        return null;
    }
    
    public void setContentHandler(ContentHandler handler) 
    {
        contentHandler = handler;
    }
    
    public void setDTDHandler(DTDHandler handler) 
    {
        // No op : not supported
    }

    public void setErrorHandler(ErrorHandler handler) 
    {
        errorHandler = handler;
    }
    
    public void setEntityResolver(EntityResolver resolver) 
    {
    }
    
    public void setFeature(String name,boolean value) 
    throws SAXNotRecognizedException, SAXNotSupportedException 
    {
        if (name.equals(SAX_NAMESPACE_FEATURE))
            {
                if (!value)
                    throw new SAXNotSupportedException("This feature can only be set to true.");
                else
                    return;
            }
        else if (name.equals(SAX_PREFIX_FEATURE))
            {
                if (value)
                    throw new SAXNotSupportedException("This feature can only be set to false.");
                else
                    return;
            }
        else if (name.equals(SAX_VALIDATION_FEATURE))
            return;
        else
            throw new SAXNotSupportedException("Unrecognized feature.");
    }
    
    public void setProperty(String name,Object value) 
    throws SAXNotRecognizedException, SAXNotSupportedException 
    {
        if (name.equals(SAX_LEXICAL_PROPERTY))
            lexicalHandler = (LexicalHandler)value;
        else
            throw new SAXNotSupportedException("Unrecognized property.");
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    //                      private Utilities
    ////////////////////////////////////////////////////////////////////////////////

    //////////////////////
    //  Inner Classes
    //////////////////////
        
} // class
