/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.dbms.TableInfo;
import org.xquark.mapper.mapping.ColumnMapping;
import org.xquark.mapper.metadata.PathNode;
import org.xquark.mapper.util.RepositoryProperties;

/**
 * Object used to perform XML structure node storage.
 *
 */
public class BatchQueryExtraDataExplorer extends QueryExtraDataExplorer
{
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private static Log log = LogFactory.getLog(BatchQueryExtraDataExplorer.class);
    
    private int currentRank = -2;
    private int rank = -1;
    
    public BatchQueryExtraDataExplorer(TableInfo tableInfo)
    {
        super(tableInfo);
    }
    /**
     * Launch the table entry retrieving for a particular bukect ID and a OID range.
     */
    public void getBranch(List paths, VarInfoSet tmpTable,
    AbstractConnection connection)
    throws SQLException
    {
        useStringDelimitor = connection.useStringDelimitor();
        stmt = connection.getConnection().createStatement();
        try
        {
            connection.setFetchSize(stmt, RepositoryProperties.getIntProperty(CONF_DATA_FETCHSIZE));
            rs = stmt.executeQuery(generateStatement(paths, tmpTable));
            notExhausted = true;
            currentResultExhausted = false;
        }
        catch (SQLException e)
        {
            stmt.close();
            throw e;
        }
    }
    
    private String generateStatement(List paths, VarInfoSet tmpTable)
    {
        StringBuffer SQLStmt = new StringBuffer();
        
        ColumnMapping refColumn;
        SubPathInfo path;
        
        for (int i = 0; i < paths.size(); i++)
        {
            path = (SubPathInfo)paths.get(i);
            
            if (i > 0)
                SQLStmt.append("\nunion all\n");
            
            SQLStmt.append("select t.");
            SQLStmt.append(tmpTable.getRankColumnName());
            SQLStmt.append(',');
            SQLStmt.append(path.subPathIndex);
            SQLStmt.append(" as n,v.uoid,v.path,v.type,v.pos,v.offset,v.data,v.num\nfrom ");
            SQLStmt.append(tmpTable.getTableName());
            SQLStmt.append(" t,");
            SQLStmt.append(tableInfo.getName());
            SQLStmt.append(" v\nwhere v.uoid between t.");
            SQLStmt.append(tmpTable.getFirstColumnName());
            SQLStmt.append(" and t.");
            SQLStmt.append(tmpTable.getLastColumnName());
            SQLStmt.append(" and t.");
            SQLStmt.append(tmpTable.getPathColumnName());
            SQLStmt.append('=');
            SQLStmt.append(path.varPath.getPathID());
            // path restriction
            List subpaths = path.subPath.getSubPaths();
            if (subpaths != null)
            {
                // last struct ancestor is added in order for the builder to initiate range
                SQLStmt.append(" and v.path");
                
                int subpathsCount = subpaths.size();
                if (subpathsCount == 1)
                {
                    SQLStmt.append("=");
                    SQLStmt.append(((PathNode)subpaths.get(0)).getPathID());
                }
                else
                {
                    SQLStmt.append(" in(");
                    
                    for (int j = 0; j < subpathsCount; j++)
                    {
                        SQLStmt.append(((PathNode)subpaths.get(j)).getPathID());
                        SQLStmt.append(',');
                    }
                    SQLStmt.setCharAt(SQLStmt.length()-1, ')');
                }
            }
        }
        
        SQLStmt.append("\norder by ");
        SQLStmt.append(tmpTable.getRankColumnName());
        SQLStmt.append(",n,num");
        
        if (log.isDebugEnabled())
        	log.debug(">>> Value stmt for extra table:\n" + SQLStmt);
        
        return SQLStmt.toString();
    }
    /**
     * get the next row from database and store it in the current object.
     */
    public boolean fetchNextRow() throws SQLException
    {
        try
        {
            if (resultActive())
            {
                notExhausted = rs.next();
                if (notExhausted)
                {
                    rank = rs.getInt(1);
                    subPathIndex = rs.getInt(2);
                    uoid = rs.getLong(3);
                    node.path = rs.getShort(4);
                    node.type = rs.getShort(5);
                    node.position = rs.getShort(6);
                    node.offset = rs.getInt(7);
                    node.data = rs.getString(8);
                    if (useStringDelimitor)
                        node.data = node.data.substring(0, node.data.length() - STRING_DELIMITOR_LENGTH);
                    node.rowNum = rs.getShort(9);
                    updateResultSegmentationFlag();
                }
                else
                    reset();
            }
            else
                return false;
            
            //                Debug.print("Tuple :: fetchNextRow for table " + mapping.getTableName() + " UOID=" + getUOID());
            return notExhausted;
        }
        catch (SQLException e)
        {
            close();
            throw e;
        }
    }
    
    void setCurrentRank(int rank)
    {
        currentSubPathIndex = 0;
        currentRank = rank;
        updateResultSegmentationFlag();
    }
    
    
    public void reset() throws SQLException
    {
        super.reset();
        currentRank = -2;
        rank = -1;
    }
    
    protected void updateResultSegmentationFlag()
    {
        currentResultExhausted =  (rank != currentRank) || (subPathIndex != currentSubPathIndex);
    }
}
