/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * DocumentExplorer.java
 *
 * Created on 18 avril 2002, 17:15
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.mapping.MappingSetIterator;
import org.xquark.mapper.metadata.CollectionMappingInfo;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 *
 */
class DocumentExplorer extends TableExplorer
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private DocumentTuple[] docTuples;
    private DocumentStructExplorer docNode;
    private DocumentExtraDataExplorer docExtraNode;
    
    /** Creates a new instance of QueryExplorer  */
    public DocumentExplorer(_RepositoryCollection collection) throws SQLException, XMLDBCException
    {
        super(collection);
        
        wNode = docNode = new DocumentStructExplorer(collection.getMetadata());
        extraNode = docExtraNode = new DocumentExtraDataExplorer(collection.getMetadata());
        
        // tuples creation
    	tuples = docTuples = new DocumentTuple[collection.getMetadata().getMappingStatementsCount()];
        MappingSetIterator it = collection.getMetadata().getMappingSetIterator();
        CollectionMappingInfo table;
        
        while (it.hasNext())
        {
            table = (CollectionMappingInfo)it.next();
        	docTuples[table.getTableMapping().getTableIndex()] =
        		new DocumentTuple(table);
	    }
    }
    
    public void initDocReading(AbstractConnection connection) throws SQLException, RepositoryException
    {
        // Table subset browsing for statement execution
        for (int i = 0; i < tuples.length; i++)
        {
            docTuples[i].init(connection, collection.getMetadata());
        }
        
        docNode.initDocReading(connection);
        docExtraNode.initDocReading(connection);
    }
    
    public void initDocumentExploration(String ID) throws SQLException, RepositoryException
    {
        // Table subset browsing for statement execution
        for (int i = 0; i < tuples.length; i++)
        {
            docTuples[i].execQuery(ID);
        }
        
        // initialize structure retrieval
        docNode.getDocument(ID);
        docExtraNode.getDocument(ID);
    }
    
}
