/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.dbms.TableSpec;
import org.xquark.mapper.metadata.CollectionMetadata;
import org.xquark.mapper.metadata.UOIDManager;
import org.xquark.mapper.util.RepositoryProperties;

/**
 * Object used to perform XML structure node storage.
 *
 */
public class DocumentExtraDataExplorer extends ExtraDataExplorer
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    private static final int RS_COLUMNOFFSET = 1; // column index start at 0 in mapping
    
    private String selectStmt;
    
    public DocumentExtraDataExplorer(CollectionMetadata collection)
    {
        super(collection.getTableInfo(TableSpec.TYPE_EXTRA));
        initStatements(collection);
        
    }
    
    private void initStatements(CollectionMetadata collection)
    {
        StringBuffer sql = new StringBuffer();
        UOIDManager manager = collection.getUOIDManager();
        
        sql.append("SELECT e.UOID,e.PATH,e.TYPE,e.POS,e.OFFSET,e.DATA,e.NUM FROM ");
        sql.append(collection.getTableInfo(TableSpec.TYPE_DOCS).getName());
        sql.append(" d,");
        sql.append(tableInfo.getName());
        sql.append(" e WHERE d.NAME= ? AND e.UOID>=(");
        sql.append(manager.getDocMultiplier());
        // assume first local node OID is 0
        sql.append("*d.UDID) AND e.UOID<=(");
        sql.append(manager.getDocMultiplier());
        sql.append("*d.UDID+");
        sql.append(manager.getDocMaxConstant());
        sql.append(") ORDER BY e.NUM");
        selectStmt = sql.toString();
    }
    
    public void initDocReading(AbstractConnection connection)
    throws SQLException
    {
        useStringDelimitor = connection.useStringDelimitor();
        try
        {
            pStmt = connection.getConnection().prepareStatement(selectStmt);
            connection.setFetchSize(pStmt, RepositoryProperties.getIntProperty(CONF_TREE_FETCHSIZE));
        }
        catch (SQLException e)
        {
            pStmt.close();
            throw e;
        }
        // NOTE test ORDER BY source, rank to see if equivalent (would match any mapping)
    }
    
    public void getDocument(String ID)
    throws SQLException
    {
        try
        {
            pStmt.setString(1, ID);
            rs = pStmt.executeQuery();
            notExhausted = true;
        }
        catch (SQLException e)
        {
            pStmt.close();
            throw e;
        }
    }
    
    /**
     * get the next row from database and store it in the current object.
     */
    public boolean fetchNextRow() throws SQLException
    {
        try
        {
            if (resultActive())
            {
                notExhausted = rs.next();
                if (notExhausted)
                {
                uoid = rs.getLong(RS_COLUMNOFFSET);
                node.path = rs.getShort(RS_COLUMNOFFSET + 1);
                node.type = rs.getShort(RS_COLUMNOFFSET + 2);
                node.position = rs.getShort(RS_COLUMNOFFSET + 3);
                node.offset = rs.getInt(RS_COLUMNOFFSET + 4);
                node.data = rs.getString(RS_COLUMNOFFSET + 5);
                if (useStringDelimitor)
                    node.data = node.data.substring(0, node.data.length() - STRING_DELIMITOR_LENGTH);
                node.rowNum = rs.getShort(RS_COLUMNOFFSET + 6);
                }
                else
                    reset();
            }
            else
                return false;
            
            return notExhausted;
        }
        catch (SQLException e)
        {
            close();
            throw e;
        }
    }
}
