/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.dbms.TableSpec;
import org.xquark.mapper.metadata.CollectionMetadata;
import org.xquark.mapper.metadata.UOIDManager;
import org.xquark.mapper.util.RepositoryProperties;

/**
 * Object used to perform XML structure node storage.
 *
 */
public class DocumentStructExplorer extends StructExplorer
{
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private static Log log = LogFactory.getLog(DocumentStructExplorer.class);
    
    private static final int RS_COLUMNOFFSET = 1; // column index start at 0 in mapping
    
    private String selectStmt;
    
    
    public DocumentStructExplorer(CollectionMetadata collection)
    {
        super(collection.getTableInfo(TableSpec.TYPE_STRUCT), collection.getUOIDManager());
        initStatements(collection);
    }
    
    private void initStatements(CollectionMetadata collection)
    {
        StringBuffer sql = new StringBuffer();
        UOIDManager manager = collection.getUOIDManager();
        
        sql.append("SELECT s.UOID,s.LAST,s.PATH FROM ");
        sql.append(collection.getTableInfo(TableSpec.TYPE_DOCS).getName());
        sql.append(" d,");
        sql.append(tableInfo.getName());
        sql.append(" s WHERE d.NAME=? AND s.BUCKET=d.UDID AND s.UOID>=(");
        sql.append(manager.getDocMultiplier());
        // assume first local node OID is 0
        sql.append("*d.UDID) AND s.UOID<=(");
        sql.append(manager.getDocMultiplier());
        sql.append("*d.UDID+");
        sql.append(manager.getDocMaxConstant());
        sql.append(") ORDER BY s.UOID");
        selectStmt = sql.toString();
        if (log.isDebugEnabled())
        	log.debug("Struct reading statement:\n" + selectStmt);
    }
    
    public void initDocReading(AbstractConnection connection)
    throws SQLException
    {
        try
        {
            pStmt = connection.getConnection().prepareStatement(selectStmt);
            connection.setFetchSize(pStmt, RepositoryProperties.getIntProperty(CONF_TREE_FETCHSIZE));
        }
        catch (SQLException e)
        {
            pStmt.close();
            throw e;
        }
        // NOTE test ORDER BY source, rank to see if equivalent (would match any mapping)
    }
    
    public void getDocument(String ID)
    throws SQLException
    {
        try
        {
            pStmt.setString(1, ID);
            rs = pStmt.executeQuery();
            notExhausted = true;
        }
        catch (SQLException e)
        {
            pStmt.close();
            throw e;
        }
    }
    
    /**
     * get the next row from database and store it in the current object.
     */
    public boolean fetchNextRow() throws SQLException
    {
        try
        {
            if (resultActive())
            {
                notExhausted = rs.next();
                if (notExhausted)
                {
                    uoid = rs.getLong(RS_COLUMNOFFSET);
                    last = rs.getLong(RS_COLUMNOFFSET + 1);
                    path = rs.getShort(RS_COLUMNOFFSET + 2);
                }
                else
                    reset();
            }
            else
                return false;
            
            return notExhausted;
        }
        catch (SQLException e)
        {
            close();
            throw e;
        }
    }
}
