/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.PreparedStatement;
import java.sql.SQLException;

import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.dbms.TableInfo;
import org.xquark.mapper.metadata.ExtraNode;

/**
 * Object used to perform XML structure node storage.
 *
 */
public class ExtraDataSaver extends ExtraDataPersistentNode
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private boolean useStringDelimitor = false;
    
    public ExtraDataSaver(TableInfo tableInfo)
    {
        super(tableInfo);
        initStatements();
    }
    
    protected void initStatements()
    {
        insertStmt = "INSERT INTO " + tableInfo.getName() + " VALUES (?,?,?,?,?,?,?)";
    }
    
    public PreparedStatement getStatement(AbstractConnection connection)
    throws SQLException
    {
        useStringDelimitor = connection.useStringDelimitor();
        // preparation of PreparedStatement for insertion
        if (pStmt == null)
            pStmt = connection.getConnection().prepareStatement(insertStmt);
        return pStmt;
    }
    
    public PreparedStatement setParameters(ExtraNode node)
    throws SQLException
    {
        try
        {
            pStmt.setLong(1, node.anchor);
            pStmt.setShort(2, node.rowNum);
            pStmt.setShort(3, node.path);
            pStmt.setShort(4, node.type);
            pStmt.setShort(5, node.position);
            pStmt.setInt(6, node.offset);
            pStmt.setString(7, useStringDelimitor?node.data + STRING_DELIMITOR:node.data);
        }
        catch (SQLException e)
        {
            pStmt.close();
            pStmt = null;
            throw e;
        }
        return pStmt;
    }
}
