/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * DocumentGenerator.java
 *
 * Created on 12 mars 2001, 18:01
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.xquark.mapper.RepositoryException;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 *
 *
 * <p>Generation is guided by the schema and indecisions are removed by
 * document instance information (stored in the structure table).</p>
 * @see org.xquark.mapper.XMLCollection
 */
public class InteractiveQueryBuilder extends QueryBuilder
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    /** To be used by query reconstruction */
    public InteractiveQueryBuilder(_RepositoryCollection collection, VarInfoSet varInfo) throws XMLDBCException
    {
        super(collection, varInfo);
        try
        {
            /* Preparing database queries */
            tuplePool = qTuplePool = new InteractiveQueryExplorer(collection);
            // prepare statements
            qTuplePool.initExploration(varInfo, collection.getRepositoryConnection().getConnection());
        }
        catch (SQLException e)
        {
            throw new RepositoryException(RepositoryException.DB_ERROR, "JDBC error while initializing tuple explorer.");
        }
    }
    
    protected void initNewResult(int rank) throws XMLDBCException
    {
        try
        {
            qTuplePool.reset();
            
            // execute prepared statement (who handles break ?)
            ((InteractiveQueryExplorer)qTuplePool).execQuery(varFirst, varLast, currentPath);
            
            // initiate row lookup
            qTuplePool.lookup();
        }
        catch (SQLException e)
        {
            throw new RepositoryException(RepositoryException.DB_ERROR,
            "Database error while performing reconstruction query", e);
        }
    }
}
