/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * QueryExplorer.java
 *
 * Created on 18 avril 2002, 17:15
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.dbms.TableSpec;
import org.xquark.mapper.mapping.TableMapping;
import org.xquark.mapper.metadata.CollectionMappingInfo;
import org.xquark.mapper.metadata.CollectionMetadata;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 *
 */
class InteractiveQueryExplorer extends QueryExplorer
{
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private static Log log = LogFactory.getLog(InteractiveQueryExplorer.class);
    
    /** Creates a new instance of QueryExplorer */
    InteractiveQueryExplorer(_RepositoryCollection collection) throws SQLException, XMLDBCException
    {
        super(collection);
        AbstractConnection connection = collection.getRepositoryConnection().getConnection();
        CollectionMetadata meta = collection.getMetadata();
        wNode = qNode = new InteractiveQueryStructExplorer(meta.getTableInfo(TableSpec.TYPE_STRUCT), meta.getUOIDManager());
        extraNode = qExtraNode = new InteractiveQueryExtraDataExplorer(meta.getTableInfo(TableSpec.TYPE_EXTRA));
    }
    
    /** Perform Tuple allocation and therefore query execution
     * @param subPaths a  {PathNode subPath, PathNode variablePath} map giving
     * for an absolute subpath the variable root path
     * @param varPaths 0 index PathNode contains the variable root path other are
     * this path subpaths
     */
    void initExploration(VarInfoSet varInfoSet, AbstractConnection connection)
    throws SQLException, XMLDBCException
    {
    	if (log.isDebugEnabled())
    		log.debug("QueryExplorer::initExploration [varInfoSet]=" + varInfoSet);
        
        // Table partitioning
        Object[] ret = dispatchSubPaths(varInfoSet);
        Map tableMappingSubset = (Map)ret[0]; 
        List subPathNodes = (List)ret[1];
        
        // Table subset browsing for statement execution
        Iterator it = tableMappingSubset.entrySet().iterator();
        Map.Entry entry;
        CollectionMappingInfo tableStmts;
        int tableIndex;
        
    	CollectionMetadata meta = collection.getMetadata();
        while (it.hasNext())
        {
            entry = (Map.Entry)it.next();
            tableIndex = ((TableMapping)entry.getKey()).getTableIndex();
            tableStmts = (CollectionMappingInfo)meta.getMappingStatements(tableIndex);
            if (queryTuples[tableIndex] == null)
                queryTuples[tableIndex] = new InteractiveQueryTuple(tableStmts, meta.getMappingPID(tableIndex));
            ((InteractiveQueryTuple)queryTuples[tableIndex]).prepareStatement((List)entry.getValue(),
            meta, connection);
        }
        
        // initialize structure retrieval
        ((InteractiveQueryStructExplorer)qNode).prepareStatement(subPathNodes, connection);
        ((InteractiveQueryExtraDataExplorer)qExtraNode).prepareStatement(subPathNodes, connection);
    }    
    
    void execQuery(long first, long last, short path) throws SQLException, RepositoryException
    {
        for (int i=0; i < queryTuples.length; i++)
        {
            if (queryTuples[i] != null)
                ((InteractiveQueryTuple)queryTuples[i]).execQuery(first, last, path);
        }
        ((InteractiveQueryStructExplorer)qNode).execQuery(first, last, path);
        ((InteractiveQueryExtraDataExplorer)qExtraNode).execQuery(first, last, path);
    }
}
