/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * LogicalStructureBuilder.java
 *
 * Created on 25 juillet 2001, 11:45
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;

import org.xml.sax.Locator;
import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.mapping.ColumnMapping;
import org.xquark.mapper.mapping.RepositoryMapping;
import org.xquark.mapper.metadata.StoragePathMetadata;
import org.xquark.schema.validation.SchemaValidationContext;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xpath.NodeKind;

/**
 *
 */
public class MapperTupleBuilder extends AbstractModelBuilder
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

  // Not final for QA purpose
    public static boolean USE_EMPTY_STRING = false;
    
    public MapperTupleBuilder(AbstractConnection connection, RepositoryMapping mapping, SchemaValidationContext schemaContext) throws XMLDBCException
    {
        super(connection, schemaContext);
        storageBuffer = new StorageBuffer(mapping, "dummy", connection, batcher, this, USE_EMPTY_STRING);
    }
    
    public  void setDocumentIDs(int docOID, String docID)
    {
        // no op
    }
    
    /**
     * For PI, comments and prefix definition.
     **/
    public  void extraNode(byte type, StoragePathMetadata path, String data)
    throws RepositoryException, SQLException
    {
        // no op
    }
    
    protected AbstractModelBuilder.AbstractModelNode getLogicalStructureNode(byte type, StoragePathMetadata path, String data, Locator locator)
    throws SQLException, RepositoryException
    {
        AbstractModelBuilder.AbstractModelNode node = null;
        
        // process mappings for elements & attributes & increment parent child counters
        short rank = 0;
        List nodeTableMappings = null, nodeColumnMappings = null;
        switch (type)
        {
            case NodeKind.ELEMENT:
                if (stack.size() != 0)  // not root element
                    rank = getCurrentNode().incChildCount();
                // no break
                
            case NodeKind.ATTRIBUTE:
                nodeTableMappings = path.getTableMappings();
                nodeColumnMappings = path.getColumnMappings();
                break;
                
            default :
        }
        
        // allocate or recycle a node
        node = (AbstractModelNode)stack.push();
        node.set(path, type, data, rank);
        
        // Process table mappings (create tuples)
        if (nodeTableMappings != null)
            node.setTupleNode(storageBuffer.bufferizeNode(nodeTableMappings, locator));
        
        // Process column mappings (set dependencies)
        if (nodeColumnMappings !=  null)
        {
            // Create dependance links between tuples (references : up to now we
            // consider that ref => (FK,PK) and thus a tuple must no be stored
            // before its master.
            Iterator it = nodeColumnMappings.iterator();
            ColumnMapping column = null;
            int refIndex;
            while (it.hasNext())
            {
                column = (ColumnMapping)it.next();
                refIndex = column.getTableRefIndex();
                if (refIndex != -1)
                    storageBuffer.getTupleFactory(column.getTableIndex()).addMasterTuple(storageBuffer.getTupleFactory(refIndex).getTuple());
            }
        }
        
        return node;
    }
    
    protected AbstractModelBuilder.AbstractModelNode finalizeNode(Locator locator)
    throws XMLDBCException, SQLException
    {
        AbstractModelBuilder.AbstractModelNode node = getCurrentNode();
        
        // process mappings for elements & attributes
        List nodeTableMappings = null, nodeColumnMappings = null;
        switch (node.type)
        {
            case NodeKind.ELEMENT:
            case NodeKind.ATTRIBUTE:
                nodeTableMappings = node.getPath().getTableMappings();
                nodeColumnMappings = node.getPath().getColumnMappings();
                break;
            default :
        }
        
        // process column mappings (simple type elements or attributes)
        if (nodeColumnMappings != null)
        {
            Iterator it = nodeColumnMappings.iterator();
            ColumnMapping column = null;
            
            while (it.hasNext())
            {
                column = (ColumnMapping)it.next();
                storageBuffer.getTupleFactory(column.getTableIndex()).getTuple().addColumnMappingData(column, this);
            }
        }
        
        // Complete table mappings
        if (nodeTableMappings != null)
            node.getTupleNode().complete();
        
        // TO IMPROVE save data : could be done on mapping complete
        if (node.type == NodeKind.ELEMENT)
        {
            // set tuple to complete, only waiting for dependences
            // TO IMPLEMENT : check before endElement that no subelement is still waited for an element completion
            //storageBuffer.getTupleFactory(index).setCompleted();
            
            // try to flush completed tuples when an element is ended
            // TO DO : refine it : do it only when necessary
            storageBuffer.flush();
        }
        return (AbstractModelNode)stack.pop();
    }
    
}
