/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * SchemaSAXHandler.java
 *
 * Created on 31 mai 2001, 16:55
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.xml.sax.InputSource;
import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.dbms._RepositoryConnection;
import org.xquark.mapper.mapping.MappingFactory;
import org.xquark.mapper.util.DestructionToken;
import org.xquark.schema.validation.SchemaValidationContext;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 *
 */
public class MappingSAXHandler extends CollectionFiler
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    public MappingSAXHandler(_RepositoryCollection collection,
    SchemaValidationContext schemaContext,
    DestructionToken destructor)
    throws XMLDBCException
    {
        super(collection, schemaContext, true, destructor);
        // TO IMPROVE in the future prefix could not be stored if schema of mapping is available
    }
    
    protected void startParsing() throws RepositoryException, SQLException
    {
        if (!performCommit)
            throw new RepositoryException(RepositoryException.NOT_ALLOWED,
            "The XML connection must be autocommit mode in order to store mapping files.");
        super.startParsing();
    }
    protected void endParsing() throws Exception
    {
        super.endParsing();
        flushBuffer();    // to be sure the mapping is stored before loading mapping
        connection.commit();    // because Sybase throw an exception if autocommit(false)
                                // and access to metadata is performed
        
        // Load the mapping in order to validate it
        RepositoryReader reader = collection.getReader();
        _RepositoryConnection repConn = collection.getRepositoryConnection();
        try
        {
            
            org.xquark.mapper.mapping.Loader loader =
            new org.xquark.mapper.mapping.Loader
            (
            reader,
            repConn.getConnection(),
            repConn.getSchemaManager(),
            false
            );
            
            MappingFactory factory = new MappingFactory
            (
            collection.getMetadata().getRepository(),
            repConn.getSchemaManager(),
            repConn
            );
            factory.createTree();
            loader.load(new InputSource(getDocumentId()), factory, getXMLErrorHandler(), false);
        }
        catch (Exception e) {
            // IMPORTANT : With Oracle (8.1.6, 8.1.7, and Sybase) access to JDBC DataBaseMetadata forces a commit (even if autocommit is false)
            ((MappingXMLCollection)collection).removeDocument(getDocumentId());
            throw e;
        }
        finally
        {
            if (reader != null)
                reader.close();
            
        }
    }
}
