/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * SchemaCollection.java
 *
 * Created on 29 mai 2001, 11:27
 */

package org.xquark.mapper.storage;

import java.util.Iterator;
import java.util.List;

import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.dbms._RepositoryConnection;
import org.xquark.mapper.metadata.CollectionMetadata;
import org.xquark.mapper.util.DestructionToken;
import org.xquark.schema.validation.SchemaValidationContext;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLDocumentFiler;

/**
 *  Special XML Collection for storing mappings
 */
public class MappingXMLCollection extends XMLCollectionImpl
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";

    /** Creates new SchemaCollection */
    public MappingXMLCollection(CollectionMetadata metadata, _RepositoryConnection connection,DestructionToken destructor)
    {
        super(metadata, connection, destructor);
    }

    /* Overriding */
    public XMLDocumentFiler getDocumentFiler() throws XMLDBCException
    {
        protectReadOnlyMethod();
        SchemaValidationContext validationContext =
        new SchemaValidationContext(repConnection.getSchemaManager());
        ValidatingFilerAdapter validator = new ValidatingFilerAdapter(validationContext, repConnection);
        validator.setDocumentBase("http://www.fake.com/");
        CollectionFiler handler =  new MappingSAXHandler
                                    (
                                    this,
                                    validationContext,
                                    new XMLCollectionImpl.HandlerDestructor()
                                    );
        validator.setSAXHandler(handler);
        filers.add(handler);
        return new XDBCFilerAdapter(validator, handler, handler, handler);
    }
    
    /** Remove an XML document from this repository.
     * @param ID The name used to identify the document in
     * the repository.
     * @throws RepositoryException Application exception.
     */
    public boolean removeDocument(String ID) throws XMLDBCException
    {
        protectReadOnlyMethod();
        // Check in the dbms that the mapping file is not used by an existing collection
        List usingCollections = repConnection.getCollectionsUsingMapping(ID);
        if (usingCollections.size() != 0)
            throw new RepositoryException(RepositoryException.NOT_ALLOWED, "Mapping is still used by the "
            + usingCollections + " collections.");
        return super.removeDocument(ID);
    }
    
    public int removeAllDocuments() throws XMLDBCException
    {
        protectReadOnlyMethod();
        Iterator it = getIdentifierList().iterator();
        int docCount = 0;
        
        while (it.hasNext())
        {
            try
            {
                removeDocument((String)it.next());
                docCount++;
            }
            catch (RepositoryException e)
            {
                if (e.getCode() != RepositoryException.NOT_ALLOWED)
                    throw e;
                // else mapping used : continue
            }
        }
        return docCount;
    }
    
    /** Rename an XML document. That is to say, change its ID.
     * @param oldID The old ID of the document.
     * @param newID new ID of the document.
     * @throws RepositoryException Application exception.
     */
    public boolean renameDocument(String oldID, String newID) throws XMLDBCException
    {
        protectReadOnlyMethod();
        // Check in the dbms that the mapping file is not used by an existing collection
        List usingCollections = repConnection.getCollectionsUsingMapping(oldID);
        if (usingCollections.size() != 0)
            throw new RepositoryException(RepositoryException.NOT_ALLOWED, "Mapping is still used by the "
            + usingCollections + " collections and cannot be renamed.");
        // TO IMPROVE change collection reference
        return super.renameDocument(oldID, newID);
    }
    
}
