/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.dbms.TableInfo;
import org.xquark.mapper.metadata.RepositoryInfo;

/** Data structure that corresponds to the storage table for repository
 * configuration and is used when creating a repository.
 * <p>This object contains all its JDBC calls and owns a connection
 * in order to perform database access on his own</p>
 */

public class PersistentRepositoryInfo
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    /////////////////////////////////////
    //  Storage JDBC variables
    /////////////////////////////////////

    private RepositoryInfo config = null;
    
    private TableInfo configTable = null;
    
    /** Connection object used by this data object. */
    private AbstractConnection connection;

    public PersistentRepositoryInfo(TableInfo configTable, AbstractConnection connection)
    {
        this.config = new RepositoryInfo();
        this.connection = connection;
        this.configTable = configTable;
    }

    public PersistentRepositoryInfo(RepositoryInfo config, TableInfo configTable, AbstractConnection connection)
    {
        this.config = config;
        this.connection = connection;
        this.configTable = configTable;
    }

    public RepositoryInfo getRepositoryInfo()
    {
        return config;
    }

    /**
     * Save this object data into a config table row with batch style.
     * Automatic flush is proceeded when max size is reached
     * @throws SQLException if a database access error occurs.
     */
    public void save()
    throws SQLException
    {
        connection.executeUpdate("DELETE FROM "+ configTable.getName());
        StringBuffer stmt = new StringBuffer();
        stmt.append("INSERT INTO ");
        stmt.append(configTable.getName());
        stmt.append(" VALUES('");
        stmt.append(config.getName());
        stmt.append("',");
        stmt.append(config.getColIDSize());
        stmt.append(",'");
        stmt.append(config.getVersion());
        stmt.append("')");
        connection.executeUpdate(stmt.toString());
    }

    public boolean fetchRow()
    throws SQLException
    {
        Statement stmt = connection.getConnection().createStatement();
        ResultSet rs = null;
        try {
            rs = stmt.executeQuery("SELECT NAME,CID_SIZE,VERSION FROM " + configTable.getName());

            if (rs.next())
            {
                config.setName(rs.getString(1));
                config.setColIDSize(rs.getByte(2));
                config.setVersion(rs.getString(3));
                return true;    // Take first row encountered
            }
            else
                return false;   // If no row found
        }
        finally {
            if (rs != null)
                rs.close();
            stmt.close();
        }
    }

}
