/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * QueryExplorer.java
 *
 * Created on 18 avril 2002, 17:15
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;
import java.util.*;

import org.xquark.mapper.dbms.AbstractConnection;
import org.xquark.mapper.mapping.TableMapping;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 *
 */
abstract class QueryExplorer extends TableExplorer
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    protected QueryTuple[] queryTuples;
    protected QueryStructExplorer qNode;
    protected QueryExtraDataExplorer qExtraNode;
    private int subPathCount;
    
    /** Creates a new instance of QueryExplorer */
    QueryExplorer(_RepositoryCollection collection) throws SQLException, XMLDBCException
    {
        super(collection);
        
        // tuples creation performed at execution (only if needed)
        tuples = queryTuples =
        	new QueryTuple[collection.getMetadata().getMappingStatementsCount()];
    }
    
    int getSubPathCount()
    {
        return subPathCount;
    }
    
    /**
	 * Method dispatchSubPaths.
	 * @param varInfoSet
	 * @return Object[]
	 */
	protected Object[] dispatchSubPaths(VarInfoSet varInfoSet)
    {
        HashMap tableMappingSubset = new HashMap();
        SubPathInfo subPathInfo;
        ArrayList subPathNodes = new ArrayList();
        TableMapping table;
        List tablePaths;

        // Table partitioning
        List rangeList = varInfoSet.getExpandedPathList();
        List subPathList;
        ExpandedPathInfoSet expandedPathInfo;
        int expandedPathCount = rangeList.size();
        
        for (int i = 0; i < expandedPathCount; i++)
        {
            expandedPathInfo = (ExpandedPathInfoSet)rangeList.get(i);
            subPathList = expandedPathInfo.getSubPaths();
            subPathCount = subPathList.size();
            
            for (int j = 0; j < subPathCount; j++)
            {
                SubPathInfoSet subPathInfoSet = (SubPathInfoSet)subPathList.get(j);
                subPathInfo = new SubPathInfo(
                subPathInfoSet.getSubPath(),
                j+1,
                subPathInfoSet.getText(),
                expandedPathInfo.getPhysicalPath());
                subPathNodes.add(subPathInfo);
                
                if ((subPathInfo.subPath.getBuildMappings() == null) || subPathInfo.isText)  // pure column mapping (not generic)
                {
                    table =  subPathInfo.subPath.getReadColumnMapping().getTableMapping();
                    tablePaths = (List)tableMappingSubset.get(table);
                    if (tablePaths == null)
                    {
                        tablePaths = new ArrayList(2);
                        tableMappingSubset.put(table, tablePaths);
                    }
                    tablePaths.add(subPathInfo);
                }
                else    // complex mapping
                {
                    Iterator it = subPathInfo.subPath.getBuilderAncestor().getBuildMappings().iterator();
                    
                    while (it.hasNext())
                    {
                        table = (TableMapping)it.next();
                        tablePaths = (List)tableMappingSubset.get(table);
                        if (tablePaths == null)
                        {
                            tablePaths = new ArrayList(2);
                            tableMappingSubset.put(table, tablePaths);
                        }
                        tablePaths.add(subPathInfo);
                    }
                }
            }
        }
        return new Object[] {tableMappingSubset, subPathNodes};
    }
    
    abstract void initExploration(VarInfoSet varInfoSet, AbstractConnection connection)
    throws SQLException, XMLDBCException;
    
    void freezeCurrentSubPath()
    {
        for (int i=0; i < queryTuples.length; i++)
        {
            if (queryTuples[i] != null)
                ((QueryTuple)queryTuples[i]).freezeCurrentSubPath();
        }
        qNode.freezeCurrentSubPath();
        qExtraNode.freezeCurrentSubPath();
    }
    
}
