/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.xquark.mapper.dbms.TableInfo;
import org.xquark.mapper.metadata.UOIDManager;

/**
 * Object used to perform XML structure node storage.
 *
 */
public abstract class QueryStructExplorer extends StructExplorer
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    protected boolean currentResultExhausted = true;
    protected int currentSubPathIndex = -2;
    protected int subPathIndex = -1;
    
    public QueryStructExplorer(TableInfo tableInfo, UOIDManager manager)
    {
        super(tableInfo, manager);
    }
    
    public short getPath()
    {
        if (resultActive())
            return super.getPath();
        else
            return -2;
    }
    
    public long getUOID()
    {
        if (resultActive())
            return super.getUOID();
        else
            return -1;
    }
    
    void freezeCurrentSubPath()
    {
        currentSubPathIndex++;
        updateResultSegmentationFlag();
    }

    public void reset() throws SQLException
    {
        super.reset();
        currentSubPathIndex = -2;
        subPathIndex = -1;
        currentResultExhausted = true;
    }
    
    protected void updateResultSegmentationFlag()
    {
        currentResultExhausted = (currentSubPathIndex != subPathIndex);
    }
    
    boolean resultActive()
    {
        return notExhausted && !currentResultExhausted;
    }
}
