/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.xquark.mapper.mapping.ColumnMapping;
import org.xquark.mapper.metadata.CollectionMappingInfo;
import org.xquark.schema.Declaration;
import org.xquark.schema.validation.ValidationContextProvider;
import org.xquark.xml.xdbc.XMLDBCException;

abstract class QueryTuple extends Tuple
{
    
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName = "$Name: XQ1_1_1 $";
    
    protected int subPathIndex = -1;
    
    protected int valueOffset = 0;
    
    protected boolean currentResultExhausted = true;
    
    protected int currentSubPathIndex = -2;
    
    protected short tablePath = 0; // meaningless if table mapping is shared
    
	QueryTuple(CollectionMappingInfo table)
	{
		super(table);
	}

	QueryTuple(CollectionMappingInfo table, short pathID)
	{
		this(table);
		tablePath = pathID;
	}

    protected String fetchValue(ColumnMapping column, Declaration decl, ValidationContextProvider nsContext)
    throws SQLException, XMLDBCException
    {
        String ret = null;
        try
        {
            if (resultActive())
            {
                    ret = column.getTypeInfo().getParameter(resultSet,
                    column.getInsertColumnIndex() + (table.getTableMapping().isClustered()?0:2) + valueOffset,
                    decl,
                    nsContext);
                    // clustered : because first index starts at 0 and OID belongs to columns
                    // not clustered : two first are XQuark index (UOID, pathOID) and index starts at 0
            }
        }
        catch (SQLException e)
        {
            selectStmt.close();
            throw e;
        }
        return ret;
    }
    
    boolean resultActive()
    {
        return notExhausted && !currentResultExhausted;
    }
    
    long getUOID()
    {
        if (resultActive())
            return uoid;
        else
            return -1;
    }
    
    short getPathID()
    {
        if (resultActive())
            return super.getPathID();
        else
            return -2;
    }
    
    void reset() throws SQLException
    {
        super.reset();
        subPathIndex = -1;
        currentResultExhausted = true;
        currentSubPathIndex = -2;
    }
    
    void freezeCurrentSubPath()
    {
        currentSubPathIndex++;
        updateResultSegmentationFlag();
    }
    
    protected void updateResultSegmentationFlag()
    {
        currentResultExhausted = (currentSubPathIndex != subPathIndex);
    }
}

