/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * RepositoryDocument.java
 *
 * Created on 25 janvier 2002, 18:45
 */
package org.xquark.mapper.storage;

import java.io.IOException;
import java.util.Date;

import org.xml.sax.SAXException;
import org.xquark.mapper.RepositoryDocument;
import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.metadata.DocumentInfo;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLDBCNotSupportedException;
import org.xquark.xquery.xdbc.AbstractXMLDocument;

/**
 *  Implementation of XMLDocument for document stored in the Repository.
 *
 */
public class RepositoryDocumentImpl extends AbstractXMLDocument
implements RepositoryDocument
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private DocumentInfo docInfo;
    
    /** Creates a new instance of RepositoryDocument
     * <p>Does not verify the document exists.</p>
     */
    public RepositoryDocumentImpl(String Id, XMLCollectionImpl collection)
    {
        super(Id, collection);
    }
    
    /** Creates a new instance of RepositoryDocument with document information
     * read from the database.
     */
    public RepositoryDocumentImpl (
                                DocumentInfo docInfo,
                                XMLCollectionImpl collection
                                )
    {
        this(docInfo.docID, collection);
        setDocumentInfo(docInfo);
    }
    
    public void setDocumentInfo(DocumentInfo docInfo)
    {
        this.docInfo = docInfo;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    //                     XMLDocument implementation
    ///////////////////////////////////////////////////////////////////////////
    public void setIdentifier(String identifier) throws XMLDBCException, XMLDBCNotSupportedException
    {
        super.setIdentifier(identifier);
        getCollection().renameDocument(getIdentifier(), identifier); // Was it meant by the API ?
    }
    
    public void remove() throws XMLDBCException, XMLDBCNotSupportedException
    {
        getCollection().removeDocument(getIdentifier());
    }
    
    public void getAsSAX() throws XMLDBCException, SAXException
    {
        _RepositoryCollection collection = (_RepositoryCollection)getCollection();
        RepositoryReader reader = collection.getReader();
        plugHandlers(reader);
        try
        {
            reader.parse(getIdentifier());
        }
        catch (IOException e)
        {
            // TO IMPROVE
            throw new RepositoryException(RepositoryException.DB_ERROR, "IOException while reading document.");
        }
        collection.releaseReader(reader);
    }
    
    ///////////////////////////////////////////////////////////////////////////
    //                     RepositoryDocument implementation
    ///////////////////////////////////////////////////////////////////////////
    public int getAverageNodeDataLength() throws XMLDBCException
    {
        if (docInfo == null)
            getDocumentInfo();
        return docInfo.avgNodeLen;
    }
    
    public Date getCreationDate() throws XMLDBCException
    {
        if (docInfo == null)
            getDocumentInfo();
        return docInfo.creationDate;
     }
    
    public long getEstimatedDocumentLength() throws XMLDBCException
    {
        if (docInfo == null)
            getDocumentInfo();
        return docInfo.getEstimatedDocumentLength();
    }
    
    public long getNodeCount() throws XMLDBCException
    {
        if (docInfo == null)
            getDocumentInfo();
        return docInfo.nodeCount;
     }
    
    ///////////////////////////////////////////////////////////////////////////
    //                              PRIVATE
    ///////////////////////////////////////////////////////////////////////////
    private void getDocumentInfo() throws RepositoryException
    {
        docInfo = ((_RepositoryCollection)getCollection()).getMetadata().getDocumentInfo(getIdentifier());
    }
}
