/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * RepositoryHandler.java
 *
 * Created on 30 mai 2001, 10:56
 */

package org.xquark.mapper.storage;

import org.w3c.dom.Document;
import org.xml.sax.XMLReader;
import org.xquark.mapper.RepositoryException;

/** This object allows to perform reading operations on an XML collection
 * using SAX2 and DOM2 interfaces.
 * This interface extends the SAX2 {@link org.xml.sax.XMLReader org.xml.sax.XMLReader}.
 * This means that you can use a RepositoryReader like any SAX2 parser <B>except that
 * the parse method only accept {@link org.xml.sax.InputSource InputSources} with a
 * sysstem ID</B>.
 * This object can be used for multiple operations increasing performance since
 * JDBC and Java resources are allocated one time.
 */
public interface RepositoryReader extends XMLReader
{
    /** Builds a DOM Level 2 document from an XML document stored in the collection
     * basing on an empty document passed as a parameter.
     * @param ID The name used to identify the document in the XML collection
     * @return a DOM Level 2 document.
     * @throws RepositoryException API exception.
     */
    public Document getDocument(String ID) throws RepositoryException;
    
    /** Release all resources. The object is no more usable.
     * @throws RepositoryException an application exception
     */    
    public void close() throws RepositoryException;
    
    /** Release handlers.
     * @throws RepositoryException an application exception
     */    
    public void unPlug();
}

