/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * SchemaSAXHandler.java
 *
 * Created on 31 mai 2001, 16:55
 */

package org.xquark.mapper.storage;

import java.util.List;

import org.xml.sax.InputSource;
import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.mapping.MappingConstants;
import org.xquark.mapper.util.DestructionToken;
import org.xquark.schema.Schema;
import org.xquark.schema.SchemaConstants;
import org.xquark.schema.SchemaManager;
import org.xquark.schema.validation.SchemaValidationContext;
import org.xquark.xml.xdbc.XMLDBCException;

/**
 *
 */
public class SchemaSAXHandler extends CollectionFiler
{
private static final String RCSRevision = "$Revision: 1.3 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    /** Creates new SchemaSAXHandler */
    public SchemaSAXHandler(_RepositoryCollection collection,
                            SchemaValidationContext schemaContext,
                            DestructionToken destructor)
    throws XMLDBCException
    {
        super(collection, schemaContext, true, destructor);    // TO IMPROVE in the future prefix could not be stored if schema of schemas is available
    }
    
    protected void endParsing()throws Exception
    {
        super.endParsing();
        flushBuffer();    // to be sure the schema is stored before loading

        // Load the schema to validate it
        SchemaManager manager = collection.getMetadata().getRepository().getSchemaManager();
        
        // load schema from input source cos' file ID ain't namespace
        Schema schema = manager.loadSchema(collection.getRepositoryConnection(), new InputSource(getDocumentId()));
        if (schema == null)
            throw new RepositoryException(RepositoryException.CONSISTENCY_ERROR, "The schema could not be loaded.");
        
        String targetNamespace = schema.getNamespace();
        if (targetNamespace == null)
            targetNamespace = NO_NAMESPACE_SCHEMA_ID;
        else if (targetNamespace.equals(SchemaConstants.XMLSCHEMA_URI)
                    || targetNamespace.equals(MappingConstants.MAPPING_URI))
            throw new RepositoryException(RepositoryException.ALREADY_EXISTS,
            "This schema could not be stored in the repository since it is a system schema.");

        // first check it's not in use
        List usingCollections = collection.getRepositoryConnection().getCollectionsUsingNamespace(targetNamespace);
        if (usingCollections.size() != 0)
            throw new RepositoryException(RepositoryException.NOT_ALLOWED,
            "This namespace is still used by documents stored in the following collection: "
            + usingCollections + ". Adding such a schema would alter the metadata for these documents.");
        else
        {
            List usingMappings = collection.getRepositoryConnection().getMappingsUsingNamespace(targetNamespace);
            if (usingMappings.size() != 0)
                throw new RepositoryException(RepositoryException.NOT_ALLOWED,
                "This namespace is still used by mapping files: "
                + usingMappings + ". Adding such a schema would alter the metadata for these documents.");
        }
        
        // Check the schema does not already exists in the system repository
        if (collection.containsDocument(targetNamespace))    // if not already opened or present in the system repository
            throw new RepositoryException(RepositoryException.ALREADY_EXISTS,
            "The namespace of the schema is already in use in the system repository.");
        // rename the schema document
        ((SchemaXMLCollection)collection).renameDocument(getDocumentId(), targetNamespace, false); // this throws an exception
        // change it so the getDocument is consistent
        docID = targetNamespace;
    }
    
}
