/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * SchemaCollection.java
 *
 * Created on 29 mai 2001, 11:27
 */

package org.xquark.mapper.storage;

import java.util.Iterator;
import java.util.List;

import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.dbms._RepositoryConnection;
import org.xquark.mapper.metadata.CollectionMetadata;
import org.xquark.mapper.util.DestructionToken;
import org.xquark.schema.validation.SchemaValidationContext;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLDocumentFiler;

/**
 *  Special XML Collection for storing XML schemas
 */
public class SchemaXMLCollection extends XMLCollectionImpl
{
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    /** Creates new SchemaCollection */
    public SchemaXMLCollection(CollectionMetadata metadata, _RepositoryConnection connection, DestructionToken destructor)
    {
        super(metadata, connection, destructor);
    }

    /* Overriding */
    public XMLDocumentFiler getDocumentFiler() throws XMLDBCException
    {
        protectReadOnlyMethod();
        SchemaValidationContext validationContext =
            new SchemaValidationContext(repConnection.getSchemaManager());
        ValidatingFilerAdapter validator = new ValidatingFilerAdapter(validationContext, repConnection);
        validator.setDocumentBase("http://www.fake.com/");
        CollectionFiler handler =  new SchemaSAXHandler
                                        (
                                        this,
                                        validationContext, 
                                        new XMLCollectionImpl.HandlerDestructor()
                                        );
        validator.setSAXHandler(handler);
        filers.add(handler);
        return new SchemaSavingFilter(new XDBCFilerAdapter(validator, handler, handler, handler));
        // TO IMPROVE : Not correct new XDBCFilerAdapter must be in front because it auto plugs
    }
    
    // TO DO : ADD a getHandler(SchemaLocator) method...
    public boolean removeDocument(String ID) throws XMLDBCException
    {
        protectReadOnlyMethod();
        // first check it's not in use
        List usingCollections = repConnection.getCollectionsUsingNamespace(ID);
        if (usingCollections.size() != 0)
            throw new RepositoryException(RepositoryException.NOT_ALLOWED,
            "This namespace is still used by documents stored in the following collection: "
            + usingCollections + ".");
        else
        {
            List usingMappings = repConnection.getMappingsUsingNamespace(ID);
            if (usingMappings.size() != 0)
                throw new RepositoryException(RepositoryException.NOT_ALLOWED,
                "This namespace is still used by mapping files: "
                + usingMappings + ".");
        }
        
        boolean ret = super.removeDocument(ID);
        
        // unload the schema
        if (ID.equals(NO_NAMESPACE_SCHEMA_ID))
            ID = null;
        getMetadata().getRepository().getSchemaManager().removeSchema(ID);
        return ret;
    }
    
    public int removeAllDocuments() throws XMLDBCException
    {
        protectReadOnlyMethod();
        Iterator it = getIdentifierList().iterator();
        int docCount = 0;
        
        while (it.hasNext())
        {
            try
            {
                removeDocument((String)it.next());
                docCount++;
            }
            catch (RepositoryException e)
            {
                if (e.getCode() != RepositoryException.NOT_ALLOWED)
                    throw e;
                // else schema used continue
            }
        }
        return docCount;
    }
    
    public boolean renameDocument(String oldID, String newID)
    throws XMLDBCException
    {
       throw new UnsupportedOperationException("A schema cannot be renamed since its ID must be the target namespace URI");
    }
}
