/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.xquark.mapper.dbms.TableInfo;
import org.xquark.mapper.metadata.UOIDManager;

/**
 * Object used to perform XML structure node storage.
 *
 */
public abstract class StructExplorer extends StructPersistentNode
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    protected boolean notExhausted = false;
    
    public StructExplorer(TableInfo tableInfo, UOIDManager manager)
    {
        super(tableInfo, manager);
    }
    
    /**
     * get the next row from database and store it in the current object.
     */
    public abstract boolean fetchNextRow() throws SQLException;
    
    boolean queryPending()
    {
        return notExhausted;
    }
    
    /** To be overridden */
    boolean resultActive()
    {
        return notExhausted;
    }
    
    public void reset() throws SQLException
    {
        super.reset();
        notExhausted = false;
    }
    
    public boolean isInRange(long first, long last, short path)
    {
        return resultActive() && super.isInRange(first, last, path);
    }
    
    public boolean isInRange(long first, long last)
    {
        return resultActive() && super.isInRange(first, last);
    }
}
