/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.sql.SQLException;

import org.xquark.mapper.dbms.TableInfo;
import org.xquark.mapper.metadata.UOIDManager;

/**
 * Object used to perform XML structure node storage.
 *
 */
public abstract class StructPersistentNode extends AbstractPersistentNode
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    protected long last = -1;
    protected short path = -2;  // -1 is ROOT path ID !
    protected UOIDManager manager = null;
    
    public StructPersistentNode(TableInfo tableInfo, UOIDManager manager)
    {
        super(tableInfo);
        this.manager = manager;
    }
    
    public long getLast()
    {
        return last;
    }
    
    public short getPath()
    {
        return path;
    }
    
    public void reset() throws SQLException
    {
        super.reset();
        last = -1;
        path = -2;
    }
    
    public boolean isInRange(long first, long last, short path)
    {
        return (uoid >= first) && (uoid <= last) && (this.path == path);
    }
    
    public boolean isInRange(long first, long last)
    {
        return (uoid >= first) && (uoid <= last);
    }
}
