/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import org.xquark.schema.SchemaLocator;
import org.xquark.schema.validation.SchemaValidationContext;
import org.xquark.schema.validation.ValidatingSchemaHandler;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLErrorHandler;

/**
 * Adapter to avoid collision between the XDBC (XMLDocumentFiler) and the SAX API
 * (ValidatingSchemaFilter) on methods like setFeature() and perform validation.
 */
public class ValidatingFilerAdapter extends ValidatingSchemaHandler
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    protected SAXHandler handler;
    
    public ValidatingFilerAdapter(SchemaValidationContext svContext, SchemaLocator locator)
    {
        super(svContext, locator, false);
    }

    public void setSAXHandler(SAXHandler handler)
    {
        this.handler = handler;
        super.setContentHandler(handler);
        super.setLexicalHandler(handler);
        super.setErrorHandler(handler);
    }
    
    ///////////////////////////////////////////////////////////////////////////
    //                     XMLDocumentFiler forward
    ///////////////////////////////////////////////////////////////////////////
    public void flushBuffer() throws XMLDBCException
    {
        handler.flushBuffer();
    }
    
    public void setErrorHandler(XMLErrorHandler handler)
    {
        this.handler.setErrorHandler(handler);
    }
    
    public XMLErrorHandler getErrorHandler()
    {
        return handler.getXMLErrorHandler();
    }
    
    public void clearBuffer() throws XMLDBCException
    {
        handler.clearBuffer();
    }
    
    public void close() throws XMLDBCException
    {
        handler.close();
    }
    
    public boolean getAutoFlush()
    {
        return handler.getAutoFlush();
    }
    
    public void setAutoFlush(boolean mode) throws XMLDBCException
    {
        handler.setAutoFlush(mode);
    }
}
