/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import java.util.ArrayList;
import java.util.List;

import org.xquark.util.NamespaceContextStack;

/** 
 * Light implementation used for the case the result table cannot be used
 * directly.
 */
public class VarBuildInfoSet implements VarInfoSet
{
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";
	private String tableName;
	private String rankColumn;
	private String firstColumn;
	private String lastColumn;
	private String pathColumn;
    
    /** List of column names(strings) representing colums on which the GROUP BY
     * clause must be applied. If null, all tuples must be generated.
     */
    private ArrayList collapseColumnList = new ArrayList(1);
    
    /** List of ExpandedPathInfoSet objects representing all paths found 
     * corresponding to a reconstruction variable.
     */
    private ArrayList expandedPathList = new ArrayList(2);
    
    /** Creates new VarBuildInfoSet */
    public VarBuildInfoSet  (
                            String tableName, String rankColumn,
                            String firstColumn, String lastColumn,
                            String pathColumn
                            )
    {
        this.tableName = tableName;
        this.rankColumn = rankColumn;
        this.firstColumn = firstColumn;
        this.lastColumn = lastColumn;
        this.pathColumn = pathColumn;
    }
    
    public String getTableName() {return tableName;}
    public String getRankColumnName() {return rankColumn;}
    public String getFirstColumnName() {return firstColumn;}
    public String getLastColumnName() {return lastColumn;}
    public String getPathColumnName() {return pathColumn;}
    
    public String getPrefix(String namespaceURI)
    {
        return null;
    }
    
	public NamespaceContextStack getNamespaceContextStack()
    {
        return null;
    }
    
    public void setCollapse(boolean collapse)
    {
        if (collapse)
            addCollapseColumn(firstColumn);
        else
            collapseColumnList.clear();
    }
    
    public void addCollapseColumn(String columnName)
    {
        collapseColumnList.add(columnName);
    }
    
    public List getCollapseColumns()
    {
        return collapseColumnList;
    }
    
    public void addExpandedVar(ExpandedPathInfoSet rangeInfo)
    {
        expandedPathList.add(rangeInfo);
    }
    
    public List getExpandedPathList()
    {
        return expandedPathList;
    }
    
}
