/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.storage;

import org.w3c.dom.Document;
import org.xml.sax.*;
import org.xml.sax.ext.LexicalHandler;
import org.xquark.xml.xdbc.*;

/**
 * Class used to convert XDBCDocumentFiler calls to basic internal calls.
 */
public class XDBCFilerAdapter extends XDBCMapperAdapter
implements XMLDocumentFiler
{
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName     = "$Name: XQ1_1_1 $";
    private CollectionFiler filer;
    
    public XDBCFilerAdapter(ContentHandler contentHandler, LexicalHandler lexicalHandler,
                    ErrorHandler errorHandler, CollectionFiler filer)
    {
        super(contentHandler, lexicalHandler, errorHandler, filer);
        this.filer = filer;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    //                     XMLDocumentFiler implementation
    ///////////////////////////////////////////////////////////////////////////
    
    public XMLDocument insertDocument(InputSource input, String id)
    throws XMLDBCException, SAXException
    {
        setDocumentId(id);
        return insertDocument(input);
    }
    
    public XMLDocument insertDocument(XMLReader parser, InputSource input, String id)
    throws XMLDBCException, SAXException
    {
        setDocumentId(id);
        return insertDocument(parser, input);
    }

    public XMLDocument insertDocument(String doc, String id)
    throws XMLDBCException, org.xml.sax.SAXException
    {
        setDocumentId(id);
        return insertDocument(doc);
    }
    
    public XMLDocument insertDocument(Document doc, String id)
    throws XMLDBCException
    {
        setDocumentId(id);
        return insertDocument(doc);
    }
    

    public XMLDocument insertDocument(InputSource input)
    throws XMLDBCException, SAXException
    {
        insertXMLDocument(input);
        return filer.getCollection().getDocument(filer.getDocumentId());
    }
    
    public XMLDocument insertDocument(XMLReader parser, InputSource input)
    throws XMLDBCException, SAXException
    {
        insertXMLDocument(parser, input);
        return filer.getCollection().getDocument(filer.getDocumentId());
    }
    
    public XMLDocument insertDocument(String doc)
    throws XMLDBCException, SAXException
    {
        insertXMLDocument(doc);
        return filer.getCollection().getDocument(filer.getDocumentId());
     }
    
    public XMLDocument insertDocument(Document document) throws XMLDBCException
    {
        insertXMLDocument(document);
        return filer.getCollection().getDocument(filer.getDocumentId());
    }
    
    public void setDocumentId(String id) throws XMLDBCException
    {
        filer.setDocumentId(id);
    }
    
    public XMLCollection getCollection()
    {
        return filer.getCollection();
    }
    
    public String getDocumentId()
    {
        return filer.getDocumentId();
    }
    
    public String[] getFeatureList()
    {
        return filer.getXMLFeatureList();
    }
    
    public String[] getPropertyList()
    {
        return filer.getXMLPropertyList();
    }
    
    public boolean getFeature(String featureId) throws XMLDBCNotRecognizedException
    {
        return filer.getXMLFeature(featureId);
    }
    
    public void setFeature(String featureId, boolean state) throws XMLDBCNotRecognizedException, XMLDBCNotSupportedException
    {
        filer.setXMLFeature(featureId, state);
    }
    
    public Object getProperty(String propertyId) throws XMLDBCNotRecognizedException
    {
        return filer.getXMLProperty(propertyId);
    }
    
    public void setProperty(String propertyId, Object value) throws XMLDBCNotRecognizedException, XMLDBCNotSupportedException
    {
        filer.setXMLProperty(propertyId, value);
    }
}
