/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * CharArraysReader.java
 *
 * Created on 23 mai 2001, 18:15
 */

package org.xquark.mapper.util;

import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;


/**
 *  Reader implementation basing on multiples char arrays avoiding allocations.
 *
 * <p>Intended to be used for direct transmission of SAX characters to JDBC2
 * setCharArrayStream.</p>
 */
public class CharArraysReader extends Reader
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";

    private ArrayList buffers;
    private int current = 0;
    private int offset = 0;
    private int totalLength = 0;
    
    /** Creates new CharArraysReader */
    public CharArraysReader()
    {
        buffers = new ArrayList();
    }
    
    public CharArraysReader(char[] str, int offset, int len)
    {
        this();
        append(str, offset, len);
    }
    
    public void append(char[] str, int offset, int len)
    {
        // MAY HAVE TO PROCEED A COPY BECAUSE CHAR BUFFER MIGHT CHANGE BEFORE END_ELEMENT ?
        buffers.add(new CharBuffer(str, offset, len));
        totalLength += len;
    }

    public int getLength()
    {
        return totalLength;
    }

    public void clear()
    {
        buffers.clear();
        current = 0;
        totalLength = 0;
        offset = 0;
    }
    
    //////////////////////////////////////////////////////
    //          Implementation of Reader
    //////////////////////////////////////////////////////
    public void close() throws IOException
    {
        clear();
    }
    
    public int read(char[] values, int off, int len) throws IOException
    {
        if (current < buffers.size())
        {
            CharBuffer cb;
            int remaining = len;
            int currentRemaining;
            int toRead;
            while (remaining > 0)
            {
                cb = (CharBuffer)buffers.get(current);
                currentRemaining = cb.len - offset;
                if (remaining >= currentRemaining)
                {
                    toRead = currentRemaining;
                    current++;
                    offset = 0;
                    if (current == buffers.size())
                        break;
                }
                else
                {
                    toRead = remaining;
                    offset += toRead;
                }
                System.arraycopy(cb.chars, cb.off + offset, values, off + (len - remaining), toRead);
                remaining -= toRead;
            }
            return (len - remaining);
        }
        else
            return -1;
    }
    
    public void reset() throws IOException
    {
        current = 0;
        offset = 0;
    }
    //////////////////////////////////////////////////////
    //              private utilities
    //////////////////////////////////////////////////////
    private class CharBuffer
    {
        char[] chars;
        int off;
        int len;
        CharBuffer(char[] str, int offset, int len)
        {
            this.chars = str;
            this.off = offset;
            this.len = len;
        }
    }
}
