/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * Pool.java
 *
 * Created on 5 mars 2001, 16:34
 */

package org.xquark.mapper.util;

import java.util.*;

/**
 * Map inherited from hashmap mainting a counter for addition/removal.
 *
 */
public class CounterMap extends HashMap
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    public boolean containsValue(Object value)
    {
        return super.containsValue(new ValueInfo(value));
    }
    
    public Object get(Object key)
    {
        ValueInfo value = (ValueInfo)super.get(key);
        if (value == null)
            return null;
        else
            return value.get();
    }
    
    public Object checkOut(Object key)
    {
        ValueInfo info = (ValueInfo)super.get(key);
        
        if (info == null)
            return null;
        else
            info.inc();
        
        return info.get();
    }
    
    /* If the key is already in the map, then counter is incremented and the
     * existing object is not replaced by the new
     */
    public Object put(Object key, Object value)
    {
        Object info = checkOut(key);
        
        if (info == null)
        {
            super.put(key, new ValueInfo(value));
            return null;
        }
        
        return info;
    }
    /* not supported */
    public void putAll(Map t) { throw new UnsupportedOperationException();}
    
    /* returns null if no mapping or if the object is not really removed
     * i.e. in use by another user
     **/
    public Object remove(Object key)
    {
        ValueInfo info = (ValueInfo)super.get(key);
        
        if (info == null)
            return null;
        else
            info.dec();
        
        if (info.isUsed())
            return null;
        else
            super.remove(key);
        
        return info.get();
    }
        
    /* removes unconditionnally the object whatever it is used.
     **/
    public Object removeAll(Object key)
    {
        ValueInfo info = (ValueInfo)super.remove(key);
        if (info == null)
            return null;
        else
            return info.get();
    }
    
    public Set entrySet()
    {
        Map.Entry wEntry;
        HashSet res = new HashSet();
        Iterator it = super.entrySet().iterator();
        while (it.hasNext())
        {
            wEntry = (Map.Entry)it.next();
            res.add(((ValueInfo)wEntry.getValue()).get());
        }
        return res;
    }
    
    public Collection values()
    {
        ArrayList list = new ArrayList();
        Iterator it = super.values().iterator();
        while (it.hasNext())
        {
            list.add(((ValueInfo)it.next()).get());
        }
        return list;
    }
    
    /* change a key for a value without losing the counter */
    public void setKey(Object oldKey, Object newKey)
    {
        if (containsKey(oldKey))
            super.put(newKey, super.remove(oldKey));
    }
    
    ///////////////////////////////////////////////////////////////////////
    //                          INNER CLASS
    ///////////////////////////////////////////////////////////////////////
    /*
     * Data structure for keeping a connection per thread and a counter
     * for the number of times the connection has been asked.
     */
    public class ValueInfo
    {
        private Object o;
        private int use = 1;
        
        ValueInfo(Object o) { this.o = o;}
        
        void inc() { use++;}
        void dec() { use--;}
        public boolean isNotUsed() { return (use < 1); }
        public boolean isUsed() { return (use > 0); }
        public Object get() { return o;}
        public boolean equals(Object o) {return o.equals(this.o);}
    }
}
