/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * NSPrefixRepositoryFilter.java
 *
 * Created on 1 juin 2001, 09:22
 */

package org.xquark.mapper.util;

import java.io.IOException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;
import org.xquark.mapper.RepositoryException;
import org.xquark.mapper.storage.RepositoryReader;

/** This filter, designed to be plugged on a RepositoryReader performs a close
 * of resources when parsing is ended.
 *
 */
public class DisposableReaderFilter extends XMLFilterImpl
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
    public DisposableReaderFilter(RepositoryReader reader) throws RepositoryException
    {
        super(reader);
    }

    public void parse(InputSource input)
    throws IOException, SAXException
    {
        super.parse(input);
        try {
            ((RepositoryReader)getParent()).close();
        }
        catch (RepositoryException e) {
            if (e.getException() instanceof Exception)
                throw new SAXException(e.getMessage(), (Exception)e.getException());
            else
                throw (Error)e.getException();
        }
    }
    public void parse(String systemID)
    throws IOException, SAXException
    {
        super.parse(systemID);
        try {
            ((RepositoryReader)getParent()).close();
        }
        catch (RepositoryException e) {
             if (e.getException() instanceof Exception)
                throw new SAXException(e.getMessage(), (Exception)e.getException());
            else
                throw (Error)e.getException();
        }
    }
}
