/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.util;

import java.util.StringTokenizer;

/**
 * Easy to use functions and methods
 */
public class Functions
{
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";
	public static final long LOCAL_BITMASK = 0x00000000FFFFFFFF;

	public static LongList StringToLongList(String s)
	{
		StringTokenizer tokens = new StringTokenizer(s, ".");
		LongList list = new LongList();

		while (tokens.hasMoreTokens())
		{
			list.add(new Long(tokens.nextToken()));
		}

		return list;
	}

	public static LongList StringToIntegerList(String s)
	{
		StringTokenizer tokens = new StringTokenizer(s, ".");
		LongList list = new LongList();

		while (tokens.hasMoreTokens())
		{
			list.add(new Long(tokens.nextToken()));
		}

		return list;
	}

	public static String replacePlaceholder(String s, String ph, String value)
	{
		int phIndex;
		if (s == null)
			return null;
		while ((phIndex = s.indexOf(ph)) >= 0)
		{
			s =
				s.substring(0, phIndex)
					+ value
					+ s.substring(phIndex + ph.length());
		}
		return s;
	}

	/* Replaces character '?' at position n with the value */
	public static String setString(String statement, String value, int pos)
	{
		StringBuffer buf = new StringBuffer(statement);

		// find the corresponding '?'
		int markPos = 0, markCount = 0, bufLen = buf.length();

		for (; markPos < bufLen; markPos++)
		{
			if (buf.charAt(markPos) == '?')
				markCount++;
			if (markCount == pos)
				break;
		}

		// replacing '?' by value
		buf.insert(markCount, value);
		buf.deleteCharAt(markCount + value.length());

		return buf.toString();
	}

	public static long makeLong(int highBits, int lowBits)
	{
		long result = (long) highBits;
		return ((result << 32) + lowBits);
	}

	public static int getHigh32(long tagCode)
	{
		return (int) (tagCode >>> 32);
	}

	public static int getLow32(long tagCode)
	{
		return (int) (tagCode & LOCAL_BITMASK);
	}

	/**
	 * Test if the character is whitespace according to XML 1.0 specifications.
	 *
	 * @param c The character to be tested.
	 * @return the result of the test.
	 */
	public static boolean isWhitespace(char c)
	{
		switch (c)
		{
			case 0x20 :
			case 0x9 :
			case 0xD :
			case 0xA :
				return true;
			default :
				return false;
		}
	}
}
