/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.util;


import java.util.ArrayList;

/** This class is a stack that never release object when popping
 * and handles object allocation or recycling when pushing.
 *
 */

public class RecyclingStack extends ArrayList
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
    private int stackSize = 0;
    private StackObjectFactory factory;
    
    public RecyclingStack(StackObjectFactory factory)
    {
        this.factory = factory;
    }
    
    public StackObject push()
    {
        StackObject o;
        if (stackSize < super.size())
        {
            o = (StackObject)get(stackSize);
            o.clear();
        }
        else
        {
            o = factory.newStackObject();
            add(o);
        }
        stackSize++;
        return o;
    }
    
    public int size()
    {
        return stackSize;
    }
    
    public StackObject pop()
    {
        StackObject o = null;
        if (stackSize > 0)
            o = (StackObject)get(--stackSize);
        // clear() is performed when resued
        return o;
    }
    
    public StackObject top()
    {
        if (stackSize > 0)
            return (StackObject)get(stackSize-1);
        else
            return null;
    }
    
    public boolean isEmpty()
    {
        return (stackSize == 0);
    }
    
    
    public String toString()
    {
        StringBuffer dummy = new StringBuffer();
        for(int i=0; i<size(); i++)
            dummy.append("\n"+get(i));
        return dummy.toString();
    }
    
    public void clear()
    {
        for (int i = 0; i < stackSize; i++)
        {
            ((StackObject)get(i)).clear();
        }
        stackSize = 0;
    }
    
    public interface StackObject
    {
        void clear();
    }
    
    public interface StackObjectFactory
    {
        StackObject newStackObject();
    }
    
}
