/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * RepositoryProperties.java
 *
 * Created on 6 novembre 2000, 11:33
 */

package org.xquark.mapper.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.xquark.mapper.AccessManager;
import org.xquark.mapper.metadata.RepositoryConstants;

/** This class manages repository configuration information.
 *
 * <p>The property file is intended to be found in the CLASSPATH, either
 * at the root or in org.xquark.mapper package. Its name is
 * </p>
 */
public class RepositoryProperties implements RepositoryConstants
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private static Properties properties = null;
    
    static
    {
        load();
    }

    /* Singleton */
    private RepositoryProperties() {}

    public static String getStringProperty(String prop)
    {
        if (properties == null)
            return null;
        return properties.getProperty(prop);
    }
    public static int getIntProperty(String prop)
    {
        if (properties == null)
            return -1;
        return Integer.parseInt(properties.getProperty(prop));
    }

    public static boolean getBooleanProperty(String prop)
    {
        if (properties == null)
            return false;
        return Boolean.valueOf(properties.getProperty(prop)).booleanValue();
    }

    public static void load()
    {
        Properties defaultProperties = null;
        if (properties != null)
            return;
        try {
            defaultProperties = new Properties();
            defaultProperties.load(AccessManager.class.getResourceAsStream(
                RESOURCES_FOLDER + DEFAULT_CONF_FILE));
            properties = new Properties(defaultProperties);
        }
        catch (IOException e) {
            throw new RuntimeException("Default configuration file couldn't be opened:" + e.getMessage());
        }
        try {
            InputStream inStream = AccessManager.class.getResourceAsStream("/" + CONF_FILE);   // look first in CLASSPATH root
            if (inStream == null)   // look then in xquark package
                inStream = AccessManager.class.getResourceAsStream(CONF_PACKAGE + CONF_FILE);   // SR 09/01/2001 GetSystemResource -> getResource
            if (inStream == null)   // look eventually in the AccessManager package
                inStream = AccessManager.class.getResourceAsStream(RESOURCES_FOLDER + DEFAULT_CONF_FILE);
            if (inStream == null)
                properties = defaultProperties;
            else
                properties.load(inStream);
        }
        catch (IOException e) {
            throw new RuntimeException("Default configuration file couldn't be opened:" + e.getMessage());
        }
    }
}
