/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapper.util;

import java.io.PrintWriter;
import java.io.Writer;

/**
 *
 */
public class Tabulator
{
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

    private PrintWriter out;
    private int[] offsets;
    private int lineLen;
    private StringBuffer buf;
    private int itemCount = 0;
    
    /** Creates a new instance of Tabulator */
    public Tabulator(Writer out, int lineLen, int[] offsets)
    {
        this.out = new PrintWriter(out);
        this.offsets = offsets;
        this.lineLen = lineLen;
        buf = new StringBuffer(lineLen);
    }
    
    public void addItem(int n)
    {
        addItem(new Integer(n));
    }
    
    public void addItem(Object item)
    {
        fillWithBlank(offsets[itemCount]);
            
        if (item != null)
        {
            String s = item.toString();
            int pad = 0;
            if (itemCount == (offsets.length - 1))
                pad = lineLen - offsets[itemCount - 1];
            else
                pad = offsets[itemCount + 1] - offsets[itemCount];
            if (s.length() > pad)
            {
                buf.append(s.substring(0, pad - 3));
                buf.append("...");
            }
            else
                buf.append(s);
        }
        
        itemCount++;
        
        // new line
        if (itemCount == offsets.length)
            flush();
    }
    
    public void flush()
    {
//        String value = buf.toString();
        out.print(buf.toString());
        newLine();
        buf.setLength(0);
        itemCount = 0;
    }
    
    public void newLine()
    {
        out.println();
        out.flush();
    }
    
    private void fillWithBlank(int offset)
    {
        for(int i = buf.length(); i < offset; i++)
            buf.append(' ');
    }
}
