/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapping;

import java.util.Properties;

import org.xquark.jdbc.typing.DBMSConstants;

/**
 * Provide access to the mapping compilation context allowing to use the same
 * generator code for multiple use. Some properties directly provided by the
 * mapping engine are listed as constants in the current class. Custom parameters
 * supplied by user in the mapping file will be provided as additional properties.
 * A user generator that intends to use this compilation context must provide a
 * constructor with this class as a unique parameter.
 * @see org.xquark.mapping.KeyGenerator for an example of use.
 */
public class CompilationContext extends Properties implements DBMSConstants {

	/**
	 * This property contains the name of the table where the generator value
	 * will be stored. 
	 */
	public static final String TABLE_NAME = "table";

	/**
	 * This property contains the name of the column where the generator value
	 * will be stored. 
	 */
	public static final String COLUMN_NAME = "column";
	
	/**
	 * Returns one of the following constants: {@link #ORACLE}, {@link #MYSQL},
	 * {@link #SQL_SERVER}, {@link #SYBASE}. This gives the database vendor Bridge
	 * is connected to and can be use to swicth between specific SQL syntaxes.
	 */
	public static final String DBMS_TYPE = "dbms";
	
	/**
	 * If "true", a SQL or DDL JDBC statement should use double quotes to surround
	 * table, column names and others DDL objects to avoid unwanted case automatic
	 * transformation. 
	 */
	public static final String USE_QUOTES_4_DDL = "use_quotes";
}

