/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.mapping;

/** Object that provide the value to store in a column
 *
 */
public interface Generator
{
    /**
     * This method is called by the XQuark storage engine when a user-generated
     * column of a relational table is filled. The Object type returned must be
     * consistent with the type returned by the XQuark schema API as defined in
     * the following table:
     * <table>
     * <tr><th>XML schema base built-in primitive type</th><th>Java class</th></tr>
     * <tr><td>string</td><td>{@link String}</td></tr>
     * <tr><td>boolean</td><td>{@link Boolean}</td></tr>
     * <tr><td>decimal</td><td>{@link java.math.BigDecimal} or {@link Long} if type facet ensure it fits.</td></tr>
     * <tr><td>float</td><td>{@link Float}</td></tr>
     * <tr><td>double</td><td>{@link Double}</td></tr>
     * <tr><td>QName</td><td>{@link org.xquark.schema.datatypes.QName}</td></tr>
     * <tr><td>NOTATION</td><td>{@link org.xquark.schema.datatypes.QName}</td></tr>
     * <tr><td>duration</td><td>{@link org.xquark.schema.datatypes.Duration}</td></tr>
     * <tr><td>dateTime</td><td>{@link org.xquark.schema.datatypes.DateTime}</td></tr>
     * <tr><td>time</td><td>{@link org.xquark.schema.datatypes.DateTime}</td></tr>
     * <tr><td>date</td><td>{@link org.xquark.schema.datatypes.DateTime}</td></tr>
     * <tr><td>gYearMonth</td><td>{@link org.xquark.schema.datatypes.DateTime}</td></tr>
     * <tr><td>gYear</td><td>{@link org.xquark.schema.datatypes.DateTime}</td></tr>
     * <tr><td>gMonthDay</td><td>{@link org.xquark.schema.datatypes.DateTime}</td></tr>
     * <tr><td>gDay</td><td>{@link org.xquark.schema.datatypes.DateTime}</td></tr>
     * <tr><td>gMonth</td><td>{@link org.xquark.schema.datatypes.DateTime}</td></tr>
     * <tr><td>hexBinary</td><td>{@link org.xquark.schema.datatypes.ByteArray}</td></tr>
     * <tr><td>base64Binary</td><td>{@link org.xquark.schema.datatypes.ByteArray}</td></tr>
     * <tr><td>anyURI</td><td>{@link org.xquark.schema.datatypes.URI}</td></tr>
     * <tr><td>union</td><td>not supported</td></tr>
     * <tr><td>list</td><td>{@link java.util.List}</td></tr>
     * <tr><td>anySimpleType</td><td>{@link String}</td></tr>
     * </table>
     * @param context Passed by the mapping algorithm to provide an access
     * to storage system variables.
     * @return An Object, that must be consistent with the type returned by the XQuark
     * schema API.
     */
    Object getValue(StorageContext context);
}
