/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * DefaultSchemaLocator.java
 *
 * Created on 14 dcembre 2001, 15:36
 */

package org.xquark.schema;

import java.net.URL;
import java.util.*;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

public class DefaultSchemaLocator implements SchemaLocator {
    private static final String RCSRevision = "$Revision: 1.2 $"; 
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    private static SAXParserFactory factory = null;

    static {
	try {
	    factory = SAXParserFactory.newInstance();
	    factory.setNamespaceAware(true);
	    factory.setValidating(false);
	}
	catch ( Exception ee ) {
	    factory = null;
	}
    }
  
    private HashMap schemaLocations = new HashMap();

    public XMLReader getSchemaReader() throws SAXException
    {
	if ( factory == null ) return null;
	try {
	    SAXParser parser = factory.newSAXParser();
	    return parser.getXMLReader();
	} catch (ParserConfigurationException e) {
	    e.fillInStackTrace();
	    throw new SAXException(e);
	}
    }
  
    public InputSource resolveLocation(String namespace, String location) 
    {
	return null;
    }
  
    public InputSource resolveLocation(String namespace, URL location)
    {
	try {
	    InputSource source = new InputSource(location.openStream());
	    source.setSystemId(location.toString());
	    return source;
	} catch (java.io.IOException ex) {
	    return null;
	}
    }

    public String resolveInclude(String location) {
        return location;
    }
 
    public void clearSchemaLocations() {
	schemaLocations.clear();
    }
    
    public Iterator getSchemaLocations(String namespace) {
	List locs = (List)schemaLocations.get(namespace);
	if (locs != null) return locs.iterator();
	else return null;
    }
    
    public void addSchemaLocation(String uri, String location) {
	List list = (List)schemaLocations.get(uri);
	if ( list == null ) {
	    list = new ArrayList();
	    schemaLocations.put(uri, list);
	}
	list.add(location);
    }
    
}
