/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema;

/**
 * class for Model Group Definition.
 *
 * <p>This is the implementation of the Model Group Definition.
 * The following properties are defined:</p>
 * <p> 1. name : optional, null or a NCName, defined in SchemaComponent.</p>
 * <p> 2. target namespace : null or a namespace name, defined in SchemaComponent.</p>
 * <p> 3. model group : a list of model group. </p>
 *
 * <p>See XML Schema Part 1: Structures 3.7
 * W3C Recommendation 2 May 2001</p>
 *
 * @see org.xquark.xml.schema.SchemaComponent
 * @see org.xquark.xml.schema.ModelGroup
 */
public final class ModelGroupDefinition extends SchemaComponent
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
  
  private ModelGroup group = null;

  /**
   * Create a new ModelGroupDefinition.
   *
   * @param schema The schema which contains this model group declaration.
   * @param name The model group name, a NCName.
   */
  public ModelGroupDefinition(Schema schema, String name) {
    super(schema, name);
  }

  public void accept(SchemaVisitor visitor) throws SchemaException {
    visitor.visit(this);
  }

  /**
   * Look up the model group
   *
   * @return The Model Group.
   * @see org.xquark.xml.schema.ModelGroup
   */
  public ModelGroup getModelGroup() {
    return group;
  }

  public void setModelGroup(ModelGroup group) {
    this.group = group;
  }
}
