/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema;

public class XPathExpr {
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
  private java.util.ArrayList fields = new java.util.ArrayList(4);
  private boolean anyLevel = false;
  private boolean isAttribute = false;

  public XPathExpr(boolean anyLevel) {
    this.anyLevel = anyLevel;
  }

  public void addStep(String namespaceURI, String localName) {
    fields.add(new XPathExpr.Step(namespaceURI, localName));
  }

  public void setAttribute(boolean isAttribute) {
    this.isAttribute = isAttribute;
  }

  public int getStepCount() {
    return fields.size();
  }

  public boolean matches(String namespaceURI, String localName, int index, boolean attr) {
    if (index >= fields.size()) return false;
    if (isAttribute && index == fields.size()-1 && !attr) return false;
    if (attr && (!isAttribute || index != fields.size()-1)) return false;
    XPathExpr.Step step = (XPathExpr.Step)fields.get(index);
    return step.matches(namespaceURI, localName);
  }

  public boolean isAnyLevel() {
    return anyLevel;
  }

  static class Step {
	private static final String RCSRevision = "$Revision: 1.2 $"; 
	private static final String RCSName     = "$Name: XQ1_1_1 $";
    String namespaceURI;
    String localName;
    boolean isAny = false;

    Step(String namespaceURI, String localName) {
      this.namespaceURI = namespaceURI;
      this.localName = localName;
      if ("*".equals(localName)) {
        this.localName = null;
        if (namespaceURI == null)
          isAny = true;
      }
    }

    boolean matches(String ns, String name) {
      if (isAny)
	return true;
      else if (localName == null) 
	return namespaceURI.equals(ns);
      else 
	return localName.equals(name) 
	  && ((namespaceURI == null && ns == null) || (namespaceURI != null && namespaceURI.equals(ns)));
    }
  }
}





















