/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.datatypes;

import java.text.SimpleDateFormat;

import org.apache.xerces.impl.xpath.regex.RegularExpression;


class GYearMonthType extends AbstractDateTimeType {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName = "$Name: XQ1_1_1 $";

	private SimpleDateFormat format;
	private RegularExpression regex;

	GYearMonthType() {
		super("gYearMonth", PrimitiveType.GYEAR_MONTH);
		format = new SimpleDateFormat("yyyy-MM");
		format.setLenient(false);
		regex = new RegularExpression("(-)?([0-9]{4,}\\-[0-9]{2})"+TIMEZONE_PATTERN, "X");
	}

	protected SimpleDateFormat getDateTimeFormat() {
		return format;
	}

	protected RegularExpression getRegularExpression() {
		return regex;
	}
	
	protected boolean isTimeFormat() {
		return false;
	}

	protected boolean isDateFormat() {
		return true;
	}
	
	protected DateTime createDateTime(long millis, long tz, boolean tzSpecified) {
		return new Value(millis, tz, tzSpecified);
	}
	
	class Value extends DateTime {

		public Value(long millis, long tz, boolean tzSpecified) {
			super(millis, tz, tzSpecified);
		 }
		
		protected boolean checkClass(Object obj) {
			return obj instanceof Value;
		}
		
		public String toString() {
			return stringValue(this);
		}
	}
}
