/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.datatypes;

import org.xquark.schema.SchemaException;
import org.xquark.schema.validation.ValidationContextProvider;


abstract class MesureableType extends EnumerableType {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName = "$Name: XQ1_1_1 $";

    private int length = -1;
    private int minLength = -1;
    private int maxLength = -1;

    MesureableType(String name, int type) {
        super(name, type);
    }

    protected void setLength(String value, boolean fixed) throws SchemaException {
        int len;
        try {
            len = Integer.parseInt(value.trim());
        } catch (Exception nfe) {
            //      "illegal length value in " + this + " : " + value;
            throw new SchemaException("unknown-error-code.2");
        }

        if (length != -1 && length != len) {
            //      "length = " + value + " in " + this + ", but his parent's length = " + length;
            throw new SchemaException("length-valid-restriction");
        }

        if (minLength != -1 || maxLength != -1) {
            throw new SchemaException("length-minLength-maxLength");
        }

        length = len;
        super.setFixed(PrimitiveType.LENGTH_FACET, fixed);
    }

    protected void setMinLength(String value, boolean fixed) throws SchemaException {
        int len;
        try {
            len = Integer.parseInt(value.trim());
        } catch (Exception nfe) {
            //      "illegal facet value";
            throw new SchemaException("unknown-error-code.2");
        }

        if ((minLength != -1 && minLength > len) || (super.isFixed(PrimitiveType.MIN_LENGTH_FACET) && minLength != len)) {
            throw new SchemaException("minLength-valid-restriction");
        }

        if (length != -1) {
            throw new SchemaException("length-minLength-maxLength");
        }

        if (maxLength != -1 && len > maxLength) {
            throw new SchemaException("minLength-less-than-equal-to-maxLength");
        }

        minLength = len;
        super.setFixed(PrimitiveType.MIN_LENGTH_FACET, fixed);
    }

    protected void setMaxLength(String value, boolean fixed) throws SchemaException {
        int len;
        try {
            len = Integer.parseInt(value.trim());
        } catch (Exception nfe) {
            //     "illegal facet value";
            throw new SchemaException("unknown-error-code.2");
        }

        if ((maxLength != -1 && maxLength < len) || (super.isFixed(PrimitiveType.MAX_LENGTH_FACET) && maxLength != len)) {
            //      "maxLength = " + value + " in " + this + ", but his parent's maxLength = " + maxLength;
            throw new SchemaException("maxLength-valid-restriction");
        }

        if (length != -1) {
            throw new SchemaException("length-minLength-maxLength");
        }

        if (minLength != -1 && len < minLength) {
            throw new SchemaException("minLength-less-than-equal-to-maxLength");
        }

        maxLength = len;
        super.setFixed(PrimitiveType.MAX_LENGTH_FACET, fixed);
    }

    protected void checkLength(int len, Object valueSpace) throws SchemaException {
        if (length != -1 && len != length)
            super.invalidFacet("cvc-length-valid", new Integer(length), valueSpace.toString());

        if (minLength != -1 && len < minLength)
            super.invalidFacet("cvc-minLength-valid", new Integer(minLength), valueSpace.toString());

        if (maxLength != -1 && len > maxLength)
            super.invalidFacet("cvc-maxLength-valid", new Integer(maxLength), valueSpace.toString());
    }

    protected Object toValueSpace(String value, ValidationContextProvider context) throws SchemaException {
        return value.toString();
    }

    public int getLength() {
        return length;
    }
    public int getMinLength() {
        return minLength;
    }
    public int getMaxLength() {
        return maxLength;
    }
}
