/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.datatypes;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.xquark.schema.SchemaException;
import org.xquark.schema.validation.ValidationContextProvider;


class PrimitiveListType extends MesureableType {
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName = "$Name: XQ1_1_1 $";

    PrimitiveType itemPrimitive;

    PrimitiveListType(PrimitiveType itemPrimitive) {
        super("list of " + itemPrimitive.getName(), PrimitiveType.LIST);
        this.itemPrimitive = itemPrimitive;
    }

	protected Object toValidType(Object data) {
		return (List) data;
	}

    private ArrayList checkPrimitiveList(String value, ValidationContextProvider context) throws SchemaException {
        StringTokenizer tok = new StringTokenizer(value);
        ArrayList result = new ArrayList();
        while (tok.hasMoreTokens()) {
            String token = tok.nextToken();
            try {
                Object actual = itemPrimitive.convert(token, context);
                result.add(actual);
            } catch (SchemaException se) {
                throw new SchemaException("cvc-datatype-valid.1.2.2", this, se);
            }
        }
        return result;
    }

    public void checkFacets(Object valueSpace) throws SchemaException {
        super.checkFacets(valueSpace);
        super.checkLength(((ArrayList)valueSpace).size(), valueSpace);
    }

    protected Object toValueSpace(String value, ValidationContextProvider context) throws SchemaException {
        return checkPrimitiveList(value, context);
    }

    public String toXMLString(Object data, ValidationContextProvider context) {
        if (data == null) return null;
        List list = (List)data;
        StringBuffer buf = new StringBuffer();
       	for (int i = 0; i < list.size(); i++) {
       		buf.append(itemPrimitive.toXMLString(list.get(i), context));
       		buf.append(' ');
       	}
       	int buflen = buf.length();
       	if (buflen > 0) buf.setLength(buflen-1);
       	return buf.toString();
    }

}
