/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.datatypes;


public class QName {
	
	private String namespaceURI;
	private String localName;
	
	public QName(String local) {
		this(null, local);
	}
	
	public QName(String uri, String local) {
		namespaceURI = uri;
		localName = local;
	}
	
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!checkClass(obj)) return false;
		QName qname = (QName)obj;
		return (namespaceURI == null ? qname.namespaceURI == null : namespaceURI.equals(qname.namespaceURI))
		       && localName.equals(qname.localName);
	}
	
	protected boolean checkClass(Object obj) {
		return obj instanceof QName;
	}
	
 	public int hashCode() {
		return (namespaceURI == null ? 0 : namespaceURI.hashCode()) ^ localName.hashCode();
	}
	
	public String getNamespaceURI() {
		return namespaceURI;
	}
	
	public String getLocalName() {
		return localName;
	}
	
	public String toString() {
		if (namespaceURI == null) return localName;
		else return "{"+namespaceURI+"}"+localName;
	}
}
