/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.loader;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xquark.schema.*;
import org.xquark.util.DefaultElementHandler;
import org.xquark.util.ElementHandler;

class AttributeTypeHandler extends DefaultElementHandler implements SchemaConstants
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
	private Loader loader = null;
  private AttributeDeclaration decl;

	AttributeTypeHandler(Loader loader, AttributeDeclaration decl) {
		this.loader = loader;
		this.decl = decl;
	}
    
  // schema 2001-05
  //   May 17,2001
	public ElementHandler startElement(String namespaceURI, String localName, Attributes atts) 
	throws SAXException 
	{
		if ( namespaceURI.equals(XMLSCHEMA_URI) ) {
        if (localName.equals(SIMPLE_TYPE_TAG)) {
          if ( decl instanceof AttributeDeclarationRef ) {
            String errCode = "src-attribute.3.2";
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, new SchemaException(errCode));
          }
          if (decl.getType() != null) {
            String errCode = "src-attribute.4";
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, new SchemaException(errCode));
          }
          SimpleType type = null;
          try {
            type = loader.buildSimpleType(atts);
          } 
          catch ( SchemaException se ) {
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, se);
          }
          decl.setType(type);
          return new SimpleTypeHandler(loader, type, false, false);  
        } 
        
        else if (localName.equals(ANNOTATION_TAG)) {
          return new AnnotationHandler();
        }
		}
		loader.notifyUnknownElement(namespaceURI, localName);
		return this;
	}
} 
	
