/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.loader;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xquark.schema.*;
import org.xquark.util.DefaultElementHandler;
import org.xquark.util.ElementHandler;

class ContentModelHandler extends DefaultElementHandler implements SchemaConstants
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
	private Loader loader = null;
	private ModelGroup group = null;
	private SchemaScope scope = null;
	private ModelGroupDefinition groupDef = null;
  private boolean redefined = false;

	ContentModelHandler(Loader loader, ModelGroupDefinition def, boolean redefined) {
		this.loader = loader;
		groupDef = def;
		this.redefined = redefined;
    if ( redefined ) {
      group = new SequenceModelGroup();
      def.setModelGroup(group);
    }
	}

	ContentModelHandler(Loader loader, ModelGroup group, SchemaScope scope) {
		this.loader = loader;
		this.group = group;
		this.scope = scope;
	}
    
	public ElementHandler startElement(String namespaceURI, String localName, Attributes atts) 
	throws SAXException 
	{
	  if ( namespaceURI.equals(XMLSCHEMA_URI) ) {
        // the first part : redefine only
        if ( redefined && localName.equals(GROUP_TAG)) {
          // check self reference
          String refName = atts.getValue("", REF_ATTR);
          // refName == null is verificated by Schema of Schemas
          if ( refName.equals(groupDef.getName()) ) {
            // self reference
            // check minOccurs, maxOccurs
            String minOccurs =  atts.getValue("", MIN_OCCURS_ATTR);
            String maxOccurs =  atts.getValue("", MAX_OCCURS_ATTR);
            if ( ( minOccurs != null && !"1".equals(minOccurs.trim()) ) ||
                 ( maxOccurs != null && !"1".equals(maxOccurs.trim()) ) ) {
              String errCode = "src-redefine.6.1.2";
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, new SchemaException(errCode));
            }
            String redefinedName = refName + ".redefine";
            ModelGroupDefinition redefinedGroup = loader.getSchema().getModelGroupDefinition(redefinedName);
            Particle result = new Particle(redefinedGroup);
            result.setMinOccurs(1);
            result.setMaxOccurs(1);
            group.add(result);
            return this;
          }
        }      

        Particle particle = null;
        try {
          particle = loader.buildParticle(localName, scope, atts);
        } 
        catch ( SchemaException se ) {
          String errMsg = "Error while processing localName -> " + localName; 
          loader.reportLoadingError(errMsg, se);
        }
        if (particle != null) {   
          if (group != null) group.add(particle);
          Object term = particle.getTerm();
          if (term instanceof ModelGroup) {
            if (group == null) {
              group = (ModelGroup)term;
              groupDef.setModelGroup(group);
            }
            return new ContentModelHandler(loader, (ModelGroup)term, scope);
          } 
          else if (term instanceof ElementDeclaration) {
            return new ElementTypeHandler(loader, (ElementDeclaration)term);
          } 
          else {
            return this;
          }
        } 

        else if (localName.equals(ANNOTATION_TAG)) {
          return new AnnotationHandler();
        }
        else
          // modefied by ZL, 26,septembre 2001
          // particle can be null in the case of minOccurs=maxOccurs=0
          return this;
	  }
	  
	  loader.notifyUnknownElement(namespaceURI, localName);
	  return this;
	}
}
