/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.loader;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xquark.schema.*;
import org.xquark.util.DefaultElementHandler;
import org.xquark.util.ElementHandler;

class RedefineHandler extends DefaultElementHandler implements SchemaConstants
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
	private Loader loader = null;

	RedefineHandler(Loader loader) {
		this.loader = loader;
	}
  
	public ElementHandler startElement(String namespaceURI, String localName, Attributes atts) 
	throws SAXException 
	{
		if ( namespaceURI.equals(XMLSCHEMA_URI) ) {
        String redefinedName = atts.getValue("", NAME_ATTR);
        // redefineName is verified by Schema of Schemas

        // case 1, 2  : SimpleType, ComplexType
        if ( localName.equals(SIMPLE_TYPE_TAG) || localName.equals(COMPLEX_TYPE_TAG) ) {
          // find Type whose name is typeName
          Type oldType = loader.getSchema().getType(redefinedName);
          if ( oldType == null ) {
            String errCode = "src-redefine.4.2";
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, new SchemaException(errCode));
          }
          oldType.setName(redefinedName + ".redefine");

          // construct new type
          if ( localName.equals(SIMPLE_TYPE_TAG) ) {
            SimpleType newType = null;
            try {
              newType = loader.buildSimpleType(atts);
              newType.setBaseType(oldType);
              loader.getSchema().removeRegisteredComponent(newType);
              loader.getSchema().register(newType);
            } 
            catch ( SchemaException se ) {
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, se);
            }
            return new SimpleTypeHandler(loader, newType, false, true);
          }
          else {
            ComplexType newType = null;
            try {
              newType = loader.buildComplexType(atts);
              newType.setBaseType(oldType);
              loader.getSchema().removeRegisteredComponent(newType);
              loader.getSchema().register(newType);
            } 
            catch ( SchemaException se ) {
              String errMsg = "Error while processing localName -> " + localName; 
              loader.reportLoadingError(errMsg, se);
            }
            return new ComplexTypeHandler(loader, newType, false, true);
          }
        }

        // case 3 : attribute group
        else if ( localName.equals(ATTRIBUTE_GROUP_TAG) ) {
          // find Type whose name is typeName
          AttributeGroupDefinition oldAttGroup = loader.getSchema().getAttributeGroupDefinition(redefinedName);
          if ( oldAttGroup == null ) {
            String errCode = "src-redefine.4.2";
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, new SchemaException(errCode));
          }
          oldAttGroup.setName(redefinedName + ".redefine");

          // construct new AttributeGroupDefinition
          AttributeGroupDefinition def = loader.buildAttributeGroupDefinition(atts, loader.getSchema());
          loader.getSchema().removeRegisteredComponent(def);
          try {
            loader.getSchema().register(def);
          } catch ( SchemaException se ) {
            // it's vide
          }
          return new AttributeGroupHandler(loader, def, oldAttGroup);
        } 

        // case 4 : group
        else	if ( localName.equals(GROUP_TAG) ) {
          ModelGroupDefinition oldGroup = loader.getSchema().getModelGroupDefinition(redefinedName);
          if ( oldGroup == null ) {
            String errCode = "src-redefine.4.2";
            String errMsg = "Error while processing localName -> " + localName; 
            loader.reportLoadingError(errMsg, new SchemaException(errCode));
          }
          oldGroup.setName(redefinedName + ".redefine");
          try {
            loader.getSchema().register(oldGroup);
          } catch ( SchemaException se ) {
            // it's vide
          }

          ModelGroupDefinition def = loader.buildModelGroupDefinition(atts, loader.getSchema());
          loader.getSchema().removeRegisteredComponent(def);
          try {
            loader.getSchema().register(def);
          } catch ( SchemaException se ) {
            // it's vide
          }
          return new ContentModelHandler(loader, def, true);
        }

        else if (localName.equals(ANNOTATION_TAG)) {
          return new AnnotationHandler();
        }
		}
		loader.notifyUnknownElement(namespaceURI, localName);
		return this;
	}
}   
	
