/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.validation;

import java.util.List;

import org.xquark.schema.ComplexType;
import org.xquark.schema.ElementDeclaration;
import org.xquark.schema.SchemaManager;

public class DefaultContentIterator extends ContentIterator {
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
  
  private SchemaManager manager = null;
  private ComplexType type = null;
  private ElementDeclaration matchedDecl = null;
  private int processContents;  
  
  public DefaultContentIterator(SchemaManager manager) {
    this(manager, LAX);
  }

  public DefaultContentIterator(SchemaManager manager, int processContents) {
    super(0, Integer.MAX_VALUE);
    this.manager = manager;
    this.processContents = processContents;
  }

  public DefaultContentIterator(ComplexType type) {
    super(0, Integer.MAX_VALUE);
    processContents = LAX;
    this.type = type;
  }

  public ElementDeclaration getMatchedDeclaration() {
    return matchedDecl;
  }

  public int getProcessContents() {
    return processContents;
  }
  
  private SchemaManager getManager() {
    if (type != null) return type.getManager();
    else return manager;
  }
  
  protected int nextElement(String namespace, String localName) {
    if (processContents == SKIP) return UNKNOWN;
    
    if (namespace != null && namespace.length() == 0) namespace = null;

    if (namespace == null && type != null) {
      matchedDecl = type.getElementDeclaration(null, localName);
    } else {
      matchedDecl = getManager().getElementDeclaration(namespace, localName);
    }
    if (matchedDecl != null)
      return MATCHED;
    else 
      return UNKNOWN;
  }

  public int stopProcessing() {
    return DONE;
  }
  
  protected boolean nextValidElements(List elements) {
    return true;
  }
  
}

