/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.schema.validation;

import org.xquark.schema.ElementDeclaration;
import org.xquark.schema.Particle;
import org.xquark.schema.SchemaConstants;

public class ElementContentIterator extends ContentIterator implements SchemaConstants {
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";

  private ElementDeclaration decl;
  private ElementDeclaration matchedDecl = null;
  private int count = 0;
  
  // local variable  
  public ElementContentIterator(ElementDeclaration decl, int min, int max)
  {
    super(min, max);
    this.decl = decl;
  }

  public ElementContentIterator(Particle particle) 
  {
    super(particle);
    decl = (ElementDeclaration)particle.getTerm();
  }
  
  public ElementDeclaration getMatchedDeclaration() {
    return matchedDecl;
  }

  public ElementDeclaration getModelDeclaration() {
    return decl;
  }
  
  public int getProcessContents() {
    return STRICT;
  }

  protected int nextElement(String namespace, String localName)
  {
    if (decl.hasName(namespace, localName)) {
      matchedDecl = decl;
      return checkMatchedDeclaration();
    } else {
      ElementDeclaration substitution = decl.getSubstitutionElement(namespace, localName);
      if (substitution != null) {
	matchedDecl = substitution;
	return checkMatchedDeclaration();
      } else if (count == 0) {
        return invalidElement("cvc-particle.2.3", parentParticle);
      } else {
	return stopProcessing();
      }
    }
  }

  private int checkMatchedDeclaration() {
    if (count == maxOccurs) {
      invalidElement("cvc-particle.2.2", parentParticle);
      return TOO_MANY;
    } else {
      if (reporter != null)
	reporter.matched(parentParticle, count);
      count++;
      return MATCHED;
    }
  }
  
  public int stopProcessing() {
    if (count < minOccurs)
      return invalidElement("cvc-particle.2.1", parentParticle);
    else if (reporter != null && parentParticle != null && count < maxOccurs)
      reporter.skipped(parentParticle);
    return DONE;
  }

  protected boolean nextValidElements(java.util.List elements) {
    if (count < maxOccurs && elements != null) {
      if (!decl.isAbstract()) elements.add(decl);
      decl.fillAllConcreteSubstitutionElements(elements);
    }
    if (count >= minOccurs) return true;
    else return false;
  }

}


