/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * IdentityConstraintPSVI.java
 *
 * Created on 1 juin 2001, 14:16
 */

package org.xquark.schema.validation;

import java.util.*;

import org.xquark.schema.IdentityConstraint;
import org.xquark.schema.SchemaException;

public class IdentityConstraintBinding extends Object {
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";

  static final String LOCAL = "localEntry";
  static final String PROPAGATED = "propagatedEntry";
  static final String INVALID = "invalidEntry";
  
  IdentityConstraint identityConstraint = null;
  HashMap nodeTable = null;
  List pendingRefs = null;
  IdentityConstraintBinding reference = null;
  
  /** Creates new IdentityConstraintPSVI */
  public IdentityConstraintBinding(IdentityConstraint identityConstraint) {
    this.identityConstraint = identityConstraint;
  }
  
  public IdentityConstraint getDefinition() {
    return identityConstraint;
  }
  
  public Map getNodeTable() { return nodeTable; }
  
  public IdentityConstraintBinding getReference() {
    return reference;
  }
  
  void setReference(IdentityConstraintBinding reference) {
    this.reference = reference;
  }
  
  void registerEntry(List keySequence) throws SchemaException {
    String prev = null;
    switch (identityConstraint.getCategory()) {
    case IdentityConstraint.CATEGORY_UNIQUE:
      if (nodeTable == null) nodeTable = new HashMap();
      prev = (String)nodeTable.put(keySequence, LOCAL);
      if (prev == LOCAL) 
      // to do: if type is binary or base64Binary, don't add keySequence
      // else add keySequence
      // noted by ZL, 08/07/2002
        throw new SchemaException("cvc-identity-constraint.4.1", identityConstraint,
                                   new SchemaException(null, "key: "+keySequence));
      break;
    case IdentityConstraint.CATEGORY_KEY:
      if (nodeTable == null) nodeTable = new HashMap();
      prev = (String)nodeTable.put(keySequence, LOCAL);
      if (prev == LOCAL) 
      // to do: if type is binary or base64Binary, don't add keySequence
      // else add keySequence
      // noted by ZL, 08/07/2002
        throw new SchemaException("cvc-identity-constraint.4.2.2", identityConstraint,
                                   new SchemaException(null, "key: "+keySequence));
      break;
    case IdentityConstraint.CATEGORY_KEYREF:
      if (pendingRefs == null) pendingRefs = new ArrayList();
      pendingRefs.add(keySequence);
      break;
    }
  }
  
  void propagateEntries(IdentityConstraintBinding binding) {
    Map table = binding.getNodeTable();
    if (table != null) {
      if (nodeTable == null) nodeTable = new HashMap();
      Iterator it = table.keySet().iterator();
      while (it.hasNext()) {
        Object key = it.next();
        String prev = (String)nodeTable.get(key);
        if (prev == null) nodeTable.put(key, PROPAGATED);
        else if (prev == PROPAGATED) nodeTable.put(key, INVALID);
      }
    }
  }
  
  void checkPendingRefs() throws SchemaException {
    if (reference != null  && pendingRefs != null ) {
      Iterator it = pendingRefs.iterator();
      Map table = reference.getNodeTable();
      StringBuffer buffer = null;
      if (table == null) {
        buffer = new StringBuffer();
        while (it.hasNext()) {
          buffer.append(" ");
          buffer.append(it.next());
        }
      } else {
        buffer = new StringBuffer();
        while (it.hasNext()) {
          List sequence = (List)it.next();
          String result = (String)table.get(sequence);
          if (result == null || result == INVALID) {
            buffer.append(" ");
            buffer.append(sequence);
          }
        }
      }
      if (buffer.length() > 0) 
        throw new SchemaException("cvc-identity-constraint.4.3", identityConstraint,
                                  new SchemaException(null, "key sequences:"+buffer));
    }
  }
}
