/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.util;

import org.xml.sax.*;
import org.xml.sax.ext.LexicalHandler;

/**
 * This class allows once you have programmed a SAX-like filter HandlerDecorator
 * (an XQuark feature) to easily produce a SAX XMLFilter sharing the same logic.
 */
public class DecoratorFilter extends DefaultXMLFilter {
	private static final String RCSRevision = "$Revision: 1.5 $"; 
	private static final String RCSName     = "$Name: XQ1_1_1 $";

  protected HandlerDecorator decorator;
  
  public DecoratorFilter(XMLReader reader, HandlerDecorator decorator) {
    super(reader);
	setHandlerDecorator(decorator);
  }

  public DecoratorFilter(HandlerDecorator decorator) {
	setHandlerDecorator(decorator);
  }
  
  private void setHandlerDecorator(HandlerDecorator decorator) {
  	this.decorator = decorator;
  	super.setContentHandler(decorator);
  }
  
  public void setContentHandler(ContentHandler handler) {
    decorator.setContentHandler(handler);
  }

  public ContentHandler getContentHandler() {
    return decorator.getContentHandler();
  }

  public void setLexicalHandler(LexicalHandler handler) {
    decorator.setLexicalHandler(handler);
  }

  public LexicalHandler getLexicalHandler() {
    return decorator.getLexicalHandler();
  }
  
  public void setErrorHandler(ErrorHandler handler) {
    decorator.setErrorHandler(handler);
  }

  public ErrorHandler getErrorHandler() {
    return decorator.getPluggedErrorHandler();
  }
  
  public void parse(InputSource source) throws SAXException, java.io.IOException {
    setupFilter();
    super.parse(source);
  }

  private void setupFilter() {
    super.setContentHandler(decorator);
    super.setLexicalHandler(decorator);
    super.setErrorHandler(decorator);
  }

}
