/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.util;

import org.xml.sax.*;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.XMLFilterImpl;

/**
 * An XMLFilter implementation which filters also LexicalHandler events
 */
public class DefaultXMLFilter extends XMLFilterImpl
  implements LexicalHandler
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
  private LexicalHandler lexicalHandler = null;

  /** Constructor
   */
  public DefaultXMLFilter() {
    super();
  }

  /** Constructor
   * @param reader The parent XMLReader that is filtered.
   */
  public DefaultXMLFilter(XMLReader reader) {
    super(reader);
  }

  //////////////////////////////////////////////////////
  //      Implementation of XMLReader interface.
  //////////////////////////////////////////////////////
  public void setProperty(String name, Object value) 
    throws SAXNotRecognizedException, SAXNotSupportedException 
  {
    if (name.equals("http://xml.org/sax/properties/lexical-handler"))
      setLexicalHandler((LexicalHandler)value);
    else
        super.setProperty(name, value);
  }
    
  public Object getProperty(String name)
    throws SAXNotRecognizedException, SAXNotSupportedException
  {
    if (name.equals("http://xml.org/sax/properties/lexical-handler"))
      return getLexicalHandler();
    else
      return super.getProperty(name);
  }

  public LexicalHandler getLexicalHandler() {
    return lexicalHandler;
  }

  public void setLexicalHandler(LexicalHandler handler) {
    try {
      if (getParent() != null)
        getParent().setProperty("http://xml.org/sax/properties/lexical-handler", this);
      lexicalHandler = handler;
    } catch (SAXException ex) {
      // Should not occur
    }
  }
  
  //////////////////////////////////////////////////////
  //      Implementation of LexicalHandler interface.
  //////////////////////////////////////////////////////
  public void startDTD(String name, String publicId,  String systemId)
    throws SAXException
  {
    lexicalHandler.startDTD(name, publicId, systemId);
  }

  public void endDTD() throws SAXException
  {
    lexicalHandler.endDTD();
  }
  
  public void startEntity(String name) throws SAXException
  {
    lexicalHandler.startEntity(name);
  }

  public void endEntity(String name) throws SAXException
  {
    lexicalHandler.endEntity(name);
  }

  public void startCDATA() throws SAXException
  {
    lexicalHandler.startCDATA();
  }
  
  public void endCDATA() throws SAXException
  {
    lexicalHandler.endCDATA();
  }
    
  public void comment(char ch[], int start, int length) throws SAXException
  {
    lexicalHandler.comment(ch, start, length);
  }
}

