/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.util;

import java.util.HashMap;

public class DefaultXMLNameFactory {
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";

  private HashMap namespaceMaps = new HashMap();
  private static DefaultXMLNameFactory instance = new DefaultXMLNameFactory();
  
  private DefaultXMLNameFactory() {}

  public static DefaultXMLNameFactory getFactory() { return instance; }
  
  public synchronized XMLName createXMLName(String namespace, String name) {
    HashMap namespaceMap = getNamespaceMap(namespace);
    XMLName result = (XMLName)namespaceMap.get(name);
    if (result == null) {
      result = new XMLNameImpl(namespace, name);
      namespaceMap.put(name, result);
    }
    return result;  
  }

  private HashMap getNamespaceMap(String namespace) {
    HashMap result = (HashMap)namespaceMaps.get(namespace);
    if (result == null) {
      result = new HashMap();
      namespaceMaps.put(namespace, result);
    }
    return result;
  }
  
  private static class XMLNameImpl implements XMLName {
	private static final String RCSRevision = "$Revision: 1.2 $"; 
	private static final String RCSName     = "$Name: XQ1_1_1 $";
    private String name;
    private String namespace;
  
    XMLNameImpl(String namespace, String name) {
      this.namespace = namespace;
      this.name = name;
    }
  
    public String getLocalName() { return name; }
  
    public String getNamespaceURI() { return namespace; }
  
    public boolean equals(Object name) {
      if (!(name instanceof XMLNameImpl)) return false;
      return equals((XMLNameImpl)name);
    }
    
    public boolean equals(XMLName name) { return this == name; }
  }
}

