/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.util;


import java.util.Collection;
import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class NamespaceContextHandler extends HandlerDecorator implements NamespaceContext
{
	private static final String RCSRevision = "$Revision: 1.4 $"; 
	private static final String RCSName     = "$Name: XQ1_1_1 $";
	
    protected NamespaceContextStack contextStack = new NamespaceContextStack();
    
	private static final byte UNKNOWN = 0;
	private static final byte START_ELEMENT = 1;
	private static final byte END_ELEMENT = 2;
	private static final byte START_PREFIX = 3;
	private static final byte END_PREFIX = 4;

	private byte status = UNKNOWN;

    /** Creates new NamespaceContextHandler */
    public NamespaceContextHandler()
    {
        super();
    }
    
    public void startPrefixMapping(String prefix, String uri)
    throws SAXException
    {
		contextStack.declarePrefix(prefix, uri);
		super.startPrefixMapping(prefix, uri);
		status = START_PREFIX;
    }
    
	public void endPrefixMapping(String prefix)
	throws SAXException
	{
		// NOTE: the case where an end prefix event follows a start prefix event
		// is due to a bug of XQuery reconstruction.
		if (status == END_ELEMENT || status == START_PREFIX)	// clean the stack of declarations of the level
		{
			contextStack.popContext();
			contextStack.pushContext();
		}
		super.endPrefixMapping(prefix);
		status = END_PREFIX;
	}
    
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException
    {
		super.startElement(namespaceURI, localName, qName, atts);
		contextStack.pushContext();
		status = START_ELEMENT;
    }
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException
    {
		contextStack.popContext();
		super.endElement(namespaceURI, localName, qName);
		status = END_ELEMENT;
    }
    
    public NamespaceContext getNamespaceContext()
    {
        return contextStack.getNamespaceContext();
    }
    
    public String getNamespaceURI(String prefix)
    {
        return contextStack.getNamespaceURI(prefix);
    }
    
    public Collection getNamespaceURIs() {
    	return contextStack.getNamespaceURIs();
    }
    
    public String getPrefix(String uri)
    {
        return contextStack.getPrefix(uri);
    }
    
    public List getDeclaredPrefixes()
    {
        return contextStack.getDeclaredPrefixes();
    }
    
    public Collection getPrefixes()
    {
        return contextStack.getPrefixes();
    }
    
    public Collection getPrefixes(String uri)
    {
        return contextStack.getPrefixes(uri);
    }
    
}
