/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xml.xdbc;

/** The interface that every data source must implement.
 * Classes implementing this interface can be instantiated
 * either directly, or through the XMLDriverManager class.
 */
public interface XMLDataSource {
 


  /** Attempts to establish a connection with the data source.
   * @throws XMLDBCException If a data source access error occurs
   * @return an XMLConnection object that represents a connection to the data source
   */  
  public XMLConnection getConnection() throws XMLDBCException;
  
  /** Attempts to establish a connection with the data source.
   * @param user the data source registered user
   * @param password the user's password
   * @throws XMLDBCException If a data source access error occurs
   * @return an XMLConnection object that represents a connection to the data source
   */  
  public XMLConnection getConnection(String user, String password) throws XMLDBCException;
  
  /** Sets the logging/tracing java.io.PrintWriter object that is used by the XDataSource object.
   * @param writer the new logging/tracing java.io.PrintWriter object, or null to disable logging and tracing
   */  
  public void setLogWriter(java.io.PrintWriter writer);
  
  /** Retrieves the log writer currently used by the XDataSource object to print messages.
   * @return the java.io.PrintWriter object used to log data source messages, or null if none is defined
   */  
  public java.io.PrintWriter getLogWriter();
  
  /** Sets the maximum time in seconds that the object will wait when attempting to log in to a data source. Setting the timeout to 0 will disable it.
   * @param seconds the login time limit in seconds
   */  
  public void setLoginTimeout(int seconds);
  
  /** Gets the maximum time in seconds that the object can wait when attempting to log in to a data source
   * @return the data source login timeout in seconds, 0 if no timeout is defined
   */  
  public int getLoginTimeout();
  
}

