/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xpath;



public final class Axis
{
    
	/**
	 * The <code>NO</code> axis contains no axis.
	 */
	public static final byte NONE = 0x00;

	/**
	 * The <code>SELF</code> axis just contains the context node itself.
	 */
	public static final byte SELF = 0x01;
    
	/**
	 * The <code>CHILD</code> axis contains the children of the context node.
	 */
	public static final byte CHILD = 0x02;
    
	/**
	 * The <code>PARENT</code> axis contains the parent node of the context node.
	 * The <code>PARENT</code> axis implies that the target node type being
	 * Type.ELEMENT.
	 */
	public static final byte PARENT = 0x03;
    
	/**
	 * The <code>ANCESTOR</code> axis contains the ancestors of the context node.
	 * The <code>ANCESTOR</code> axis implies that the target node type being
	 * Type.ELEMENT.
	 */
	public static final byte ANCESTOR = 0x04;
    
	/**
	 * The <code>ANCESTOR_OR_SELF</code> axis contains the context node and
	 *  the ancestors of the context node; thus, the ancestor axis will always
	 * include the root node.
	 */
	public static final byte ANCESTOR_OR_SELF = 0x05;
    
	/**
	 * The <code>DESCENDANT</code> axis contains the descendants of the
	 * context node; a descendant is a child or a child of a child and so on;
	 * thus the descendant axis never contains attribute or namespace nodes.
	 */
	public static final byte DESCENDANT = 0x06;
    
	/**
	 * The <code>descendant-or-self</code> axis contains the context node
	 * and the descendants of the context node.
	 */
	public static final byte DESCENDANT_OR_SELF = 0x07;
    
	/**
	 * The <code>PRECEDING</code> axis contains all nodes in the same document
	 *  as the context node that are before the context node in document order,
	 *  excluding any ancestors and excluding attribute nodes and namespace nodes.
	 */
	public static final byte PRECEDING = 0x08;
    
	/**
	 * The <code>PRECEDING_SIBLING</code> axis contains all the preceding
	 * siblings of the context node; if the context node is an attribute node
	 * or namespace node, the <code>PRECEDING_SIBLING</code> axis is empty.
	 */
	public static final byte PRECEDING_SIBLING = 0x09;
    
	/**
	 * The <code>FOLLOWING</code> axis contains all nodes in the same document
	 *  as the context node that are after the context node in document order,
	 * excluding any descendants and excluding attribute nodes and namespace
	 * nodes
	 */
	public static final byte FOLLOWING = 0x0a;
    
	/**
	 * The <code>FOLLOWING_SIBLING</code> axis contains all the following
	 * siblings of the context node; if the context node is an attribute node
	 * or namespace node, the <code>FOLLOWING_SIBLING</code> axis is empty.
	 */
	public static final byte FOLLOWING_SIBLING = 0x0b;
    
	/**
	 * The <code>ATTRIBUTE</code> axis contains the attributes of the context
	 *  node; the axis will be empty unless the context node is an element.
	 */
	public static final byte ATTRIBUTE = 0x0c;
    
	/**
	 * The <code>namespace</code> axis contains the namespace nodes of the
	 * context node; the axis will be empty unless the context node is an element.
	 */
	public static final byte NAMESPACE = 0x0d;

	/**
	 * The string values of the different axis
	 */
	static public final  String[] AXISSTRINGS = {
		"",
		"self::",
		"child::",
		"parent::",
		"ancestor::",
		"ancestor-or-self::",
		"descendant::",
		"descendant-or-self::",
		"preceding::",
		"preceding-sibling::",
		"following::",
		"following-sibling::",
		"attribute::",
		"namespace::"
	};
	
	public static String stringValue(byte axis)
	{
		switch(axis)
		{
			case SELF:
				return "self::";
 			case CHILD:
				return "child::";
			case PARENT:
				return "parent::";                
			case ANCESTOR:
				return "ancestor::";
			case ANCESTOR_OR_SELF:
				return "ancestor-or-self::";
			case DESCENDANT:
				return "descendant::";
			case DESCENDANT_OR_SELF:
				return "descendant-or-self::";
 			case PRECEDING:
				return "preceding::";                
			case PRECEDING_SIBLING:
				return "preceding-sibling::";                
			case FOLLOWING:
				return "following::";                
			case FOLLOWING_SIBLING:
				return "following-sibling::";                
			case ATTRIBUTE:
				return "attribute::";                
			case NAMESPACE:
				return "namespace::";                
			default:
				return "unknown-axis::";
		}
	}

}
