/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */


package org.xquark.xpath;


public interface StepExpr {


/**
 * The <code>NO_WILDCARD</code> wildcard contains no wildcard.
 */
static public final byte NO_WILDCARD = 0x00;

/**
 * The <code>NO_WILDCARD</code> wildcard contains a wildcard.
 */
static public final byte WILDCARD = 0x01;

/**
 * The <code>NO_WILDCARD</code> wildcard contains a wildcard in the localname part.
 */
static public final byte LOCALNAME_WILDCARD = 0x02;

/**
 * The <code>NO_WILDCARD</code> wildcard contains a wildcard in the namspace part.
 */
static public final byte NAMESPACE_WILDCARD = 0x03;

/**
 * 
 * @return true if there is a separator in front of the step
 */
public boolean hasSeparator();

/**
 * 
 * @return the constant representing the axis
 */
public byte getAxis();

/**
 * 
 * @return the namespace if any null otherwise
 */
public String getNameSpace();

/**
 * 
 * @return the localname if any null otherwise
 */
public String getLocalName();

/**
 * 
 * @return the constant representing the wilcard
 */
public byte getWildCardType();

/**
 * 
 * @return false if no wildcard is present
 */
public boolean hasWildcard();

/**
 * 
 * @return true if step if *
 */
public boolean isWildcard();

/**
 * 
 * @return true if step is qname with * as namespace
 */
public boolean isNameSpaceWildcard();

/**
 * 
 * @return true if step is qname with * as localname
 */
public boolean isLocalNameWildcard();

/**
 * 
 * @return the constant representing the kind of the step
 */
public byte getKindTest();

/**
 * 
 * @return the (schema) type used in element or attribute test
 */
public org.xquark.schema.Type getType();

/**
 * 
 * @return true if the type used in element test is nillable
 */
public boolean isTypeNillable();

// methods for path resolving

public XNode getXNode();

public boolean match(XNode node);

public byte getTypeTest();

public boolean isTypeTest();

public String toHashCodeString();

}
