/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xpath;

import org.xquark.schema.Type;

public class StepExprImpl implements StepExpr {

	/**
	 * 
	 */
	private boolean hasSeparator = false;

	/**
	 * 
	 */
	private byte axis = Axis.NONE;

	/**
	 * 
	 */
	private String localName = null;

	/**
	 * 
	 */
	private String nameSpace = null;

	/**
	 * 
	 */
	private byte wild = StepExpr.NO_WILDCARD;

	/**
	 * 
	 */
	private byte kindtest = NodeKind.NONE;

	/**
	 * 
	 */
	private Type type = null;
	/**
	 * 
	 */
	private boolean isTypeNillable = false;

	public StepExprImpl(byte axis, String nameSpace, String localName) {
		this.axis = axis;
		this.nameSpace = nameSpace;
		this.localName = localName;
	}

	/**
	 * 
	 * @return true if there is a separator in front of the step
	 */
	public boolean hasSeparator() {
		return hasSeparator;
	}

	/**
	 * 
	 * @return the constant representing the axis
	 */
	public byte getAxis() {
		return axis;
	}

	/**
	 * 
	 * @return the namespace if any null otherwise
	 */
	public String getNameSpace() {
		return nameSpace;
	}

	/**
	 * 
	 * @return the localname if any null otherwise
	 */
	public String getLocalName() {
		return localName;
	}

	/**
	 * 
	 * @return the constant representing the wilcard
	 */
	public byte getWildCardType() {
		return wild;
	}

	/**
	 * 
	 * @return false if no wildcard is present
	 */
	public boolean hasWildcard() {
		return wild != StepExpr.NO_WILDCARD;
	}

	/**
	 * 
	 * @return true if step if *
	 */
	public boolean isWildcard() {
		return wild == StepExpr.WILDCARD;
	}

	/**
	 * 
	 * @return true if step is qname with * as namespace
	 */
	public boolean isNameSpaceWildcard() {
		return wild == StepExpr.NAMESPACE_WILDCARD;
	}

	/**
	 * 
	 * @return true if step is qname with * as localname
	 */
	public boolean isLocalNameWildcard() {
		return wild == StepExpr.LOCALNAME_WILDCARD;
	}

	/**
	 * 
	 * @return the constant representing the kind of the step
	 */
	public byte getKindTest() {
		return kindtest;
	}

	/**
	 * 
	 * @return the (schema) type used in element or attribute test
	 */
	public org.xquark.schema.Type getType() {
		return type;
	}

	/**
	 * 
	 * @return true if the type used in element test is nillable
	 */
	public boolean isTypeNillable() {
		return isTypeNillable;
	}

	/**
	 * @return the string value of the step
	 */
	public String toHashCodeString() {
		return toString();
	}
	public String toString() {
		StringBuffer buf = new StringBuffer();
		if (axis != Axis.NONE)
			buf.append('/');
		buf.append(Axis.AXISSTRINGS[axis]);
		switch (kindtest) {
			case NodeKind.ATTRIBUTE :
//				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
//				buf.append('(');
				if (isWildcard())
					buf.append('*');
				else if (localName != null) {
//					buf.append('@');
					if (nameSpace != null) {
						buf.append('{');
						buf.append(nameSpace); 
						buf.append('}');
					}
					buf.append(localName);
//					if (type != null) {
//						buf.append(" ,");
//						buf.append(type.toString());
//					}
//				} else if (type != null) {
//					buf.append(type.toString());
				}
//				buf.append(')');
				break;
			case NodeKind.COMMENT :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				buf.append(')');
				break;
			case NodeKind.DOCUMENT :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				if (localName != null || type != null) {
//					buf.append(NodeKind.NODEKINDSTRINGS[NodeKind.ELEMENT]);
//					buf.append('(');
					if (isWildcard())
						buf.append('*');
					else if (localName != null) {
						if (nameSpace != null) {
							buf.append('{');
							buf.append(nameSpace); 
							buf.append('}');
						}
						buf.append(localName);
//						if (type != null) {
//							buf.append(" ,");
//							buf.append(type.toString());
//							if (isTypeNillable) {
//								buf.append(" nillable");
//							}
//						}
//					} else if (type != null) {
//						buf.append(type.toString());
					}
//					buf.append(')');
				}
				buf.append(')');
				break;
			case NodeKind.ELEMENT :
//				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
//				buf.append('(');
				if (isWildcard())
					buf.append('*');
				else if (localName != null) {
					if (nameSpace != null) {
						buf.append('{');
						buf.append(nameSpace); 
						buf.append('}');
					}
					buf.append(localName);
//					if (type != null) {
//						buf.append(" ,");
//						buf.append(type.toString());
//						if (isTypeNillable) {
//							buf.append(" nillable");
//						}
//					}
//				} else if (type != null) {
//					buf.append(type.toString());
				}
//				buf.append(')');
				break;
				//case NodeKind.NAMESPACE :
			case NodeKind.NODE :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				buf.append(')');
				break;
			case NodeKind.PI :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				if (localName != null)
					buf.append(localName);
				buf.append(')');
				break;
			case NodeKind.TEXT :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				buf.append(')');
				break;
		}
		return buf.toString();
	}

	//	methods for path resolving

	public boolean equals(Object o)
	{
		return (o instanceof StepExpr) 
			&& (nameSpace == null || nameSpace.equals(((StepExpr)o).getNameSpace())) 
			&& localName.equals(((StepExpr)o).getLocalName())
			&& kindtest ==  ((StepExpr)o).getKindTest()
//			&& type ==  ((StepExpr)o).getType()
//			&& wild ==  ((StepExpr)o).getWildCardType()
			&& (axis == ((StepExpr)o).getAxis());
	}

	public static final String NAME_WILDCARD = "*";
	public XNode xnode = null;

	public StepExprImpl(byte axis, XNode xnode) {
		this.axis = axis;
		this.xnode = xnode;
		this.localName = xnode.getLocalName();
		this.nameSpace = xnode.getNamespace();
		this.kindtest = xnode.getType();
	}

	public StepExprImpl(XNode xnode) {
		this.axis = Axis.CHILD;
		this.xnode = xnode;
		this.localName = xnode.getLocalName();
		this.nameSpace = xnode.getNamespace();
		this.kindtest = xnode.getType();
	}

	public XNode getXNode() {
		return xnode;
	}

	public boolean match(XNode node) {
		if (node == null)
			return false;
		if (isTypeTest()) { // true if local name == null
			return typeMatch(node);
		} else // namespace matching
			{
			if (typeMatch(node) == false) {
				return false;
			}
			if (getNamespace() == null) {
				if (node.getNamespace() == null) {
					// namespace match
				} else {
					return false;
				}
			} else if (isNamespaceWildcard()) {
				// namespace match
			} else if (node.getNamespace() == null) {
				return false;
			} else if (getNamespace().equals(node.getNamespace())) {
				// namespace match
			} else {
				return false;
			}

			// matching local name
			if (this.getLocalName() == null) {
				if (node.getLocalName() == null) {
					// local name match
				} else {
					return false;
				}
			} else if (this.isLocalNameWildcard()) {
				// local name match
			} else if (node.getLocalName() == null) {
				return false;
			} else if (this.getLocalName().equals(node.getLocalName())) {
				// local name match
			} else {
				return false;
			}
			return true;
		}
	}
	public boolean isTypeTest()
	{
		return xnode.getLocalName() == null;
	}
	private boolean typeMatch(XNode node)
	{
		return ((getTypeTest() == node.getType()) || (getTypeTest() == NodeKind.NODE));
	}
	public byte getTypeTest()
	{
		return xnode.getType();
	}
	public String getNamespace()
	{
		return xnode.getNamespace();
	}
	public boolean isNamespaceWildcard()
	{
		return isNameTest() && (xnode.getNamespace() != null) && xnode.getNamespace().equals(NAME_WILDCARD);
	}
	public boolean isNameTest()
	{
		return xnode.getLocalName() != null;
	}
	
}
