/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xpath;

import org.xquark.schema.Declaration;

/**
 * This class allows the construction of specific implementations of XTree
 * giving direct access to XML schema components.
 *
 * <p>Adding data to XTreeNode implementations is made possible by derivation
 * of this object.</p>
 * <p>Methods in this base class (except allocateNode) must not be overriden
 * since they are used by other base implementation classes.</p>
 *
 */
public abstract class TypedXTreeBuilder extends XTreeBuilder
{
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	/**
	 * Default constructor.
	 */
	public TypedXTreeBuilder()
	{}

	/**
	 * Constructor for adding nodes to an existing tree.
	 * @param tree an existing tree to enrich.
	 */
	public TypedXTreeBuilder(XTree tree)
	{
		super(tree);
	}

	/**
	 * Creates an instance of <code>XTreeNode</code> or of its sub-class
	 * given a declaration.
	 *
	 * @param parent the parent node of the new node
	 * @param decl the XML schema declaration that will be used to create
	 */
	public final TypedXTreeNode createTypedNode(
		XTreeNode parent,
		Declaration decl)
	{
		return (TypedXTreeNode) tree.register(allocateNode(parent, decl));
	}

	/**
	 * XTreeNode factory to implement. Implementation should create a new node
	 * derived from the base XTreeNode implementation and calling the "super"
	 * constructor that performs parent/child registration.
	 * @param parent parent node of the new node
	 * @param decl the XML schema declaration that will be used to create
	 * @return the new TypedXTreeNode (upcasted)
	 */
	public abstract XTreeNode allocateNode(XTreeNode parent, Declaration decl);
}
