/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */


package org.xquark.xpath;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 *  Simple XTree depth-first iterator.
 */
public class XTreeIterator implements Iterator
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
    private ArrayList childIterators = new ArrayList();
    private XTreeNode nextNode;
    private Iterator it;
    
    /** Creates new XTreeIterator */
    public XTreeIterator(XTree tree)
    {
        nextNode = tree.getRoot();
    }
    
    /** Creates new XTreeIterator */
    public XTreeIterator(XTreeNode iterationRoot)
    {
        nextNode = iterationRoot;
    }
    
    /**
     * Returns <tt>true</tt> if the iteration has more elements. (In other
     * words, returns <tt>true</tt> if <tt>next</tt> would return an element
     * rather than throwing an exception.)
     *
     * @return <tt>true</tt> if the iterator has more elements.
     */
    public boolean hasNext()
    {
        return nextNode != null;
    }
    
    /**
     * Returns the next element in the interation.
     *
     * @return the next element in the iteration.
     * @exception NoSuchElementException iteration has no more elements.
     */
    public Object next()
    {
        if (!hasNext())
            throw new NoSuchElementException();
        // prepare return
        XTreeNode wNode = nextNode;
        nextNode = null;
        
        // prepare next
        
        // explore
        if ((wNode.getChildren() != null) && !wNode.getChildren().isEmpty())
        {
            childIterators.add(it);
            it = wNode.getChildren().iterator();
            nextNode = (XTreeNode)it.next();
        }
        else    // going backwards
        {
            while (!childIterators.isEmpty() && (nextNode == null))
            {
                if (it.hasNext())
                    nextNode = (XTreeNode)it.next();
                else
                    it = (Iterator)childIterators.remove(childIterators.size() - 1);
            }
        }
        return wNode;
    }
    
    /**
     *
     * Removes from the underlying collection the last element returned by the
     * iterator (optional operation).  This method can be called only once per
     * call to <tt>next</tt>.  The behavior of an iterator is unspecified if
     * the underlying collection is modified while the iteration is in
     * progress in any way other than by calling this method.
     *
     * @exception UnsupportedOperationException if the <tt>remove</tt>
     * 		  operation is not supported by this Iterator.
     *
     * @exception IllegalStateException if the <tt>next</tt> method has not
     * 		  yet been called, or the <tt>remove</tt> method has already
     * 		  been called after the last call to the <tt>next</tt>
     * 		  method.
     */
    public void remove()
    {
        throw new UnsupportedOperationException();
    }
    
}
