/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xpath.schema;


import java.util.Iterator;

import org.xquark.schema.ComplexType;
import org.xquark.schema.Declaration;
import org.xquark.schema.Schema;
import org.xquark.xpath.*;

/**
 * Specific implementation of XTreeBuilder.
 *
 */
public class SchemaTreeBuilder extends TypedXTreeBuilder
{
private static final String RCSRevision = "$Revision: 1.2 $"; 
private static final String RCSName     = "$Name: XQ1_1_1 $";
    public XTreeNode allocateNode(XTreeNode parent, String namespace, String localName, byte type)
    {
        return new SchemaNode(tree, (SchemaNode)parent, namespace, localName, type);
    }
    public XTreeNode allocateNode(XTreeNode parent, Declaration decl)
    {
        return new SchemaNode(tree, (SchemaNode)parent, decl);
    }
    
    public XTree allocateTree()
    {
        return new XTree();
    }
    
    private void expand(TypedXTreeNode node)
    {
        if (node.getDeclaration().getType() instanceof ComplexType)
        {
            ComplexType type = (ComplexType)node.getDeclaration().getType();
            Declaration decl = null;
            
            // process attributes
            Iterator it = type.getAttributeDeclarations().iterator();
            while(it.hasNext())
            {
                decl = (Declaration) it.next();
                createNamedNode(node, decl.getNamespace(), decl.getName(), NodeKind.ATTRIBUTE);
            }
        
            // process elements
            it = type.getElementDeclarations().iterator();
            while(it.hasNext())
            {
                decl = (Declaration) it.next();
                expand((SchemaNode)createNamedNode(node, decl.getNamespace(), decl.getName(), NodeKind.ELEMENT));
            }
        }
    }
    public void expandSchemaTree(Schema schema)
    {
        Iterator it = schema.getElementDeclarations().iterator();
        
        while(it.hasNext())
        {   // process every root element declarations
            expand(createTypedNode(tree.getRoot(), (Declaration)it.next()));
        }
    }
}
