/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.datastructure;

import java.math.BigDecimal;
import java.sql.Date;
import java.util.ArrayList;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xquark.schema.SchemaManager;
import org.xquark.xpath.Axis;
import org.xquark.xpath.NodeKind;
import org.xquark.xpath.datamodel.TypedAttributeImpl;
import org.xquark.xpath.datamodel.TypedDocumentImpl;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xpath.datamodel.TypedValue;
import org.xquark.xquery.parser.ListOpUNIONExpression;
import org.xquark.xquery.parser.LocatedExpression;
import org.xquark.xquery.parser.NodeTest;
import org.xquark.xquery.parser.ParenthesizedExpression;
import org.xquark.xquery.parser.QName;
import org.xquark.xquery.parser.Step;
import org.xquark.xquery.parser.Value;
import org.xquark.xquery.parser.ValueDecimal;
import org.xquark.xquery.parser.ValueDouble;
import org.xquark.xquery.parser.ValueFloat;
import org.xquark.xquery.parser.ValueInteger;
import org.xquark.xquery.parser.ValueString;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;
import org.xquark.xquery.parser.primitivefunctions.xsfunctions.FunctionDATETIME;
import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.TypeException;

public class DOMUtils {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	protected static TypedDocumentImpl docimpl = null;
	static {
		docimpl = new TypedDocumentImpl();
	}

	// ***********************************************************************
	// * DOM Tools
	// ***********************************************************************
	public static boolean compareTree(ArrayList trees1, ArrayList trees2) {
		if ((trees1 == null) && (trees2 == null))
			return true;
		if ((trees1 != null) && (trees2 == null))
			return false;
		if ((trees1 == null) && (trees2 != null))
			return false;
		if (trees1.size() != trees2.size())
			return false;
		for (int i = 0; i < trees1.size(); i++)
			if (compareTree((TypedNode) trees1.get(i), (TypedNode) trees2.get(i)) == false)
				return false;
		return true;
	}

	/**
	 * Compare two trees. Considerate multivalue, disorder, attributes
	 * and all :-).
	 *
	 * @param node1 the first node to compare.
	 * @param node2 the second node to compare.
	 * @return true if the two tree are similar (even if the order is
	 *             different), false otherwise.
	 */
	public static boolean compareTree(TypedNode node1, TypedNode node2) {
		if (!node1.getNodeName().equals(node2.getNodeName()))
			return false;
		Comparable value1 = (Comparable) node1.getTypedValue();
		Comparable value2 = (Comparable) node2.getTypedValue();
		if ((value1 == null) && (value2 != null))
			return false;
		if ((value1 != null) && (value2 == null))
			return false;
		if ((value1 != null) && (value2 != null))
			if (!value1.equals(value2))
				return false;
		org.w3c.dom.NodeList nl1 = node1.getChildNodes();
		org.w3c.dom.NodeList nl2 = node2.getChildNodes();

		if (nl1.getLength() != nl2.getLength())
			return false;

		for (int i = 0; i < nl1.getLength(); i++) {
			TypedNode n1 = (TypedNode) nl1.item(i);
			String label = n1.getNodeName();
			ArrayList v = getAllNodeByName(node2, label);
			if (v.size() < 1)
				return false;
			boolean b = false;
			for (int j = 0; j < v.size(); j++) {
				TypedNode n2 = (TypedNode) v.get(j);
				b = compareTree(n1, n2);
				if (b == true)
					break;
			}
			if (b == false)
				return false;
		}
		org.w3c.dom.NamedNodeMap nnm1 = node1.getAttributes();
		org.w3c.dom.NamedNodeMap nnm2 = node2.getAttributes();
		if ((nnm1 == null) && (nnm1 == null))
			return true;
		if (nnm1 == null)
			return false;
		if (nnm2 == null)
			return false;
		if (nnm1.getLength() != nnm2.getLength())
			return false;
		for (int i = 0; i < nnm1.getLength(); i++) {
			TypedNode n1 = (TypedNode) nnm1.item(i);
			TypedNode n2 = (TypedNode) nnm2.getNamedItem(n1.getNodeName());
			if (n2 == null)
				return false;
			if (n1.getTypedValue() == null && n2.getTypedValue() == null)
				continue;
			if (n1.getTypedValue() == null || n2.getTypedValue() == null)
				return false;
			if (n1.getTypedValue() instanceof Comparable && n2.getTypedValue() instanceof Comparable) {
				if (DOMUtils.compareObjects((Comparable) n1.getTypedValue(), (Comparable) n2.getTypedValue()) != 0)
					return false;
			} else {
				if (!n1.getTypedValue().equals(n2.getTypedValue()))
					return false;
			}
		}
		return true;
	}

	/* 
	-1 if node1 is greater
	0 if equal
	1 if node2 is greater
	*/
	public static int simpleCompareNodeValue(ArrayList nodes1, ArrayList nodes2) {
		if (nodes1 == null || nodes1.isEmpty()) return -1;
		if (nodes2 == null || nodes2.isEmpty()) return 1;
		TypedNode node1 = (TypedNode)nodes1.get(0);
		TypedNode node2 = (TypedNode)nodes2.get(0);
		Comparable value1 = (Comparable) node1.getTypedValue();
		Comparable value2 = (Comparable) node2.getTypedValue();
		if (value1 == null)
			value1 = (Comparable) ((TypedNode) node1.getChildNodes().item(0)).getTypedValue();
		if (value2 == null)
			value2 = (Comparable) ((TypedNode) node2.getChildNodes().item(0)).getTypedValue();
		return compareObjects(value1, value2);
	}

	public static int compareObjects(Comparable value1, Comparable value2) {
		// TODO optimize comparison
		// long > decimal > float > double > string
		if (value1.getClass() == value2.getClass())
			return value1.compareTo(value2);

		if (value1 instanceof Double) {
			value2 = new Double(value2.toString());
		} else if (value2 instanceof Double) {
			value1 = new Double(value1.toString());
		} else if (value1 instanceof Float) {
			value2 = new Float(value2.toString());
		} else if (value2 instanceof Float) {
			value1 = new Float(value1.toString());
		} else if (value1 instanceof BigDecimal) {
			value2 = new BigDecimal(value2.toString());
		} else if (value2 instanceof BigDecimal) {
			value1 = new BigDecimal(value1.toString());
		} else if (value1 instanceof Integer) {
			value2 = new Integer(value2.toString());
		} else if (value2 instanceof Integer) {
			value1 = new Integer(value1.toString());
		}
		return value1.compareTo(value2);
	}

	/**
	 * Return the child with the specified name.
	 *
	 * @param root the root node where to find the child with the right node
	 * @param label the label of the child to find
	 * @return the child with the specified label. Null if not found.
	 */
	public static ArrayList getAllNodeByName(TypedNode root, String label) {
		org.w3c.dom.NodeList nl = root.getChildNodes();
		ArrayList vect = new ArrayList();

		for (int i = 0; i < nl.getLength(); i++) {
			if (nl.item(i).getNodeName().equalsIgnoreCase(label)) {
				vect.add(nl.item(i));
			}
		}
		return vect;
	}

	/**
	 * Get the text of a leaf.
	 * If the node is a final node in our definition :
	 * For example :
	 * <pre>
	 *      #Element : name
	 *      |
	 *      +---- #Text : scott
	 * </pre>
	 *
	 * or :
	 * <pre>
	 *      #Element : name
	 *      |
	 *      +---- #Attribute : String
	 *      |
	 *      +---- #Text : scott
	 * </pre>
	 * but not
	 * <pre>
	 *      #Element : name
	 *      |
	 *      +---- #Element : age
	 *      |        |
	 *      |        +---- #Text : scott
	 *      |
	 *      +---- #Text : scott
	 * </pre>
	 * @return the text of the leaf if any, null otherwise.
	 */
	public static Object getTextLeaf(TypedNode node) {
		if (node == null)
			return null;
		if (node instanceof TypedValue) {
			return ((TypedValue) node).getData();
		}
		if (node instanceof TypedAttributeImpl) {
			return ((TypedAttributeImpl) node).getNodeValue();
		}
		org.w3c.dom.NodeList nl = node.getChildNodes();
		if (nl.getLength() != 1)
			return null;
		TypedNode n = (TypedNode) nl.item(0);
		return n.getTypedValue();
	}

	//later an arraylist
	public static Object getTextLeaf(TypedNode node, LocatedExpression paths) {
		ArrayList path = new ArrayList();
		ArrayList steps = paths.getSteps();
		for (int i = 0; i < steps.size(); i++) {
			Step step = (Step) steps.get(i);
			XQueryExpression exp = step.getExpression();
			// TODO later, consider here namespace and prefix comparison
			if (exp instanceof QName)
				path.add(((QName) exp).getLocalName());
		}
		TypedNode leafnode = getNodeByPath(path, node);
		return getTextLeaf(leafnode);
	}

	public static void getStringValue(Node node, StringBuffer sb) {
		if (node == null)
			return;
		if (node.getNodeType() == Node.TEXT_NODE) {
			sb.append(node.getNodeValue());
		}
		org.w3c.dom.NodeList nl = node.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			getStringValue(nl.item(i), sb);
		}
	}

	/**
	 * Access to a node given by a path given in argument. A path is
	 * given here as a vector of string.
	 *
	 * @param path the vector of string representing the path
	 * @return the node accessed
	 */
	private static TypedNode getNodeByPath(ArrayList path, TypedNode root) {

		if (root == null)
			return null;
		if (path == null || path.isEmpty())
			return root;

		org.w3c.dom.NodeList nl = root.getChildNodes();
		boolean found = false;
		TypedNode n = null;
		for (int i = 0; i < nl.getLength(); i++) {
			n = (TypedNode) nl.item(i);
			if (n.getNodeName().equals((String) path.get(0))) {
				path.remove(0);
				found = true;
				break;
			}
		}
		if (found == false)
			return null;
		if (path.size() <= 0)
			return n;
		return (getNodeByPath(path, n));
	}

	/**
	 * Access to a node given by a path given in argument. A path is
	 * given here as a vector of string.
	 *
	 * @param path the vector of string representing the path
	 * @return the node accessed
	 */
	private static ArrayList getNodeListByPath(ArrayList path, TypedNode root) {
		if ((path == null) || (path.size() <= 0))
			return null;

		String str = (String) path.get(path.size() - 1);
		path.remove(path.size() - 1);
		TypedNode n = getNodeByPath(path, root);

		if (n == null)
			return null;

		org.w3c.dom.NodeList nl = n.getChildNodes();

		ArrayList v = new ArrayList();
		for (int i = 0; i < nl.getLength(); i++) {
			n = (TypedNode) nl.item(i);
			if (n.getNodeName().equals(str))
				v.add(n);
		}
		return v;
	}

	private static boolean compareQNames(String nameSpace1, String localName1, String nameSpace2, String localName2) {
		if ((((nameSpace1 == null || nameSpace1.equals("")) && (nameSpace2 == null || nameSpace2.equals(""))) || (nameSpace1 != null && nameSpace1.equals("*")) || (nameSpace2 != null) && (nameSpace2.equals("*") || nameSpace2.equals(nameSpace2))) && localName1 != null && localName2 != null && localName1.equals(localName2))
			return true;
		return false;
	}

	private static ArrayList getStep(TypedNode node, Step step) throws XQueryException {
		return getStep(node, step, null);
	}

	private static ArrayList getStep(TypedNode node, Step step, ArrayList reslist) throws XQueryException {
		if (step == null)
			return null;
		//		if (!step.hasSeparator())
		//			return null;
		XQueryExpression stepExpr = step.getExpression();
		while (stepExpr instanceof ParenthesizedExpression)
		    stepExpr = ((ParenthesizedExpression)stepExpr).getExpression();		
		int nodeType = node.getNodeType();
		switch (step.getAxis()) {
			//			case -1 :
			case Axis.NONE :
				if (stepExpr instanceof ListOpUNIONExpression) {
					ArrayList nodelist = new ArrayList(1);
					nodelist.add(node);
					ArrayList steps = ((LocatedExpression) ((ListOpUNIONExpression) stepExpr).getExpression1()).getSteps();
					for (int i = 0; i < steps.size(); i++) {
						nodelist = getStep(nodelist, (Step) steps.get(i));
						if (nodelist == null)
							break;
					}
					if (nodelist != null && !nodelist.isEmpty()) {
						if (reslist == null)
							reslist = new ArrayList();
						reslist.addAll(nodelist);
					}
					nodelist = new ArrayList(1);
					nodelist.add(node);
					steps = ((LocatedExpression) ((ListOpUNIONExpression) stepExpr).getExpression2()).getSteps();
					for (int i = 0; i < steps.size(); i++) {
						nodelist = getStep(nodelist, (Step) steps.get(i));
						if (nodelist == null)
							break;
					}
					if (nodelist != null && !nodelist.isEmpty()) {
						if (reslist == null)
							reslist = new ArrayList();
						reslist.addAll(nodelist);
					}
					return reslist;
				} else
					return null;
			case Axis.CHILD :
				// child::node()
				// child::NT
				// child::text()
				if (nodeType == Node.ATTRIBUTE_NODE)
					return null;
				if (stepExpr instanceof QName) {
					NodeList nl = node.getChildNodes();
					for (int i = 0; i < nl.getLength(); i++) {
						Node tmpnode = nl.item(i);
						if (tmpnode.getNodeType() == Node.ELEMENT_NODE) {
							if (compareQNames(((QName) stepExpr).getNameSpace(), ((QName) stepExpr).getLocalName(), tmpnode.getNamespaceURI(), tmpnode.getLocalName())) {
								if (reslist == null)
									reslist = new ArrayList();
								reslist.add(tmpnode);
							}
						}
					}
					return reslist;
				} else if (stepExpr instanceof NodeTest) {
					NodeList nl = node.getChildNodes();
					switch (((NodeTest) stepExpr).getKind()) {
						case NodeKind.TEXT :
							for (int i = 0; i < nl.getLength(); i++) {
								Node tmpnode = nl.item(i);
								if (tmpnode.getNodeType() == Node.TEXT_NODE) {
									if (reslist == null)
										reslist = new ArrayList();
									reslist.add(tmpnode);
								}
							}
							return reslist;
						case NodeKind.NODE :
							for (int i = 0; i < nl.getLength(); i++) {
								Node tmpnode = nl.item(i);
								if (tmpnode.getNodeType() == Node.TEXT_NODE || tmpnode.getNodeType() == Node.ELEMENT_NODE) {
									if (reslist == null)
										reslist = new ArrayList();
									reslist.add(tmpnode);
								}
							}
							return reslist;
					}
				}
				break;
			case Axis.DESCENDANT_OR_SELF :
				// descendant-or-self::node()
				// this code assumes that only descendant-or-self::node() can occur
				if (stepExpr instanceof NodeTest) {
					if (reslist == null) {
						reslist = new ArrayList();
						reslist.add(node);
					}
					if (nodeType == Node.ATTRIBUTE_NODE)
						return reslist;
					NodeList nl = node.getChildNodes();
					switch (((NodeTest) step.getExpression()).getKind()) {
						case NodeKind.NODE :
							for (int i = 0; i < nl.getLength(); i++) {
								Node tmpnode = nl.item(i);
								if (tmpnode.getNodeType() == Node.TEXT_NODE || tmpnode.getNodeType() == Node.ELEMENT_NODE) {
									reslist.add(tmpnode);
									getStep((TypedNode) tmpnode, step, reslist);
								}
							}
							return reslist;
					}
				}
				break;
			case Axis.ATTRIBUTE :
				// attribute::NT
				if (stepExpr instanceof QName) {
					NamedNodeMap nnm = node.getAttributes();
					if (nnm != null) {
						Node tmpnode = nnm.getNamedItem(step.getExpression().getStringValue());
						if (tmpnode != null) {
							if (reslist == null)
								reslist = new ArrayList(1);
							reslist.add(tmpnode);
							return reslist;
						}
					}
					return reslist;
				}
				break;
			case Axis.SELF :
				// self::node()
				// self::text()
				// self::NT
				if (stepExpr instanceof QName) {
					if (nodeType != Node.ELEMENT_NODE && nodeType != Node.ATTRIBUTE_NODE)
						return null;
					if (compareQNames(((QName) stepExpr).getNameSpace(), ((QName) stepExpr).getLocalName(), node.getNamespaceURI(), node.getLocalName())) {
						if (reslist == null)
							reslist = new ArrayList(1);
						reslist.add(node);
					}
					return reslist;
				} else if (stepExpr instanceof NodeTest) {
					switch (((NodeTest) stepExpr).getKind()) {
						case NodeKind.TEXT :
							if (nodeType != Node.TEXT_NODE)
								return null;
							String text = node.getNodeValue();
							if (text == null || text.length() == 0)
								return null;
							if (reslist == null)
								reslist = new ArrayList(1);
							reslist.add(node);
							//							String text = node.getNodeValue();
							//							if (text == null || text.length() == 0)
							//								return null;
							//							if (reslist == null)
							//								reslist = new ArrayList(1);
							//							reslist.add(new TypedValueImpl(docimpl, text));
							return reslist;
						case NodeKind.NODE :
							if (reslist == null)
								reslist = new ArrayList(1);
							reslist.add(node);
							return reslist;
					}
					break;
				}
		}
		throw new XQueryException("getStep : could not handle step : " + step);
	}

	public static ArrayList getStep(ArrayList nodelist, Step step) throws XQueryException {
		if (nodelist == null || nodelist.isEmpty())
			return null;
		ArrayList reslist = new ArrayList();
		for (int i = 0; i < nodelist.size(); i++) {
			TypedNode nodei = (TypedNode) nodelist.get(i);
			ArrayList tmplist = DOMUtils.getStep(nodei, step);
			if (tmplist != null)
				reslist.addAll(tmplist);
		}
		if (reslist.isEmpty())
			reslist = null;
		return reslist;
	}

	public static ArrayList getSteps(TypedNode node, XQueryExpression expr) throws XQueryException {
		if (node == null)
			return null;
		if (expr instanceof Variable) {
			ArrayList reslist = new ArrayList(1);
			reslist.add(node);
			return reslist;
		}
		if (!(expr instanceof LocatedExpression))
			return null;
		ArrayList reslist = new ArrayList(1);
		reslist.add(node);
		ArrayList steps = ((LocatedExpression) expr).getSteps();
		for (int i = 1; i < steps.size(); i++) {
			reslist = getStep(reslist, (Step) steps.get(i));
			if (reslist == null)
				break;
		}
		if (reslist != null && reslist.isEmpty())
			reslist = null;
		return reslist;
	}
	public static ArrayList getSteps(ArrayList nodelist, XQueryExpression expr) throws XQueryException {
		if (nodelist == null || nodelist.isEmpty())
			return null;
		if (expr instanceof Variable)
			return nodelist;
		if (!(expr instanceof LocatedExpression))
			return null;
		ArrayList reslist = (ArrayList) nodelist.clone();
		ArrayList steps = ((LocatedExpression) expr).getSteps();
		for (int i = 1; i < steps.size(); i++) {
			reslist = getStep(reslist, (Step) steps.get(i));
			if (reslist == null)
				break;
		}
		if (reslist != null && reslist.isEmpty())
			reslist = null;
		return reslist;
	}

	// method telling whether first tuple should be placed before second tuple given a set of ordering elements
	public static boolean orderTuple(Tuple tuple1, Tuple tuple2, ArrayList orderbys) throws XQueryException {
		for (int i = 0; i < orderbys.size(); i++) {
			String stri = ((XQueryExpression) orderbys.get(i)).getStringValue(); // to avoid braces !
			int order = ((XQueryExpression) orderbys.get(i)).getOrder(0);
			if (order == -1)
				order = Constants.ASCENDING_ORDER;
			ArrayList list1 = tuple1.getPath(stri);
			ArrayList list2 = tuple2.getPath(stri);
			if (list1 == null || list1.isEmpty()) {
				if (list2 == null || list2.isEmpty())
					continue;
				else {
					if (list2.size() > 1)
						throw new XQueryException("Could not compare more than one element!");
					if (order == Constants.DESCENDING_ORDER)
						return false;
				}
			} else {
				if (list1.size() > 1)
					throw new XQueryException("Could not compare more than one element!");
				if (list2 == null || list2.isEmpty()) {
					if (order == Constants.ASCENDING_ORDER)
						return false;
				} else {
					if (list2.size() > 1)
						throw new XQueryException("Could not compare more than one element!");
					int comp = fullCompareTree((TypedNode) list1.get(0), (TypedNode) list2.get(0), order);
					if (comp == 0)
						continue;
					else if (comp > 0) {
						if (order == Constants.ASCENDING_ORDER)
							return false;
						else
							return true;
					} else if (comp < 0) {
						if (order == Constants.DESCENDING_ORDER)
							return false;
						else
							return true;
					}
				}
			}
		}
		return true;
	}

	private static int fullCompareTree(TypedNode node1, TypedNode node2, int order) throws XQueryException {
		if (!node1.getNodeName().equals(node2.getNodeName()))
			throw new XQueryException("Could not compare elements with different names!");
		Comparable value1 = (Comparable) node1.getTypedValue();
		Comparable value2 = (Comparable) node2.getTypedValue();
		if (value1 != null && value2 != null)
			return compareObjects(value1, value2);
		if (order != Constants.PLACE_ORDER) {
			TypedNode tmpnode1 = node1;
			TypedNode tmpnode2 = node2;
			if (value1 == null) {
				org.w3c.dom.NodeList nl = node1.getChildNodes();
				if (nl.getLength() != 1)
					throw new XQueryException("Could not compare elements, at least one incorrect value");
				node1 = (TypedNode) nl.item(0);
			}
			if (value2 == null) {
				org.w3c.dom.NodeList nl = node2.getChildNodes();
				if (nl.getLength() != 1)
					throw new XQueryException("Could not compare elements, at least one incorrect value");
				node2 = (TypedNode) nl.item(0);
			}
			return fullCompareTree(node1, node2, order);
		} else { // PLACE_ORDER
			if (compareTree(node1, node2))
				return 0;
			else
				return 1;
		}
	}

	//public static TextImpl makeTextNode(String strvalue) { return new TextImpl(docimpl, strvalue) ; }
	public static TypedNode makeTextNode(Value value, SchemaManager schemamanager) {
		TypedNode text = (TypedNode) docimpl.createTextNode(value.getValue());
		if (value instanceof ValueInteger) {
			text.setType(schemamanager.getIntegerType());
			text.setTypedValue(((ValueInteger) value).getIntValue());
		} else if (value instanceof ValueDecimal) {
			text.setType(schemamanager.getDecimalType());
			text.setTypedValue(((ValueDecimal) value).getDecimalValue());
		} else if (value instanceof ValueFloat) {
			text.setType(schemamanager.getFloatType());
			text.setTypedValue(((ValueFloat) value).getFloatValue());
		} else if (value instanceof ValueDouble) {
			text.setType(schemamanager.getDoubleType());
			text.setTypedValue(((ValueDouble) value).getDoubleValue());
		} else if (value instanceof ValueString) {
			text.setType(schemamanager.getStringType());
			text.setTypedValue(value.getValue());
		}
		return text;
	}

	public static XQueryExpression getXQueryExpression(Object obj) throws TypeException, XQueryException {

		if (obj instanceof Integer)
			return new ValueInteger(obj.toString(), null);
		else if (obj instanceof Double)
			return new ValueDouble(obj.toString(), null);
		else if (obj instanceof Float)
			return new ValueFloat(obj.toString(), null);
		else if (obj instanceof BigDecimal)
			return new ValueDecimal(obj.toString(), null);
		else if (obj instanceof String)
			return new ValueString(obj.toString(), null);
		else if (obj instanceof Date) {
			ArrayList list = new ArrayList(1);
			list.add(new ValueString(obj.toString(), null));
			return new FunctionDATETIME(list, null);
		}
		throw new XQueryException("Impossible to convert object : " + obj);
	}

}
