/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2004 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery;

import java.net.URL;
import java.util.*;

import org.xml.sax.InputSource;

public class DefaultModuleLocator implements ModuleLocator {
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	private HashMap moduleLocations = new HashMap();

	public InputSource resolveLocation(String namespace, String location) {
		return null;
	}

	public InputSource resolveLocation(String namespace, URL location) {
		try {
			InputSource source = new InputSource(location.openStream());
			source.setSystemId(location.toString());
			return source;
		} catch (java.io.IOException ex) {
			return null;
		}
	}

	public String resolveInclude(String location) {
		return location;
	}

	public void clearModuleLocations() {
		moduleLocations.clear();
	}

	public Iterator getModuleLocations(String namespace) {
		List locs = (List) moduleLocations.get(namespace);
		if (locs != null)
			return locs.iterator();
		else
			return null;
	}

	public void addModuleLocation(String uri, String location) {
		List list = (List) moduleLocations.get(uri);
		if (list == null) {
			list = new ArrayList();
			moduleLocations.put(uri, list);
		}
		list.add(location);
	}

}
