/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2004 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.TypeException;
import org.xquark.xquery.typing.TypeVisitor;

public class JavaModule extends XQueryModule {

	private String className = null;
	private Class javaClass = null;
	private ArrayList javaClassMethods = new ArrayList();
	
	public JavaModule(String className, TypeVisitor typeVisitor) throws XQueryException {
		this.className = className;
		// create all stuff necessary
		// create function declaration
		try {
			javaClass = Class.forName(className);
			Method methods[] = javaClass.getMethods();
			ArrayList funcDeclList = new ArrayList();
			HashMap funcDeclMap = new HashMap();
			for (int i = 0; i < methods.length; i++) {
				Method method = (Method) methods[i];
				if (method.getDeclaringClass() == Object.class)
					continue;
				javaClassMethods.add(method);
				String name = method.getName();
				Class parameterTypes[] = method.getParameterTypes();
				ArrayList parameters = new ArrayList(parameterTypes.length);
				int index = 1;
				for (int j = 0; j < parameterTypes.length; j++) {
					Class parameterType = (Class) parameterTypes[j];
					Variable varj = new Variable(new QName("param"+i++, null), null);
					XQueryExpression retExpr = getQNameFromJavaType(parameterType);
					ItemType itemType = new ItemType(retExpr, this);
					SequenceType seqType = new SequenceType(itemType, Constants.NOTHING, this);
					varj.setTypeDeclaration(seqType);
					varj.setBindingType(Constants.PARAMETER_BINDINGTYPE);
					varj.accept(typeVisitor);
					parameters.add(varj);
				}
				Class returnType = method.getReturnType();
				XQueryExpression retExpr = getQNameFromJavaType(returnType);
				ItemType itemType = new ItemType(retExpr, this);
				SequenceType seqType = new SequenceType(itemType, Constants.NOTHING, this);

				FunctionDeclaration funcDecl = new FunctionDeclaration(new QName(name, null), parameters, seqType, null, this);
				funcDecl.setMethod(method);
				funcDeclList.add(funcDecl);
				funcDeclMap.put(funcDecl.getFuncName().getLocalName(), funcDecl);
			}
			HashMap map = new HashMap();
			map.put("java:" + className, funcDeclMap);
			this.setFunctions(map);
			this.setDeclList(funcDeclList);
		} catch (ClassNotFoundException e) {
			throw new XQueryException(e.getMessage());
		}
	}
	
	public Class getJavaClass() {
		return javaClass;
	}
	
	private final String XSPREFIX = "xs";
	private final String XSURI = "http://www.w3.org/2001/XMLSchema";
	
	private QName getQNameFromJavaType(Class javaType) throws TypeException, XQueryException {
		if (javaType.getName() == "String")
			return new QName(XSURI,XSPREFIX,"string",null,null);
		else if (javaType.getName() == "int")
			return new QName(XSURI,XSPREFIX,"integer",null,null);
		return null;
	}

}
