/*
 * This file belongs to the XQuare distribution. Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307. You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.metadata;

import java.util.*;

import org.xquark.xquery.metadata.resolver.CollectionMetadata;
import org.xquark.xquery.metadata.resolver.SourceMetadata;

// ***************************************************************************
// * Class 'MetaWrapper'
// ***************************************************************************

public class MetaWrapper implements SourceMetadata {

    // **********************************************************************
    // * Constant
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.6 $";

    private static final String RCSName = "$Name: XQ1_1_1 $";

    // **********************************************************************
    // * Fields
    // **********************************************************************
    private HashMap metaModules = null;

    // defined for compatibility purposes
    private HashMap metaCollections = null;

    private String sourcename = null;

    private MetaDataImpl metadata = null;

    // **********************************************************************
    // * Constructor
    // **********************************************************************
    /**
     * Construct the class MetaWrapper
     * 
     * @param
     */
    public MetaWrapper(MetaDataImpl metadata) {
        this.metadata = metadata;
    }

    //	// use for importing this with new MetaData reference
    //    public MetaWrapper(MetaDataImpl metadata, MetaWrapper oldwrapper) {
    //        initData();
    //        this.metadata = metadata;
    //// HashMap hm = oldwrapper.getMetaCollections () ;
    //// for (Iterator it = hm.values ().iterator () ; it.hasNext () ; )
    //// {
    //// MetaCollection col = (MetaCollection) it.next () ;
    //// MetaCollection newmetacol = new MetaCollection (metadata, col) ;
    //// addMetaCollection (newmetacol) ;
    //// }
    //    }

    // **********************************************************************
    // * Methods
    // **********************************************************************
    public void addMetaModule(MetaModule metaModule) {
        // incorrect argument
        if (metaModule == null)
            return;
        // get name of collection
        String key = metaModule.getNamespace();
//        // incorrect name
//        if (key == null)
//            return;
        // initialize collections if necessary
        if (metaModules == null)
            metaModules = new HashMap();
        // verify collection of same name not present
        if (metaModules.get(key) != null)
            return;
        // add collection
        metaModules.put(key, metaModule);
    }

    /**
     *  
     */
    public void addMetaCollection(MetaCollection metacollection) {
        // incorrect argument
        if (metacollection == null)
            return;
        // get name of collection
        String key = metacollection.getCollectionName();
        // incorrect name
        if (key == null)
            return;
        // initialize collections if necessary
        if (metaCollections == null)
            metaCollections = new HashMap();
        // verify collection of same name not present
        if (metaCollections.get(key) != null)
            return;
        // add collection
        metaCollections.put(key, metacollection);
    }

    /**
     *  
     */
    public String getSourceName() {
        return sourcename;
    }

    /**
     *  
     */
    public void setSourceName(String sourcename) {
        this.sourcename = sourcename;
    }

    /**
     * @deprecated
     */
        public Collection getCollectionsMetadata() {
            if (metaCollections == null) return null;
            return metaCollections.values();
        }

    /**
     * Return the schematrees associated with this wrapper
     */
    //    public HashMap getSchemaTrees() throws SAXException {
    //        HashMap collections = new HashMap();
    //        Iterator it = metacollections.entrySet().iterator();
    //        while (it.hasNext()) {
    //            Map.Entry entry = (Map.Entry)it.next();
    //            MetaCollection col = (MetaCollection)entry.getValue();
    //            if (col.getXTree() != null) collections.put((String)entry.getKey(),col.getXTree());
    //        }
    //        return collections;
    //    }
	public MetaCollection getMetaCollection(String colName) {
		if (metaCollections == null) return null;
		return (MetaCollection)metaCollections.get(colName);
	}
    public HashMap getMetaCollections() {
        return metaCollections ;
    }
    public MetaDataImpl getMetaData() {
        return metadata;
    }

    /**
     * @deprecated
     */
    public CollectionMetadata getCollectionMetadata(String colname) {
        if (metaCollections == null) return null;
        CollectionMetadata retColl = (CollectionMetadata)metaCollections.get(colname);
        if (retColl == null)
            retColl = (CollectionMetadata)metaCollections.get(colname.toUpperCase());
        return retColl;
    }

    public boolean isCaseSensitive() {
        return false;
    }

    public boolean isStrict() {
        return false;
    }

	public List getCollectionsName ()
	{
		if (metaCollections == null) return null;
		List al = new ArrayList () ;
		al.addAll(metaCollections.keySet());
		return al ;
	}
	
    public HashMap getMetaModules() {
        return metaModules;
    }

//    public Collection getMetaModulesMetadata() {
//        // TODO Auto-generated method stub
//        return null;
//    }
//
//    public CollectionMetadata getMetaModuleMetadata(String namespace) {
//        // TODO Auto-generated method stub
//        return null;
//    }

    
    // **********************************************************************
    // * Destructor
    // **********************************************************************
    /**
     * Dereference allocated object which will be further be destroy by the garbage collector.
     */
    protected void finalize() throws Throwable {

    }

}