/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package  org.xquark.xquery.metadata.resolver;

import java.util.Collection;

/**
 * Interface that provides a common access to the metadata of an XML data
 * source.
 */
public interface SourceMetadata
{

    /**
     * Returns a collection of metadata objects for all collections of the
     * source.
     * @return a Collection object containing CollectionMetadata objects.
     * @deprecated
     */
    public Collection getCollectionsMetadata();
    
    /** Provide access to a particular collection's metadata given by name.
     * <B>Warning:</B> implementation must be consistent with the case flag
     * , i.e., must be case unsensitive if the flag is false.
     * @see #isCaseSensitive()
     * @param collectionName the ID of the collection. Case is considered or not
     * according to the case flag.
     * @return a SourceMetadata object
     * @deprecated
     */    
    public CollectionMetadata getCollectionMetadata(String collectionName);
    
	/**
	 * Accessor to the source name.
	 * @return the source symbolic name
	 */
    public String getSourceName();
    
    /**
     * Returns the value of the case flag. If false, the case of the collection
     * name retrieved with the getCollections() method must be ignored.
     * @see #isCaseSensitive()
     * @see CollectionMetadata#getCollectionName()
     * @return the value of the case flag.
     */
    public boolean isCaseSensitive();

	/**
	 * Returns the value of the strict flag. If true, only those paths present in
	 * the metadata can be found, none can be extrapolated.
	 * @return the value of the strict flag.
	 */
	public boolean isStrict();
}
