/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.normalize;

import java.util.ArrayList;

import org.xquark.xquery.parser.*;

public class AddLoopInfoVisitor extends DefaultParserVisitor {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private ArrayList loopIDs = new ArrayList(1);
    private String varliststring = null;
    private int varlistsize = 0;
    
    public AddLoopInfoVisitor(ArrayList varlist) {
        if (varlist != null && !varlist.isEmpty()) {
            this.varlistsize = varlist.size();
            StringBuffer buf = new StringBuffer();
            for (int i=0;i<varlist.size();i++) buf.append(((Variable)varlist.get(i)).toString());
            this.varliststring = buf.toString();
        }
    }
    
    private void addLoopIDs(ArrayList list) {
        if (list == null) return;
        if (list.isEmpty()) return;
        for (int i=0;i<list.size();i++) {
            Variable var = (Variable)list.get(i);
            if (!(loopIDs.contains(var)))
                loopIDs.add(var);
        }
    }
        
    //	public void visit(AggregateFunctionCall arg) throws XQueryException;
    public void visit(AttributeValuePair arg) throws XQueryException {
        XQueryExpression attValue = arg.getAttributeValue();
        attValue.accept(this);
    }
    //	public void visit(BinOpANDExpression arg) throws XQueryException;
    //	public void visit(BinOpORExpression arg) throws XQueryException;
    //	public void visit(CastTreatExpression arg) throws XQueryException;
    //	public void visit(ContextDeclaration arg) throws XQueryException;
    //	public void visit(Dereference arg) throws XQueryException;
    
    public void visit(Element arg) throws XQueryException {
        ArrayList prevLoopIDs = (ArrayList)loopIDs.clone();
        addLoopIDs(arg.getDependIDs());
        addLoopIDs(arg.getSkolemIDs());
        ArrayList attributes = arg.getAttributes();
        if (attributes != null)
            for (int i=0;i<attributes.size();i++)
                ((AttributeValuePair)attributes.get(i)).accept(this);
        ArrayList subExpressions = arg.getSubExpressions();
        if (subExpressions != null)
            for (int i=0;i<subExpressions.size();i++)
                ((XQueryExpression)subExpressions.get(i)).accept(this);
        loopIDs = prevLoopIDs;
    }
    
    //	public void visit(FLWRExpression arg) throws XQueryException;
    //	public void visit(FunctionCall arg) throws XQueryException;
    //	public void visit(FunctionDefinition arg) throws XQueryException;
    //	public void visit(InstanceOfExpression arg) throws XQueryException;
    //	public void visit(InternalFunctionCall arg) throws XQueryException;
    public void visit(ITEExpression arg) throws XQueryException {
		ArrayList prevLoopIDs = (ArrayList)loopIDs.clone();
		addLoopIDs(arg.getDependIDs());
		addLoopIDs(arg.getSkolemIDs());
		arg.getIfExpression().accept(this);
		arg.getThenExpression().accept(this);
		arg.getElseExpression().accept(this);
		loopIDs = prevLoopIDs;
    }
    //	public void visit(LibraryFunctionCall arg) throws XQueryException;
    //	public void visit(ListOpAFTERExpression arg) throws XQueryException;
    //	public void visit(ListOpArithExpression arg) throws XQueryException;
    //	public void visit(ListOpBEFOREExpression arg) throws XQueryException;
    //	public void visit(ListOpCompExpression arg) throws XQueryException;
    //	public void visit(ListOpEXCEPTExpression arg) throws XQueryException;
    //	public void visit(ListOpINTERSECTExpression arg) throws XQueryException;
    //	public void visit(ListOpUNIONExpression arg) throws XQueryException;
    
    public void visit(LocatedExpression arg) throws XQueryException {
        ArrayList prevLoopIDs = (ArrayList)loopIDs.clone();
        addLoopIDs(arg.getDependIDs());
        addLoopIDs(arg.getSkolemIDs());
        if (loopIDs.isEmpty())arg.setLoopIDs(varliststring);
        else {
            if (loopIDs.size() > this.varlistsize) arg.setLoopIDs(null);
            else {
                StringBuffer buf = new StringBuffer();
                for (int i=0;i<loopIDs.size();i++) buf.append(((Variable)loopIDs.get(i)).toString());
                arg.setLoopIDs(buf.toString());
            }
        }
        loopIDs = prevLoopIDs;
        // for sub path list
        ArrayList steps = arg.getSteps();
        XQueryExpression tmpExpr = ((Step)steps.get(0)).getExpression();
        if (tmpExpr instanceof Variable) {
            ((Variable)tmpExpr).addSubPath(arg); // CHANGE   29/05/2002
        }
    }
    
    //	public void visit(NodeType arg) throws XQueryException;
    //	public void visit(Parameter arg) throws XQueryException;
    // public void visit(PrimitiveFunctionCall arg) throws XQueryException;
    //	public void visit(QName arg) throws XQueryException;
    //	public void visit(QuantifiedExpression arg) throws XQueryException;
    //	public void visit(RangeExpression arg) throws XQueryException;
    //	public void visit(RootNodeFunctionCall arg) throws XQueryException;
    //	public void visit(SortedExpression arg) throws XQueryException;
    //	public void visit(Step arg) throws XQueryException;
    //	public void visit(TypeSwitchExpression arg) throws XQueryException;
    //	public void visit(UnOpMinusExpression arg) throws XQueryException;
    //	public void visit(Value arg) throws XQueryException;
    //	public void visit(ValueBoolean arg) throws XQueryException;
    //	public void visit(ValueFloat arg) throws XQueryException;
    //	public void visit(ValueInteger arg) throws XQueryException;
    //	public void visit(ValueString arg) throws XQueryException;
    //	public void visit(ValueText arg) throws XQueryException;
    
    public void visit(Variable arg) throws XQueryException {
        ArrayList prevLoopIDs = (ArrayList)loopIDs.clone();
        addLoopIDs(arg.getDependIDs());
        addLoopIDs(arg.getSkolemIDs());
        if (loopIDs.isEmpty()) arg.setLoopIDs(varliststring);
        else {
            if (loopIDs.size() > this.varlistsize) arg.setLoopIDs(null);
            else {
                StringBuffer buf = new StringBuffer();
                for (int i=0;i<loopIDs.size();i++) buf.append(((Variable)loopIDs.get(i)).toString());
                arg.setLoopIDs(buf.toString());
            }
        }
        loopIDs = prevLoopIDs;
        // for sub path list
        arg.addSubPath(arg);
    }
    
    //	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException;
    //	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;
    
    public void visit(XQueryExpressionSequence arg) throws XQueryException {
        ArrayList prevLoopIDs = (ArrayList)loopIDs.clone();
        addLoopIDs(arg.getDependIDs());
        addLoopIDs(arg.getSkolemIDs());
        ArrayList subExpressions = arg.getSubExpressions();
        if (subExpressions != null)
            for (int i=0;i<subExpressions.size();i++)
                ((XQueryExpression)subExpressions.get(i)).accept(this);
        loopIDs = prevLoopIDs;
    }
    
    //	public void visit(XQueryFile arg) throws XQueryException;
    //	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException;
    //	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException;
    //	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException;
    //	public void visit(XQueryUnit arg) throws XQueryException;
    //	public void visit(XQueryVoid arg) throws XQueryException;
    
    // primitive functions
    
}
