/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.normalize;

import java.util.ArrayList;

import org.xquark.xquery.parser.*;
import org.xquark.xquery.typing.TypeVisitor;

public class AtomizeVisitor extends DefaultParserVisitor {
	

	private XQueryExpression resultExpr = null;
	private TypeVisitor typeVisitor = null;
	private boolean isSpecial = false;
	
	
	AtomizeVisitor(TypeVisitor typeVisitor) {
		this.typeVisitor = typeVisitor;	 
	}
	
	public XQueryExpression getExpression() {
		isSpecial = false;
		return resultExpr;
	}
	
	public void isSpecial() {
		isSpecial= true;
	}
	
//	public void visit(AggregateFunctionCall arg) throws XQueryException;

	public void visit(AttributeValuePair arg) throws XQueryException {
		arg.getExpression2().accept(this);
	}

//	public void visit(BinOpANDExpression arg) throws XQueryException;
//	public void visit(BinOpORExpression arg) throws XQueryException;
//	public void visit(CaseClauseExpression arg) throws XQueryException;
//	public void visit(CastTreatExpression arg) throws XQueryException;
//	public void visit(CData arg) throws XQueryException;
//	public void visit(ComputedNamespace arg) throws XQueryException;
//	public void visit(ComputedText arg) throws XQueryException;
//	public void visit(ContextDeclaration arg) throws XQueryException;
//	public void visit(Document arg) throws XQueryException;

	public void visit(Element arg) throws XQueryException {
		if (isSpecial) {
			resultExpr = new ValueInteger("1",arg.getParentModule()); 
			return;
		}
		ArrayList list = arg.getSubExpressions();
		if (list != null) {
			if (list.size() == 1)
				resultExpr = (XQueryExpression)list.get(0);
			else
				resultExpr = new XQueryExpressionSequence(list, arg.getParentModule());
		} else
			resultExpr = null;
	}
	
	public void visit(FLWRExpression arg) throws XQueryException {
		if (isSpecial) {
			if (!(arg.getReturnClause() instanceof Value))
				//arg.setReturnClause((XQueryExpression)arg.getVariables().get(0));
				arg.setReturnClause(new ValueInteger("1",arg.getParentModule()));
		} else {
			arg.getReturnClause().accept(this);
			arg.setReturnClause(resultExpr);
		}
		resultExpr = arg; 
	}
	
//	public void visit(FunctionCall arg) throws XQueryException;
//	public void visit(FunctionDeclaration arg) throws XQueryException;
//	public void visit(InstanceOfExpression arg) throws XQueryException;
//	public void visit(InternalFunctionCall arg) throws XQueryException;
//	public void visit(ITEExpression arg) throws XQueryException;
//	public void visit(ItemType arg) throws XQueryException;
//	public void visit(LibraryFunctionCall arg) throws XQueryException;
//	public void visit(ListOpAFTERExpression arg) throws XQueryException;
//	public void visit(ListOpArithExpression arg) throws XQueryException;
//	public void visit(ListOpBEFOREExpression arg) throws XQueryException;
//	public void visit(ListOpCompExpression arg) throws XQueryException;
//	public void visit(ListOpEXCEPTExpression arg) throws XQueryException;
//	public void visit(ListOpINTERSECTExpression arg) throws XQueryException;
//	public void visit(ListOpUNIONExpression arg) throws XQueryException;
//	public void visit(LocatedExpression arg) throws XQueryException;
//	public void visit(ElementTest arg) throws XQueryException;
//	public void visit(AttributeTest arg) throws XQueryException;
//	public void visit(NodeTest arg) throws XQueryException;
//	public void visit(PrimitiveFunctionCall arg) throws XQueryException;
//	public void visit(QName arg) throws XQueryException;
//	public void visit(QuantifiedExpression arg) throws XQueryException;
//	public void visit(RangeExpression arg) throws XQueryException;
//	public void visit(RootNodeFunctionCall arg) throws XQueryException;
//	public void visit(SchemaContextPath arg) throws XQueryException;
//	public void visit(SchemaContextStep arg) throws XQueryException;
//	public void visit(SequenceType arg) throws XQueryException;
//	public void visit(SortedExpression arg) throws XQueryException;
//	public void visit(Step arg) throws XQueryException;
//	public void visit(TypeSwitchExpression arg) throws XQueryException;
//	public void visit(UnOpMinusExpression arg) throws XQueryException;
//	public void visit(ValidateExpression arg) throws XQueryException;
//	public void visit(Value arg) throws XQueryException;
//	public void visit(ValueBoolean arg) throws XQueryException;
//	public void visit(ValueDecimal arg) throws XQueryException;
//	public void visit(ValueDouble arg) throws XQueryException;
//	public void visit(ValueFloat arg) throws XQueryException;
//	public void visit(ValueInteger arg) throws XQueryException;
//	public void visit(ValueString arg) throws XQueryException;
//	public void visit(ValueText arg) throws XQueryException;
//	public void visit(Variable arg) throws XQueryException;
//	public void visit(XMLComment arg) throws XQueryException;
//	public void visit(XMLProcessingInstruction arg) throws XQueryException;
//	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException;
//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;

	public void visit(XQueryExpression arg) throws XQueryException {
		resultExpr = arg;
	}
	
	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		ArrayList list = arg.getSubExpressions();
		if (list != null) {
			if (list.size() == 1) {
				((XQueryExpression)list.get(0)).accept(this);
			} else {
				for (int i=0;i<list.size();i++) {
					XQueryExpression expri = (XQueryExpression)list.get(i);
					expri.accept(this);
					if (resultExpr != null)
						list.set(i,resultExpr);
					else {
						list.remove(i);
						i--;
					}						
				}
				if (list.size() == 0)
					resultExpr = null;
				else if (list.size() == 1)
					resultExpr = (XQueryExpression)list.get(0);
				else
					resultExpr = arg;
			}
		}
	}
//	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException;
//	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException;
//	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException;
//	public void visit(XQueryModule arg) throws XQueryException;
//	public void visit(XQueryVoid arg) throws XQueryException;
	
	// primitive functions
	
//	public void visit(FunctionABS arg) throws XQueryException ;
//	public void visit(FunctionAVG arg) throws XQueryException ;
//	public void visit(FunctionCEILING arg) throws XQueryException;
//	public void visit(FunctionCOLLECTION arg) throws XQueryException;
//	public void visit(FunctionCONCAT arg) throws XQueryException;
//	public void visit(FunctionCONTAINS arg) throws XQueryException;
//	public void visit(FunctionCOUNT arg) throws XQueryException;
//	public void visit(FunctionCURRENT_DATE arg) throws XQueryException;
//	public void visit(FunctionCURRENT_DATETIME arg) throws XQueryException;
//	public void visit(FunctionCURRENT_TIME arg) throws XQueryException;
//	public void visit(FunctionDATA arg) throws XQueryException;
//	public void visit(FunctionDATE arg) throws XQueryException;
//	public void visit(FunctionDATETIME arg) throws XQueryException;
//	public void visit(FunctionDECIMAL arg) throws XQueryException;
//	public void visit(FunctionDISTINCT_VALUES arg) throws XQueryException;
//	public void visit(FunctionDOC arg) throws XQueryException;
//	public void visit(FunctionDOUBLE arg) throws XQueryException;
//	public void visit(FunctionEMPTY arg) throws XQueryException;
//	public void visit(FunctionENDS_WITH arg) throws XQueryException;
//	public void visit(FunctionEXISTS arg) throws XQueryException;
//	public void visit(FunctionGET_MONTH_FROM_DATE arg) throws XQueryException;
//	public void visit(FunctionGET_YEAR_FROM_DATE arg) throws XQueryException;
//	public void visit(FunctionFALSE arg) throws XQueryException;
//	public void visit(FunctionFLOOR arg) throws XQueryException;
//	public void visit(FunctionINTEGER arg) throws XQueryException;
//	public void visit(FunctionLOCAL_NAME arg) throws XQueryException;
//	public void visit(FunctionLOWER_CASE arg) throws XQueryException;
//	public void visit(FunctionMATCHES arg) throws XQueryException;
//	public void visit(FunctionMAX arg) throws XQueryException;
//	public void visit(FunctionMIN arg) throws XQueryException;
//	public void visit(FunctionNAMESPACE_URI arg) throws XQueryException;
//	public void visit(FunctionNOT arg) throws XQueryException;
//	public void visit(FunctionNUMBER arg) throws XQueryException;
//	public void visit(FunctionROUND arg) throws XQueryException;
//	public void visit(FunctionSTARTS_WITH arg) throws XQueryException;
//	public void visit(FunctionSTRING arg) throws XQueryException;
//	public void visit(FunctionSTRING_LENGTH arg) throws XQueryException;
//	public void visit(FunctionSUBSTRING arg) throws XQueryException;
//	public void visit(FunctionSUM arg) throws XQueryException;
//	public void visit(FunctionTIME arg) throws XQueryException;
//	public void visit(FunctionTRUE arg) throws XQueryException;
//	public void visit(FunctionUPPER_CASE arg) throws XQueryException;
//	public void visit(FunctionDEEP_EQUALS arg) throws XQueryException;
}
