/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package  org.xquark.xquery.normalize;

import java.util.ArrayList;
import java.util.List;

import org.xquark.xquery.parser.FLWRExpression;

public class DependNode {

    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    protected ArrayList sons;
    protected DependNode parent = null;
    protected FLWRExpression expr = null;


    public DependNode(DependNode son, DependNode parent, FLWRExpression expr) {
        addSon(son);
        setParent(parent);
        setExpression(expr);
    }

    public DependNode(ArrayList sons, DependNode parent, FLWRExpression expr) {
        setSons(sons);
        setParent(parent);
        setExpression(expr);
    }

    public ArrayList getSons() { return sons; }

    public void setSons(ArrayList sons) { this.sons = sons; }

    public void addSon(DependNode son) {
        if (son == null) return;
        if (sons == null) sons = new ArrayList(1);
        this.sons.add(son);
        son.setParent(this);
    }
    public void addSon(int index, DependNode son) {
        if (son == null) return;
        if (sons == null) sons = new ArrayList(1);
        this.sons.add(index,son);
        son.setParent(this);
    }
    public void addSons(ArrayList sons) {
        if (sons == null) return;
        if (this.sons == null) this.sons = sons;
        else this.sons.addAll(sons);
        for (int i=0;i<sons.size();i++) ((DependNode)sons.get(i)).setParent(this);
    }

    public DependNode getParent() { return parent; }

    public void setParent(DependNode parent) { this.parent = parent; }

    public FLWRExpression getExpression() { return expr; }

    public void setExpression(FLWRExpression expr) { this.expr = expr; }


    // For debugging
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(expr.getParentExpression().get(0));
        if (sons != null && sons.size() > 0) {
            sb.append(" [");
            for (int i=0;i<sons.size();i++) {
                sb.append(" ");
                sb.append(((DependNode)sons.get(i)).toString());
            }
            sb.append(" ] ");
        }
        return sb.toString();
    }

	protected  List varLevels = null ;
	public  List varLevels() {
		if (null == varLevels) {
			if (null == parent) {
				varLevels = new ArrayList();
				varLevels.add(getExpression().getVariables().clone());
			}
			else {
				varLevels = new ArrayList();
				varLevels.addAll(parent.varLevels());
				List thisLevel = (List)getExpression().getVariables().clone();
				for (int i = 0; i < varLevels.size(); i++) {
					thisLevel.removeAll((List)varLevels.get(i));
				}
				varLevels.add(thisLevel);
			}
		}
		return varLevels;
	}

	protected  List sortExprLevels = null ;
	public  List sortExprLevels() {

		ArrayList  sortExprList = null;
		if (null == sortExprLevels) {
			if (null == parent) {
				sortExprLevels = new ArrayList();
				sortExprList = getExpression().getOrderBy();
				if (null != sortExprList) {
					sortExprLevels.add(sortExprList.clone());
				}
				else {
					sortExprLevels.add(null);
				}

			}
			else {
				sortExprLevels = new ArrayList();
				sortExprLevels.addAll(parent.sortExprLevels());

				sortExprList = getExpression().getOrderBy();
				if (null != sortExprList) {
					List thisLevel = (ArrayList) sortExprList.clone();
					for (int i = 0; i < sortExprLevels.size(); i++) {
						List  curLevel = (List)sortExprLevels.get(i);
						if (null != curLevel) {
							thisLevel.removeAll(curLevel);
						}
					}
					sortExprLevels.add(thisLevel);
				}
				else {
					sortExprLevels.add(null);
				}
			}
		}
		return sortExprLevels;
	}

}
