/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.normalize;

// import org.xquark.xml.xqueryparser.common.primitivefunctions.*;

import java.util.ArrayList;

import org.xquark.xquery.parser.*;

public class EraseSkolemIDVisitor extends DefaultParserVisitor {

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	private Variable var = null;

	public EraseSkolemIDVisitor(Variable var) {
		this.var = var;
	}

	//	public void visit(AggregateFunctionCall arg) throws XQueryException
	//	public void visit(AttributeValuePair arg) throws XQueryException;
	//	public void visit(BinOpANDExpression arg) throws XQueryException;
	//	public void visit(BinOpORExpression arg) throws XQueryException;
	//	public void visit(CastTreatExpression arg) throws XQueryException;
	//	public void visit(ContextDeclaration arg) throws XQueryException;
	//	public void visit(Dereference arg) throws XQueryException;

	public void visit(Element arg) throws XQueryException {
		if (arg.getSkolemIDs() != null)
			arg.getSkolemIDs().remove(var);
		ArrayList subExpressions = arg.getSubExpressions();
		if (subExpressions != null)
			for (int i = 0; i < subExpressions.size(); i++)
				 ((XQueryExpression) subExpressions.get(i)).accept(this);
	}

	//	public void visit(FLWRExpression arg) throws XQueryException;
	//	public void visit(FunctionCall arg) throws XQueryException;
	//	public void visit(FunctionDefinition arg) throws XQueryException;
	//	public void visit(InstanceOfExpression arg) throws XQueryException;
	//	public void visit(InternalFunctionCall arg) throws XQueryException;
	public void visit(ITEExpression arg) throws XQueryException {
		if (arg.getSkolemIDs() != null)
			arg.getSkolemIDs().remove(var);
	}
	//	public void visit(LibraryFunctionCall arg) throws XQueryException;
	//	public void visit(ListOpAFTERExpression arg) throws XQueryException;
	//	public void visit(ListOpArithExpression arg) throws XQueryException;
	//	public void visit(ListOpBEFOREExpression arg) throws XQueryException;
	//	public void visit(ListOpCompExpression arg) throws XQueryException;
	//	public void visit(ListOpEXCEPTExpression arg) throws XQueryException;
	//	public void visit(ListOpINTERSECTExpression arg) throws XQueryException;
	//	public void visit(ListOpUNIONExpression arg) throws XQueryException;
	//    public void visit(LocatedExpression arg) throws XQueryException {
	//		if (arg.getSkolemIDs() != null) arg.getSkolemIDs().remove(var);
	//    }
	//	public void visit(NodeType arg) throws XQueryException;
	//	public void visit(Parameter arg) throws XQueryException;
	//	public void visit(PrimitiveFunctionCall arg) throws XQueryException;
	//	public void visit(QName arg) throws XQueryException;
	//	public void visit(QuantifiedExpression arg) throws XQueryException;
	//	public void visit(RangeExpression arg) throws XQueryException;
	//	public void visit(RootNodeFunctionCall arg) throws XQueryException;
	//	public void visit(SortedExpression arg) throws XQueryException;
	//	public void visit(Step arg) throws XQueryException;
	//	public void visit(TypeSwitchExpression arg) throws XQueryException;
	//	public void visit(UnOpMinusExpression arg) throws XQueryException;
	//	public void visit(Value arg) throws XQueryException;
	//	public void visit(ValueBoolean arg) throws XQueryException;
	//	public void visit(ValueFloat arg) throws XQueryException;
	//	public void visit(ValueInteger arg) throws XQueryException;
	//	public void visit(ValueString arg) throws XQueryException;
	//	public void visit(ValueText arg) throws XQueryException;
	//    public void visit(Variable arg) throws XQueryException {
	//		if (arg.getSkolemIDs() != null) arg.getSkolemIDs().remove(var);
	//    }
	//	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException;
	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
		if (arg.getSkolemIDs() != null)
			arg.getSkolemIDs().remove(var);
		arg.getExpression1().accept(this);
		arg.getExpression2().accept(this);
	}
	//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;
	// public void visit(XQueryExpression arg) throws XQueryException;

	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		if (arg.getSkolemIDs() != null)
			arg.getSkolemIDs().remove(var);
		ArrayList subExpressions = arg.getSubExpressions();
		for (int i = 0; i < subExpressions.size(); i++)
			 ((XQueryExpression) subExpressions.get(i)).accept(this);
	}

	//	public void visit(XQueryFile arg) throws XQueryException;
	//	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException;
	//	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException;
	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
		if (arg.getSkolemIDs() != null)
			arg.getSkolemIDs().remove(var);
		if (arg.getExpression() != null)
			arg.getExpression().accept(this);
	}
	//	public void visit(XQueryUnit arg) throws XQueryException;
	//	public void visit(XQueryVoid arg) throws XQueryException;

	// primitive functions

}
