/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.normalize;

import java.util.ArrayList;

import org.xquark.xquery.parser.*;



public class SetVariablePathsVisitor extends DefaultParserVisitor {
    
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
	    
    public SetVariablePathsVisitor() {}
           
    public void visit(Element arg) throws XQueryException {
        if (arg.getStartTag() != null) {
            arg.getStartTag().accept(this);
        }
        if (arg.getAttributes() != null) {
            for (int i = 0; i < arg.getAttributes().size(); i++) {
                ((AttributeValuePair)arg.getAttributes().get(i)).accept(this);
            }
        }
        if (arg.getSubExpressions() != null) {
            for (int j = 0; j < arg.getSubExpressions().size(); j++) {
                ((XQueryExpression)arg.getSubExpressions().get(j)).accept(this);
            }
        }
    }
        
    public void visit(FunctionCall arg) throws XQueryException {
        if (arg.getArguments() != null) {
            for (int i = 0; i < arg.getArguments().size(); i++) {
                ((XQueryExpression)arg.getArguments().get(i)).accept(this);
            }
        }
    }
    
	public void visit(FLWRExpression arg) throws XQueryException {
		ArrayList list = arg.getVariables();
		for (int i=0;i<list.size();i++) {
			XQueryExpression expri = ((Variable)list.get(i)).getExpression();
			if (expri instanceof FLWRExpression) 
				expri.accept(this);
		}
		list = arg.getOrderBy();
		if (list != null)
			for (int i=0;i<list.size();i++)
				((XQueryExpression)list.get(i)).accept(this);
		if (arg.getReturnClause() instanceof FLWRExpression)
			arg.getReturnClause().accept(this);
	}

    public void visit(LocatedExpression arg) throws XQueryException {
		if (arg.getExpression() instanceof Variable) {
			((Variable)arg.getExpression()).addSubPath(arg);
        }
    }
    
    public void visit(Variable arg) throws XQueryException {
        arg.addSubPath(arg);
    }
    
	public void visit(ITEExpression arg) throws XQueryException {
		arg.getIfExpression().accept(this);
		arg.getThenExpression().accept(this);
		arg.getElseExpression().accept(this);
	}

    public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
        if (arg.getExpression1() != null) {
            arg.getExpression1().accept(this);
        }
        if (arg.getExpression2() != null) {
            arg.getExpression2().accept(this);
        }
    }
    
    
    //public void visit(XQueryExpression arg) throws XQueryException {}
    
    
    public void visit(XQueryExpressionSequence arg) throws XQueryException {
        for (int i = 0; i < arg.getSubExpressions().size(); i++) {
            ((XQueryExpression)arg.getSubExpressions().get(i)).accept(this);
        }
    }
    
    public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
        if (arg.getExpression() != null) {
            arg.getExpression().accept(this);
        }
    }
    
    
}
