/*
 * This file belongs to the XQuare distribution. Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307. You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.normalize;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.xpath.Axis;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.typing.TypeVisitor;

public class SubstituteVisitor extends DefaultParserVisitor {
    private static final String RCSRevision = "$Revision: 1.8 $";

    private static final String RCSName = "$Name: XQ1_1_1 $";

    private HashMap map = null;

    private boolean inReturn = false;

    private TypeVisitor typeVisitor = null;

    private XQueryModule module = null;

    public SubstituteVisitor(HashMap map, TypeVisitor typeVisitor, XQueryModule module) {
        setMap(map);
        this.typeVisitor = typeVisitor;
        this.module = module;
    }

    public void setMap(HashMap map) {
        this.map = map;
    }

    //	public void visit(AggregateFunctionCall arg) throws XQueryException

    public void visit(AttributeValuePair arg) throws XQueryException {
        XQueryExpression expr = null;
        XQueryExpression valueExpr = arg.getAttributeValue();
        if ((expr = (XQueryExpression) map.get(valueExpr)) != null) {
            ArrayList parenti = valueExpr.getParentExpression();
            if (parenti != null) {
                int indexi = parenti.indexOf(arg);
                if (indexi != -1)
                    parenti.remove(indexi);
                try {
                    XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                    arg.setAttributeValue(exprClone);
                } catch (CloneNotSupportedException e) {
                    throw new XQueryException("Could not clone expression : " + expr);
                }
            }
        } else
            valueExpr.accept(this);
    }

    //	public void visit(BinOpANDExpression arg) throws XQueryException;
    //	public void visit(BinOpORExpression arg) throws XQueryException;
    //	public void visit(CastTreatExpression arg) throws XQueryException;
    //	public void visit(ContextDeclaration arg) throws XQueryException;
    //	public void visit(Dereference arg) throws XQueryException;

    public void visit(Element arg) throws XQueryException {
        XQueryExpression expr = null;
        expr = arg.getStartTag();
        if ((expr = (XQueryExpression) map.get(expr)) != null)
            arg.setStartTag(expr);
        else
            arg.getStartTag().accept(this);
        ArrayList attributes = arg.getAttributes();
        if (attributes != null) {
            for (int i = 0; i < attributes.size(); i++) {
                AttributeValuePair subi = (AttributeValuePair) attributes.get(i);
                subi.accept(this);
            }
        }
        ArrayList subExpressions = arg.getSubExpressions();
        if (subExpressions != null) {
            for (int i = 0; i < subExpressions.size(); i++) {
                XQueryExpression subi = (XQueryExpression) subExpressions.get(i);
                if ((expr = (XQueryExpression) map.get(subi)) != null) {
                    subExpressions.remove(i);
                    ArrayList parenti = subi.getParentExpression();
                    if (parenti != null) {
                        int indexi = parenti.indexOf(arg);
                        if (indexi != -1)
                            parenti.remove(indexi);
                        try {
                            XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                            arg.addSubExpressions(i, exprClone);
                        } catch (CloneNotSupportedException e) {
                            throw new XQueryException("Could not clone expression : " + expr);
                        }
                    }
                } else
                    subi.accept(this);
            }
        }
    }

    public void visit(FLWRExpression arg) throws XQueryException {
        ArrayList variables = arg.getVariables();
        XQueryExpression expr = null;
        for (int i = 0; i < variables.size(); i++) {
            Variable vari = (Variable) variables.get(i);
            XQueryExpression variExpr = vari.getExpression();
            if ((expr = (XQueryExpression) map.get(variExpr)) != null) {
                ArrayList parenti = variExpr.getParentExpression();
                if (parenti != null) {
                    int indexi = parenti.indexOf(arg);
                    if (indexi != -1)
                        parenti.remove(indexi);
                    try {
                        XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                        vari.setExpression(exprClone, false);
                    } catch (CloneNotSupportedException e) {
                        throw new XQueryException("Could not clone expression : " + expr);
                    }
                }
            } else
                variExpr.accept(this);
        }
        ArrayList orderbys = arg.getOrderBy();
        if (orderbys != null) {
            for (int i = 0; i < orderbys.size(); i++) {
                XQueryExpression expri = (XQueryExpression) orderbys.get(i);
                if ((expr = (XQueryExpression) map.get(expri)) != null) {
                    ArrayList parenti = expri.getParentExpression();
                    if (parenti != null) {
                        int indexi = parenti.indexOf(arg);
                        if (indexi != -1)
                            parenti.remove(indexi);
                        try {
                            XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                            orderbys.set(i, exprClone);
                        } catch (CloneNotSupportedException e) {
                            throw new XQueryException("Could not clone expression : " + expr);
                        }
                    }
                } else
                    expri.accept(this);
            }
        }
        XQueryExpression whereClause = arg.getWhereClause();
        if (whereClause != null) {
            if ((expr = (XQueryExpression) map.get(whereClause)) != null) {
                try {
                    XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                    arg.setWhereClause(exprClone);
                } catch (CloneNotSupportedException e) {
                    throw new XQueryException("Could not clone expression : " + expr);
                }
            } else
                whereClause.accept(this);
        }
        XQueryExpression returnClause = arg.getReturnClause();
        if ((expr = (XQueryExpression) map.get(returnClause)) != null) {
            try {
                XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                arg.setReturnClause(exprClone);
            } catch (CloneNotSupportedException e) {
                throw new XQueryException("Could not clone expression : " + expr);
            }
        } else
            returnClause.accept(this);
    }

    public void visit(FunctionCall arg) throws XQueryException {
        ArrayList arguments = arg.getArguments();
        if (arguments == null)
            return;
        XQueryExpression expr = null;
        for (int i = 0; i < arguments.size(); i++) {
            XQueryExpression argi = (XQueryExpression) arguments.get(i);
            if ((expr = (XQueryExpression) map.get(argi)) != null) {
                arguments.remove(i);
                ArrayList parenti = argi.getParentExpression();
                if (parenti != null) {
                    int indexi = parenti.indexOf(arg);
                    if (indexi != -1)
                        parenti.remove(indexi);
                    try {
                        XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                        arg.addArguments(i, exprClone);
                    } catch (CloneNotSupportedException e) {
                        throw new XQueryException("Could not clone expression : " + expr);
                    }
                }
            } else
                argi.accept(this);
        }
    }

    //	public void visit(FunctionDefinition arg) throws XQueryException;
    //	public void visit(InstanceOfExpression arg) throws XQueryException;
    //	public void visit(InternalFunctionCall arg) throws XQueryException;
    //	public void visit(ITEExpression arg) throws XQueryException;
    //	public void visit(LibraryFunctionCall arg) throws XQueryException;
    //	public void visit(ListOpAFTERExpression arg) throws XQueryException;
    //	public void visit(ListOpArithExpression arg) throws XQueryException;
    //	public void visit(ListOpBEFOREExpression arg) throws XQueryException;
    //	public void visit(ListOpCompExpression arg) throws XQueryException;
    //	public void visit(ListOpEXCEPTExpression arg) throws XQueryException;
    //	public void visit(ListOpINTERSECTExpression arg) throws XQueryException;
    //	public void visit(ListOpUNIONExpression arg) throws XQueryException;
    public void visit(LocatedExpression arg) throws XQueryException {
        ArrayList steps = arg.getSteps();
        XQueryExpression expr = null;
        boolean change = false;
        for (int i = 0; i < steps.size(); i++) {
            Step stepi = (Step) steps.get(i);
            XQueryExpression stepExpr = stepi.getExpression();
            if ((expr = (XQueryExpression) map.get(stepExpr)) != null) {
                // if (expr instanceof LocatedExpression) {
                steps.remove(i);
                if (stepi.getPredicates() != null) {
                    ArrayList addSteps = expr.getSteps();
                    Step lastStep = (Step) addSteps.get(addSteps.size() - 1);
                    lastStep.addPredicates(stepi.getPredicates());
                }
                if (expr instanceof LocatedExpression) {
                    try {
                        LocatedExpression locClone = (LocatedExpression) expr.clone(module);
                        arg.addStepsAt(i, locClone.getSteps());
                    } catch (CloneNotSupportedException e) {
                        throw new XQueryException("Could not clone expression : " + expr);
                    }
                } else {
                    try {
                        ArrayList list = new ArrayList(1);
                        list.add(new Step(false, Axis.NONE, (XQueryExpression) expr.clone(module), null, arg.getParentModule()));
                        arg.addStepsAt(i, list);
                    } catch (CloneNotSupportedException e) {
                        throw new XQueryException("Could not clone expression : " + expr);
                    }
                }
                change = true;

                /*
                 * } else if (expr instanceof FLWRExpression) { if (((FLWRExpression)expr).getReturnClause() instanceof LocatedExpression) { typeVisitor.setForce(true); LocatedExpression tmpLoc = (LocatedExpression)((FLWRExpression)expr).getReturnClause(); steps.remove(i); if (stepi.getPredicates() != null) { ArrayList addSteps = tmpLoc.getSteps(); Step lastStep = (Step)addSteps.get(addSteps.size()-1); lastStep.addPredicates(stepi.getPredicates()); } tmpLoc.addSteps(steps); tmpLoc.accept(typeVisitor); HashMap tmpMap = new HashMap(1); tmpMap.put(arg,expr); for (int j=0;j <arg.getParentExpression().size();j++) ((XQueryExpression)arg.getParentExpression().get(j)).substitute(tmpMap,typeVisitor); typeVisitor.setForce(false); return; } else if (((FLWRExpression)expr).getReturnClause() instanceof Variable) { typeVisitor.setForce(true); Variable tmpVar = (Variable)((FLWRExpression)expr).getReturnClause(); Step tmpStep = new Step(-1,-1,tmpVar,null,typeVisitor); ArrayList tmpSteps = new
                 * ArrayList(1); tmpSteps.add(tmpStep); steps.remove(i); LocatedExpression tmpLoc = null; if (stepi.getPredicates() != null) { Step tmpPredStep = new Step(-1,-1,tmpVar,stepi.getPredicates(),typeVisitor); ArrayList tmpPredSteps = new ArrayList(1); tmpPredSteps.add(tmpPredStep); tmpLoc = new LocatedExpression(tmpPredSteps,typeVisitor); ((FLWRExpression)expr).addWhereClause(tmpLoc); } tmpLoc = new LocatedExpression(tmpSteps,typeVisitor); tmpLoc.addSteps(steps); tmpLoc.accept(typeVisitor); ((FLWRExpression)expr).setReturnClause(tmpLoc); ((FLWRExpression)expr).accept(typeVisitor); HashMap tmpMap = new HashMap(1); arg.getSteps().add(i,stepi); tmpMap.put(arg,expr); for (int j=0;j <arg.getParentExpression().size();j++) ((XQueryExpression)arg.getParentExpression().get(j)).substitute(tmpMap,typeVisitor); typeVisitor.setForce(false); return; } } else { throw new Exception("Impossible substitution : " + stepExpr + " ( -> " + expr + " ) in " + arg + " , please rewrite your
                 * request"); }
                 */
            }
            stepi.accept(this);
        }
        if (change)
            arg.accept(this.typeVisitor);
    }

    //	public void visit(NodeType arg) throws XQueryException;
    //	public void visit(Parameter arg) throws XQueryException;
    //	public void visit(PrimitiveFunctionCall arg) throws XQueryException;
    //	public void visit(QName arg) throws XQueryException;
    public void visit(QuantifiedExpression arg) throws XQueryException {
        ArrayList variables = arg.getVariables();
        XQueryExpression expr = null;
        for (int i = 0; i < variables.size(); i++) {
            Variable vari = (Variable) variables.get(i);
            XQueryExpression variExpr = vari.getExpression();
            if ((expr = (XQueryExpression) map.get(variExpr)) != null) {
                ArrayList parenti = variExpr.getParentExpression();
                if (parenti != null) {
                    int indexi = parenti.indexOf(arg);
                    if (indexi != -1)
                        parenti.remove(indexi);
                    try {
                        XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                        vari.setExpression(exprClone, false);
                    } catch (CloneNotSupportedException e) {
                        throw new XQueryException("Could not clone expression : " + expr);
                    }
                }
            } else
                variExpr.accept(this);
        }
        XQueryExpression tmpExpr = arg.getExpression();
        if ((expr = (XQueryExpression) map.get(tmpExpr)) != null) {
            try {
                XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                arg.setExpression(exprClone);
            } catch (CloneNotSupportedException e) {
                throw new XQueryException("Could not clone expression : " + expr);
            }
        } else
            tmpExpr.accept(this);
    }

    //	public void visit(RangeExpression arg) throws XQueryException;
    //	public void visit(RootNodeFunctionCall arg) throws XQueryException;
    //	public void visit(SortedExpression arg) throws XQueryException;

    public void visit(Step arg) throws XQueryException {
        XQueryExpression stepExpr = arg.getExpression();
        XQueryExpression expr = null;
        if ((expr = (XQueryExpression) map.get(stepExpr)) != null && !(expr instanceof LocatedExpression)) {
            ArrayList parenti = stepExpr.getParentExpression();
            if (parenti != null) {
                int indexi = parenti.indexOf(arg);
                if (indexi != -1)
                    parenti.remove(indexi);
                try {
                    XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                    arg.setExpression(exprClone);
                } catch (CloneNotSupportedException e) {
                    throw new XQueryException("Could not clone expression : " + expr);
                }
            }
        } else {
            stepExpr.accept(this);
            ArrayList predicates = arg.getPredicates();
            expr = null;
            if (predicates != null)
                for (int i = 0; i < predicates.size(); i++) {
                    XQueryExpression subi = (XQueryExpression) predicates.get(i);
                    if ((expr = (XQueryExpression) map.get(subi)) != null) {
                        predicates.remove(i);
                        ArrayList parenti = subi.getParentExpression();
                        if (parenti != null) {
                            int indexi = parenti.indexOf(arg);
                            if (indexi != -1)
                                parenti.remove(indexi);
                            try {
                                XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                                arg.addPredicate(i, exprClone);
                            } catch (CloneNotSupportedException e) {
                                throw new XQueryException("Could not clone expression : " + expr);
                            }
                        }
                    } else
                        subi.accept(this);
                }
        }
    }

    //	public void visit(TypeSwitchExpression arg) throws XQueryException;
    //	public void visit(UnOpMinusExpression arg) throws XQueryException;
    //	public void visit(Value arg) throws XQueryException;
    //	public void visit(ValueBoolean arg) throws XQueryException;
    //	public void visit(ValueFloat arg) throws XQueryException;
    //	public void visit(ValueInteger arg) throws XQueryException;
    //	public void visit(ValueString arg) throws XQueryException;
    //	public void visit(ValueText arg) throws XQueryException;

    public void visit(Variable arg) throws XQueryException {
        XQueryExpression expr = null;
        XQueryExpression varExpr = arg.getExpression();
        if (varExpr == null)
            return;
        if ((expr = (XQueryExpression) map.get(varExpr)) != null) {
            try {
                XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                arg.setExpression(exprClone, false);
            } catch (CloneNotSupportedException e) {
                throw new XQueryException("Could not clone expression : " + expr);
            }
        }
        // else varExpr.accept(this);
    }

    public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
        XQueryExpression expr = null;
        XQueryExpression tmpExpr = arg.getExpression1();
        if ((expr = (XQueryExpression) map.get(tmpExpr)) != null) {
            try {
                XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                arg.setExpression1(exprClone);
            } catch (CloneNotSupportedException e) {
                throw new XQueryException("Could not clone expression : " + expr);
            }
        } else
            tmpExpr.accept(this);
        tmpExpr = arg.getExpression2();
        if ((expr = (XQueryExpression) map.get(tmpExpr)) != null) {
            try {
                XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                arg.setExpression2(exprClone);
            } catch (CloneNotSupportedException e) {
                throw new XQueryException("Could not clone expression : " + expr);
            }
        } else
            tmpExpr.accept(this);
    }

    //	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;
    public void visit(XQueryExpression arg) throws XQueryException {
    }

    public void visit(XQueryExpressionSequence arg) throws XQueryException {
        ArrayList subExpressions = arg.getSubExpressions();
        XQueryExpression expr = null;
        for (int i = 0; i < subExpressions.size(); i++) {
            XQueryExpression subi = (XQueryExpression) subExpressions.get(i);
            if ((expr = (XQueryExpression) map.get(subi)) != null) {
                subExpressions.remove(i);
                ArrayList parenti = subi.getParentExpression();
                if (parenti != null) {
                    int indexi = parenti.indexOf(arg);
                    if (indexi != -1)
                        parenti.remove(indexi);
                    try {
                        XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                        arg.addSubExpressions(i, exprClone);
                    } catch (CloneNotSupportedException e) {
                        throw new XQueryException("Could not clone expression : " + expr);
                    }
                }
            } else
                subi.accept(this);
        }
    }

    //	public void visit(XQueryFile arg) throws XQueryException;
    //	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException;
    //	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException;

    public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
        XQueryExpression expr = null;
        XQueryExpression tmpExpr = arg.getExpression();
        if ((expr = (XQueryExpression) map.get(tmpExpr)) != null) {
            try {
                XQueryExpression exprClone = (XQueryExpression) expr.clone(module);
                arg.setExpression(exprClone);
            } catch (CloneNotSupportedException e) {
                throw new XQueryException("Could not clone expression : " + expr);
            }
        } else
            tmpExpr.accept(this);
    }

    //	public void visit(XQueryUnit arg) throws XQueryException;
    //	public void visit(XQueryVoid arg) throws XQueryException;

    // primitive functions

    //	public void visit(FunctionATTRIBUTE arg) throws XQueryException ;
    //	public void visit(FunctionAVG arg) throws XQueryException ;
    //	public void visit(FunctionCOLLECTION arg) throws XQueryException;
    //	public void visit(FunctionCONTAINS arg) throws XQueryException;
    //	public void visit(FunctionCOUNT arg) throws XQueryException;
    //  public void visit(FunctionCURRENT_DATETIME arg) throws XQueryException
    //	public void visit(FunctionDATA arg) throws XQueryException;
    //	public void visit(FunctionDISTINCT_VALUES arg) throws XQueryException;
    //	public void visit(FunctionDOCUMENT arg) throws XQueryException;
    //	public void visit(FunctionEMPTY arg) throws XQueryException;
    //	public void visit(FunctionFILTER arg) throws XQueryException;
    //	public void visit(FunctionMAX arg) throws XQueryException;
    //	public void visit(FunctionMIN arg) throws XQueryException;
    //	public void visit(FunctionNAME arg) throws XQueryException;
    //	public void visit(FunctionNOT arg) throws XQueryException;
    //	public void visit(FunctionNUMBER arg) throws XQueryException;
    //	public void visit(FunctionPART arg) throws XQueryException;
    //	public void visit(FunctionPOSITION arg) throws XQueryException;
    //	public void visit(FunctionSHALLOW arg) throws XQueryException;
    //	public void visit(FunctionSTRING arg) throws XQueryException;
    //	public void visit(FunctionSTRINGVALUE arg) throws XQueryException;
    //	public void visit(FunctionSUM arg) throws XQueryException;
    //	public void visit(FunctionTEXT arg) throws XQueryException;
}