/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */


package  org.xquark.xquery.parser;

import java.util.ArrayList;

public abstract class FunctionCall extends XQueryExpression implements Cloneable {

    private static final String RCSRevision = "$Revision: 1.8 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";

    // cannot be null or empty string
    protected QName funcName = null;
    protected ArrayList arguments = null;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public FunctionCall(QName name,ArrayList arguments) throws XQueryException {
        setFuncName(name);
        setArguments(arguments);
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public XQueryExpression getArgument(int pos) {
        if (arguments == null) return null;
        if (pos >= arguments.size()) return null;
        return (XQueryExpression) this.arguments.get(pos);
    }
    
    public ArrayList getArguments() { return arguments; }
    
    public void setArguments(ArrayList arguments) throws XQueryException {
        this.arguments = arguments;
        if (this.arguments != null)
            for (int i=0;i<this.arguments.size();i++) {
                XQueryExpression expr = (XQueryExpression)this.arguments.get(i);
                expr.setParentExpression(this);
                expr.setParentModule(parentModule);
            }
    }
        
    public QName getFuncName() { return funcName; }
    public void setFuncName(QName funcName) throws XQueryException {
        if (funcName == null || funcName.getName().equals("")) throw new XQueryException("funcName of FunctionCall cannot be null");
        this.funcName = funcName;
    }
    
    public void addArguments(XQueryExpression subExpression) throws XQueryException {
        if (arguments == null) arguments = new ArrayList(1);
        arguments.add(subExpression);
        subExpression.setParentExpression(this);
        subExpression.setParentModule(parentModule);
    }
    
    public void addArguments(int index,XQueryExpression subExpression) throws XQueryException {
        if (arguments == null || arguments.size() < index) return;
        arguments.add(index,subExpression);
        subExpression.setParentExpression(this);
        subExpression.setParentModule(parentModule);
    }
        
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    public void addParentExpression(XQueryExpression parentExpression) {
        addParentExpression(parentExpression) ;
        funcName.addParentExpression(parentExpression);
        if (arguments != null)
            for (int i=0;i<arguments.size();i++)
                ((XQueryExpression)arguments.get(i)).addParentExpression(parentExpression);
    }    
    
}



