/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */


package  org.xquark.xquery.parser;

import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.TypeException;

public class ListOpArithExpression extends XQueryListBinaryOperatorExpression implements Cloneable {
    
    private static final String RCSRevision = "$Revision: 1.7 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    // cannot be -1
    protected int operator = -1;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public ListOpArithExpression(XQueryExpression expression1, XQueryExpression expression2, int operator, XQueryModule parentModule) throws TypeException, XQueryException {
        super(expression1,expression2);
        this.operator = operator;
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public int getOperator() { return operator; }
    public void setOperator(int operator) throws XQueryException {
        if (operator == -1 || operator > Constants.MAX_ARITHMETICS)
            throw new XQueryException("operator of ListOpArithExpression has incorrect value " + operator);
        this.operator = operator ;
    }
            
}



