/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.xquery.parser.hinttree.HintTree;
import org.xquark.xquery.typing.TypeException;
import org.xquark.util.NamespaceContext;

public interface ParserFactoryInterface {
	
	AttributeValuePair createAttributeValuePair(XQueryExpression attributeName, XQueryExpression attributeValue, boolean isXmlns, XQueryModule parentModule) throws TypeException, XQueryException;

	BinOpANDExpression createBinOpANDExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	BinOpORExpression createBinOpORExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	CData createCData(String data, XQueryModule parentModule) throws TypeException, XQueryException;

	CaseClauseExpression createCaseClauseExpression(SequenceType sequenceType, Variable variable, XQueryExpression returnExpression, XQueryModule parentModule) throws TypeException, XQueryException;

	CastTreatExpression createCastTreatExpression(int kind, SequenceType sequenceType, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	Document createDocument(XQueryExpression element, XQueryModule parentModule) throws TypeException, XQueryException;

	Element createElement(XQueryExpression startTag, HashMap localPrefixes, ArrayList vectAttributes, ArrayList vectExpressions, XQueryModule parentModule) throws TypeException, XQueryException;

	FLWRExpression createFLWRExpression(ArrayList bindings, HintTree hintClause, XQueryExpression whereClause, ArrayList orderbys, XQueryExpression returnClause, XQueryModule parentModule) throws TypeException, XQueryException;

	FLWRExpression createFLWRExpression(ArrayList bindings, XQueryExpression whereClause, ArrayList orderbys, XQueryExpression returnClause, XQueryModule parentModule) throws TypeException, XQueryException;

	FLWRExpression createFLWRExpression(ArrayList bindings, XQueryExpression whereClause, XQueryExpression returnClause, XQueryModule parentModule) throws TypeException, XQueryException;

	FunctionDeclaration createFunctionDeclaration(QName funcName, ArrayList tmpVect, SequenceType sequenceType, EnclosedExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;
	
	HintTree createHintTree(byte type, byte side, Object left, Object right);

	ITEExpression createITEExpression(XQueryExpression expressionIf, XQueryExpression expressionThen, XQueryExpression expressionElse, XQueryModule parentModule) throws TypeException, XQueryException;

	InstanceOfExpression createInstanceOfExpression(XQueryExpression expression, SequenceType sequenceType, XQueryModule parentModule) throws TypeException, XQueryException;

	InternalFunctionCall createInternalFunctionCall(QName funcName, ArrayList argsVect, XQueryModule parentModule) throws TypeException, XQueryException;

	ItemType createItemType(XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	LibraryFunctionCall createLibraryFunctionCall(QName funcName, ArrayList argsVect, FunctionDeclaration functionDefinition, XQueryModule parentModule) throws TypeException, XQueryException;

	ListOpAFTERExpression createListOpAFTERExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	ListOpArithExpression createListOpArithExpression(XQueryExpression mainExpression, XQueryExpression expression, int kind, XQueryModule parentModule) throws TypeException, XQueryException;

	ListOpBEFOREExpression createListOpBEFOREExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	ListOpCompExpression createListOpCompExpression(XQueryExpression mainExpression, XQueryExpression expression, int kind, XQueryModule parentModule) throws TypeException, XQueryException;

	ListOpEXCEPTExpression createListOpEXCEPTExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	ListOpINTERSECTExpression createListOpINTERSECTExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	ListOpUNIONExpression createListOpUNIONExpression(XQueryExpression mainExpression, XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	LocatedExpression createLocatedExpression(ArrayList pathVect, boolean inPredicate, XQueryModule parentModule) throws TypeException, XQueryException;

	ElementTest createElementTest(QName elementQName, QName typeQName, boolean isOptional, XQueryModule parentModule) throws TypeException, XQueryException;

	AttributeTest createAttributeTest(QName elementQName, QName typeQName, XQueryModule parentModule) throws TypeException, XQueryException;

	SchemaElementTest createSchemaElementTest(QName elementQName, XQueryModule parentModule) throws TypeException, XQueryException;

	SchemaAttributeTest createSchemaAttributeTest(QName AttributeQName, XQueryModule parentModule) throws TypeException, XQueryException;

	NodeTest createNodeTest(byte kind, XQueryExpression arg, XQueryModule parentModule) throws TypeException, XQueryException;

	QName createQName(String namespace, String prefixname, String typeName, XQueryModule parentModule, NamespaceContext namespaceContext) throws TypeException, XQueryException;

	QuantifiedExpression createQuantifiedExpression(int kind, ArrayList variables, XQueryExpression expressionSat, XQueryModule parentModule) throws TypeException, XQueryException;

	RangeExpression createRangeExpression(XQueryExpression expression1, XQueryExpression expression2, XQueryModule parentModule) throws TypeException, XQueryException;

	SequenceType createSequenceType(ItemType itemType, int occurrence, XQueryModule parentModule) throws TypeException, XQueryException;

	SortedExpression createSortedExpression(XQueryExpression expression, ArrayList tmpVect, boolean stable, XQueryModule parentModule) throws TypeException, XQueryException;

	Step createStep(boolean separator, byte axis, XQueryExpression expression, ArrayList predicates, XQueryModule parentModule) throws TypeException, XQueryException;

	TypeSwitchExpression createTypeSwitchExpression(XQueryExpression switchExpression, HashMap cases, Variable defaultVariable, XQueryExpression defaultExpression, XQueryModule parentModule) throws TypeException, XQueryException;

	UnOpMinusExpression createUnOpMinusExpression(XQueryExpression expression, boolean minus, XQueryModule parentModule) throws TypeException, XQueryException;

	ValidateExpression createValidateExpression(int validationType, XQueryExpression validateExpression, XQueryModule parentModule) throws TypeException, XQueryException;

	ValueBoolean createValueBoolean(String str, XQueryModule parentModule) throws TypeException, XQueryException;

	ValueDecimal createValueDecimal(String str, XQueryModule parentModule) throws TypeException, XQueryException;

	ValueDouble createValueDouble(String str, XQueryModule parentModule) throws TypeException, XQueryException;

	ValueInteger createValueInteger(String str, XQueryModule parentModule) throws TypeException, XQueryException;

	ValueString createValueString(String str, XQueryModule parentModule) throws TypeException, XQueryException;

	ValueText createValueText(String str, XQueryModule parentModule) throws TypeException, XQueryException;

//	ExternalVariable createExternalVariable(QName varName, XQueryModule parentModule) throws TypeException, XQueryException;

	Variable createVariable(QName var, XQueryModule parentModule) throws TypeException, XQueryException;

	XMLComment createXMLComment(XQueryExpression value, XQueryModule parentModule) throws TypeException, XQueryException;

	ComputedNamespace createComputedNamespace(String ncname, XQueryExpression value, XQueryModule parentModule) throws TypeException, XQueryException;

	ComputedText createComputedText(XQueryExpression value, XQueryModule parentModule) throws TypeException, XQueryException;

	XMLProcessingInstruction createXMLProcessingInstruction(XQueryExpression PITarget, XQueryExpression data, XQueryModule parentModule) throws TypeException, XQueryException;

	XQueryModule createXQueryModule();

	ImportSchema createImportSchema(boolean hasPrefix, String prefix, String name, String location);

	ImportModule createImportModule(String prefix, String name, String location, XQueryModule module);

	ParenthesizedExpression createParenthesizedExpression(XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;

	XQueryExpressionSequence createXQueryExpressionSequence(ArrayList expressions, XQueryModule parentModule) throws TypeException, XQueryException;

	EnclosedExpression createEnclosedExpression(XQueryExpression expression, XQueryModule parentModule) throws TypeException, XQueryException;
}
