/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */


package  org.xquark.xquery.parser;

import org.xquark.xquery.typing.TypeException;


public class ValueBoolean extends Value implements Cloneable {
    private static final String RCSRevision = "$Revision: 1.8 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public ValueBoolean(String value, XQueryModule parentModule) throws TypeException, XQueryException {
        super(value);
        setValue(value);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public boolean getBooleanValue() { if (value.equalsIgnoreCase("true")) return true; else return false; }
	public Object getObjectValue() { return null; } 
    public void setValue(String value) throws XQueryException {
        // must be FALSE or TRUE
        if (!value.equalsIgnoreCase("true") && !value.equalsIgnoreCase("false")) throw new XQueryException("value of ValueBoolean must be a Valid Boolean");
        this.value = value;
    }
    public void setValue(boolean value) throws XQueryException {
        setValue(String.valueOf(value));
    }
            
}



