/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2004 XQuark Group
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xquery.parser.util.Constants;
import org.xquark.xquery.typing.TypeException;

public class Variable extends XQueryExpression implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.36 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	protected SequenceType typeDeclaration = null;
	// cannot be null or empty string
	protected QName varName = null;
	protected XQueryExpression expression = null;
	// cannot be -1
	protected int bindingtype = -1;
	// paths beginning with variable
	protected ArrayList xpaths = null;
	protected FLWRExpression parentFLWR = null;
	protected Variable positionVar = null;
	protected byte creator = 0;
	protected boolean isDummy = false;
	
	protected boolean isExternal = false;
	protected Object externalValue = null;

	public static final byte PARSER_CREATOR = 0;
	public static final byte NORM_CREATOR = 1;
	public static final byte RES_CREATOR = 2;
	public static final byte CAN_CREATOR = 3;
	public static final byte CLONE_CREATOR = 4;
	public static final byte DEPMAN_CREATOR = 5;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public Variable(QName varName, XQueryModule parentModule) throws TypeException, XQueryException {
		setVarName(varName);
		setParentModule(parentModule);
		//		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
		//			accept(parentModule.getStaticContext().getTypeVisitor());
	}

	// #############################################################################
	// ATTRIBUTES ACCESS STUFF
	// #############################################################################

	public String toString() {
		return toString(null, false, false, false);
	}

	public String getStringValue() {
		return getStringValue(false, false);
	}

	public FLWRExpression getParentFLWRExpression() {
		return parentFLWR;
	}

	public void setParentFLWRExpression(FLWRExpression parentFLWR) {
		this.parentFLWR = parentFLWR;
	}

	public void setParentExpression(XQueryExpression expression) {
		if (expression == null)
			return;
		if (parentExpression == null)
			parentExpression = new ArrayList(1);
		// else parentExpression.clear();
		if (!parentExpression.contains(expression))
			parentExpression.add(expression);
	}

	public void setExpression(XQueryExpression expression, boolean reType) throws TypeException, XQueryException {
		if (expression != this.expression) {
			this.expression = expression;
			if (this.expression != null) {
				expression.setParentExpression(this);
			}
			if (reType && parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
				accept(parentModule.getStaticContext().getTypeVisitor());
		}
	}

	//	public void setExpression(XQueryExpression expression) throws XQueryException {
	//		this.expression = expression;
	//		if (this.expression != null) {
	//			expression.setParentExpression(this);
	//			expression.setParentModule(parentModule);
	//		}
	//	}

	public void setBindingType(int bindingtype) throws XQueryException {
		if (bindingtype == -1 || bindingtype > Constants.MAX_BINDINGTYPE)
			throw new XQueryException("bindingtype of Variable has incorrect value " + bindingtype);
		this.bindingtype = bindingtype;
	}

	public Variable getPositionVar() {
		return positionVar;
	}

	public XQueryExpression getExpression() {
		return expression;
	}
	public int getBindingType() {
		return bindingtype;
	}

	public boolean isInput() {
		if (expression == null)
			return false;
		if (expression instanceof InputFunctionCall || (expression instanceof LocatedExpression && ((LocatedExpression) expression).getExpression() instanceof InputFunctionCall))
			return true;
		return false;
	}

	public ArrayList getSubPathList() {
		return xpaths;
	}

	// attention if a variable is present this method only returns the variable
	// attention this method erases the NodeTypes found at the end of a path
	// DANGER do not use this method !!!!
	public ArrayList getUniqueSubPathList() {
		if (xpaths == null)
			return null;
		ArrayList tmpList = null;
		if (xpaths.contains(this)) {
			tmpList = new ArrayList(1);
			tmpList.add(this);
		} else {
			tmpList = new ArrayList();
			for (int i = 0; i < xpaths.size(); i++) {
				XQueryExpression tmpExpr = (XQueryExpression) xpaths.get(i);
				if (tmpExpr instanceof LocatedExpression) {
					LocatedExpression tmpLoc = (LocatedExpression) tmpExpr;
					ArrayList steps = tmpLoc.getSteps();
					Step lastStep = tmpLoc.getStepNum(steps.size() - 1);
					if (lastStep.getExpression() instanceof NodeTest) {
						tmpLoc.setNodeAccessor(lastStep.getStepKind());
						steps.remove(steps.size() - 1);
					}
				}
				if (tmpList.isEmpty())
					tmpList.add(tmpExpr);
				else {
					boolean found = false;
					for (int j = 0; j < tmpList.size(); j++)
						if (tmpExpr.getStringValue().equals(((XQueryExpression) tmpList.get(j)).getStringValue())) {
							found = true;
							break;
						}
					if (!found)
						tmpList.add(tmpExpr);
				}
			}
		}
		return tmpList;
	}

	/**
	 * Returns the PathNode list of the defining expression
	 *
	 */
	public ArrayList getXTrees() {
	    if (expression == null)
	        return null;
		return expression.getXTrees();
	}

	//  W3C WD 16 August 2002
	public void setTypeDeclaration(SequenceType typeDeclaration) {
		this.typeDeclaration = typeDeclaration;
		if (typeDeclaration != null) {
			this.typeDeclaration.setParentModule(parentModule);
			this.typeDeclaration.setParentExpression(this);
			qtype = typeDeclaration.getQType();
		}
	}
	public SequenceType getTypeDeclaration() {
		return typeDeclaration;
	}

	public void setVarName(QName varName) throws XQueryException {
		if (varName == null || varName.getName().equals(""))
			throw new XQueryException("name of Variable cannot be null or empty string");
		this.varName = varName;
		this.varName.setParentModule(parentModule);
		this.varName.setParentExpression(this);
	}

	public QName getVarName() {
		return varName;
	}
	public String getName() {
		return varName.getStringValue();
	}
	public void setName(String name) {
		varName.setName(name);
	}

	public void addSubPath(XQueryExpression expr) {
		if (xpaths == null)
			xpaths = new ArrayList();
		if (expr != null && !xpaths.contains(expr))
			xpaths.add(expr);
	}

	public byte getCreator() {
		return creator;
	}

	public void setCreator(byte creator) {
		this.creator = creator;
	}

	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		varName.addParentExpression(parentExpression);
		if (expression != null)
			expression.addParentExpression(parentExpression);
	}

	public void isDummy(boolean b) {
		isDummy = b;
	}
	
	public boolean isDummy() {
		return isDummy;		
	}
	
	public boolean isExternal() {
	    return isExternal;
	}
	
	public void isExternal(boolean isExternal) {
	    this.isExternal = isExternal;
	}
	
	public Object getExternalValue() {
	    return externalValue;
	}
	
	public void setExternalvalue(Object value, byte valueType) {
	    externalValue = value;
	}

	public boolean startsWithVariable() {
		return true;
	}

}
