/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.parser.primitivefunctions.fnfunctions;

import java.util.ArrayList;

import org.xquark.xquery.parser.*;
import org.xquark.xquery.typing.TypeException;

public class FunctionCOLLECTION extends InputFunctionCall {
    
    private static final String RCSRevision = "$Revision: 1.9 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private final static String STAR = "*";
	private ArrayList sources = new ArrayList(1);
	private String collectionName = STAR;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public FunctionCOLLECTION(ArrayList args, XQueryModule parentModule) throws TypeException, XQueryException {
		super(new QName("collection",null),null);
		if (args == null) 
			args = new ArrayList(1);
		if (args.size() == 0) {
			args.add(new ValueString("*:*",parentModule));
		}
        setArguments(args);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
    }
    
    public void setArguments(ArrayList arguments) throws XQueryException {
        // verify that arguments are strings
        // of the following format "abc" or "*.abc" or "*" or "abc.*" or "abc.def" or "*.*"
        // transforms () -> ("*.*")
        // transforms ("*") -> ("*.*")
        // transforms ("abc") -> ("*.abc")
        
        if (arguments == null || arguments.size() == 0) {
        	return;
        } else if (arguments.size() == 1) {
            for (int i=0;i<arguments.size();i++) {
                ValueString valStr = null;
                try {
                    valStr = (ValueString)arguments.get(i);
                }
                catch (ClassCastException e) {
                    throw new XQueryException("Argument of collection function must be a literal string");
                }
                String arg = valStr.getValue();
				// change LARS 07/04/04
				int colonIndex = -1;
				int startIndex = 0;
				while ((colonIndex = arg.indexOf(":",startIndex)) != -1) {
					sources.add(arg.substring(startIndex,colonIndex));
					startIndex = colonIndex+1;
				}
				collectionName = arg.substring(startIndex);
				if (sources.isEmpty()) {
					sources.add(STAR);
					valStr.setValue("*:"+arg);
				}
            }
        }
        else {
			throw new XQueryException("Function collection should have at most one argument");
        }
        super.setArguments(arguments);
    }
    
    public String getCollectionName() {
        return collectionName;
    }
    public String getSourceName() {
    	return (String)sources.get(0);
     }
    public void eraseSourceName() throws XQueryException {
    	sources.remove(0);
		if (sources.isEmpty()) {
			sources.add(STAR);
		}
		StringBuffer buf = new StringBuffer();
		for (int i=0;i<sources.size();i++) {
			buf.append(sources.get(i));
			buf.append(":");
		}
		buf.append(collectionName);
    	((ValueString)arguments.get(0)).setValue(buf.toString());
    }
        
}
