/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */


package org.xquark.xquery.parser.primitivefunctions.fnfunctions;

import java.util.ArrayList;

import org.xquark.xquery.parser.*;
import org.xquark.xquery.typing.TypeException;

/*
Returns the local part of the name of $srcval as a string that will either be the zero-length string, or will have the lexical form of an NCName.
If the argument is omitted, it defaults to the context node. 
If there is no context node (that is, if the context item is not a node), the function returns the zero-length string.
If the argument is supplied and is the empty sequence, the function returns the zero-length string.
If the target node has no name (that is, if it is a document node, a comment, or a text node), the function returns the zero-length string.
Otherwise, the value returned will be the local part of the expanded-QName of the target node (as determined by the name accessor in the data model). 
This will be a string whose lexical form is an NCName.
*/
public class FunctionNAMESPACE_URI extends PrimitiveFunctionCall {
    
    private static final String RCSRevision = "$Revision: 1.7 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public FunctionNAMESPACE_URI(ArrayList args, XQueryModule parentModule) throws TypeException, XQueryException {
        super(new QName("namespace-uri",null),args);
		setParentModule(parentModule);
		if (parentModule != null && parentModule.getStaticContext().getTypeVisitor() != null)
			accept(parentModule.getStaticContext().getTypeVisitor());
    }
    
    // this function should be called with one argument
    public void setArguments(ArrayList arguments) throws XQueryException {
        if (arguments == null || arguments.size() != 1) throw new XQueryException("Only partial specification, function namespace-uri should have one argument");
        super.setArguments(arguments);
    }
        
}


