/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */


package org.xquark.xquery.parser.util;


public interface Constants {


    
    public final static int NOTHING = -1;
    
    /**
     * These constants represent the
     * different <code>AxisSpecifier</code>
     * in <i>XPath</i>.
     */
    
    // separators
    public final static String[] SEPARATORSTRINGS = {
        "/",
//        "//",
    };
    
    public final static int CHILD_SEPARATOR = 0;
    // public final static int DESCENDANT_SELF_SEPARATOR = 1;
    
    public final static int MAX_SEPARATOR = 0;
    
    /**
     * These constants represent the
     * quantifiers used in <i>Xpath</i>
     * expressions.
     */
    public final static String[] QUANTIFIERSSTRINGS = {
        "some",
        "every"
    };
    
    public final static int SOME_QUANTIFIER = 0;
    public final static int EVERY_QUANTIFIER = 1;
    
    public final static int MAX_QUANTIFIER = 1;
    
    
    /**
     * These constants represent the
     * comparison operators used in
     * <i>Quilt</i>.
     */
    public final static String[] COMPOPSSTRINGS = {
        "=",
        ">",
        ">=",
        "<=",
        "<",
        "!=",

		"eq",
		"gt",
		"ge",
		"le",
		"lt",
		"ne",

		"is",
		"<<",
		">>"
    };
    
    public final static int EQ_COMPOP  = 0;
    public final static int GT_COMPOP  = 1;
    public final static int GEQ_COMPOP = 2;
    public final static int LEQ_COMPOP = 3;
    public final static int LT_COMPOP  = 4;
    public final static int NEQ_COMPOP = 5;
	// danger keep same order as above!!!
	public final static int EQ_VALUECOMP = 6;
	public final static int GT_VALUECOMP = 7;
	public final static int GE_VALUECOMP = 8;
	public final static int LE_VALUECOMP = 9;
	public final static int LT_VALUECOMP = 10;
	public final static int NE_VALUECOMP = 11;

    public final static int IS_NODECOMP = 12;
	public final static int LEAST_NODECOMP = 13;
	public final static int GREATEST_NODECOMP = 14;
       
	public final static int MAX_COMPOP = 14;
	
//    /**
//     * The entry in this array gives the
//     * key of the reverse comparison
//     * operator in the array <code>COMPOPS</code>.
//     */
//    public final static int[] COMPOPS_REVERSE = {
//        1,
//        0,
//        3,
//        2,
//        4,
//        5,
//    };
    
    /**
     * These constants represent the
     * binary operators on node-lists
     * used in <i>Quilt</i>.
     */
    
    public final static String[] NODELISTOPERATORSSTRINGS = {
        " | ",
        " union ",
        " after ",
        " before ",
        " except ",
        " or ",
        " and ",
        " intersect "
    };
    
    public final static int UNION_NODELISTOPERATOR = 0;
    public final static int INTER_NODELISTOPERATOR = 1;
    public final static int AFTER_NODELISTOPERATOR = 2;
    public final static int BEFORE_NODELISTOPERATOR = 3;
    public final static int EXCEPT_NODELISTOPERATOR = 4;
    public final static int OR_NODELISTOPERATOR = 5;
    public final static int AND_NODELISTOPERATOR = 6;
    public final static int INTERSECT_NODELISTOPERATOR = 7;
    
    public final static int MAX_NODELISTOPERATOR = 7;
    
    /**
     * These constants refer to the
     * different types of basic
     * <i>Quilt</i> values used in our
     * evaluation semantics.
     */
    
    // arithmetic operations
    
    public final static String[] ARITHMETICSSTRINGS = {
        "+",
        "-",
        "*",
        "/",
        "mod",
		"idiv"
    };
    
    public final static int PLUS_ARITHMETICS = 0;
    public final static int MINUS_ARITHMETICS = 1;
    public final static int MULTIPLY_ARITHMETICS = 2;
    public final static int DIVIDE_ARITHMETICS = 3;
    public final static int MODULO_ARITHMETICS = 4;
	public final static int INTEGER_DIVIDE_ARITHMETICS = 5;
    
    public final static int MAX_ARITHMETICS = 5;
    
    // typing operations
    
    public final static String[] TYPINGSTRINGS = {
        "cast",
        "treat",
    };
    
    public final static int CAST_TYPING = 0;
    public final static int TREAT_TYPING = 1;
    
    public final static int MAX_TYPING = 1;
    
    // order operations
    public final static String[] ORDERSSTRINGS = {
        "ascending",
        "descending",
		"greatest",
		"least",
		"place_order"
    };
    
    public final static int ASCENDING_ORDER = 0;
    public final static int DESCENDING_ORDER = 1;
    public final static int GREATEST_ORDER = 2;
    public final static int LEAST_ORDER = 3;
	public final static int PLACE_ORDER = 4;
    
    public final static int MAX_ORDER = 4;
    
    // binding type
    public final static int LET_BINDINGTYPE = 0;
    public final static int FOR_BINDINGTYPE = 1;
    public final static int QUANTIFIER_BINDINGTYPE = 2;
    public final static int FUNCTION_BINDINGTYPE = 3;
    public final static int OUTERFOR_BINDINGTYPE = 4;
    public final static int TYPESWITCH_BINDINGTYPE = 5;
    public final static int OUTERLET_BINDINGTYPE = 6;
    public final static int POSITION_BINDINGTYPE = 7;
	public final static int DECLARATION_BINDINGTYPE = 8;
	public final static int PARAMETER_BINDINGTYPE = 9;
    
    public final static int MAX_BINDINGTYPE = 9;
    
    // declaration type
    public final static int NAMESPACE_DECL = 0;
    public final static int SCHEMA_DECL = 1;
    
    public final static int MAX_DECL = 1;
    
    // WRAPPER SQL STUFF
    
    // Boolean Operators
    public final static int AND = 0;
    public final static int OR = 1;
    public final static int NOT = 2;
    public final static String[] BOOLEAN_OPERATORS = {
        "and",
        "or",
        "not"
    };
    // Aggregation functions
    public final static int AVG = 0;
    public final static int COUNT = 1;
    public final static int MAX = 2;
    public final static int MIN = 3;
    public final static int SUM = 4;
    public final static String[] AGGREGATEOPS = {
        "avg",
        "count",
        "max",
        "min",
        "sum"
    }	;
    
	// Occurrence Indicator
	public final static String[] OCCURRENCE_INDICATORS = {
		"*",
		"+",
		"?"
	};
    public final static int OCCURRENCE_STAR = 0;
    public final static int OCCURRENCE_PLUS = 1;
    public final static int OCCURRENCE_QUESTION_MARK = 2;
	public final static int MAX_OCCURRENCE_INDICATORS = 3;
    
    
	// Validation
	public final static String[] VALIDATIONSTRINGS = {
		"lax",
		"strict",
		"skip"
	};
	public final static int VALIDATION_LAX = 0;
	public final static int VALIDATION_STRICT = 1;
	public final static int VALIDATION_SKIP = 2;
	public final static int VALIDATION_PRESERVE = 3;

	public final static String[] SCOPE_STRINGS = {
		"global",
		"context"
	};
	public final static int SCOPE_GLOBAL = 0;
	public final static int SCOPE_CONTEXT = 1;

	// xmlspace
	public final static String[] XMLSPACESTRINGS = {
		"preserve",
		"strip"
	};
	public final static int XMLSPACE_PRESERVE = 0;
	public final static int XMLSPACE_STRIP = 1;
    
	// copy namespace
	public final static String[] COPYNAMESPACESTRINGS = {
	        "preserve",
	        "no-preserve",
	        "inherit",
	        "no-inherit"
	};
	public final static int COPYNAMESPACE_PRESERVE = 0;
	public final static int COPYNAMESPACE_NO_PRESERVE = 1;
	public final static int COPYNAMESPACE_INHERIT = 2;
	public final static int COPYNAMESPACE_NO_INHERIT = 3;
	
    // schema context path
    public final static int NOAXIS = 0;
	public final static int WITHAXIS = 1;
	public final static int WITHATTAXIS = 2;
	public final static int WITHTYPE = 3;
	
	// Validation
	public final static String[] ORDERSTRINGS = {
		"ordered",
		"unordered"
	};
	public final static int ORDERED = 0;
	public final static int UNORDERED = 1;
	
}








