/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2004 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.parser.util;

public class ErrorCodes {

	static public final String XP0001 = "err:XP0001";
	//It is a static error if analysis of an expression relies on some component of the static context that has not been assigned a value.

	static public final String XP0002 = "err:XP0002"; 
	//It is a dynamic error if evaluation of an expression relies on some part of the dynamic context that has not been assigned a value.

	static public final String XP0003 = "err:XP0003"; 
	//It is a static error if an expression is not a valid instance of the grammar defined in A.1 EBNF.

	static public final String XP0004 = "err:XP0004"; 
	//During the analysis phase, it is a type error if the static typing feature is in effect and an expression is found to have a static type that is not appropriate for the context in which the expression occurs.

	static public final String XP0005 = "err:XP0005"; 
	//During the analysis phase, it is a type error if the static typing feature is in effect and the static type assigned to an expression other than the expression () is the empty type.

	static public final String XP0006 = "err:XP0006"; 
	//During the evaluation phase, it is a type error if a value does not match a required type as specified by the matching rules in 2.4.4 SequenceType Matching.

	static public final String XP0007 = "err:XP0007"; 
	//It is a type error if the fn:data function is applied to a node whose type annotation denotes a complex type with non-mixed complex content.

	static public final String XP0008 = "err:XP0008"; 
	//It is a static error if an expression refers to a type name, function name, namespace prefix, or variable name that is not defined in the static context.

	static public final String XP0009 = "err:XQ0009"; 
	//An implementation that does not support the Schema Import Feature may raise a static error if a prolog contains a schema import.

	static public final String XP0010 = "err:XQ0010"; 
	//An implementation that does not support the Full Axis Feature may raise a static error if a path expression references an unsupported axis (ancestor, ancestor-or-self, following, following-sibling, preceding, or preceding-sibling).

	static public final String XP0012 = "err:XQ0012"; 
	//If the Schema Import Feature is in effect, it is a static error if the set of definitions contained in all imported schemas do not satisfy the conditions for schema validity specified in Sections 3 and 5 of [XML Schema] Part 1. In particular, the definitions must be valid, they must be complete, and they must be unique -- that is, the pool of definitions must not contain two or more schema components with the same name and target namespace.

	static public final String XP0013 = "err:XQ0013"; 
	//It is a static error if an implementation supports a pragma and the implementation determines that the PragmaContents are invalid.

	static public final String XP0014 = "err:XQ0014"; 
	//It is a static error if an implementation does not support a must-understand extension or an implementation does support a must-understand extension and the implementation determines that the ExtensionContents are invalid.

	static public final String XP0015 = "err:XQ0015"; 
	//It is a static error if the XQuery Flagger is enabled and the query contains a must-understand extension.

	static public final String XP0016 = "err:XQ0016";
	//An implementation that does not support the Module Feature raises a static error if it encounters a module declaration or a module import.

	static public final String XP0017 = "err:XP0017"; 
	//It is a static error if the expanded QName and number of arguments in a function call do not match the name and arity of an in-scope function in the static context.

	static public final String XP0018 = "err:XP0018"; 
	//It is a static error for an expression to depend on the focus when the focus is undefined.

	static public final String XP0019 = "err:XP0019"; 
	//It is a type error if the result of a step expression (StepExpr) is not a sequence of nodes.

	static public final String XP0020 = "err:XP0020"; 
	//It is a type error if in an axis expression, the context item is not a node.

	static public final String XP0021 = "err:XP0021"; 
	//It is a dynamic error if a value in a cast expression cannot be cast to the required type.

	static public final String XP0022 = "err:XQ0022"; 
	//It is a static error if the value of a namespace declaration attribute is not a literal string.

	static public final String XP0023 = "err:XQ0023"; 
	//It is a type error if the content sequence in an element constructor contains a document node.

	static public final String XP0024 = "err:XQ0024"; 
	//It is a type error if the content sequence in an element constructor contains an attribute node following a node that is not an attribute node or a namespace node.

	static public final String XP0025 = "err:XQ0025"; 
	//It is a dynamic error if two or more attribute values in the content sequence of an element constructor have the same name.

	static public final String XP0026 = "err:XQ0026"; 
	//In an element-constructor expression, it is a dynamic error if the validation mode is strict and the in-scope element declarations do not contain an element declaration whose unique name matches the name of the constructed element.

	static public final String XP0027 = "err:XQ0027"; 
	//In an element-constructor or validate expression, it is a dynamic error if validation fails.

	static public final String XP0028 = "err:XQ0028"; 
	//It is a type error if the content sequence in a document constructor contains a document, attribute, or namespace node.

	static public final String XP0029 = "err:XP0029"; 
	//It is a dynamic error in a cast expression if the input value does not satisfy the facets of the target type.

	static public final String XP0030 = "err:XQ0030"; 
	//It is a type error if the argument of a validate expression does not evaluate to exactly one document or element node.

	static public final String XP0031 = "err:XQ0031"; 
	//It is a static error if the version number specified in a version declaration is not supported by the implementation.

	static public final String XP0032 = "err:XQ0032"; 
	//A static error is raised if the query prolog contains multiple declarations for the base URI.

	static public final String XP0033 = "err:XQ0033"; 
	//It is a static error if the query prolog contains multiple declarations for the same namespace prefix.

	static public final String XP0034 = "err:XQ0034"; 
	//It is a static error if more than one function declared or imported by a module has the same expanded QName.

	static public final String XP0035 = "err:XQ0035"; 
	//It is a static error to import two schemas that both define the same name in the same symbol space and in the same scope.

	static public final String XP0036 = "err:XQ0036"; 
	//It is a type error to import a module if the importing module's in-scope type definitions do not include definitions for the type names that appear in variable declarations, function parameters, or function returns found in the imported module.

	static public final String XP0037 = "err:XQ0037"; 
	//It is a static error to import a module that contains function declarations or variable declarations whose names are already declared in the static context of the importing module.

	static public final String XP0038 = "err:XQ0038"; 
	//It is a static error if a query prolog specifies more than one default collation, or value specified does not identify a collation known to the implementation.

	static public final String XP0039 = "err:XQ0039"; 
	//It is an static error for a function declaration to have more than one parameter with the same name.

	static public final String XP0040 = "err:XQ0040"; 
	//It is a type error if the content sequence in an element constructor contains a namespace node node following a node that is not a namespace node.

	static public final String XP0041 = "err:XQ0041"; 
	//It is a dynamic error if the name expression in a computed processing instruction or computed namespace constructor returns a QName whose URI part is not empty.

	static public final String XP0042 = "err:XQ0042"; 
	//It is a static error if the enclosing expression of a computed namespace constructor is not a computed element constructor.

	static public final String XP0043 = "err:XQ0043"; 
	//It is a dynamic error if two or more computed namespace constructors within the same computed element constructor attempt to bind the same namespace prefix.

	static public final String XP0044 = "err:XQ0044"; 
	//It is a dynamic error if the name expression of a computed attribute constructor returns a QName that is in the namespace http://www.w3.org/TR/REC-xml-names (corresponding to namespace prefix xmlns).

	static public final String XP0045 = "err:XQ0045"; 
	//It is a static error if the declared function name in a function declaration has no namespace prefix or has one of the predefined namespace prefixes other than local.

	static public final String XP0046 = "err:XQ0046"; 
	//It is a static error if a string that is required to contain a valid URI does not contain a valid lexical form according to the definition of xs:anyURI in [XML Schema].

	static public final String XP0047 = "err:XQ0047"; 
	//It is a static error if the target URI (and location hint, if present) in a module import do not identify an available module.

	static public final String XP0048 = "err:XQ0048"; 
	//It is a static error if a function or variable declared in a library module is not in the target namespace of the library module.

	static public final String XP0049 = "err:XQ0049"; 
	//It is a static error if more than one variable declared or imported by a module has the same expanded QName.

	static public final String XP0050 = "err:XP0050"; 
	//It is a dynamic error if dynamic type of the operand of a treat expression does not match the type specified by the treat expression.

	static public final String XP0051 = "err:XP0051"; 
	//It is a static error if a QName that is used as an AtomicType in a SequenceType is not defined in the in-scope type definitions as an atomic type.

	static public final String XP0052 = "err:XQ0052"; 
	//It is a dynamic error if the content of an element or attribute constructor includes an atomic value that cannot be cast into a string, such as a value of type xs:QName or xs:NOTATION.

	static public final String XP0053 = "err:XQ0053"; 
	//It is a static error if the string literal used in a namespace declaration to specify a URI is a zero-length string.

	static public final String XP0054 = "err:XQ0054"; 
	//It is a dynamic error if the initializing expression in a variable declaration cannot be executed because of a circularity (for example, the expression depends on a function that in turn depends on the value of the initialized variable).

	static public final String XP0055 = "err:XP0055"; 
	//It is a static error if an ElementTest specifies a schema path that is not found in the in-scope schema definitions.


	// parser errors
	
}
