/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.typing;

import java.util.*;

import org.xquark.schema.*;
import org.xquark.xpath.Axis;
import org.xquark.xpath.NodeKind;
import org.xquark.xpath.StepExpr;
import org.xquark.xquery.parser.QName;
import org.xquark.xquery.parser.XQueryException;

public class ComputeSubTypeVisitor extends DefaultQTypeVisitor {

	private List historyList = null;
	private PrimeList resultList = null;
	private QType resultQType = null;
	private StepExpr step = null;
	private SchemaManager schemamanager = null;
	private byte axis = Axis.NONE;
	private byte kind = NodeKind.NONE;
	private QType ANYELT = null;
	private QType ANYATT = null;
	//	private QType STRING_0_1 = null;
	//	private QType STRING_1_1 = null;
	//	private QType STRING_0_N = null;
	private QType TEXT_0_1 = null;
	private QType TEXT_1_1 = null;
	private QType TEXT_0_N = null;

	public ComputeSubTypeVisitor() {
		historyList = new ArrayList();
		resultList = new PrimeList();
	}

	public void reset(byte contextType, StepExpr step, SchemaManager schemamanager) throws XQueryException {
		this.step = step;
		this.schemamanager = schemamanager;
		historyList.clear();
		resultList.clear();
		resultQType = null;
		axis = step.getAxis();
		kind = step.getKindTest();
		if (ANYELT == null)
			ANYELT = new QTypeElement(new QName("*", null, "*", null, null), schemamanager.getType(SchemaConstants.XMLSCHEMA_URI, TypeVisitor.SC_ANYTYPE), QType.OCC_0_N);
		if (ANYATT == null)
			ANYATT = new QTypeAttribute(new QName("*", null, "*", null, null), (SimpleType) schemamanager.getAnySimpleType());
		if (TEXT_0_1 == null)
			TEXT_0_1 = new QTypeText((SimpleType) schemamanager.getAnySimpleType(), QType.OCC_0_1);
		if (TEXT_1_1 == null)
			TEXT_1_1 = new QTypeText((SimpleType) schemamanager.getAnySimpleType(), QType.OCC_0_N);
		if (TEXT_0_N == null)
			TEXT_0_N = new QTypeText((SimpleType) schemamanager.getAnySimpleType(), QType.OCC_0_N);
	}

	public QType getResultQType() {
		return resultQType;
	}

	public void visit(QType arg) throws XQueryException {
		resultQType = null;
	}

	public void visit(QTypeText arg) throws XQueryException {
		resultQType = null;
		Type type = arg.getType();
		switch (kind) {
			case NodeKind.TEXT :
				// -> self::text()
				switch (axis) {
					case Axis.SELF :
						resultQType = arg;
						break;
						// not possible
					case Axis.CHILD :
					case Axis.DESCENDANT :
					case Axis.DESCENDANT_OR_SELF :
					case Axis.PARENT :
					case Axis.ANCESTOR :
					case Axis.ANCESTOR_OR_SELF :
					case Axis.ATTRIBUTE :
					case Axis.FOLLOWING :
					case Axis.FOLLOWING_SIBLING :
					case Axis.NAMESPACE :
					case Axis.PRECEDING :
					case Axis.PRECEDING_SIBLING :
						break;
				}
				break;
			case NodeKind.NODE :
				// -> self::node()
				// -> parent::node()
				switch (axis) {
					case Axis.SELF :
					case Axis.DESCENDANT_OR_SELF :
						resultQType = arg;
						break;
						// not supported
					case Axis.PARENT :
						break;
						// not possible
					case Axis.CHILD :
					case Axis.DESCENDANT :
					case Axis.ANCESTOR :
					case Axis.ANCESTOR_OR_SELF :
					case Axis.ATTRIBUTE :
					case Axis.FOLLOWING :
					case Axis.FOLLOWING_SIBLING :
					case Axis.NAMESPACE :
					case Axis.PRECEDING :
					case Axis.PRECEDING_SIBLING :
						break;
				}
				break;
			case NodeKind.ELEMENT :
				// -> parent::NT
				switch (axis) {
					// not supported
					case Axis.PARENT :
						break;
						// not possible
					case Axis.SELF :
					case Axis.CHILD :
					case Axis.DESCENDANT :
					case Axis.DESCENDANT_OR_SELF :
					case Axis.ANCESTOR :
					case Axis.ANCESTOR_OR_SELF :
					case Axis.ATTRIBUTE :
					case Axis.FOLLOWING :
					case Axis.FOLLOWING_SIBLING :
					case Axis.NAMESPACE :
					case Axis.PRECEDING :
					case Axis.PRECEDING_SIBLING :
						break;
				}
				break;
		}
	}

	public void visit(QTypeAttribute arg) throws XQueryException {
		resultQType = null;
		Type type = arg.getType();
		switch (kind) {
			case NodeKind.NODE :
				// -> self::node()
				// -> parent::node()
				switch (axis) {
					case Axis.SELF :
					case Axis.DESCENDANT_OR_SELF :
						resultQType = arg;
						break;
						// not supported
					case Axis.PARENT :
						break;
						// not possible
					case Axis.CHILD :
					case Axis.DESCENDANT :
					case Axis.ANCESTOR :
					case Axis.ANCESTOR_OR_SELF :
					case Axis.ATTRIBUTE :
					case Axis.FOLLOWING :
					case Axis.FOLLOWING_SIBLING :
					case Axis.NAMESPACE :
					case Axis.PRECEDING :
					case Axis.PRECEDING_SIBLING :
						break;
				}
				break;
			case NodeKind.ELEMENT :
				// -> parent::NT
				switch (axis) {
					// not supported
					case Axis.PARENT :
						break;
						// not possible
					case Axis.SELF :
					case Axis.CHILD :
					case Axis.DESCENDANT :
					case Axis.DESCENDANT_OR_SELF :
					case Axis.ANCESTOR :
					case Axis.ANCESTOR_OR_SELF :
					case Axis.ATTRIBUTE :
					case Axis.FOLLOWING :
					case Axis.FOLLOWING_SIBLING :
					case Axis.NAMESPACE :
					case Axis.PRECEDING :
					case Axis.PRECEDING_SIBLING :
						break;
				}
				break;
		}
	}

	//	public void visit(QTypeComment arg) throws XQueryException {
	//		visit((QTypeNode) arg);
	//	}

	public void visit(QTypeDocument arg) throws XQueryException {
		resultQType = null;
		if (arg.getQType() != null) {
			switch (kind) {
				case NodeKind.ATTRIBUTE :
				case NodeKind.TEXT :
					// error
					return;
				case NodeKind.NODE :
					// -> self::node()
					// -> child::node()
					// -> descendant-or-self::node()
					// -> descendant::node()
					switch (axis) {
						case Axis.SELF :
							resultQType = arg;
							break;
						case Axis.CHILD :
						case Axis.DESCENDANT :
							arg.getQType().accept(this);
							break;
						case Axis.DESCENDANT_OR_SELF :
							arg.getQType().accept(this);
							if (resultQType == null)
								resultQType = arg;
							else {
								if (resultQType instanceof QTypeUnion) {
									List list = ((QTypeUnion) resultQType).getPrimeList();
									list.add(arg);
									resultQType = new QTypeUnion(list, ((QTypeUnion) resultQType).getOccurence());
								} else {
									List list = new ArrayList(2);
									list.add(arg);
									list.add(resultQType);
								}
							}
							break;
					}
					break;
					//				case NodeKind.ELEMENT :
					//					// -> self::NT
					//					// -> child::NT
					//					// -> descendant-or-self::NT
					//					// -> descendant::NT
					//					arg.getQType().accept(this);
					//					break;
			}
		}
		if (resultQType != null)
			return;
		if (kind == NodeKind.ELEMENT) {
			String namespace = step.getNameSpace();
			String localname = step.getLocalName();
			if (namespace != null && namespace.equals("*")) {

				Collection schemas = schemamanager.getSchemas();
				for (Iterator it = schemas.iterator(); it.hasNext();) {
					Schema schema = (Schema) it.next();
					ElementDeclaration eltdecl = schema.getElementDeclaration(localname);
					if (eltdecl != null) {
						resultQType = new QTypeElement(new QName(namespace, null, localname, null, null), eltdecl, eltdecl.getType());
						break;
					}
				}
				if (resultQType == null && arg.isFromDocument())
					resultQType = new QTypeElement(new QName(namespace, null, localname, null, null), schemamanager.getType(SchemaConstants.XMLSCHEMA_URI, TypeVisitor.SC_ANYTYPE));

			} else {
				ElementDeclaration eltdecl = schemamanager.getElementDeclaration(namespace, localname);
				if (eltdecl != null)
					resultQType = new QTypeElement(new QName(namespace, null, localname, null, null), eltdecl.getType());
				else if (arg.isFromDocument())
					resultQType = new QTypeElement(new QName(namespace, null, localname, null, null), schemamanager.getType(SchemaConstants.XMLSCHEMA_URI, TypeVisitor.SC_ANYTYPE));
				//				}
			}
			//if (resultQType != null) arg.setQType(resultQType);
		} else {
			if (arg.getQType() == null)
				return;
			arg.getQType().accept(this);
		}
	}
	/* TODO
	 * typing should use anysimpletype not string
	 */
	public void visit(QTypeElement arg) throws XQueryException {
		resultQType = null;
		Type type = arg.getType();
		switch (kind) {
			case NodeKind.TEXT :
				// -> child::text()
				// -> descendant-or-self::text()
				// -> descendant::text()
				switch (axis) {
					// supported
					case Axis.CHILD :
					case Axis.DESCENDANT_OR_SELF :
						if (type.getValueType() != null) {
							resultQType = TEXT_0_1;
						} else if (type.getContentType() == Type.MIXED) {
							resultQType = TEXT_0_N;
						} else {
							resultQType = null;
						}
						// break if axis is child or resultQType is Multiple
						if (axis == Axis.CHILD || (resultQType != null && resultQType.isMultiple()))
							break;
					case Axis.DESCENDANT :
						// add current resultQType
						resultList.add(resultQType);
						// reset step axis
						axis = Axis.DESCENDANT_OR_SELF;
						Collection decls = type.getElementDeclarations();
						if (decls != null && !decls.isEmpty()) {
							Iterator it = decls.iterator();
							while (it.hasNext()) {
								// test stop condition
								Declaration decl = (Declaration) it.next();
								StringBuffer buf = new StringBuffer();
								if (decl.getNamespace() != null) {
									buf.append(decl.getNamespace());
									buf.append(":");
								}
								buf.append(decl.getName());
								// this is to handle recursions
								if (historyList.contains(buf.toString())) {
									continue;
								}
								historyList.add(buf.toString());
								QType tmpQType = null;
								try {
									tmpQType = getQTypeFromDeclaration(decl);
								} catch (XQueryException e) {
									// do nothing
								}
								if (tmpQType != null) {
									tmpQType.accept(this);
									resultList.add(resultQType);
								}
								historyList.remove(buf.toString());
								// to stop recursion
								if (resultQType.isMultiple())
									break;
								// erase   02/09/2003
								/*
								if (resultList.size() > 1) {
									resultQType = TEXT_0_N;
									break;
								}
								*/
							}
						}
						break;
						// not supported
						// not possible
					case Axis.ANCESTOR :
					case Axis.ANCESTOR_OR_SELF :
					case Axis.ATTRIBUTE :
					case Axis.FOLLOWING :
					case Axis.FOLLOWING_SIBLING :
					case Axis.NAMESPACE :
					case Axis.PARENT :
					case Axis.PRECEDING :
					case Axis.PRECEDING_SIBLING :
					case Axis.SELF :
						break;
				}
				break;
			case NodeKind.NODE :
				// -> self::node()
				// -> child::node()
				// -> descendant-or-self::node()
				// -> descendant::node()
				switch (axis) {
					// supported
					case Axis.SELF :
						resultQType = arg;
						break;
					case Axis.CHILD :
						ArrayList list = new ArrayList();
						if (type.getValueType() != null) {
							list.add(TEXT_0_1);
						} else if (((ComplexType) type).getContentType() == Type.MIXED) {
							list.add(TEXT_0_N);
						}
						Collection col = type.getElementDeclarations();
						if (col != null && !col.isEmpty()) {
							Iterator it = col.iterator();
							while (it.hasNext()) {
								ElementDeclaration eltDecl = (ElementDeclaration) it.next();
								try {
									QType tmpQType = getQTypeFromDeclaration(eltDecl);
									if (tmpQType != null && !list.contains(tmpQType))
										list.add(tmpQType);
								} catch (XQueryException e) {
									// do nothing
								}
							}
						} else {
							if (compareQNames(type.getNamespace(), type.getName(), SchemaConstants.XMLSCHEMA_URI, SchemaConstants.ANY_TYPE))
								list.add(new QTypeElement(arg.getName(), type));
						}
						if (!list.isEmpty()) {
							if (list.size() > 1)
								resultQType = new QTypeUnion(list);
							else
								resultQType = (QType) list.get(0);
						}
						break;
					case Axis.DESCENDANT_OR_SELF :
					case Axis.DESCENDANT :
						// make union of processing instruction, comment, element, text
						// TODO
						// test for complex type and mixed
						if (type.getValueType() != null) {
							resultList.add(TEXT_0_1);
						} else if (((ComplexType) type).getContentType() == Type.MIXED) {
							resultList.add(TEXT_0_N);
						}
						// add current type
						if (axis == Axis.DESCENDANT_OR_SELF)
							resultList.add(arg);
						// reset step axis
						axis = Axis.DESCENDANT_OR_SELF;
						// add test for anyType
						if (arg.getType().getName() != null && arg.getType().getName().equals(TypeVisitor.SC_ANYTYPE) && !resultList.contains(ANYELT)) {
							resultList.add(ANYELT);
						}
						// first get all elements
						Collection decls = type.getElementDeclarations();
						if (decls != null && !decls.isEmpty()) {
							Iterator it = decls.iterator();
							while (it.hasNext()) {
								Declaration decl = (Declaration) it.next();
								StringBuffer buf = new StringBuffer();
								if (decl.getNamespace() != null) {
									buf.append(decl.getNamespace());
									buf.append(":");
								}
								buf.append(decl.getName());
								if (historyList.contains(buf.toString())) { // this is to handle recursions
									continue;
								}
								historyList.add(buf.toString());
								QType tmpQType = null;
								try {
									tmpQType = getQTypeFromDeclaration(decl);
								} catch (XQueryException e) {
									// do nothing
								}
								if (tmpQType != null) {
									tmpQType.accept(this);
									resultList.add(resultQType);
								}
								historyList.remove(buf.toString());
							}
						}
						if (historyList.isEmpty()) {
							if (resultList.size() == 1)
								resultQType = (QType) resultList.get(0);
							else if (!resultList.isEmpty())
								resultQType = new QTypeUnion(resultList);
							resultList.clear();
						}
						break;
						// not supported
					case Axis.ANCESTOR :
					case Axis.ANCESTOR_OR_SELF :
					case Axis.ATTRIBUTE :
					case Axis.FOLLOWING :
					case Axis.FOLLOWING_SIBLING :
					case Axis.NAMESPACE :
					case Axis.PARENT :
					case Axis.PRECEDING :
					case Axis.PRECEDING_SIBLING :
						break;
						// not possible
				}
				break;
			case NodeKind.ATTRIBUTE :
				break;
			case NodeKind.ELEMENT :
				// -> self::NT
				// -> child::NT
				// -> descendant-or-self::NT
				// -> descendant::NT
				// -> attribute::NT
				String namespace = step.getNameSpace();
				String localname = step.getLocalName();
				Type argType = arg.getType();
				switch (axis) {
					// supported
					case Axis.ATTRIBUTE :
						if (argType.getName() != null && argType.getName().equals(TypeVisitor.SC_ANYTYPE)) {
							// TODO maybe test if the namespace exists ...
							resultQType = new QTypeAttribute(new QName(namespace, null, localname, null, null),schemamanager.getAnySimpleType());
							// TODO maybe clone is necessary ....
						} else {
							AttributeDeclaration attDecl = type.getAttributeDeclaration(namespace, localname);
							if (attDecl == null) {
								//if (namespace == null) namespace = "";
								if (localname == null)
									localname = "";
								Collection decls = type.getAttributeDeclarations();
								if (decls != null && !decls.isEmpty()) {
									Iterator it = decls.iterator();
									ArrayList list = new ArrayList();
									while (it.hasNext()) {
										AttributeDeclaration decl = (AttributeDeclaration) it.next();
										if (compareQNames(namespace, localname, decl.getNamespace(), decl.getName())) {
											QType tmpQType = null;
											try {
												tmpQType = getQTypeFromDeclaration(decl);
											} catch (XQueryException e) {
												// do nothing
											}
											if (tmpQType != null && !list.contains(tmpQType))
												list.add(tmpQType);
										}
									}
									if (!list.isEmpty())
										//if (list.size() > 1) obj = new QTypeSequence(list);
										if (list.size() > 1)
											resultQType = new QTypeUnion(list);
										else
											resultQType = (QType) list.get(0);
								}
							} else { //if (resultQType == null) {
								try {
									resultQType = this.getQTypeFromDeclaration((Declaration) attDecl);
								} catch (XQueryException e) {
									// do nothing
									resultQType = null;
								}
							}
						}
						break;
					case Axis.CHILD :
						if (type.getValueType() != null || type.getContentType() == Type.EMPTY)
							break;
						Utils.getSubType((Particle) ((ComplexType) type).getContentModel().getModel(), namespace, localname, schemamanager);
						resultQType = Utils.getOccurrenceVisitor().getQType();
						break;
					case Axis.DESCENDANT_OR_SELF :
					case Axis.SELF :
						QName argqname = (QName) arg.getName();
						if (compareQNames(namespace, localname, argqname.getNameSpace(), argqname.getLocalName())) {
							resultQType = new QTypeElement(new QName(namespace, null, localname, null, null), argType, QType.OCC_0_N);
						}
						if (axis == Axis.SELF)
							break;
						// add current to history
						resultList.add(resultQType);
					case Axis.DESCENDANT :
						// reset step axis
						axis = Axis.DESCENDANT_OR_SELF;
						// iterate on sub elements
						Collection decls = type.getElementDeclarations();
						if (decls != null && !decls.isEmpty()) {
							Iterator it = decls.iterator();
							while (it.hasNext()) {
								Declaration decl = (Declaration) it.next();
								StringBuffer buf = new StringBuffer();
								if (decl.getNamespace() != null) {
									buf.append(decl.getNamespace());
									buf.append(":");
								}
								buf.append(decl.getName());
								if (historyList.contains(buf.toString())) { // this is to handle recursions
									continue;
								}
								historyList.add(buf.toString());
								QType tmpQType = null;
								try {
									tmpQType = getQTypeFromDeclaration(decl);
								} catch (XQueryException e) {
									// do nothing
								}
								if (tmpQType != null) {
									tmpQType.accept(this);
									resultList.add(resultQType);
								}
								historyList.remove(buf.toString());
							}
						}
						if (historyList.isEmpty()) {
							if (resultList.size() == 1)
								resultQType = (QType) resultList.get(0);
							else if (!resultList.isEmpty())
								resultQType = new QTypeUnion(resultList);
							resultList.clear();
						}
						break;
						// not supported
					case Axis.ANCESTOR :
					case Axis.ANCESTOR_OR_SELF :
					case Axis.FOLLOWING :
					case Axis.FOLLOWING_SIBLING :
					case Axis.NAMESPACE :
					case Axis.PARENT :
					case Axis.PRECEDING :
					case Axis.PRECEDING_SIBLING :
						break;
				}
				break;
			case NodeKind.COMMENT :
				// -> self::comment()
				// -> child::comment()
				// -> descendant-or-self::comment()
				// -> descendant::comment()
				resultQType = new QTypeComment(QType.OCC_0_N);
				break;
			case NodeKind.PI :
				// -> self::processing-intruction()
				// -> child::processing-intruction()
				// -> descendant-or-self::processing-intruction()
				// -> descendant::processing-intruction()
				resultQType = new QTypeProcessingInstruction();
				break;
				// not supported
			case NodeKind.DOCUMENT :
			case NodeKind.NAMESPACE :
				break;
		}
	}
	//	public void visit(QTypeProcessingInstruction arg) throws XQueryException {
	//		visit((QTypeNode) arg);
	//	}

	public void visit(QTypeSequence arg) throws XQueryException {
		resultQType = null;
		ArrayList newList = new ArrayList();
		Iterator it = arg.getPrimeList().iterator();
		while (it.hasNext()) {
			QType qtype = (QType) it.next();
			qtype.accept(this);
			if (resultQType != null) {
				newList.add(resultQType);
			}
		}
		if (!newList.isEmpty()) {
			if (newList.size() == 1)
				resultQType = (QType) newList.get(0);
			else
				resultQType = new QTypeSequence(newList);
		}
	}

	public void visit(QTypeUnion arg) throws XQueryException {
		resultQType = null;
		ArrayList newList = new ArrayList();
		Iterator it = arg.getPrimeList().iterator();
		while (it.hasNext()) {
			QType qtype = (QType) it.next();
			qtype.accept(this);
			if (resultQType != null && !newList.contains(resultQType)) {
				newList.add(resultQType);
			}
		}
		if (!newList.isEmpty()) {
			if (newList.size() == 1)
				resultQType = (QType) newList.get(0);
			else
				resultQType = new QTypeUnion(newList);
		}
	}
	//	public void visit(QTypeNode arg) throws XQueryException {
	//		visit((QTypeItem) arg);
	//	}
	//	public void visit(QTypePrime arg) throws XQueryException {
	//		visit((QType) arg);
	//	}
	//	public void visit(QTypeItem arg) throws XQueryException {
	//		visit((QType) arg);
	//	}

	// private methods
	private QType getQTypeFromDeclaration(Declaration decl) throws XQueryException {
		if (decl == null)
			throw new TypeException("Expression not found in metadata");
		QType retQType = null;
		QName qname = new QName(decl.getNamespace(), null, decl.getName(), null, null);
		if (decl instanceof AttributeDeclaration)
			retQType = new QTypeAttribute(qname, (AttributeDeclaration)decl, (SimpleType) decl.getType());
		if (decl instanceof ElementDeclaration) {
			boolean multi = false;
			ElementDeclaration eltDecl = (ElementDeclaration) decl;
			SchemaScope scopeType = eltDecl.getScope();
			if (scopeType != null && scopeType instanceof ComplexType) {
				int size[] = { 0, 0 };
				size = ((ComplexType) scopeType).getDeclarationOccurrence(eltDecl.getNamespace(), eltDecl.getName());
				if (size != null && size[1] > 1)
					multi = true;
			}
			retQType = new QTypeElement(qname, decl.getType(), multi ? QType.OCC_0_N : QType.OCC_1_1);
		}
		return retQType;
	}

	private boolean compareQNames(String nameSpace1, String localName1, String nameSpace2, String localName2) {
		// conditions 
		// ( NS1 == * || NS2 == * || ( NS1 empty && NS2 empty) || NS1 == NS2 ) && LN1 != null && LN2 != null && (LN1 = LN2 || LN1 = * || LN2 = *)
		boolean sameNS = false;
		if (!sameNS && (nameSpace1 == null || nameSpace1.length() == 0) && (nameSpace2 == null || nameSpace2.length() == 0))
			sameNS = true;
		if (!sameNS && nameSpace1 != null && (nameSpace1.equals("*") || nameSpace1.equals(nameSpace2)))
			sameNS = true;
		if (!sameNS && nameSpace2 != null && nameSpace2.equals("*"))
			sameNS = true;
		if (sameNS && localName1 != null && localName1 != null && (localName1.equals(localName2) || localName1.equals("*") || localName2.equals("*")))
			return true;
		return false;
	}
}
