/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.typing;

import org.xquark.xquery.parser.XQueryException;

public class QTypeDocument extends QTypeNode implements QTypeVisitable {
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	public static final byte ANY_ORIGIN = 0;
	public static final byte COLLECTION_ORIGIN = 1;
	public static final byte DOCUMENT_ORIGIN = 2;

	private QType qtype = null;
	private byte origin = ANY_ORIGIN;

	public QTypeDocument(QType qtype, byte origin) {
		this.qtype = qtype;
		subclass = DOCUMENT;
		occurence = OCC_1_1;
		this.origin = origin; 
	}
	public QTypeDocument(QType qtype, byte occurence, byte origin) {
		this.qtype = qtype;
		subclass = DOCUMENT;
		this.occurence = occurence;
		this.origin = origin;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(QTypeVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// get/set
	public QType getQType() {
		return qtype;
	}

	public void setQType(QType qtype) throws TypeException {
		this.qtype = qtype;
	}

	//	public Type getType() {
	//		return qtype.getType();
	//	}

	// equals

	public boolean equals(Object type) {
		if (!(type instanceof QTypeDocument))
			return false;
//		if (((QTypeDocument) qtype).getOccurence() != occurence)
//			return false;
		return qtype.equals(((QTypeDocument) type).getQType());
	}

	// tells whether qtype is a node
	public boolean isNode() {
		if (qtype == null)
			return false;
		return qtype.isNode();
	}

	// tells whether qtype is a node with simple type
	public boolean isSimpleTypeNode() {
		if (qtype == null)
			return false;
		return qtype.isSimpleTypeNode();
	}

	// tells whether qtype is a collection
	public boolean isCollection() {
		return false;
	}

	// tells whether qtype is a document
	public boolean isDocument() {
		return true;
	}

	// tells whether qtype is multiple
//	public boolean isMultiple() {
//		return false;
//	}

	// tells whether qtype is mixed
	public boolean isMixed() {
		if (qtype == null)
			return false;
		return qtype.isMixed();
	}

	// tells whether qtype is element
	public boolean isElement() {
		if (qtype == null)
			return false;
		return qtype.isElement();
	}

	public boolean canBeComparedTo(QType compType) {
		if (compType == null)
			return false;
		if (!(compType instanceof QTypeDocument))
			return qtype.canBeComparedTo(compType);
		return qtype.canBeComparedTo(((QTypeDocument) compType).getQType());
	}

	//public Object getSubType(String namespace, String localname, boolean isAttribute, boolean inDepth, ArrayList history) throws TypeException {
	// Olivier: 2002/11/19
	//	public Object getSubType(QName qname, int axis, boolean inDepth, ArrayList history, ArrayList reslist, SchemaManager schemamanager) throws TypeException {
	//		if (qtype == null)
	//			return null;
	//		Object obj = qtype.getSubType(qname, axis, inDepth, history, reslist, schemamanager);
	//		return obj;
	//	}
	//
	//	public QType applyTEXTFunction(boolean inDepth, ArrayList history, SchemaManager schemamanager) throws TypeException {
	//		if (qtype == null)
	//			return null;
	//		QType type = qtype.applyTEXTFunction(inDepth, history, schemamanager);
	//		if (type != null)
	//			return new QTypeDocument(type);
	//		return null;
	//	}
	//
	
	public QType applyDATAFunction() throws TypeException {
		if (qtype == null)
			return null;
		return qtype.applyDATAFunction();
	}
		
	//
	//	public QType applyNODEFunction(int axis, ArrayList history, ArrayList reslist, SchemaManager schemamanager) throws TypeException {
	//		if (qtype == null)
	//			return null;
	//		QType type = qtype.applyNODEFunction(axis, history, reslist, schemamanager);
	//		if (type != null)
	//			return new QTypeDocument(type);
	//		return null;
	//	}

	// get origin
	public boolean isFromDocument() {
		return (origin == DOCUMENT_ORIGIN);
	}
	// clone
	public Object clone() throws CloneNotSupportedException {
		QTypeDocument newObj = new QTypeDocument(qtype, occurence, origin);
		return newObj;
	}
	// print
	public String toSimpleString() {
		return "QTypeDocument(" + qtype + ")";
	}

	public String toString() {
		return "QTypeDocument(" + qtype + ")";
	}

}
