/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.typing;

import java.util.ArrayList;

import org.xquark.schema.SchemaConstants;
import org.xquark.schema.SchemaManager;
import org.xquark.schema.SimpleType;
import org.xquark.xquery.parser.XQueryException;

public class QTypeText extends QTypeNode implements QTypeVisitable {
	private static final String RCSRevision = "$Revision: 1.8 $";
	private static final String RCSName = "$Name: XQ1_1_1 $";

	public QTypeText(SimpleType simpleType) {
		setType(simpleType);
		subclass = TEXT;
		occurence = OCC_1_1;
	}
	public QTypeText(SimpleType simpleType, byte occurence) {
		setType(simpleType);
		subclass = TEXT;
		this.occurence = occurence;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(QTypeVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// get/set
	//    public SimpleType getSimpleType() {
	//        return simpleType;
	//    }

	// equals
	public boolean equals(Object qtype) {
		if (!(qtype instanceof QTypeText))
			return false;
//		if (((QTypeText) qtype).getOccurence() != occurence)
//			return false;
		return compare(type, ((QTypeText) qtype).getSimpleType());
	}

	// tells whether qtype is numeric
	public boolean isNumeric() {
		if (type == null)
			return false;
		else
			return isNumeric(type);
	}

	// tells whether qtype is boolean
	public boolean isBoolean() {
		if (type == null)
			return false;
		else
			return isBoolean(type);
	}

	// tells whether qtype is integer
	public boolean isInteger() {
		if (type == null)
			return false;
		else
			return isInteger(type);
	}

	// tells whether qtype is string
	public boolean isString() {
		if (type == null || isMultiple())
			return false;
		else
			return isString(type);
	}

	//    // tells whether qtype is an atom
	//    public boolean isAtom() { return false; }

//	// tells whether qtype can react to text()
//	public boolean isText() {
//		return true;
//	}

	// tells whether qtype is a date or collection of dates
	public boolean isDate() {
		if (type == null)
			return false;
		else
			return isDate(type);
	}

	// tells whether qtype is a attribte or collection or sequence
	public boolean isAttribute() {
		return false;
	}

//	// tells whether qtype is a ID, IDREF or IDREFS type
//	public boolean isIDAttribute() {
//		if (type == null)
//			return false;
//		if (type.getNamespace().equals(SchemaConstants.XMLSCHEMA_URI) && (type.getName().equals("ID") || type.getName().equals("IDREF") || type.getName().equals("IDREFS")))
//			return true;
//		return false;
//	}

	// tells whether qtype is a node
	public boolean isNode() {
		return true;
	}

	// tells whether qtype is a node with simple type
	public boolean isSimpleTypeNode() {
		return true;
	}

	// tells whether qtype is atomic (atom or element with simple content
	public boolean isAtomic() {
		return !isMultiple();
	}

	public boolean canBeComparedTo(QType compType) {
		if (compType == null)
			return false;
		if (!(compType instanceof QTypeText))
			return compType.canBeComparedTo(this);
		if (type == null)
			return true;
		if (((QTypeText) compType).getSimpleType() == null)
			return true;
		return canBeCompared(((SimpleType)type).getPrimitive().getType(), ((QTypeText) compType).getSimpleType().getPrimitive().getType());
	}

	public QType applyTEXTFunction(boolean inDepth, ArrayList history, SchemaManager schemamanager) throws TypeException {
		return new QTypeAtom((SimpleType) schemamanager.getType(SchemaConstants.XMLSCHEMA_URI, TypeVisitor.SC_STRING),occurence);
	}

	public QType applyDATAFunction() throws TypeException {
		return new QTypeAtom((SimpleType)type,occurence);
	}

	public QType applyFILTERFunction() throws TypeException {
		return this;
	}

	// clone
	public Object clone() throws CloneNotSupportedException {
		QTypeText newObj = new QTypeText((SimpleType)type,occurence);
		return newObj;
	}
	// print
	public String toSimpleString() {
		return "QTypeText(" + type.getName() + ")";
	}

	public String toString() {
		return "QTypeText(" + type + ")";
	}

}
