/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Odonata.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.typing;

import org.xquark.schema.Particle;
import org.xquark.schema.SchemaException;
import org.xquark.schema.SchemaManager;
import org.xquark.xpath.StepExpr;
import org.xquark.xquery.parser.XQueryException;

public class Utils {

	private static ComputeSubTypeVisitor subTypeVisistor = null;

	public static QType getSubType(byte contextType, QType qtype, StepExpr step, SchemaManager schemamanager, boolean inPredicate) throws TypeException, XQueryException {
		if (subTypeVisistor == null)
			subTypeVisistor = new ComputeSubTypeVisitor();
		subTypeVisistor.reset(contextType, step, schemamanager);
		try {
			qtype.accept(subTypeVisistor);
		} catch (XQueryException e) {
			throw new TypeException(e.getMessage());
		}
		QType result = subTypeVisistor.getResultQType();
		if (result != null && !inPredicate)
			result = result.changeOccurence(QType.computeOccurence(qtype.getOccurence(), result.getOccurence()));
		return result;
	}

	private static OccurrenceVisitor occurrenceVisistor = null;

	public static OccurrenceVisitor getOccurrenceVisitor() {
		return occurrenceVisistor;
	}

	public static void getSubType(Particle particle, String namespace, String name, SchemaManager schemamanager) throws TypeException {
		if (occurrenceVisistor == null)
			occurrenceVisistor = new OccurrenceVisitor();
		occurrenceVisistor.reset(namespace, name, schemamanager);
		try {
			particle.accept(occurrenceVisistor);
		} catch (SchemaException e) {
			throw new TypeException(e.getMessage());
		}
	}

}
