/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.xdbc;

import java.util.*;

import org.xml.sax.helpers.AttributesImpl;
import org.xquark.util.NamespaceContextStack;
import org.xquark.util.XMLDocumentFragmentor;
import org.xquark.xml.xdbc.*;
import org.xquark.xpath.XTreeReader;
import org.xquark.xquery.metadata.MetaModule;
import org.xquark.xquery.parser.FunctionDeclaration;
import org.xquark.xquery.parser.Variable;

/**
 * Implementation of XMLDataSourceMetaData for the method getMetaData()
 */
public abstract class AbstractXMLDataSourceMetaData implements XMLDataSourceMetaData {
    private static final String RCSRevision = "$Revision: 1.5 $";

    private static final String RCSName = "$Name: XQ1_1_1 $";

    public static final String METADATA_ELEMENT_NAME = "metadata";

    public static final String SOURCE_ATTR = "source";

    public static final String SCHEMAS_ELEMENT_NAME = "schemas";

    public static final String COLLECTIONS_ELEMENT_NAME = "collections";
    public static final String VARIABLE_ELEMENT_NAME = "variable";

    public static final String FUNCTION_ELEMENT_NAME = "function";

    public static final String MODULES_ELEMENT_NAME = "modules";

    public static final String XQUERYTYPE_ELEMENT_NAME = "xquerytype";

    public static final String MODULE_ELEMENT_NAME = "module";

    public static final String PARAMETERS_ELEMENT_NAME = "parameters";

    public static final String PARAMETER_ELEMENT_NAME = "parameter";

    public static final String NAME_ATTR = "name";

    public static final String TARGETNAMESPACE_ATTR = "targetNamespace";

    public static final String[] PROPERTY_LIST = { "xmldbc.maxConnections", "xmldbc.maxStatementLength", "xmldbc.maxActiveStatements", "xmldbc.maxResultSetLength", "xmldbc.maxUserNameLength", "xmldbc.dataSourceVersion", "xmldbc.vendor.name", "xmldbc.vendor.product", "xmldbc.vendor.version", "xmldbc.driver.version", "xmldbc.driver.product", "xmldbc.xpath", "xmldbc.xpath.version", "xmldbc.xquery", "xmldbc.xquery.version", "xmldbc.xquery.outerJoin", "xmldbc.twoPhaseCommit", "xmldbc.xquery.equality", "xmldbc.xquery.result", "xmldbc.xmlCollection", "xmldbc.xmlCollection.id", "xmldbc.xmlCollection.iterator", "xmldbc.xquery.multiCollection" };

    XMLConnection connection;

    protected AbstractXMLDataSourceMetaData(XMLConnection connection) {
        this.connection = connection;
    }

    public java.util.Set getCapabilities() {
        return Collections.EMPTY_SET;
    }

    public XMLDocument getMetaData() throws XMLDBCException {
        return new MetaDataDocument();
    }

    /**
     * To get the XMLConnection object that produced this XMLDataSourceMetaData object.
     * 
     * @return the XMLConnection object that produced this object.
     * @throws XMLDBCException
     *             if a data source access error occurs.
     */
    public XMLConnection getConnection() throws XMLDBCException {
        return connection;
    }

    public void setConnection(XMLConnection connection) throws XMLDBCException {
        this.connection = connection;
    }

    public String[] getPropertyList() {
        return PROPERTY_LIST;
    }

    ///////////////////////////////////////////////////////////////////////////
    //                          INNER CLASSES
    ///////////////////////////////////////////////////////////////////////////
    private class MetaDataDocument extends AbstractXMLDocument {
        private static final String RCSRevision = "$Revision: 1.5 $";

        private static final String RCSName = "$Name: XQ1_1_1 $";

        private static final String METADATA_ID = "XMLDBC DataSource Metadata";

        public MetaDataDocument() {
            super(METADATA_ID, null, true);
        }

        public void remove() throws XMLDBCException, XMLDBCNotSupportedException {
            throw new XMLDBCNotSupportedException("This document is read-only");
        }

        public void getAsSAX() throws XMLDBCException, org.xml.sax.SAXException {
            AttributesImpl atts = new AttributesImpl();

            // start doc
            contentHandler.startDocument();

            // start root element
            contentHandler.startPrefixMapping(XTreeReader.SOURCE_METADATA_PREFIX, XTreeReader.SOURCE_METADATA_NAMESPACE);
            atts.addAttribute("", SOURCE_ATTR, "", "CDATA", connection == null ? "" : connection.getURL());
            contentHandler.startElement(XTreeReader.SOURCE_METADATA_NAMESPACE, METADATA_ELEMENT_NAME, "", atts);
            atts.clear();

            // serialize schemas
            contentHandler.startElement("", SCHEMAS_ELEMENT_NAME, "", atts);

            XMLDocumentFragmentor fragmentor = new XMLDocumentFragmentor(contentHandler, lexicalHandler);

            Iterator it = AbstractXMLDataSourceMetaData.this.getSchemaNamespaces().iterator();
            XMLDocument fragment;

            while (it.hasNext()) {
                fragment = AbstractXMLDataSourceMetaData.this.getSchema((String) it.next());
                if (fragment == null)
                    continue;
                fragment.setContentHandler(fragmentor);
                fragment.setLexicalHandler(fragmentor);
                fragment.getAsSAX();
            }

            contentHandler.endElement("", SCHEMAS_ELEMENT_NAME, "");

            // serialize path sets
            //			contentHandler.startElement("", COLLECTIONS_ELEMENT_NAME, "", atts);
            //			
            //			it = AbstractXMLDataSourceMetaData.this.getCollectionNames().iterator();
            //			
            //			while (it.hasNext()) {
            //				fragment = AbstractXMLDataSourceMetaData.this.getPathSet((String)it.next());
            //				if (fragment == null)
            //					continue ;
            //				fragment.setContentHandler(fragmentor);
            //				fragment.setLexicalHandler(fragmentor);
            //				fragment.getAsSAX();
            //			}
            //			
            //			contentHandler.endElement("", COLLECTIONS_ELEMENT_NAME, "");

            List list = AbstractXMLDataSourceMetaData.this.getMetaModules();
            if (!list.isEmpty()) {
                contentHandler.startElement("", MODULES_ELEMENT_NAME, "", atts);
                for (int i = 0; i < list.size(); i++) {
                    MetaModule module = (MetaModule) list.get(i);

                    if (module.getDeclarations() != null) {
                        Collection declarationsPrefixes = module.getDeclarations().getPrefixes();
                        Iterator prefixes = declarationsPrefixes.iterator();
                        while (prefixes.hasNext()) {
                            String prefix = (String) prefixes.next();
                            String uri = (String) module.getDeclarations().getNamespaceURI(prefix);
                            contentHandler.startPrefixMapping(prefix, uri);
                        }
                    }

                    atts.addAttribute("", TARGETNAMESPACE_ATTR, "", "CDATA", module.getNamespace());
                    contentHandler.startElement("", MODULE_ELEMENT_NAME, "", atts);

                    atts.clear();
                    List tmplist = module.getDeclList();
                    if (tmplist != null) {
                        for (int j = 0; j < tmplist.size(); j++) {
                            Object obj = tmplist.get(j);
                            if (obj instanceof Variable) {
                                Variable var = (Variable) obj;
                                if (var.getVarName().getNameSpace().equals(module.getNamespace())) {
                                    atts.addAttribute("", NAME_ATTR, "", "CDATA", var.getVarName().getStringValue(module.getDeclarations()));
                                    contentHandler.startElement("", VARIABLE_ELEMENT_NAME, "", atts);
                                    atts.clear();
                                    contentHandler.startElement("", XQUERYTYPE_ELEMENT_NAME, "", atts);
                                    String varTypeStr = var.getTypeDeclaration().getStringValue(module.getDeclarations());
                                    contentHandler.characters(varTypeStr.toCharArray(), 0, varTypeStr.length());
                                    contentHandler.endElement("", XQUERYTYPE_ELEMENT_NAME, "");
                                    contentHandler.endElement("", VARIABLE_ELEMENT_NAME, "");
                                }
                            } else if (obj instanceof FunctionDeclaration) {
                                FunctionDeclaration funDecl = (FunctionDeclaration) obj;
                                if (funDecl.getFuncName().getNameSpace().equals(module.getNamespace())) {
                                    atts.addAttribute("", NAME_ATTR, "", "CDATA", funDecl.getFuncName().getStringValue(module.getDeclarations()));
                                    contentHandler.startElement("", FUNCTION_ELEMENT_NAME, "", atts);
                                    atts.clear();
                                    contentHandler.startElement("", XQUERYTYPE_ELEMENT_NAME, "", atts);
                                    String funcTypeStr = funDecl.getReturnType().getStringValue(module.getDeclarations());
                                    contentHandler.characters(funcTypeStr.toCharArray(), 0, funcTypeStr.length());
                                    contentHandler.endElement("", XQUERYTYPE_ELEMENT_NAME, "");
                                    ArrayList params = funDecl.getParameters();
                                    if (params != null && !params.isEmpty()) {
                                        contentHandler.startElement("", PARAMETERS_ELEMENT_NAME, "", atts);
                                        for (int k = 0; k < params.size(); k++) {
                                            Variable vark = (Variable) params.get(k);
                                            atts.addAttribute("", NAME_ATTR, "", "CDATA", vark.getVarName().getStringValue(module.getDeclarations()));
                                            contentHandler.startElement("", PARAMETER_ELEMENT_NAME, "", atts);
                                            atts.clear();
                                            contentHandler.startElement("", XQUERYTYPE_ELEMENT_NAME, "", atts);
                                            String varTypeStr = vark.getTypeDeclaration().getStringValue(module.getDeclarations());
                                            contentHandler.characters(varTypeStr.toCharArray(), 0, varTypeStr.length());
                                            contentHandler.endElement("", XQUERYTYPE_ELEMENT_NAME, "");
                                            contentHandler.endElement("", PARAMETER_ELEMENT_NAME, "");
                                        }
                                        contentHandler.endElement("", PARAMETERS_ELEMENT_NAME, "");
                                    }
                                    contentHandler.endElement("", FUNCTION_ELEMENT_NAME, "");
                                }
                            }
                        }
                    }
                    contentHandler.endElement("", MODULE_ELEMENT_NAME, "");

                    if (module.getDeclarations() != null) {
                        Collection declarationsPrefixes = module.getDeclarations().getPrefixes();
                        Iterator prefixes = declarationsPrefixes.iterator();
	                    while (prefixes.hasNext()) {
	                        String prefix = (String) prefixes.next();
	                        contentHandler.endPrefixMapping(prefix);
	                    }
                    }

                }
                contentHandler.endElement("", MODULES_ELEMENT_NAME, "");
            }

            
			// serialize path sets
			contentHandler.startElement("", COLLECTIONS_ELEMENT_NAME, "", atts);
			
			List collectionList = AbstractXMLDataSourceMetaData.this.getCollectionNames();
			if (collectionList != null) {
				it = AbstractXMLDataSourceMetaData.this.getCollectionNames().iterator();
				
				while (it.hasNext()) {
					fragment = AbstractXMLDataSourceMetaData.this.getPathSet((String)it.next());
					if (fragment == null)
						continue ;
					fragment.setContentHandler(fragmentor);
					fragment.setLexicalHandler(fragmentor);
					fragment.getAsSAX();
				}
			}
			
			contentHandler.endElement("", COLLECTIONS_ELEMENT_NAME, "");
            
            // end root element
            contentHandler.endElement(XTreeReader.SOURCE_METADATA_NAMESPACE, METADATA_ELEMENT_NAME, "");
            contentHandler.endPrefixMapping(XTreeReader.SOURCE_METADATA_PREFIX);

            // end doc
            contentHandler.endDocument();
        }

    }
}