/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

package org.xquark.xquery.xdbc;

import java.io.IOException;

import org.xml.sax.XMLReader;
import org.xquark.xml.xdbc.XMLCollection;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xpath.XTree;
import org.xquark.xpath.XTreeReader;

public class PathSetXMLDocument extends AbstractXMLDocument {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    private XTree pathSet;
    
    public PathSetXMLDocument(String collectionName, XTree pathSet)
    {
        super(collectionName, true);
        this.pathSet = pathSet;
    }
    
    public PathSetXMLDocument(String collectionName, XMLCollection collection, XTree pathSet)
    {
        super(collectionName, collection, true);
        this.pathSet = pathSet;
    }
    
    /**
     * Returns the document content as a SAX2 event flow.
     * This method uses the various SAX2 handlers set by the setXXXHandler() methods.
     * <P>*** ONLY FOR SAX2 ***</P>
     * @throws XMLDBCException if a data source access error occurs
     * or if the user has not set a content handler.
     * @throws org.xml.sax.SAXException if a handler exception occurs.
     */
    public void getAsSAX() throws XMLDBCException, org.xml.sax.SAXException
    {
        XMLReader reader = new XTreeReader(pathSet);
        plugHandlers(reader);
        try {
            reader.parse(getIdentifier());
        }
        catch (IOException e) {
            // TO IMPROVE
            throw new XMLDBCException("IOException while reading document.", e);
        }
    }
    
}
