/*
 * This file belongs to the XQuare distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquare.org.
 */

/*
 * XMLDocumentImpl.java
 *
 * Created on 12 juin 2002, 14:46
 */

package org.xquark.xquery.xdbc;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLResultSet;

public class XMLDocumentImpl extends AbstractXMLDocument
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name: XQ1_1_1 $";
    
    private XMLResultSet xmlResultSet = null;
    private String namespace = null;
    private String localName = null;
    private String qName = null;
    
    /** Creates a new instance of XMLDocumentImpl */
    public XMLDocumentImpl(XMLResultSet xmlResultSet, String namespace, String localName, String qName)
    {
        super(true);
        this.xmlResultSet = xmlResultSet;
        this.namespace = namespace;
        this.localName = localName;
        this.qName = qName;
    }
    
    /**
     * Returns the document content as a SAX2 event flow.
     * This method uses the various SAX2 handlers set by the setXXXHandler() methods.
     * <P>*** ONLY FOR SAX2 ***</P>
     * @throws XMLDBCException if a data source access error occurs
     * or if the user has not set a content handler.
     * @throws org.xml.sax.SAXException if a handler exception occurs.
     */
    public void getAsSAX() throws XMLDBCException, SAXException
    {
        if (contentHandler != null)
            xmlResultSet.setContentHandler(contentHandler);
        if (errorHandler != null)
            xmlResultSet.setErrorHandler(errorHandler);
        if (lexicalHandler != null)
         xmlResultSet.setLexicalHandler(lexicalHandler);
        contentHandler.startDocument();
        contentHandler.startElement(namespace, localName, qName, new AttributesImpl());
        while (xmlResultSet.hasNext())
            xmlResultSet.nextAsSAX();
        contentHandler.endElement(namespace, localName, qName);
        contentHandler.endDocument();
    }
    
}
